FUNCTION GET_BAND_STRUCT, shape, $
                          nu_c=nu_c, $
                          nu_min=nu_min, nu_max=nu_max, $
                          npts=npts, nutab=nutab, value=value, $
                          instr=instr, version=version, channel=channel, $
                          help=help
; set nu_c for band of type 'DIRAC' (default 0., to be filled in later on)
; set nu_min and nu_max for bands of type 'TOPHAT' (default 0., to be filled in later on)
; set either npts or (nu and value) for bands of type 'TABLE'

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'GET_BAND_STRUCT: function to get a psm band structure'
     PSM_INFO_MESSAGE, 'SYNTAX: result = GET_BAND_STRUCT(shape, nu_c=, nu_min=, nu_max=, npts=, nutab=, value=, instr=, version=, channel=)'
     PSM_INFO_MESSAGE, "'shape' can be either 'DIRAC' or 'TOPHAT' or 'TABLE' or 'INSTR'"
     PSM_INFO_MESSAGE, "set nu_c for band of type 'DIRAC' (default 0., to be filled in later on)"
     PSM_INFO_MESSAGE, "set nu_min and nu_max for bands of type 'TOPHAT' (default 0., to be filled in later on)"
     PSM_INFO_MESSAGE, "set version and channel for bands of type 'INSTR'"
     PSM_INFO_MESSAGE, "set either npts or (nutab and value) for bands of type 'TABLE'"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF

  CASE shape OF
     'DIRAC': BEGIN
        IF NOT KEYWORD_SET(nu_c) THEN nu_c=0.
        band =  { shape: 'DIRAC', $              ; 
                  nu_c: nu_c, $                  ; 
                  integ_dnu: 1d0 $               ;
                }
     END
     'TOPHAT':BEGIN
        IF NOT KEYWORD_SET(nu_min) THEN nu_min=0.
        IF NOT KEYWORD_SET(nu_max) THEN nu_max=0.
        nu_c = (nu_min + nu_max)/2.
        band =  { shape: 'TOPHAT', $             ;
                  nu_c: nu_c, $                  ; 
                  nu_min: nu_min, $              ;
                  nu_max: nu_max, $              ;
                  integ_dnu: nu_max-nu_min $     ;
                } 
     END
     'TABLE':BEGIN
        IF NOT KEYWORD_SET(nutab) THEN BEGIN
           IF NOT DEFINED(npts) THEN npts=1
           nutab = FLTARR(npts)
        ENDIF
        IF NOT KEYWORD_SET(value) THEN value = FLTARR(npts)
        IF NOT KEYWORD_SET(npts) THEN npts = N_ELEMENTS(nutab)
        band = { shape: 'TABLE', $               ;
                 nu: nutab, $                    ;
                 value: value, $                 ; 
                 integ_dnu: 0d0 $                ;
                 }
        IF ((KEYWORD_SET(value) NE 0) AND (KEYWORD_SET(nutab) NE 0)) THEN band.integ_dnu = INT_TABULATED(band.nu, band.value)
     END
     'INSTR': BEGIN
        IF NOT KEYWORD_SET(instr) THEN instr=''
        IF NOT KEYWORD_SET(channel) THEN channel=''
        IF NOT KEYWORD_SET(version) THEN version=''
        band = { shape:'INSTR', $
                 instr:instr, $
                 version:version, $
                 channel:STRTRIM(STRING(channel),2), $
                 data:PTR_NEW() $
               }
     END
     'UNKNOWN': BEGIN
        band = {shape:'UNKNOWN'}
     END
     ELSE: MESSAGE, 'Band shape not implemented'
  ENDCASE

  RETURN, band

END
