module healmod

contains

subroutine asciicambcl2fitscl(pol,lm,llm,cambscalclfile,camblensedclfile,cmbclthfile,potclthfile,lensedcmbclthfile)

  use healpix_types
  use head_fits
  use fitstools

  implicit none

  integer(i4b), intent(in) :: lm,llm,pol
  character(len=filenamelen), intent(in)  :: cambscalclfile, camblensedclfile
  character(len=filenamelen), intent(in) :: cmbclthfile, potclthfile, lensedcmbclthfile 

  integer(i4b) :: status, l, ll, nlh, ncl
  real(dp)  ::  t0, cltt, clee, clbb, clte, clpp, clpt
  real(dp), dimension(:,:), allocatable       :: cmbcl, potcl, lensedcmbcl
  character(len=80), dimension(:), allocatable  :: hdr

  ncl=3*pol+1

  allocate(cmbcl(0:lm,1:ncl),stat = status)
  if (status /= 0) stop
  allocate(potcl(0:lm,1:1),stat = status)
  if (status /= 0) stop
  allocate(lensedcmbcl(0:llm,1:ncl),stat = status)
  if (status /= 0) stop

  cmbcl(:,:)=0.d0
  potcl(:,:)=0.d0
  lensedcmbcl(:,:)=0.d0

  t0=2.726*1.0E6

  open(unit=1,file=cambscalclfile,action="read",status= "old")
    do l=2,lm 
      if(pol.eq.0) then
        read(1,*) ll, cltt, clee, clte, clpp, clpt
        cmbcl(l,1)=2.d0*PI*cltt/dble(ll)/dble(ll+1)
        potcl(l,1)=clpp/dble(ll)/dble(ll)/dble(ll)/dble(ll)/t0/t0
      else 
        read(1,*) ll, cltt, clee, clte, clpp, clpt
        cmbcl(l,1)=2.d0*PI*cltt/dble(ll)/dble(ll+1)
        cmbcl(l,2)=2.d0*PI*clee/dble(ll)/dble(ll+1)
        cmbcl(l,3)=0.d0
        cmbcl(l,4)=2.d0*PI*clte/dble(ll)/dble(ll+1)
        potcl(l,1)=clpp/dble(ll)/dble(ll)/dble(ll)/dble(ll)/t0/t0
      endif
   end do
  close(unit=1)

  nlh=1000

  allocate(hdr(1:nlh),stat = status)
  if (status /= 0) stop

  hdr(:)=" "

  call add_card(hdr,"COMMENT","-----------------------------------------------")
  call add_card(hdr,"COMMENT"," CMB power spectrum C(l) keywords          ")
  call add_card(hdr,"COMMENT","-----------------------------------------------")
  call add_card(hdr,"EXTNAME","'ANALYSED POWER SPECTRUM'")
  call add_card(hdr,"COMMENT"," POWER SPECTRUM : C(l) ")
  call add_card(hdr)
  call add_card(hdr,"POLAR",.true.," Polarisation included (True/False)")
  call add_card(hdr,"BCROSS",.false.," Magnetic cross terms included (True/False)")
  call add_card(hdr,"MAX-LPOL",lm," Maximum L multipole order")
  call add_card(hdr,"LREGRESS",0," Regression of low multipoles (0/1/2)")
  call add_card(hdr)
  call add_card(hdr,"TEMPTYPE","THERMO"," temperature : either THERMO or ANTENNA")
  if(pol.eq.1) then
    call add_card(hdr,'POLNORM','CMBFAST','Uses E-B conventions')
  endif
  call add_card(hdr,"TTYPE1", "TEMPERATURE","Temperature C(l)")
  call add_card(hdr,"TUNIT1", "uK^2","square of map units")
  call add_card(hdr)
  if(pol.eq.1) then
    call add_card(hdr,"TTYPE2", "GRADIENT","Gradient (=ELECTRIC) polarisation C(l)")
    call add_card(hdr,"TUNIT2", "uK^2","power spectrum units")
    call add_card(hdr)
    call add_card(hdr,"TTYPE3", "CURL","Curl (=MAGNETIC) polarisation C(l)")
    call add_card(hdr,"TUNIT3", "uK^2","power spectrum units")
    call add_card(hdr)
    call add_card(hdr,"TTYPE4", "G-T","Gradient-Temperature (=CROSS) terms")
    call add_card(hdr,"TUNIT4", "uK^2","power spectrum units")
    call add_card(hdr)
    call add_card(hdr,"COMMENT","The polarisation power spectra have the same definition as in CMBFAST")
    call add_card(hdr)
  endif

  call write_asctab(cmbcl,lm,ncl,hdr,nlh,cmbclthfile) 

  hdr(:)=" "

  call add_card(hdr,'COMMENT','-----------------------------------------------')
  call add_card(hdr,'COMMENT','     CMB power spectrum C(l) keywords          ')
  call add_card(hdr,'COMMENT','-----------------------------------------------')
  call add_card(hdr,'EXTNAME','''COMPUTED POWER SPECTRUM''')
  call add_card(hdr,'COMMENT',' POWER SPECTRUM : C(l) ')
  call add_card(hdr)
  call add_card(hdr,"TEMPTYPE","THERMO"," temperature : either THERMO or ANTENNA")
  call add_card(hdr)
  call add_card(hdr,'TTYPE1', 'TEMPERATURE','Potential C(l)')
  call add_card(hdr,'TUNIT1', "uK^2",'unit')
  call add_card(hdr)

  call write_asctab(potcl,lm,1,hdr,nlh,potclthfile) 

  open(unit=1,file=camblensedclfile,action="read",status= "old")
    do l=2,llm 
      if(pol.eq.0) then
        read(1,*) ll, cltt
        lensedcmbcl(l,1)=2.d0*PI*cltt/dble(ll)/dble(ll+1)
      else 
        read(1,*) ll, cltt, clee, clbb, clte
        lensedcmbcl(l,1)=2.d0*PI*cltt/dble(ll)/dble(ll+1)
        lensedcmbcl(l,2)=2.d0*PI*clee/dble(ll)/dble(ll+1)
        lensedcmbcl(l,3)=2.d0*PI*clbb/dble(ll)/dble(ll+1)
        lensedcmbcl(l,4)=2.d0*PI*clte/dble(ll)/dble(ll+1)
      endif
   end do
  close(unit=1)

  hdr(:)=" "

  call add_card(hdr,"COMMENT","-----------------------------------------------")
  call add_card(hdr,"COMMENT"," CMB power spectrum C(l) keywords          ")
  call add_card(hdr,"COMMENT","-----------------------------------------------")
  call add_card(hdr,"EXTNAME","'ANALYSED POWER SPECTRUM'")
  call add_card(hdr,"COMMENT"," POWER SPECTRUM : C(l) ")
  call add_card(hdr)
  call add_card(hdr,"POLAR",.true.," Polarisation included (True/False)")
  call add_card(hdr,"BCROSS",.false.," Magnetic cross terms included (True/False)")
  call add_card(hdr,"MAX-LPOL",llm," Maximum L multipole order")
  call add_card(hdr,"LREGRESS",0," Regression of low multipoles (0/1/2)")
  call add_card(hdr)
  call add_card(hdr,"TEMPTYPE","THERMO"," temperature : either THERMO or ANTENNA")
  if(pol.eq.1) then
    call add_card(hdr,'POLNORM','CMBFAST','Uses E-B conventions')
  endif
  call add_card(hdr,"TTYPE1", "TEMPERATURE","Temperature C(l)")
  call add_card(hdr,"TUNIT1", "uK^2","square of map units")
  call add_card(hdr)
  if(pol.eq.1) then
    call add_card(hdr,"TTYPE2", "GRADIENT","Gradient (=ELECTRIC) polarisation C(l)")
    call add_card(hdr,"TUNIT2", "uK^2","power spectrum units")
    call add_card(hdr)
    call add_card(hdr,"TTYPE3", "CURL","Curl (=MAGNETIC) polarisation C(l)")
    call add_card(hdr,"TUNIT3", "uK^2","power spectrum units")
    call add_card(hdr)
    call add_card(hdr,"TTYPE4", "G-T","Gradient-Temperature (=CROSS) terms")
    call add_card(hdr,"TUNIT4", "uK^2","power spectrum units")
    call add_card(hdr)
    call add_card(hdr,"COMMENT","The polarisation power spectra have the same definition as in CMBFAST")
    call add_card(hdr)
  endif

  call write_asctab(lensedcmbcl,llm,ncl,hdr,nlh,lensedcmbclthfile) 

  deallocate(cmbcl)
  deallocate(potcl)
  deallocate(lensedcmbcl)
  deallocate(hdr)

end subroutine asciicambcl2fitscl

subroutine fitsmap2fitscl(pol,ns,mapfile,nl,clfile,iter)

  use healpix_types
  use pix_tools
  use fitstools 
  use misc_utils
  use alm_tools
  use head_fits

  implicit none

  integer(i4b), intent(in) :: iter
  integer(i4b), intent(in) :: nl, ns, pol
  character(len=filenamelen), intent(in) :: mapfile, clfile

  integer(kind=i4b) :: status, nlh
  real(dp),  dimension(:), allocatable :: zb
  real(dp),  dimension(:,:), allocatable :: map, w8, cl  
  complex(dpc), dimension(:,:,:), allocatable :: alm
  character(len=80), dimension(:), allocatable  :: hdr

  allocate(map(0:12*ns**2-1,1:2*pol+1),stat = status)
  if (status /= 0) stop

  call input_map(mapfile,map,12*ns*ns,2*pol+1) 
  
  allocate(w8(1:2*ns,1:2*pol+1),stat = status)
  if (status /= 0) stop
  allocate(zb(1:2),stat = status)
  if (status /= 0) stop 
  allocate(alm(1:2*pol+1,0:nl,0:nl),stat = status)
  if (status /= 0) stop 

  w8(:,:) = 1.d0
  zb(:) = 0.d0

  call map2alm_iterative(ns,nl,nl,iter,map,alm,zb,w8)

  deallocate(map)
  deallocate(w8)
  deallocate(zb)

  allocate(cl(0:nl,1:3*pol+1),stat = status)
  if (status /= 0) stop

  call alm2cl(nl,nl,alm,cl)
  
  deallocate(alm)
 
  nlh=1000

  allocate(hdr(1:nlh),stat = status)
  if (status /= 0) stop
  hdr(:)=" "

  call add_card(hdr,"COMMENT","-----------------------------------------------")
  call add_card(hdr,"COMMENT"," CMB power spectrum C(l) keywords          ")
  call add_card(hdr,"COMMENT","-----------------------------------------------")
  call add_card(hdr,"EXTNAME","'ANALYSED POWER SPECTRUM'")
  call add_card(hdr,"COMMENT"," POWER SPECTRUM : C(l) ")
  call add_card(hdr)
  call add_card(hdr,"POLAR",.true.," Polarisation included (True/False)")
  call add_card(hdr,"BCROSS",.false.," Magnetic cross terms included (True/False)")
  call add_card(hdr,"MAX-LPOL",nl," Maximum L multipole order")
  call add_card(hdr,"LREGRESS",0," Regression of low multipoles (0/1/2)")
  call add_card(hdr)
  call add_card(hdr,"TEMPTYPE","THERMO"," temperature : either THERMO or ANTENNA")
  if(pol.eq.1) call add_card(hdr,'POLNORM','CMBFAST','Uses E-B conventions')
  call add_card(hdr,"TTYPE1", "TEMPERATURE","Temperature C(l)")
  call add_card(hdr,"TUNIT1", "uK^2","square of map units")
  call add_card(hdr)
  if(pol.eq.1) then
    call add_card(hdr,"TTYPE2", "GRADIENT","Gradient (=ELECTRIC) polarisation C(l)")
    call add_card(hdr,"TUNIT2", "uK^2","power spectrum units")
    call add_card(hdr)
    call add_card(hdr,"TTYPE3", "CURL","Curl (=MAGNETIC) polarisation C(l)")
    call add_card(hdr,"TUNIT3", "uK^2","power spectrum units")
    call add_card(hdr)
    call add_card(hdr,"TTYPE4", "G-T","Gradient-Temperature (=CROSS) terms")
    call add_card(hdr,"TUNIT4", "uK^2","power spectrum units")
    call add_card(hdr)
    call add_card(hdr,"COMMENT","The polarisation power spectra have the same definition as in CMBFAST")
    call add_card(hdr)
  endif

  call write_asctab(cl,nl,3*pol+1,hdr,nlh,clfile)
  
  deallocate(cl)
  deallocate(hdr)

end subroutine fitsmap2fitscl

subroutine rotate_map(ns,crottp,srottp,maptp)

  use healpix_types

  implicit none

  integer(i4b), intent(in)  :: ns
  real(dp), dimension(0:12*ns*ns-1), intent(in)  :: crottp, srottp
  real(dp), dimension(0:12*ns*ns-1,1:3), intent(inout)  :: maptp

  integer(i4b) :: ip
  real(dp) :: qmap, umap, crot, srot

  do ip=0,12*ns*ns-1
    qmap=maptp(ip,2)
    umap=maptp(ip,3)
    crot=crottp(ip)
    srot=srottp(ip)
    maptp(ip,2)=crot*qmap-srot*umap
    maptp(ip,3)=srot*qmap+crot*umap
  enddo

end subroutine rotate_map

subroutine change_direction(ns,thtp,phtp,gthtp,gphtp,thnewtp,phnewtp,crottp,srottp)

  use healpix_types

  implicit none

  integer(i4b), intent(in)  :: ns
  real(dp), dimension(0:12*ns*ns-1), intent(in)  :: thtp, phtp
  real(dp), dimension(0:12*ns*ns-1,1:1), intent(in)  :: gthtp, gphtp
  real(dp), dimension(0:12*ns*ns-1), intent(out) :: thnewtp, phnewtp
  real(dp), dimension(0:12*ns*ns-1), intent(out), optional :: crottp, srottp
  integer(i4b) :: ip
  real(dp) :: th, ph, thnew, phnew, gth, gph, gtp, sincgtp, tamg

  do ip=0,12*ns*ns-1
    th=thtp(ip)
    ph=phtp(ip)
    gth=gthtp(ip,1)
    gph=gphtp(ip,1)
    gtp=dsqrt(gth*gth+gph*gph)
    if (gtp.gt.0.d0) then
      sincgtp=(dsin(gtp)/gtp);
      if (present(crottp) .or. present(srottp)) then
        tamg=gph*dsin(th)/(gtp*gtp*sincgtp*dcos(th)+gth*dcos(gtp)*dsin(th))
        crottp(ip)=2.d0*((gth+gph*tamg)/gtp)*((gth+gph*tamg)/gtp)/(1.d0+tamg*tamg)-1.d0
        srottp(ip)=2.d0*((gth+gph*tamg)/gtp)*((gph-gth*tamg)/gtp)/(1.d0+tamg*tamg)
      endif
      thnew=dacos(dcos(gtp)*dcos(th)-sincgtp*dsin(th)*gth)
      if (dabs(dsin(thnew)).gt.1.0d-10) then
         phnew=ph+dasin(sincgtp*gph/dsin(thnew))
      else
         phnew=ph
      endif
    else
      thnew=th
      phnew=ph
    endif
    thnewtp(ip)=thnew
    phnewtp(ip)=phnew
  enddo

end subroutine change_direction


subroutine simulate_nfft_map(pol,nl,alm,ns,sig,kap,thmap,phmap,map)

  use healpix_types

  implicit none

  !declaration of input and output variables
  integer(i4b),   intent(in) :: nl,ns,pol,sig,kap
  complex(dpc), dimension(1:2*pol+1,0:nl,0:nl), intent(in)  :: alm
  real(dp), dimension(0:12*ns**2-1), intent(in) :: thmap, phmap
  real(dp), dimension(0:12*ns**2-1,1:2*pol+1), intent(out) :: map

  !declaration of local variables
  integer(i4b) :: status, l,m,s,j,hj,lmm,mml,idx
  real(dp)     :: p, q, pj, qj, fj
  real(dp),dimension(:,:), allocatable :: dlms, dd
  real(dp),dimension(:), allocatable   :: sqt, rsqt, fl
  real(dp), dimension(:), allocatable  :: tmap,umap,qmap
  complex(dpc) :: ii
  complex(dpc), dimension(:), allocatable :: tms,pms
  complex(dpc), dimension(:,:,:), allocatable :: xlm


  ii=dcmplx(0.d0,1.d0)

  allocate(sqt(0:2*nl),stat = status)
  if (status /= 0) stop
  allocate(rsqt(0:2*nl),stat = status)
  if (status /= 0) stop
  allocate(fl(0:nl),stat = status)
  if (status /= 0) stop

  sqt(:)=0.d0
  rsqt(:)=0.d0
  fl(:)=0.d0

  p=1.d0/dsqrt(2.d0) 
  q=p

  do l=0,2*nl
    sqt(l)=dsqrt(dble(l))
  end do

  do l=0,nl
    fl(l)=dsqrt(dble(l*(l+1)))
  end do

  allocate(xlm(1:2*pol+1,0:nl,0:nl),stat = status)
  if (status /= 0) stop 

  call comp_xlm(pol,nl,alm,xlm)

  allocate(dlms(-1:2*nl,-1:nl),stat = status)
  if (status /= 0) stop
  allocate(dd(-1:2*nl,-1:nl),stat = status)
  if (status /= 0) stop

  allocate(tms(0:4*nl*(nl+1)),stat=status)
  if (status /= 0) stop 
  if(pol.eq.1) then
    allocate(pms(0:4*nl*(nl+1)),stat=status)
    if (status /= 0) stop 
  endif 
 
  dlms(:,:)=0.d0
  dd(:,:)=0.d0
  
  tms(:)=dcmplx(0.d0,0.d0)
  if(pol.eq.1) then
    pms(:)=dcmplx(0.d0,0.d0)
  endif 

  do l=0,nl 
    if(l.eq.0) then
      dlms(0,0) = 1.d0
      tms(nl*(2*nl+1)+nl)=tms(nl*(2*nl+1)+nl)+xlm(1,0,0)
      if(pol.eq.1) then
      pms(nl*(2*nl+1)+nl)=pms(nl*(2*nl+1)+nl)+(xlm(2,0,0)-ii*xlm(3,0,0))
      endif
      goto 2000
    end if
  
    if(l.eq.1) then
      dd(0,0)  =  q
      dd(1,0)  = -p
      dd(0,1)  =  p
      dd(1,1)  =  q
      goto 1000
    end if

    j=2*l-1

    do m=0,j
      rsqt(m) = sqt(j-m)
    end do
       
    fj=dble(j)
    qj=q/fj
    pj=p/fj

    do s=0,j/2
      do m=0,j
        dd(m,s)=rsqt(m)*(dlms(m,s)*(sqt(j-s)*qj)+dlms(m,s-1)*(sqt(s)*pj)) &
             & + sqt(m)*(dlms(m-1,s-1)*(sqt(s)*qj)-dlms(m-1,s)*(sqt(j-s)*pj))
      enddo
    end do

    hj=l-1
    if(mod(l,2).eq.0) then
      do s=0,j-1,2
        dd(s,l)=dd(j-s,hj)
        dd(s+1,l)=-dd(j-s-1,hj)
      end do
    else
      do s=0,j-1,2
        dd(s,l)=-dd(j-s,hj)
        dd(s+1,l)=dd(j-s-1,hj)
      end do
    endif

    1000   continue

    j=2*l
    do m=0,j
      rsqt(m)=sqt(j-m)
    end do

    fj = dble(j)
    qj = q / fj
    pj = p / fj

    do m=0,j
      do s=0,j/2
        dlms(m,s)= rsqt(m)*(dd(m,s)*(sqt(j-s)*qj)+dd(m,s-1)*(sqt(s)*pj)) &
               & + sqt(m)*(dd(m-1,s-1)*(sqt(s)*qj)-dd(m-1,s)*(sqt(j-s)*pj))
      enddo
    enddo

    do m=0,j/2 
      do s=0,j/2
        lmm=l-m
        idx=(nl+m-l)*(2*nl+1)+(nl+s-l)
        tms(idx)=tms(idx)+conjg(xlm(1,l,lmm))*dlms(m,s)*dlms(l,s)
        if(pol.eq.1) then
          pms(idx)=pms(idx)+(conjg(xlm(2,l,lmm))-ii*conjg(xlm(3,l,lmm)))* &
                &   dlms(m,s)*dlms(l+2,s)
        endif
      enddo
    enddo

    do m=j/2+1,j 
      do s=0,j/2
        mml=m-l
         idx=(nl+m-l)*(2*nl+1)+(nl+s-l)
         tms(idx)=tms(idx)+dble(1-2*mod(mml,2))*xlm(1,l,mml)*dlms(m,s)*dlms(l,s)
         if(pol.eq.1) then
            pms(idx)=pms(idx)+dble(1-2*mod(mml,2))*  &
                   & (xlm(2,l,mml)-ii*xlm(3,l,mml))*dlms(m,s)*dlms(l+2,s)
         endif
      enddo
    enddo

    2000   continue
  end do

  do m=0,2*nl
    do s=nl+1,2*nl
      idx=m*(2*nl+1)+s
      tms(idx)=dble(1-2*mod(abs(m-nl),2))*tms(idx+2*(nl-s))
      if(pol.eq.1) then
        pms(idx)=dble(1-2*mod(abs(m-nl),2))*pms(idx+2*(nl-s))
      endif
    enddo
  enddo

  deallocate(dd)
  deallocate(dlms)
  deallocate(xlm)
  deallocate(fl)
  deallocate(sqt)
  deallocate(rsqt)

  if(pol.eq.0) then
    allocate(tmap(0:12*ns**2-1),stat=status)
    if (status /= 0) stop
    tmap(:)=0.d0

    call get_lensed_temp_map(nl,tms,ns,sig,kap,thmap,phmap,tmap)

    deallocate(tms)

    map(:,1)=tmap(:)

    deallocate(tmap)
  endif

  if(pol.eq.1) then
    allocate(tmap(0:12*ns**2-1),stat=status)
    if (status /= 0) stop
    tmap(:)=0.d0
    allocate(qmap(0:12*ns**2-1),stat=status)
    if (status /= 0) stop
    qmap(:)=0.d0
    allocate(umap(0:12*ns**2-1),stat=status)
    if (status /= 0) stop
    umap(:)=0.d0

    call get_lensed_pol_map(nl,tms,pms,ns,sig,kap,thmap,phmap,tmap,qmap,umap)

    deallocate(tms)
    deallocate(pms)

    map(:,1)=tmap(:)
    map(:,2)=qmap(:)
    map(:,3)=umap(:)

    deallocate(tmap)
    deallocate(qmap)
    deallocate(umap)
  endif

end subroutine simulate_nfft_map

subroutine direction(ns,thtp,phtp)

  use healpix_types
  use pix_tools

  implicit none

  integer(i4b), intent(in)  :: ns
  real(dp), dimension(0:12*ns*ns-1), intent(out)  :: thtp, phtp
 
  integer(i4b) :: ip
  real(dp) :: th, ph


  do ip=0,12*ns*ns-1
    call pix2ang_ring(ns,ip,th,ph)
    thtp(ip)=th
    phtp(ip)=ph
  end do
end subroutine direction



subroutine asciialm2asciimapder(pol,nl,alm,ns,map,mapderth,mapderph)

  use healpix_types
  use pix_tools
  use fitstools 
  use misc_utils
  use alm_tools
  use head_fits
  
  implicit none

  integer(i4b), intent(in)  :: ns,nl,pol
  complex(dpc), dimension(1:2*pol+1,0:nl,0:nl), intent(in)  :: alm
  real(dp), dimension(0:12*ns**2-1,1:2*pol+1), intent(out) :: map
  real(dp), dimension(0:12*ns**2-1,1:2*pol+1), intent(out) :: mapderth,mapderph

  integer(i4b) :: status
  real(dp), dimension(:), allocatable    :: mapaux
  real(dp), dimension(:,:), allocatable  :: mapder

  allocate(mapder(0:12*ns**2-1,1:2*(2*pol+1)),stat=status)
  if(status /= 0) stop

  if(pol.eq.0) then
    allocate(mapaux(0:12*ns**2-1),stat=status)
    if(status /= 0) stop
    call alm2map_der(ns,nl,nl,alm,mapaux,mapder)
    map(:,1)=mapaux(:)
    deallocate(mapaux)
  else if(pol.eq.1) then
    call alm2map_der(ns,nl,nl,alm,map,mapder)
  endif

  mapderth(:,1:2*pol+1)=mapder(:,1:2*pol+1)
  mapderph(:,1:2*pol+1)=mapder(:,2*(pol+1):2*(2*pol+1))

  deallocate(mapder)

end subroutine asciialm2asciimapder




subroutine fitscl2asciialm(pol,clfile,ns,nl,is,alm)

  use healpix_types
  use alm_tools
  use rngmod

  implicit none

  integer(i4b), intent(in)  :: is,ns,nl,pol
  character(len=filenamelen), intent(in) :: clfile
  complex(dpc), dimension(1:2*pol+1,0:nl,0:nl), intent(out)  :: alm

  type(planck_rng) :: rng_handle
  integer(i4b) :: nlh, status 
  real(dp) :: fwhm
  character(len=80), dimension(:), allocatable  :: hdr

  nlh=1000
  fwhm=0.d0

  allocate(hdr(1:nlh),stat = status)
  if (status /= 0) STOP 

  hdr(:)=" "

  call rand_init(rng_handle,is)

  call create_alm(ns,nl,nl,pol,clfile,rng_handle,fwhm,alm,hdr) 

  deallocate(hdr)

end subroutine fitscl2asciialm

subroutine asciialm2fitsalm(pol,nl,alm,almfile)

  use healpix_types
  use fitstools 
  use alm_tools
  use head_fits

  implicit none

  integer(i4b), intent(in)  :: nl,pol
  complex(dpc), dimension(1:2*pol+1,0:nl,0:nl), intent(in)  :: alm
  character(len=filenamelen), intent(in) ::  almfile

  integer(i4b) :: i,nlh, status  
  character(len=80), dimension(:), allocatable  :: hdr


  nlh=1000

  allocate(hdr(1:nlh),stat = status)
  if (status /= 0) STOP 

  do i = 1,2*pol+1
    hdr = " "
    call add_card(hdr,"EXTNAME","SIMULATION")
    call add_card(hdr,"COMMENT","-----------------------------------------------")
    call add_card(hdr,"COMMENT","     Map Simulation Specific Keywords      ")
    call add_card(hdr,"COMMENT","-----------------------------------------------")
    if (i.eq.1) then
      call add_card(hdr,"EXTNAME","'SIMULATED a_lms (TEMPERATURE)'")
    elseif (i.eq.2) then
      call add_card(hdr,"EXTNAME","'SIMULATED a_lms (GRAD / ELECTRIC component)'")
    elseif (i.eq.3) then
      call add_card(hdr,"EXTNAME","'SIMULATED a_lms (CURL / MAGNETIC component)'")
    endif
    call add_card(hdr)
    call add_card(hdr,"MAX-LPOL",nl      ,"Maximum multipole l simulated")
    call add_card(hdr,"MAX-MPOL",nl      ,"Maximum m simulated")
    call add_card(hdr,"COMMENT","-----------------------------------------------")
    if (i.eq.1) then
      call add_card(hdr,"COMMENT","Temperature a_lms")
    else
      call add_card(hdr,"COMMENT","Polarisation a_lms")
    endif
    call add_card(hdr,"COMMENT"," The real and imaginary part of the a_lm with m>=0")
    call add_card(hdr)
    call add_card(hdr,"TTYPE1", "INDEX"," i = l^2 + l + m + 1")
    call add_card(hdr,"TUNIT1", "   "," index")
    call add_card(hdr)
    call add_card(hdr,"TTYPE2", "REAL"," REAL a_lm")
    call add_card(hdr,"TUNIT2", "UNKNOWN"," alm units")
    call add_card(hdr)
    call add_card(hdr,"TTYPE3", "IMAG"," IMAGINARY a_lm")
    call add_card(hdr,"TUNIT3", "UNKNOWN"," alm units")
    call add_card(hdr)
    call add_card(hdr)
    call add_card(hdr)
    call add_card(hdr)
    call dump_alms(almfile,alm(i,0:nl,0:nl),nl,hdr,nlh,i-1)
  end do


  deallocate(hdr)

end subroutine asciialm2fitsalm

subroutine fitsalm2asciialm(pol,nl,almfile,alm)

  use healpix_types
  use fitstools 
  use alm_tools
  use head_fits

  implicit none

  integer(i4b), intent(in)  :: nl,pol
  character(len=filenamelen), intent(in) ::  almfile
  complex(dpc), dimension(1:2*pol+1,0:nl,0:nl), intent(out)  :: alm

  integer(i4b) :: i,j,nlh,ncl,next,nalms,status
  real(dp), dimension(:,:,:), allocatable   :: almaux
  character(len=80), dimension(:,:), allocatable  :: hdr

  nlh=1000
  nalms=(nl+1)*(nl+2)/2
  ncl=3
  next=2*pol+1

  allocate(hdr(1:nlh,1:next),stat = status)
  if (status /= 0) STOP 
  allocate(almaux(1:nalms,1:ncl+1,1:next),stat = status)
  if (status /= 0) STOP 

  call fits2alms(almfile,nalms,almaux,ncl,hdr,nlh,next) 

  do j = 1,next
   do i = 1,nalms
     alm(j,int(almaux(i,1,j)),int(almaux(i,2,j)))=dcmplx(almaux(i,3,j),almaux(i,4,j))
   end do
  end do

  deallocate(hdr)
  deallocate(almaux)

end subroutine fitsalm2asciialm

subroutine fitsalm2fitsmapder(pol,nl,almfile,ns,mapfile,mapderthfile,mapderphfile)

  use healpix_types
  use pix_tools
  use fitstools 
  use misc_utils
  use alm_tools
  use head_fits
  
  implicit none

  integer(i4b), intent(in)  :: ns,nl,pol
  character(len=filenamelen), intent(in) :: almfile
  character(len=filenamelen), intent(out) :: mapfile,mapderthfile,mapderphfile

  integer(i4b) :: np,nlh,status
  complex(dpc), dimension(:,:,:), allocatable :: alm
  real(dp), dimension(:,:), allocatable         :: map,mapder
  real(dp), dimension(:), allocatable         :: mapaux
  character(len=80), dimension(:), allocatable  :: hdr

  np=12*ns**2
  nlh=1000

  allocate(hdr(1:nlh),stat = status)
  if (status /= 0) STOP 

  hdr(:)=" "

  call add_card(hdr,"COMMENT","----------------------------------------")
  call add_card(hdr,"COMMENT"," Sky Map Pixelisation Specific Keywords ")
  call add_card(hdr,"COMMENT","----------------------------------------")
  call add_card(hdr,"PIXTYPE","HEALPIX","HEALPIX Pixelisation")
  call add_card(hdr,"ORDERING","RING",  "Pixel ordering scheme, either RING or NESTED")
  call add_card(hdr,"NSIDE",ns,  "Resolution parameter for HEALPIX")
  call add_card(hdr,"FIRSTPIX",0,"First pixel # (0 based)")
  call add_card(hdr,"LASTPIX",np-1,"Last pixel # (0 based)")
  call add_card(hdr,"POLCCONV","COSMO"," Coord. convention for polarisation (COSMO/IAU)")
  call add_card(hdr)
  if(pol.eq.0) then
    call add_card(hdr,"TTYPE1","TEMPERATURE","Temperature map")
    call add_card(hdr)
  elseif(pol.eq.1) then
    call add_card(hdr,"TTYPE1","TEMPERATURE","Temperature map")
    call add_card(hdr)
    call add_card(hdr,"TTYPE2", "Q-POLARISATION","Q Polarisation map")
    call add_card(hdr) 
    call add_card(hdr,"TTYPE3", "U-POLARISATION","U Polarisation map")
    call add_card(hdr)
  endif

  allocate(alm(1:2*pol+1,0:nl,0:nl),stat=status)
  if(status /= 0) stop

  call fitsalm2asciialm(pol,nl,almfile,alm)

  allocate(map(0:np-1,1:2*pol+1),stat=status)
  if(status /= 0) stop
  allocate(mapder(0:np-1,1:2*(2*pol+1)),stat=status)
  if(status /= 0) stop

  if(pol.eq.0) then
    allocate(mapaux(0:np-1),stat=status)
    if(status /= 0) stop
    call alm2map_der(ns,nl,nl,alm,mapaux,mapder)
    map(:,1)=mapaux(:)
    deallocate(mapaux)
  else if(pol.eq.1) then
    call alm2map_der(ns,nl,nl,alm,map,mapder)
  endif
  deallocate(alm)

  call output_map(map,hdr,mapfile) 
  
  if(pol.eq.0) then
    map(:,1)=mapder(:,1)
    call output_map(map,hdr,mapderthfile)
    map(:,1)=mapder(:,2)
    call output_map(map,hdr,mapderphfile)
  elseif(pol.eq.1) then
    map(:,1)=mapder(:,1)
    map(:,2)=mapder(:,2)
    map(:,3)=mapder(:,3)
    call output_map(map,hdr,mapderthfile)
    map(:,1)=mapder(:,4)
    map(:,2)=mapder(:,5)
    map(:,3)=mapder(:,6)
    call output_map(map,hdr,mapderphfile)
  endif

  deallocate(map)
  deallocate(mapder)
  deallocate(hdr)

end subroutine fitsalm2fitsmapder

subroutine comp_xlm(pol,nl,alm,xlm)

  use healpix_types

  implicit none

  integer(i4b), intent(in)  :: nl,pol
  complex(dpc), dimension(1:2*pol+1,0:nl,0:nl), intent(in)  :: alm
  complex(dpc), dimension(1:2*pol+1,0:nl,0:nl), intent(out) :: xlm

  integer(i4b) :: status,l,m
  real(dp),  dimension(:), allocatable :: fl
  complex(dpc), dimension(:), allocatable :: el

  allocate(fl(0:nl),stat = status)
  if (status /= 0) stop 
  allocate(el(0:nl),stat = status)
  if (status /= 0) stop 

  do l=0,nl
    fl(l)=dsqrt(dble(2*l+1)/(4.d0*pi))
  end do

  call comp_expm(nl,el)

  do l=0,nl
    do m=0,l
      xlm(:,l,m)=fl(l)*el(m)*alm(:,l,m)
    end do    
  end do

  deallocate(fl)
  deallocate(el)
 
end subroutine comp_xlm

subroutine comp_expm(nl,el)

  use healpix_types

  implicit none

  integer(i4b), intent(in)  :: nl
  complex(dpc),  dimension(0:nl), intent(out) :: el

  integer(i4b) :: l,l2

  do l=0,nl,2
    l2=l/2
    el(l)=dcmplx(dble(1-2*mod(l2,2)),0.d0)
  end do

  do l=1,nl,2
    l2=(l-1)/2
    el(l)=dcmplx(0.d0,-dble(1-2*mod(l2,2)))
  end do

end subroutine comp_expm

subroutine asciimap2fitsmap(pol,ns,map,outfile)

  use healpix_types
  use pix_tools
  use fitstools 
  use misc_utils
  use alm_tools
  use head_fits
  
  implicit none

  integer(i4b), intent(in)  :: ns,pol
  real(dp), dimension(0:12*ns**2-1,1:2*pol+1) , intent(in) :: map
  character(len=filenamelen), intent(in) :: outfile 
  
  integer(i4b) :: status, nlh,np
  character(len=80), dimension(:), allocatable  :: hdr

  nlh=1000
  np=12*ns**2

  allocate(hdr(1:nlh),stat = status)
  if (status /= 0) STOP 

  hdr(:) = " "

  call add_card(hdr,"COMMENT","----------------------------------------")
  call add_card(hdr,"COMMENT"," Sky Map Pixelisation Specific Keywords ")
  call add_card(hdr,"COMMENT","----------------------------------------")
  call add_card(hdr,"PIXTYPE","HEALPIX","HEALPIX Pixelisation")
  call add_card(hdr,"ORDERING","RING",  "Pixel ordering scheme, either RING or NESTED")
  call add_card(hdr,"NSIDE",ns,  "Resolution parameter for HEALPIX")
  call add_card(hdr,"FIRSTPIX",0,"First pixel # (0 based)")
  call add_card(hdr,"LASTPIX",np-1,"Last pixel # (0 based)")
  call add_card(hdr,"POLCCONV","COSMO"," Coord. convention for polarisation (COSMO/IAU)")
  call add_card(hdr)
  if(pol.eq.0) then
    call add_card(hdr,"TTYPE1","TEMPERATURE","Temperature map")
    call add_card(hdr) 
  elseif(pol.eq.1) then
    call add_card(hdr,"TTYPE1","TEMPERATURE","Temperature map")
    call add_card(hdr) 
    call add_card(hdr,"TTYPE2", "Q-POLARISATION","Q Polarisation map")
    call add_card(hdr) 
    call add_card(hdr,"TTYPE3", "U-POLARISATION","U Polarisation map")
    call add_card(hdr)
  endif

  call output_map(map,hdr,outfile)

  deallocate(hdr)
  
end subroutine asciimap2fitsmap

subroutine asciialm2fitsmap(pol,nl,alm,ns,outfile)

  use healpix_types
  use pix_tools
  use fitstools 
  use misc_utils
  use alm_tools
  use head_fits
  
  implicit none

  integer(i4b), intent(in)  :: ns,nl,pol
  complex(dpc), dimension(1:2*pol+1,0:nl,0:nl), intent(in)  :: alm
  character(len=filenamelen), intent(in) :: outfile 


  integer(i4b) :: np,nlh,status
  real(dp), dimension(:,:), allocatable         :: map
  real(dp), dimension(:), allocatable         :: mapaux
  character(len=80), dimension(:), allocatable  :: hdr

  np=12*ns**2
  nlh=1000

  allocate(hdr(1:nlh),stat = status)
  if (status /= 0) STOP 

  hdr(:) = " "

  call add_card(hdr,"COMMENT","----------------------------------------")
  call add_card(hdr,"COMMENT"," Sky Map Pixelisation Specific Keywords ")
  call add_card(hdr,"COMMENT","----------------------------------------")
  call add_card(hdr,"PIXTYPE","HEALPIX","HEALPIX Pixelisation")
  call add_card(hdr,"ORDERING","RING",  "Pixel ordering scheme, either RING or NESTED")
  call add_card(hdr,"NSIDE",ns,  "Resolution parameter for HEALPIX")
  call add_card(hdr,"FIRSTPIX",0,"First pixel # (0 based)")
  call add_card(hdr,"LASTPIX",np-1,"Last pixel # (0 based)")
  call add_card(hdr,"POLCCONV","COSMO"," Coord. convention for polarisation (COSMO/IAU)")
  call add_card(hdr)
  if(pol.eq.0) then  
    call add_card(hdr,"TTYPE1","TEMPERATURE","Temperature map")
    call add_card(hdr)  
  elseif(pol.eq.1) then
    call add_card(hdr,"TTYPE1","TEMPERATURE","Temperature map")
    call add_card(hdr)  
    call add_card(hdr,"TTYPE2", "Q-POLARISATION","Q Polarisation map")
    call add_card(hdr) 
    call add_card(hdr,"TTYPE3", "U-POLARISATION","U Polarisation map")
    call add_card(hdr)
  endif

  allocate(map(0:np-1,1:2*pol+1),stat=status)
  if(status /= 0) stop

  if(pol.eq.0) then
    allocate(mapaux(0:np-1),stat=status)
    if(status /= 0) stop
    call alm2map(ns,nl,nl,alm,mapaux)
    map(:,1)=mapaux(:)
    deallocate(mapaux)
  else if(pol.eq.1) then
    call alm2map(ns,nl,nl,alm,map)
  endif

  call output_map(map,hdr,outfile)

  deallocate(map)
  deallocate(hdr)

end subroutine asciialm2fitsmap

function inttostr(i)
  
   use healpix_types

   implicit none

   integer(i4b) , intent(in) :: i
   character(len=30) inttostr

   write (inttostr,*) i
   inttostr = adjustr(adjustl(inttostr))

end function inttostr

end module healmod

