MODULE myhealmodule

contains


subroutine readcambscalcl(pol,nl,scalclfile,cmbcl,potcl)

  use healpix_types

  implicit none

  integer(i4b), intent(in) :: nl,pol
  character(len=filenamelen), intent(in)  :: scalclfile
  real(dp), dimension(0:nl,1:3*pol+1), intent(out)  :: cmbcl
  real(dp), dimension(0:nl,1:2), intent(out)  :: potcl
  integer(i4b) :: l, ll
  real(dp)  ::  t0, cltt, clee,clte, clpp, clpt

  t0=2.726*1.0E6

  cmbcl(:,:)=0.d0
  potcl(:,:)=0.d0

  open(unit=1,file=scalclfile,action="read",status= "old")
    do l=2,nl 
      if(pol.eq.0) then
        read(1,*) ll, cltt, clpp, clpt
        cmbcl(l,1)=2.d0*PI*cltt/dble(ll)/dble(ll+1)
      else 
        read(1,*) ll, cltt, clee, clte, clpp, clpt
        cmbcl(l,1)=2.d0*PI*cltt/dble(ll)/dble(ll+1)
        cmbcl(l,2)=2.d0*PI*clee/dble(ll)/dble(ll+1)
        cmbcl(l,3)=0.d0
        cmbcl(l,4)=2.d0*PI*clte/dble(ll)/dble(ll+1)
      endif
      potcl(l,1)=clpp/dble(ll)/dble(ll)/dble(ll)/dble(ll)/t0/t0
      potcl(l,2)=clpt/dble(ll)/dble(ll)/dble(ll)/t0
   end do
  close(unit=1)

end subroutine readcambscalcl

subroutine cambscalcl2asciialm(pol,nl,cmbcl,potcl,is,alm,plm)

  use healpix_types
  use rngmod

  implicit none
  
  integer(i4b), intent(in) :: is,pol,nl
  real(dp),  dimension(0:nl,1:3*pol+1),intent(in) :: cmbcl
  real(dp),  dimension(0:nl,1:2),intent(in) :: potcl
  complex(dpc), dimension(1:2*pol+1,0:nl,0:nl), intent(out)  :: alm
  complex(dpc), dimension(1:1,0:nl,0:nl), intent(out)  :: plm

  type(planck_rng) :: rng_handle
  integer(i4b)     :: l,m
  real(dp)         :: tamp, xamp, bamp, corr, examp
  complex(dpc)     :: glm

  call rand_init(rng_handle,is)

  alm(:,:,:)=0.d0
  plm(:,:,:)=0.d0

  do l=2,nl
    alm(1,l,0)=dcmplx(rand_gauss(rng_handle),0.d0)*dsqrt(cmbcl(l,1))
    tamp=dsqrt(cmbcl(l,1)/2.d0)
    do m=1,l
     alm(1,l,m)=dcmplx(rand_gauss(rng_handle),rand_gauss(rng_handle))*tamp
    end do
  end do

  if(pol.eq.1) then
    do l=2,nl
      if (cmbcl(l,1).eq.0.d0) then
        tamp=1.d0
      else
        tamp=cmbcl(l,1)
      endif
      corr=cmbcl(l,4)/tamp
      xamp=dsqrt(cmbcl(l,2)-corr*cmbcl(l,4))
      bamp=dsqrt(cmbcl(l,3))
      glm=dcmplx(rand_gauss(rng_handle),0.d0)
      plm(1,l,0)=glm
      alm(2,l,0)=corr*alm(1,l,0)+glm*xamp
      alm(3,l,0)=dcmplx(rand_gauss(rng_handle),0.d0)*bamp
      xamp=xamp/dsqrt(2.d0)
      bamp=bamp/dsqrt(2.d0)
      do m=1,l
        glm=dcmplx(rand_gauss(rng_handle),rand_gauss(rng_handle))
        alm(2,l,m)=corr*alm(1,l,m)+glm*xamp
        alm(3,l,m)=dcmplx(rand_gauss(rng_handle),rand_gauss(rng_handle))*bamp
        plm(1,l,m)=glm
      end do
    end do
  end if

  do l=2,nl
    if (cmbcl(l,1).eq.0.d0) then
      tamp=1.d0
    else
      tamp=cmbcl(l,1)
    endif
    corr=potcl(l,2)/tamp
    if(pol.eq.1) then
      examp=corr*cmbcl(l,4)*dsqrt(tamp/(cmbcl(l,2)*tamp -cmbcl(l,4)*cmbcl(l,4)))
      xamp=dsqrt(max(0.d0,potcl(l,1)-corr*potcl(l,2)-examp*examp))
      plm(1,l,0)=-examp*plm(1,l,0)
      examp=examp/dsqrt(2.d0)
    else
      xamp=dsqrt(max(0.d0,potcl(l,1)-corr*potcl(l,2)))
    endif
    plm(1,l,0)=plm(1,l,0)+corr*alm(1,l,0) + &
                 & dcmplx(rand_gauss(rng_handle),0.d0)*xamp
    xamp=xamp/dsqrt(2.d0)
    do m=1,l
      if(pol.eq.1) then
        plm(1,l,m)=-examp*plm(1,l,m) 
      endif
      plm(1,l,m)=plm(1,l,m)+corr*alm(1,l,m)+ &
           & dcmplx(rand_gauss(rng_handle),rand_gauss(rng_handle))*xamp
    end do
  end do

end subroutine cambscalcl2asciialm


subroutine asciiscalcl2fitsscalcl(pol,lm,infile1,outfile1,outfile2)

  use healpix_types
  use head_fits
  use fitstools

  implicit none

  integer(i4b), intent(in) :: lm,pol
  character(len=filenamelen), intent(in)  :: infile1
  character(len=filenamelen), intent(in) :: outfile1, outfile2 

  integer(i4b) :: status, l, ll, nlh, ncl
  real(dp)  ::  t0, cltt, clee,clte, clpp, clpt
  real(dp), dimension(:,:), allocatable       :: cmbcl, potcl
  character(len=80), dimension(:), allocatable  :: hdr

  ncl=3*pol+1

  allocate(cmbcl(0:lm,1:ncl),stat = status)
  if (status /= 0) stop
  allocate(potcl(0:lm,1:1),stat = status)
  if (status /= 0) stop

  cmbcl(:,:)=0.d0
  potcl(:,:)=0.d0

  t0=2.726*1.0E6

  open(unit=1,file=infile1,action="read",status= "old")
    do l=2,lm 
      if(pol.eq.0) then
        read(1,*) ll, cltt, clee, clte, clpp, clpt
        cmbcl(l,1)=2.d0*PI*cltt/dble(ll)/dble(ll+1)
        potcl(l,1)=clpp/dble(ll)/dble(ll)/dble(ll)/dble(ll)/t0/t0
      else 
        read(1,*) ll, cltt, clee, clte, clpp, clpt
        cmbcl(l,1)=2.d0*PI*cltt/dble(ll)/dble(ll+1)
        cmbcl(l,2)=2.d0*PI*clee/dble(ll)/dble(ll+1)
        cmbcl(l,3)=0.d0
        cmbcl(l,4)=2.d0*PI*clte/dble(ll)/dble(ll+1)
        potcl(l,1)=clpp/dble(ll)/dble(ll)/dble(ll)/dble(ll)/t0/t0
      endif
   end do
  close(unit=1)

  nlh=1000

  allocate(hdr(1:nlh),stat = status)
  if (status /= 0) stop

  hdr(:)=" "

  call add_card(hdr,"COMMENT","-----------------------------------------------")
  call add_card(hdr,"COMMENT"," CMB power spectrum C(l) keywords          ")
  call add_card(hdr,"COMMENT","-----------------------------------------------")
  call add_card(hdr,"EXTNAME","'ANALYSED POWER SPECTRUM'")
  call add_card(hdr,"COMMENT"," POWER SPECTRUM : C(l) ")
  call add_card(hdr)
  call add_card(hdr,"POLAR",.true.," Polarisation included (True/False)")
  call add_card(hdr,"BCROSS",.false.," Magnetic cross terms included (True/False)")
  call add_card(hdr,"MAX-LPOL",lm," Maximum L multipole order")
  call add_card(hdr,"LREGRESS",0," Regression of low multipoles (0/1/2)")
  call add_card(hdr)
  call add_card(hdr,"TEMPTYPE","THERMO"," temperature : either THERMO or ANTENNA")
  if(pol.eq.1) then
    call add_card(hdr,'POLNORM','CMBFAST','Uses E-B conventions')
  endif
  call add_card(hdr,"TTYPE1", "TEMPERATURE","Temperature C(l)")
  call add_card(hdr,"TUNIT1", "uK^2","square of map units")
  call add_card(hdr)
  if(pol.eq.1) then
    call add_card(hdr,"TTYPE2", "GRADIENT","Gradient (=ELECTRIC) polarisation C(l)")
    call add_card(hdr,"TUNIT2", "uK^2","power spectrum units")
    call add_card(hdr)
    call add_card(hdr,"TTYPE3", "CURL","Curl (=MAGNETIC) polarisation C(l)")
    call add_card(hdr,"TUNIT3", "uK^2","power spectrum units")
    call add_card(hdr)
    call add_card(hdr,"TTYPE4", "G-T","Gradient-Temperature (=CROSS) terms")
    call add_card(hdr,"TUNIT4", "uK^2","power spectrum units")
    call add_card(hdr)
    call add_card(hdr,"COMMENT","The polarisation power spectra have the same definition as in CMBFAST")
    call add_card(hdr)
  endif

  call write_asctab(cmbcl,lm,ncl,hdr,nlh,outfile1) 

  hdr(:)=" "

  call add_card(hdr,'COMMENT','-----------------------------------------------')
  call add_card(hdr,'COMMENT','     CMB power spectrum C(l) keywords          ')
  call add_card(hdr,'COMMENT','-----------------------------------------------')
  call add_card(hdr,'EXTNAME','''COMPUTED POWER SPECTRUM''')
  call add_card(hdr,'COMMENT',' POWER SPECTRUM : C(l) ')
  call add_card(hdr)
  call add_card(hdr,"TEMPTYPE","THERMO"," temperature : either THERMO or ANTENNA")
  call add_card(hdr)
  call add_card(hdr,'TTYPE1', 'TEMPERATURE','Potential C(l)')
  call add_card(hdr,'TUNIT1', "uK^2",'unit')
  call add_card(hdr)


  call write_asctab(potcl,lm,1,hdr,nlh,outfile2) 

  deallocate(cmbcl)
  deallocate(potcl)
  deallocate(hdr)

end subroutine asciiscalcl2fitsscalcl



subroutine asciiscalcl2asciialm(pol,crs,cmbcl,potcl,nl,is,alm,plm)

  use healpix_types
  use pix_tools
  use fitstools 
  use misc_utils
  use alm_tools
  use head_fits
  use rngmod

  implicit none
  
  integer(i4b), intent(in) :: is,pol,nl,crs
  real(dp),  dimension(0:nl,1:3*pol+1),intent(in) :: cmbcl
  real(dp),  dimension(0:nl,1:crs+1),intent(in) :: potcl
  complex(dpc), dimension(1:2*pol+1,0:nl,0:nl), intent(out)  :: alm
  complex(dpc), dimension(1:1,0:nl,0:nl), intent(out)  :: plm

  type(planck_rng) :: rng_handle
  integer(i4b)     :: l,m
  real(dp)         :: tamp, xamp, bamp, cros, examp
  complex(dpc)     :: crg

  call rand_init(rng_handle,is)

  alm(:,:,:)=0.d0

  do l=2,nl
    tamp = dsqrt(cmbcl(l,1))
    alm(1,l,0)=dcmplx(rand_gauss(rng_handle),0.d0)*tamp
    tamp = tamp/dsqrt(2.d0)
    do m=1,l
     alm(1,l,m)=dcmplx(rand_gauss(rng_handle),rand_gauss(rng_handle))*tamp
    end do
  end do

  plm(:,:,:)=0.d0

  if(pol.eq.1) then

    do l=2,nl
      tamp=cmbcl(l,1)
      cros=cmbcl(l,4)/tamp
      xamp=dsqrt(cmbcl(l,2) - cros*cmbcl(l,4))
      bamp=dsqrt(cmbcl(l,3))
      crg=dcmplx(rand_gauss(rng_handle),0.d0)
      plm(1,l,0)=crg
      alm(2,l,0)=cros*alm(1,l,0) + crg*xamp
      alm(3,l,0)=dcmplx(rand_gauss(rng_handle),0.d0)*bamp
      xamp=xamp/dsqrt(2.d0)
      bamp=bamp/dsqrt(2.d0)
      do m=1,l
        crg=dcmplx(rand_gauss(rng_handle),rand_gauss(rng_handle))
        alm(2,l,0)=cros*alm(1,l,0) + crg*xamp
        alm(3,l,0)=dcmplx(rand_gauss(rng_handle),0.d0)*bamp
        plm(1,l,m)=crg
      end do
    end do

  end if

  do l=2,nl
    tamp=cmbcl(l,1)
    cros=0.d0
    if(crs.eq.1) then
      cros=potcl(l,2)/tamp
    end if

    if(pol.eq.0) then
      xamp = dsqrt(max(0.d0,potcl(l,1) - cros*potcl(l,2)))
    else if(pol.eq.1) then
      examp = cros*cmbcl(l,4)*dsqrt(tamp/(cmbcl(l,2)*tamp -cmbcl(l,4)*cmbcl(l,4)))
      xamp = dsqrt(max(0.d0,potcl(l,1)-cros*potcl(l,2)-examp*examp))
      plm(1,l,0) = -examp*plm(1,l,0)
      examp = examp/dsqrt(2.d0)
    endif

    plm(1,l,0) = plm(1,l,0) + cros*alm(1,l,0) + &
                 & dcmplx(rand_gauss(rng_handle),0.d0)*xamp
    xamp=xamp/dsqrt(2.d0)

    do m = 1, l
      if(pol.eq.1) then
        plm(1,l,m) = -examp*plm(1,l,m) 
      endif
      plm(1,l,m) = plm(1,l,m) + cros*alm(1,l,m) + &
           & dcmplx(rand_gauss(rng_handle),rand_gauss(rng_handle))*xamp
    end do
  end do


end subroutine asciiscalcl2asciialm



subroutine simul_all_map(pol,nl,alm,plm,ns,sig,kap,ulmap,gmap,lmap,method)

  use healpix_types

  implicit none


  !declaration of input and output variables
  integer(i4b),   intent(in) :: nl,ns,pol,sig,kap
  complex(dpc), dimension(1:2*pol+1,0:nl,0:nl), intent(in)  :: alm
  complex(dpc), dimension(1:1,0:nl,0:nl), intent(in)  :: plm
  real(dp), dimension(0:12*ns**2-1,1:2*pol+1) , intent(out) :: ulmap, lmap
  real(dp), dimension(0:12*ns**2-1,1:3) , intent(out) :: gmap
  character(len=80), intent(in) :: method

  !declaration of local variables
  integer(i4b) :: status, l,m,s,j,hj,lmm,mml,idx !,dim
  real(dp)     :: p, q, pj, qj, fj
  real(dp),dimension(:,:), allocatable :: dlms, dd
  real(dp),dimension(:,:), allocatable :: map, mapderth, mapderph
  real(dp),dimension(:), allocatable   :: sqt, rsqt,fl
  real(dp), dimension(:), allocatable :: ulttp,ulutp,ulqtp
  real(dp), dimension(:), allocatable :: phtp,gthtp,gphtp
  real(dp), dimension(:), allocatable :: lttp,lutp,lqtp
  real(dp) :: finish, start, finish_loop, start_loop, st, fi, stnew, finew
  complex(dpc) :: ii
  complex(dpc), dimension(:), allocatable :: tms,gms,pms,phms
  complex(dpc), dimension(:,:,:), allocatable :: xlm,ylm


  call cpu_time(start)

  ii=dcmplx(0.d0,1.d0)  !ii=sqrt(-1)

  allocate(sqt(0:2*nl),stat = status)
  if (status /= 0) stop
  allocate(rsqt(0:2*nl),stat = status)
  if (status /= 0) stop
  allocate(fl(0:nl),stat = status)
  if (status /= 0) stop

  sqt(:)=0.d0
  rsqt(:)=0.d0
  fl(:)=0.d0

  p=1.d0/dsqrt(2.d0)   !p=q=cos(pi/4)
  q=p

  do l=0,2*nl
    sqt(l)=dsqrt(dble(l))
  end do

  do l=0,nl
    fl(l)=dsqrt(dble(l*(l+1)))
  end do

  allocate(xlm(1:2*pol+1,0:nl,0:nl),stat = status)
  if (status /= 0) stop 
  allocate(ylm(1:1,0:nl,0:nl),stat = status)
  if (status /= 0) stop 

  call comp_xlm(pol,nl,alm,xlm)
  call comp_xlm(0,nl,plm,ylm)

  allocate(dlms(-1:2*nl,-1:nl),stat = status)
  if (status /= 0) stop
  allocate(dd(-1:2*nl,-1:nl),stat = status)
  if (status /= 0) stop

  if(pol.eq.0) then 
    allocate(tms(0:4*nl*(nl+1)),stat=status)
    if (status /= 0) stop 
  elseif(pol.eq.1) then
    allocate(tms(0:4*nl*(nl+1)),stat=status)
    if (status /= 0) stop 
    allocate(pms(0:4*nl*(nl+1)),stat=status)
    if (status /= 0) stop 
  endif 
 
  allocate(phms(0:4*nl*(nl+1)),stat=status)
  if (status /= 0) stop 
  allocate(gms(0:4*nl*(nl+1)),stat=status)
  if (status /= 0) stop 

  dlms(:,:)=0.d0
  dd(:,:)=0.d0
  if(pol.eq.0) then
    tms(:)=dcmplx(0.d0,0.d0)
  elseif(pol.eq.1) then
    tms(:)=dcmplx(0.d0,0.d0)
    pms(:)=dcmplx(0.d0,0.d0)
  endif 

  phms(:)=dcmplx(0.d0,0.d0)
  gms(:)=dcmplx(0.d0,0.d0)
    
!!$  open(unit=1111,file='fool.dat',action="write",status="replace")

  call cpu_time(start_loop)

  do l=0,nl 
    call cpu_time(st)
    if(l.eq.0) then
      dlms(0,0) = 1.d0
      call cpu_time(stnew)
      if(pol.eq.0) then
      tms(nl*(2*nl+1)+nl)=tms(nl*(2*nl+1)+nl)+xlm(1,0,0)
      elseif(pol.eq.1) then
      tms(nl*(2*nl+1)+nl)=tms(nl*(2*nl+1)+nl)+xlm(1,0,0)
      pms(nl*(2*nl+1)+nl)=pms(nl*(2*nl+1)+nl)+(xlm(2,0,0)-ii*xlm(3,0,0))
      endif
      phms(nl*(2*nl+1)+nl)=phms(nl*(2*nl+1)+nl)+ylm(1,0,0)
      gms(nl*(2*nl+1)+nl)=gms(nl*(2*nl+1)+nl)+dcmplx(0.d0,0.d0)
      call cpu_time(finew)
      goto 2000
    end if
  
    if(l.eq.1) then
      dd(0,0)  =  q
      dd(1,0)  = -p
      dd(0,1)  =  p
      dd(1,1)  =  q
      goto 1000
    end if

    j=2*l-1

    do m=0,j
      rsqt(m) = sqt(j-m)
    end do
       
    fj=dble(j)
    qj=q/fj
    pj=p/fj

    do s=0,j/2
      do m=0,j
        dd(m,s)=rsqt(m)*(dlms(m,s)*(sqt(j-s)*qj)+dlms(m,s-1)*(sqt(s)*pj)) &
             & + sqt(m)*(dlms(m-1,s-1)*(sqt(s)*qj)-dlms(m-1,s)*(sqt(j-s)*pj))
      enddo
    end do

    hj=l-1
    if(mod(l,2).eq.0) then
      do s=0,j-1,2
        dd(s,l)=dd(j-s,hj)
        dd(s+1,l)=-dd(j-s-1,hj)
      end do
    else
      do s=0,j-1,2
        dd(s,l)=-dd(j-s,hj)
        dd(s+1,l)=dd(j-s-1,hj)
      end do
    endif

    1000   continue

    j=2*l
    do m=0,j
      rsqt(m)=sqt(j-m)
    end do

    fj = dble(j)
    qj = q / fj
    pj = p / fj

    do m=0,j
      do s=0,j/2
        dlms(m,s)= rsqt(m)*(dd(m,s)*(sqt(j-s)*qj)+dd(m,s-1)*(sqt(s)*pj)) &
               & + sqt(m)*(dd(m-1,s-1)*(sqt(s)*qj)-dd(m-1,s)*(sqt(j-s)*pj))
      enddo
    enddo

    call cpu_time(stnew)
    do m=0,j/2 
      do s=0,j/2
        lmm=l-m
        idx=(nl+m-l)*(2*nl+1)+(nl+s-l)
        if(pol.eq.0) then
          tms(idx)=tms(idx)+conjg(xlm(1,l,lmm))*dlms(m,s)*dlms(l,s)
        elseif(pol.eq.1) then
          tms(idx)=tms(idx)+conjg(xlm(1,l,lmm))*dlms(m,s)*dlms(l,s)
          pms(idx)=pms(idx)+(conjg(xlm(2,l,lmm))-ii*conjg(xlm(3,l,lmm)))* &
                &   dlms(m,s)*dlms(l+2,s)
        endif
        phms(idx)=phms(idx)+conjg(ylm(1,l,lmm))*dlms(m,s)*dlms(l,s)
        gms(idx)=gms(idx)+ii*fl(l)*conjg(ylm(1,l,lmm))*dlms(m,s)*dlms(l+1,s)
      enddo
    enddo

    do m=j/2+1,j 
      do s=0,j/2
        mml=m-l
         idx=(nl+m-l)*(2*nl+1)+(nl+s-l)
         if(pol.eq.0) then
           tms(idx)=tms(idx)+dble(1-2*mod(mml,2))*xlm(1,l,mml)*dlms(m,s)*dlms(l,s)
         elseif(pol.eq.1) then
           tms(idx)=tms(idx)+dble(1-2*mod(mml,2))*xlm(1,l,mml)*dlms(m,s)*dlms(l,s)
           pms(idx)=pms(idx)+dble(1-2*mod(mml,2))*  &
                   & (xlm(2,l,mml)-ii*xlm(3,l,mml))*dlms(m,s)*dlms(l+2,s)
         endif
         phms(idx)=phms(idx)+dble(1-2*mod(mml,2))*ylm(1,l,mml)*dlms(m,s)*dlms(l,s)
         gms(idx)=gms(idx)+ii*fl(l)*dble(1-2*mod(mml,2))*ylm(1,l,mml)*dlms(m,s)*dlms(l+1,s)
      enddo
    enddo
    call cpu_time(finew)

    2000   continue
    call cpu_time(fi)  
!!$    write(1111,'(i8,2e16.8)'), l, fi-st, finew-stnew
  end do

  call cpu_time(finish_loop)
!!$  close(unit=1111)
  print *, "time to compute fourier modes from harmonic modes =",finish_loop-start_loop

  do m=0,2*nl
    do s=nl+1,2*nl
      idx=m*(2*nl+1)+s
      if(pol.eq.0) then
        tms(idx)=dble(1-2*mod(abs(m-nl),2))*tms(idx+2*(nl-s))
      elseif(pol.eq.1) then
        tms(idx)=dble(1-2*mod(abs(m-nl),2))*tms(idx+2*(nl-s))
        pms(idx)=dble(1-2*mod(abs(m-nl),2))*pms(idx+2*(nl-s))
      endif
      phms(idx)=dble(1-2*mod(abs(m-nl),2))*phms(idx+2*(nl-s))
      gms(idx)=-dble(1-2*mod(abs(m-nl),2))*gms(idx+2*(nl-s))
    enddo
  enddo

  deallocate(dd)
  deallocate(dlms)
  deallocate(xlm)
  deallocate(ylm)
  deallocate(fl)
  deallocate(sqt)
  deallocate(rsqt)

  call cpu_time(finish)

  print *, "time to compute fourier modes from harmonic modes =",finish-start

  call cpu_time(start)

  allocate(phtp(0:12*ns**2-1),stat=status)
  if (status /= 0) stop
  allocate(gthtp(0:12*ns**2-1),stat=status)
  if (status /= 0) stop
  allocate(gphtp(0:12*ns**2-1),stat=status)
  if (status /= 0) stop

  phtp(:)=0.d0
  gthtp(:)=0.d0
  gphtp(:)=0.d0

  if(pol.eq.0) then
    allocate(ulttp(0:12*ns**2-1),stat=status)
    if (status /= 0) stop
    allocate(lttp(0:12*ns**2-1),stat=status)
    if (status /= 0) stop

    ulttp(:)=0.d0
    lttp(:)=0.d0

    print *, "********************************************************************"
    
    if (method .eq. 'nfft') then
      print *, "using nfft for unlensed map"
      call get_temp_map(nl,tms,phms,gms,ns,sig,kap,ulttp,phtp,gthtp,gphtp)
    else if (method .eq. 'hpix') then
      print *, "using healpix for unlensed map"
      allocate(map(0:12*ns**2-1,2*pol+1),stat=status)

      call asciialm2asciimap(pol,nl,alm,ns,map)
      ulttp(:)=map(:,1)

      deallocate(map)

      allocate(map(0:12*ns**2-1,1),stat=status)
      allocate(mapderth(0:12*ns**2-1,1),stat=status)
      allocate(mapderph(0:12*ns**2-1,1),stat=status)

      call asciialm2asciimapder(0,nl,plm,ns,map,mapderth,mapderph)
      phtp(:)=map(:,1)
      gthtp(:)=mapderth(:,1)
      gphtp(:)=mapderph(:,1)

      deallocate(map)
      deallocate(mapderth)
      deallocate(mapderph)    
    endif

    call get_lensed_temp_map(nl,tms,ns,sig,kap,ulttp,phtp,gthtp,gphtp,lttp)

!!$    call get_all_temp_map(nl,tms,phms,gms,ns,sig,kap,ulttp,phtp,gthtp,gphtp,lttp)

    print *, "********************************************************************"

    deallocate(tms)

    ulmap(:,1)=ulttp(:)

    deallocate(ulttp)

    lmap(:,1)=lttp(:)

    deallocate(lttp)

  elseif(pol.eq.1) then

    allocate(ulttp(0:12*ns**2-1),stat=status)
    if (status /= 0) stop
    allocate(ulqtp(0:12*ns**2-1),stat=status)
    if (status /= 0) stop
    allocate(ulutp(0:12*ns**2-1),stat=status)
    if (status /= 0) stop

    allocate(lttp(0:12*ns**2-1),stat=status)
    if (status /= 0) stop
    allocate(lqtp(0:12*ns**2-1),stat=status)
    if (status /= 0) stop
    allocate(lutp(0:12*ns**2-1),stat=status)
    if (status /= 0) stop

    ulttp(:)=0.d0
    ulqtp(:)=0.d0
    ulutp(:)=0.d0

    lttp(:)=0.d0
    lqtp(:)=0.d0
    lutp(:)=0.d0

    print *, "********************************************************************"

    if (method .eq. 'nfft') then
      print *, "using nfft for unlensed map"
      call get_pol_map(nl,tms,pms,phms,gms,ns,sig,kap,ulttp,ulqtp,ulutp,phtp,gthtp,gphtp)
    else if (method .eq. 'hpix') then
      print *, "using healpix for unlensed map"
      allocate(map(0:12*ns**2-1,2*pol+1),stat=status)

      call asciialm2asciimap(pol,nl,alm,ns,map)
      ulttp(:)=map(:,1)
      ulqtp(:)=map(:,2)
      ulutp(:)=map(:,3)

      deallocate(map)

      allocate(map(0:12*ns**2-1,1),stat=status)
      allocate(mapderth(0:12*ns**2-1,1),stat=status)
      allocate(mapderph(0:12*ns**2-1,1),stat=status)

      call asciialm2asciimapder(0,nl,plm,ns,map,mapderth,mapderph)
      phtp(:)=map(:,1)
      gthtp(:)=mapderth(:,1)
      gphtp(:)=mapderph(:,1)

      deallocate(map)
      deallocate(mapderth)
      deallocate(mapderph)    
    endif

    call get_lensed_pol_map(nl,tms,pms,ns,sig,kap,ulttp,ulqtp,ulutp,phtp,gthtp,gphtp,lttp,lqtp,lutp)

!!$    call get_all_pol_map(nl,tms,pms,phms,gms,ns,sig,kap,ulttp,ulqtp,ulutp,phtp,gthtp,gphtp,lttp,lqtp,lutp)

    print *, "********************************************************************"

    deallocate(tms)
    deallocate(pms)

    ulmap(:,1)=ulttp(:)

    deallocate(ulttp)

    ulmap(:,2)=ulqtp(:)
    
    deallocate(ulqtp)

    ulmap(:,3)=ulutp(:)

    deallocate(ulutp)


    lmap(:,1)=lttp(:)

    deallocate(lttp)

    lmap(:,2)=lqtp(:)
    
    deallocate(lqtp)

    lmap(:,3)=lutp(:)

    deallocate(lutp)

  endif

  deallocate(phms)
  deallocate(gms)

  gmap(:,1)=phtp(:)

  deallocate(phtp)

  gmap(:,2)=gthtp(:)
    
  deallocate(gthtp)

  gmap(:,3)=gphtp(:)
    
  deallocate(gphtp)

  call cpu_time(finish)

  print *, "time to simulate maps from fourier modes using nfft =",finish-start

end subroutine simul_all_map


subroutine comp_xlm(pol,nl,alm,xlm)

  use healpix_types

  implicit none

  integer(i4b), intent(in)  :: nl,pol
  complex(dpc), dimension(1:2*pol+1,0:nl,0:nl), intent(in)  :: alm
  complex(dpc), dimension(1:2*pol+1,0:nl,0:nl), intent(out) :: xlm

  integer(i4b) :: status,l,m
  real(dp),  dimension(:), allocatable :: fl
  complex(dpc), dimension(:), allocatable :: el

  allocate(fl(0:nl),stat = status)
  if (status /= 0) stop 
  allocate(el(0:nl),stat = status)
  if (status /= 0) stop 

  do l=0,nl
    fl(l)=dsqrt(dble(2*l+1)/(4.d0*pi))
  end do

  call comp_expm(nl,el)

  do l=0,nl
    do m=0,l
      xlm(:,l,m)=fl(l)*el(m)*alm(:,l,m)
    end do    
  end do

  deallocate(fl)
  deallocate(el)
 
end subroutine comp_xlm

subroutine comp_expm(nl,el)

  use healpix_types

  implicit none

  integer(i4b), intent(in)  :: nl
  complex(dpc),  dimension(0:nl), intent(out) :: el

  integer(i4b) :: l,l2

  do l=0,nl,2
    l2=l/2
    el(l)=dcmplx(dble(1-2*mod(l2,2)),0.d0)
  end do

  do l=1,nl,2
    l2=(l-1)/2
    el(l)=dcmplx(0.d0,-dble(1-2*mod(l2,2)))
  end do

end subroutine comp_expm

subroutine asciimap2fitsmap(pol,ns,map,outfile)

  use healpix_types
  use pix_tools
  use fitstools 
  use misc_utils
  use alm_tools
  use head_fits
  
  implicit none

  integer(i4b), intent(in)  :: ns,pol
  real(dp), dimension(0:12*ns**2-1,1:2*pol+1) , intent(in) :: map
  character(len=filenamelen), intent(in) :: outfile 
  
  integer(i4b) :: status, nlh,np
  character(len=80), dimension(:), allocatable  :: hdr

  nlh=1000
  np=12*ns**2

  allocate(hdr(1:nlh),stat = status)
  if (status /= 0) STOP 

  hdr(:) = " "

  call add_card(hdr,"COMMENT","----------------------------------------")
  call add_card(hdr,"COMMENT"," Sky Map Pixelisation Specific Keywords ")
  call add_card(hdr,"COMMENT","----------------------------------------")
  call add_card(hdr,"PIXTYPE","HEALPIX","HEALPIX Pixelisation")
  call add_card(hdr,"ORDERING","RING",  "Pixel ordering scheme, either RING or NESTED")
  call add_card(hdr,"NSIDE",ns,  "Resolution parameter for HEALPIX")
  call add_card(hdr,"FIRSTPIX",0,"First pixel # (0 based)")
  call add_card(hdr,"LASTPIX",np-1,"Last pixel # (0 based)")
  call add_card(hdr,"POLCCONV","COSMO"," Coord. convention for polarisation (COSMO/IAU)")
  call add_card(hdr)
  if(pol.eq.0) then
    call add_card(hdr,"TTYPE1","TEMPERATURE","Temperature map")
    call add_card(hdr) 
  elseif(pol.eq.1) then
    call add_card(hdr,"TTYPE1","TEMPERATURE","Temperature map")
    call add_card(hdr) 
    call add_card(hdr,"TTYPE2", "Q-POLARISATION","Q Polarisation map")
    call add_card(hdr) 
    call add_card(hdr,"TTYPE3", "U-POLARISATION","U Polarisation map")
    call add_card(hdr)
  endif

  call output_map(map,hdr,outfile)

  deallocate(hdr)
  
end subroutine asciimap2fitsmap

subroutine fitscl2asciialm(pol,clf,ns,nl,is,alm)

  use healpix_types
  use alm_tools
  use rngmod

  implicit none

  integer(i4b), intent(in)  :: is,ns,nl,pol
  character(len=filenamelen), intent(in) :: clf
  complex(dpc), dimension(1:2*pol+1,0:nl,0:nl), intent(out)  :: alm


  type(planck_rng) :: rng_handle
  integer(i4b) :: nlh, status 
  real(dp) :: fwhm
  character(len=80), dimension(:), allocatable  :: hdr

  nlh=1000
  fwhm=0.d0

  allocate(hdr(1:nlh),stat = status)
  if (status /= 0) STOP 

  hdr(:)=" "

  call rand_init(rng_handle,is)

  call create_alm(ns,nl,nl,pol,clf,rng_handle,fwhm,alm,hdr) 

  deallocate(hdr)

end subroutine fitscl2asciialm

subroutine asciialm2fitsalm(pol,nl,alm,almfile)

  use healpix_types
  use fitstools 
  use alm_tools
  use head_fits

  implicit none

  integer(i4b), intent(in)  :: nl,pol
  complex(dpc), dimension(1:2*pol+1,0:nl,0:nl), intent(in)  :: alm
  character(len=filenamelen), intent(in) ::  almfile

  integer(i4b) :: i,nlh, status  
  character(len=80), dimension(:), allocatable  :: hdr


  nlh=1000

  allocate(hdr(1:nlh),stat = status)
  if (status /= 0) STOP 

  do i = 1,2*pol+1
    hdr = " "
    call add_card(hdr,"EXTNAME","SIMULATION")
    call add_card(hdr,"COMMENT","-----------------------------------------------")
    call add_card(hdr,"COMMENT","     Map Simulation Specific Keywords      ")
    call add_card(hdr,"COMMENT","-----------------------------------------------")
    if (i.eq.1) then
      call add_card(hdr,"EXTNAME","'SIMULATED a_lms (TEMPERATURE)'")
    elseif (i.eq.2) then
      call add_card(hdr,"EXTNAME","'SIMULATED a_lms (GRAD / ELECTRIC component)'")
    elseif (i.eq.3) then
      call add_card(hdr,"EXTNAME","'SIMULATED a_lms (CURL / MAGNETIC component)'")
    endif
    call add_card(hdr)
    call add_card(hdr,"MAX-LPOL",nl      ,"Maximum multipole l simulated")
    call add_card(hdr,"MAX-MPOL",nl      ,"Maximum m simulated")
    call add_card(hdr,"COMMENT","-----------------------------------------------")
    if (i.eq.1) then
      call add_card(hdr,"COMMENT","Temperature a_lms")
    else
      call add_card(hdr,"COMMENT","Polarisation a_lms")
    endif
    call add_card(hdr,"COMMENT"," The real and imaginary part of the a_lm with m>=0")
    call add_card(hdr)
    call add_card(hdr,"TTYPE1", "INDEX"," i = l^2 + l + m + 1")
    call add_card(hdr,"TUNIT1", "   "," index")
    call add_card(hdr)
    call add_card(hdr,"TTYPE2", "REAL"," REAL a_lm")
    call add_card(hdr,"TUNIT2", "UNKNOWN"," alm units")
    call add_card(hdr)
    call add_card(hdr,"TTYPE3", "IMAG"," IMAGINARY a_lm")
    call add_card(hdr,"TUNIT3", "UNKNOWN"," alm units")
    call add_card(hdr)
    call add_card(hdr)
    call add_card(hdr)
    call add_card(hdr)
    call dump_alms(almfile,alm(i,0:nl,0:nl),nl,hdr,nlh,i-1)
  end do


  deallocate(hdr)

end subroutine asciialm2fitsalm


subroutine asciialm2fitsmap(pol,nl,alm,ns,outfile)

  use healpix_types
  use pix_tools
  use fitstools 
  use misc_utils
  use alm_tools
  use head_fits
  
  implicit none

  integer(i4b), intent(in)  :: ns,nl,pol
  complex(dpc), dimension(1:2*pol+1,0:nl,0:nl), intent(in)  :: alm
  character(len=filenamelen), intent(in) :: outfile 


  integer(i4b) :: np,nlh,status
  real(dp), dimension(:,:), allocatable         :: map
  real(dp), dimension(:), allocatable         :: mapaux
  character(len=80), dimension(:), allocatable  :: hdr

  np=12*ns**2
  nlh=1000

  allocate(hdr(1:nlh),stat = status)
  if (status /= 0) STOP 

  hdr(:) = " "

  call add_card(hdr,"COMMENT","----------------------------------------")
  call add_card(hdr,"COMMENT"," Sky Map Pixelisation Specific Keywords ")
  call add_card(hdr,"COMMENT","----------------------------------------")
  call add_card(hdr,"PIXTYPE","HEALPIX","HEALPIX Pixelisation")
  call add_card(hdr,"ORDERING","RING",  "Pixel ordering scheme, either RING or NESTED")
  call add_card(hdr,"NSIDE",ns,  "Resolution parameter for HEALPIX")
  call add_card(hdr,"FIRSTPIX",0,"First pixel # (0 based)")
  call add_card(hdr,"LASTPIX",np-1,"Last pixel # (0 based)")
  call add_card(hdr,"POLCCONV","COSMO"," Coord. convention for polarisation (COSMO/IAU)")
  call add_card(hdr)
  if(pol.eq.0) then  
    call add_card(hdr,"TTYPE1","TEMPERATURE","Temperature map")
    call add_card(hdr)  
  elseif(pol.eq.1) then
    call add_card(hdr,"TTYPE1","TEMPERATURE","Temperature map")
    call add_card(hdr)  
    call add_card(hdr,"TTYPE2", "Q-POLARISATION","Q Polarisation map")
    call add_card(hdr) 
    call add_card(hdr,"TTYPE3", "U-POLARISATION","U Polarisation map")
    call add_card(hdr)
  endif


  allocate(map(0:np-1,1:2*pol+1),stat=status)
  if(status /= 0) stop

  if(pol.eq.0) then
    allocate(mapaux(0:np-1),stat=status)
    if(status /= 0) stop
    call alm2map(ns,nl,nl,alm,mapaux)
    map(:,1)=mapaux(:)
    deallocate(mapaux)
  else if(pol.eq.1) then
    call alm2map(ns,nl,nl,alm,map)
  endif

  call output_map(map,hdr,outfile)

  deallocate(map)
  deallocate(hdr)

end subroutine asciialm2fitsmap

subroutine fitsalm2asciialm(pol,nl,almfile,alm)

  use healpix_types
  use fitstools 
  use alm_tools
  use head_fits

  implicit none

  integer(i4b), intent(in)  :: nl,pol
  character(len=filenamelen), intent(in) ::  almfile
  complex(dpc), dimension(1:2*pol+1,0:nl,0:nl), intent(out)  :: alm

  integer(i4b) :: i,j,nlh,ncl,next,nalms,status
  real(dp), dimension(:,:,:), allocatable   :: almaux
  character(len=80), dimension(:,:), allocatable  :: hdr

  nlh=1000
  nalms=(nl+1)*(nl+2)/2
  ncl=3
  next=2*pol+1

  allocate(hdr(1:nlh,1:next),stat = status)
  if (status /= 0) STOP 
  allocate(almaux(1:nalms,1:ncl+1,1:next),stat = status)
  if (status /= 0) STOP 

  call fits2alms(almfile,nalms,almaux,ncl,hdr,nlh,next) 

  do j = 1,next
   do i = 1,nalms
     alm(j,int(almaux(i,1,j)),int(almaux(i,2,j)))=dcmplx(almaux(i,3,j),almaux(i,4,j))
   end do
  end do

  deallocate(hdr)
  deallocate(almaux)

end subroutine fitsalm2asciialm

subroutine asciimap2asciicl(ns,map,nl,pol,cl)!!!(ns,map,nl,pol,cl,iter)

  use healpix_types
  use pix_tools
  use fitstools 
  use misc_utils
  use alm_tools
  
  implicit none

  integer(i4b), intent(in) :: nl, ns, pol !!!, iter
  real(dp),  dimension(0:12*ns**2-1,1:2*pol+1),intent(in) :: map
  real(dp),  dimension(0:nl,1:3*pol+1),intent(out) :: cl

  integer(kind=i4b) :: status
  real(dp),  dimension(:), allocatable :: zb, mapaux
  real(dp),  dimension(:,:), allocatable :: w8  
  complex(dpc), dimension(:,:,:), allocatable :: alm

  allocate(w8(1:2*ns,1:2*pol+1),stat = status)
  if (status /= 0) stop
  allocate(zb(1:2),stat = status)
  if (status /= 0) stop 
  allocate(alm(1:2*pol+1,0:nl,0:nl),stat = status)
  if (status /= 0) stop 

  w8(:,:) = 1.d0
  zb(:) = 0.d0

  if(pol.eq.0) then
    allocate(mapaux(0:12*ns**2-1),stat=status)
    if(status /= 0) stop    
    mapaux(:)=map(:,1)
    call map2alm(ns,nl,nl,mapaux,alm,zb,w8)
    deallocate(mapaux)
  elseif(pol.eq.1) then
    call map2alm(ns,nl,nl,map,alm,zb,w8)
  endif

!!$  call map2alm_iterative(ns,nl,nl,0,map,alm,zb,w8)
  
  call alm2cl(nl,nl,alm,cl)

  deallocate(w8)
  deallocate(zb)
  deallocate(alm)

end subroutine asciimap2asciicl


subroutine asciialm2fitsmapder(pol,nl,alm,ns,derthfile,derphfile)

  use healpix_types
  use pix_tools
  use fitstools 
  use misc_utils
  use alm_tools
  use head_fits
  
  implicit none

  integer(i4b), intent(in)  :: ns,nl,pol
  complex(dpc), dimension(1:2*pol+1,0:nl,0:nl), intent(in)  :: alm
  character(len=filenamelen), intent(out) :: derthfile,derphfile!,mapfile


  integer(i4b) :: np,nlh,status
  real(dp), dimension(:,:), allocatable         :: map,der
  real(dp), dimension(:), allocatable         :: mapaux
  character(len=80), dimension(:), allocatable  :: hdr

  np=12*ns**2
  nlh=1000

  allocate(hdr(1:nlh),stat = status)
  if (status /= 0) STOP 

  hdr(:) = " "

  call add_card(hdr,"COMMENT","----------------------------------------")
  call add_card(hdr,"COMMENT"," Sky Map Pixelisation Specific Keywords ")
  call add_card(hdr,"COMMENT","----------------------------------------")
  call add_card(hdr,"PIXTYPE","HEALPIX","HEALPIX Pixelisation")
  call add_card(hdr,"ORDERING","RING",  "Pixel ordering scheme, either RING or NESTED")
  call add_card(hdr,"NSIDE",ns,  "Resolution parameter for HEALPIX")
  call add_card(hdr,"FIRSTPIX",0,"First pixel # (0 based)")
  call add_card(hdr,"LASTPIX",np-1,"Last pixel # (0 based)")
  call add_card(hdr,"POLCCONV","COSMO"," Coord. convention for polarisation (COSMO/IAU)")
  call add_card(hdr)
  if(pol.eq.0) then
    call add_card(hdr,"TTYPE1","TEMPERATURE","Temperature map")
    call add_card(hdr)
  elseif(pol.eq.1) then
    call add_card(hdr,"TTYPE1","TEMPERATURE","Temperature map")
    call add_card(hdr)
    call add_card(hdr,"TTYPE2", "Q-POLARISATION","Q Polarisation map")
    call add_card(hdr) 
    call add_card(hdr,"TTYPE3", "U-POLARISATION","U Polarisation map")
    call add_card(hdr)
  endif

  allocate(map(0:np-1,1:2*pol+1),stat=status)
  if(status /= 0) stop
  allocate(der(0:np-1,1:2*(2*pol+1)),stat=status)
  if(status /= 0) stop

  if(pol.eq.0) then
    allocate(mapaux(0:np-1),stat=status)
    if(status /= 0) stop
    call alm2map_der(ns,nl,nl,alm,mapaux,der)
    map(:,1)=mapaux(:)
    deallocate(mapaux)
  else if(pol.eq.1) then
    call alm2map_der(ns,nl,nl,alm,map,der)
  endif
    
  if(pol.eq.0) then
    map(:,1)=der(:,1)
    call output_map(map,hdr,derthfile)
    map(:,1)=der(:,2)
    call output_map(map,hdr,derphfile)
  elseif(pol.eq.1) then
    map(:,1)=der(:,1)
    map(:,2)=der(:,2)
    map(:,3)=der(:,3)
    call output_map(map,hdr,derthfile)
    map(:,1)=der(:,4)
    map(:,2)=der(:,5)
    map(:,3)=der(:,6)
    call output_map(map,hdr,derphfile)
  endif

  deallocate(map)
  deallocate(der)
  deallocate(hdr)

end subroutine asciialm2fitsmapder


subroutine asciialm2asciimap(pol,nl,alm,ns,map)

  use healpix_types
  use pix_tools
  use fitstools 
  use misc_utils
  use alm_tools
  use head_fits
  
  implicit none

  integer(i4b), intent(in)  :: ns,nl,pol
  complex(dpc), dimension(1:2*pol+1,0:nl,0:nl), intent(in)  :: alm
  real(dp), dimension(0:12*ns**2-1,1:2*pol+1), intent(out)  :: map

  integer(i4b) :: status
  real(dp), dimension(:), allocatable         :: mapaux

  if(pol.eq.0) then
    allocate(mapaux(0:12*ns**2-1),stat=status)
    if(status /= 0) stop
    call alm2map(ns,nl,nl,alm,mapaux)
    map(:,1)=mapaux(:)
    deallocate(mapaux)
  else if(pol.eq.1) then
    call alm2map(ns,nl,nl,alm,map)
  endif

end subroutine asciialm2asciimap

subroutine asciialm2asciimapder(pol,nl,alm,ns,map,mapderth,mapderph)

  use healpix_types
  use pix_tools
  use fitstools 
  use misc_utils
  use alm_tools
  use head_fits
  
  implicit none

  integer(i4b), intent(in)  :: ns,nl,pol
  complex(dpc), dimension(1:2*pol+1,0:nl,0:nl), intent(in)  :: alm
  real(dp), dimension(0:12*ns**2-1,1:2*pol+1), intent(out) :: map
  real(dp), dimension(0:12*ns**2-1,1:2*pol+1), intent(out) :: mapderth,mapderph

  integer(i4b) :: status
  real(dp), dimension(:), allocatable    :: mapaux
  real(dp), dimension(:,:), allocatable  :: mapder

  allocate(mapder(0:12*ns**2-1,1:2*(2*pol+1)),stat=status)
  if(status /= 0) stop

  if(pol.eq.0) then
    allocate(mapaux(0:12*ns**2-1),stat=status)
    if(status /= 0) stop
    call alm2map_der(ns,nl,nl,alm,mapaux,mapder)
    map(:,1)=mapaux(:)
    deallocate(mapaux)
  else if(pol.eq.1) then
    call alm2map_der(ns,nl,nl,alm,map,mapder)
  endif

  mapderth(:,1:2*pol+1)=mapder(:,1:2*pol+1)
  mapderph(:,1:2*pol+1)=mapder(:,2*(pol+1):2*(2*pol+1))

  deallocate(mapder)

end subroutine asciialm2asciimapder

subroutine fitsmap2fitscl(pol,ns,mapfile,nl,clfile,iter)

  use healpix_types
  use pix_tools
  use fitstools 
  use misc_utils
  use alm_tools
  use head_fits

  implicit none

  integer(i4b), intent(in) :: iter
  integer(i4b), intent(in) :: nl, ns, pol
  character(len=filenamelen), intent(in) :: mapfile, clfile

  integer(kind=i4b) :: status, nlh
  real(dp),  dimension(:), allocatable :: zb
  real(dp),  dimension(:,:), allocatable :: map, w8, cl  
  complex(dpc), dimension(:,:,:), allocatable :: alm
  character(len=80), dimension(:), allocatable  :: hdr

  allocate(map(0:12*ns**2-1,1:2*pol+1),stat = status)
  if (status /= 0) stop

  call input_map(mapfile,map,12*ns*ns,2*pol+1) 
  
  allocate(w8(1:2*ns,1:2*pol+1),stat = status)
  if (status /= 0) stop
  allocate(zb(1:2),stat = status)
  if (status /= 0) stop 
  allocate(alm(1:2*pol+1,0:nl,0:nl),stat = status)
  if (status /= 0) stop 

  w8(:,:) = 1.d0
  zb(:) = 0.d0

  call map2alm_iterative(ns,nl,nl,iter,map,alm,zb,w8)

  deallocate(map)
  deallocate(w8)
  deallocate(zb)

  allocate(cl(0:nl,1:3*pol+1),stat = status)
  if (status /= 0) stop

  call alm2cl(nl,nl,alm,cl)
  
  deallocate(alm)
 
  nlh=1000

  allocate(hdr(1:nlh),stat = status)
  if (status /= 0) stop
  hdr(:)=" "

  call add_card(hdr,"COMMENT","-----------------------------------------------")
  call add_card(hdr,"COMMENT"," CMB power spectrum C(l) keywords          ")
  call add_card(hdr,"COMMENT","-----------------------------------------------")
  call add_card(hdr,"EXTNAME","'ANALYSED POWER SPECTRUM'")
  call add_card(hdr,"COMMENT"," POWER SPECTRUM : C(l) ")
  call add_card(hdr)
  call add_card(hdr,"POLAR",.true.," Polarisation included (True/False)")
  call add_card(hdr,"BCROSS",.false.," Magnetic cross terms included (True/False)")
  call add_card(hdr,"MAX-LPOL",nl," Maximum L multipole order")
  call add_card(hdr,"LREGRESS",0," Regression of low multipoles (0/1/2)")
  call add_card(hdr)
  call add_card(hdr,"TEMPTYPE","THERMO"," temperature : either THERMO or ANTENNA")
  if(pol.eq.1) call add_card(hdr,'POLNORM','CMBFAST','Uses E-B conventions')
  call add_card(hdr,"TTYPE1", "TEMPERATURE","Temperature C(l)")
  call add_card(hdr,"TUNIT1", "uK^2","square of map units")
  call add_card(hdr)
  if(pol.eq.1) then
    call add_card(hdr,"TTYPE2", "GRADIENT","Gradient (=ELECTRIC) polarisation C(l)")
    call add_card(hdr,"TUNIT2", "uK^2","power spectrum units")
    call add_card(hdr)
    call add_card(hdr,"TTYPE3", "CURL","Curl (=MAGNETIC) polarisation C(l)")
    call add_card(hdr,"TUNIT3", "uK^2","power spectrum units")
    call add_card(hdr)
    call add_card(hdr,"TTYPE4", "G-T","Gradient-Temperature (=CROSS) terms")
    call add_card(hdr,"TUNIT4", "uK^2","power spectrum units")
    call add_card(hdr)
    call add_card(hdr,"COMMENT","The polarisation power spectra have the same definition as in CMBFAST")
    call add_card(hdr)
  endif

  call write_asctab(cl,nl,3*pol+1,hdr,nlh,clfile)
  
  deallocate(cl)
  deallocate(hdr)

end subroutine fitsmap2fitscl

subroutine asciimap2fitscl(pol,ns,map,nl,clfile)

  use healpix_types
  use pix_tools
  use fitstools 
  use misc_utils
  use alm_tools
  use head_fits

  implicit none

  integer(i4b), intent(in) :: nl, ns, pol
  character(len=filenamelen), intent(in) :: clfile
  real(dp), dimension(0:12*ns**2-1,1:2*pol+1), intent(in) :: map

  integer(kind=i4b) :: status, nlh
  real(dp),  dimension(:), allocatable :: zb, mapaux
  real(dp),  dimension(:,:), allocatable :: w8, cl  
  complex(dpc), dimension(:,:,:), allocatable :: alm
  character(len=80), dimension(:), allocatable  :: hdr

  allocate(w8(1:2*ns,1:2*pol+1),stat = status)
  if (status /= 0) stop
  allocate(zb(1:2),stat = status)
  if (status /= 0) stop 
  allocate(alm(1:2*pol+1,0:nl,0:nl),stat = status)
  if (status /= 0) stop 

  w8(:,:) = 1.d0
  zb(:) = 0.d0

  if(pol.eq.0) then
    allocate(mapaux(0:12*ns**2-1),stat=status)
    if(status /= 0) stop    
    mapaux(:)=map(:,1)
    call map2alm(ns,nl,nl,mapaux,alm,zb,w8)
    deallocate(mapaux)
  elseif(pol.eq.1) then
    call map2alm(ns,nl,nl,map,alm,zb,w8)
  endif

  deallocate(w8)
  deallocate(zb)

  allocate(cl(0:nl,1:3*pol+1),stat = status)
  if (status /= 0) stop

  call alm2cl(nl,nl,alm,cl)
  
  deallocate(alm)
 
  nlh=1000

  allocate(hdr(1:nlh),stat = status)
  if (status /= 0) stop
  hdr(:)=" "

  call add_card(hdr,"COMMENT","-----------------------------------------------")
  call add_card(hdr,"COMMENT"," CMB power spectrum C(l) keywords          ")
  call add_card(hdr,"COMMENT","-----------------------------------------------")
  call add_card(hdr,"EXTNAME","'ANALYSED POWER SPECTRUM'")
  call add_card(hdr,"COMMENT"," POWER SPECTRUM : C(l) ")
  call add_card(hdr)
  call add_card(hdr,"POLAR",.true.," Polarisation included (True/False)")
  call add_card(hdr,"BCROSS",.false.," Magnetic cross terms included (True/False)")
  call add_card(hdr,"MAX-LPOL",nl," Maximum L multipole order")
  call add_card(hdr,"LREGRESS",0," Regression of low multipoles (0/1/2)")
  call add_card(hdr)
  call add_card(hdr,"TEMPTYPE","THERMO"," temperature : either THERMO or ANTENNA")
  if(pol.eq.1) call add_card(hdr,'POLNORM','CMBFAST','Uses E-B conventions')
  call add_card(hdr,"TTYPE1", "TEMPERATURE","Temperature C(l)")
  call add_card(hdr,"TUNIT1", "uK^2","square of map units")
  call add_card(hdr)
  if(pol.eq.1) then
    call add_card(hdr,"TTYPE2", "GRADIENT","Gradient (=ELECTRIC) polarisation C(l)")
    call add_card(hdr,"TUNIT2", "uK^2","power spectrum units")
    call add_card(hdr)
    call add_card(hdr,"TTYPE3", "CURL","Curl (=MAGNETIC) polarisation C(l)")
    call add_card(hdr,"TUNIT3", "uK^2","power spectrum units")
    call add_card(hdr)
    call add_card(hdr,"TTYPE4", "G-T","Gradient-Temperature (=CROSS) terms")
    call add_card(hdr,"TUNIT4", "uK^2","power spectrum units")
    call add_card(hdr)
    call add_card(hdr,"COMMENT","The polarisation power spectra have the same definition as in CMBFAST")
    call add_card(hdr)
  endif

  call write_asctab(cl,nl,3*pol+1,hdr,nlh,clfile)
  
  deallocate(cl)
  deallocate(hdr)

end subroutine asciimap2fitscl


END MODULE myhealmodule
