#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <complex.h>

#include "chealpix.h"
#include "nfft3.h"
#include "nfft3conf.h"
#include "nfft3util.h"
#include "kernels.h"
#include "fastsum.h"

/********************************************************************/
void norm_angle(double* , double*);

void get_fourier_mode(double , double complex* ,double complex*);

void get_lensed_temp_map_(int*, 
                          double complex*,
		          int*, int*, int*,
                          double*, double*,
			  double*);

void get_lensed_pol_map_(int*, 
                         double complex*, double complex*,
		         int*, int*, int*,
                         double*, double*,
			 double*, double*, double*);

/********************************************************************/
void get_lensed_temp_map_(int *lm, 
                          double complex *tms,
   		          int *ns, int *sig, int *kap, 
                          double *thmap, double *phmap,
		          double *ltmap){

  int np;
  nfft_plan pl;
  int N[2],n[2];
  int i,m,s,p,q,mps,delp;
  time_t timer1,timer2;
  double th, ph;
  /*HEALPix C Subroutine to compute number of pixel given te nside*/

  np=nside2npix(*ns); 

  /* Dimensions of fourier modes */

  N[0]=2*(*lm+1);
  N[1]=N[0];
  n[0]=(*sig)*N[0];
  n[1]=(*sig)*N[1];

  /*Initialization of NFFT*/

  nfft_init_guru(&pl,2, N, np, n,*kap,
		 PRE_PHI_HUT| PRE_PSI| MALLOC_F_HAT| MALLOC_X| MALLOC_F |
		 FFTW_INIT| FFT_OUT_OF_PLACE,
		 FFTW_ESTIMATE| FFTW_DESTROY_INPUT);

 /*Computaion of non-equisapced nodes before lensing
  The the range of  new theta and phi are scaled down to 
  [-0.5,0.5]*[-0.5,0.5]*/

  for (i = 0;i<=(np-1);i++){
    th=thmap[i];
    ph=phmap[i];
    if(th>=2.0*PI){
      th=th-2.0*PI;    
    }
    if(th<0.0){
      th=2.0*PI+th;
    }
    if(ph>=2.0*PI){
      ph=ph-2.0*PI; 
    }
    if(phmap[i]<0.0){
      ph=2.0*PI+ph;
    }
    th=-0.5+th/(2.0*PI);
    ph=-0.5+ph/(2.0*PI);
    pl.x[2*i]=ph;
    pl.x[2*i+1]=th;
  }

  /*Checks a transform plan for frequently used bad parameter*/

  nfft_check(&pl);

  /*Precompute psi, the entries of the matrix B*/

  if(pl.nfft_flags & PRE_ONE_PSI) nfft_precompute_one_psi(&pl);

  /*Computation of fourier modes for temperature after lensing
  The extra factor (double)(1-2*mps) and complex conjugation are required  
  to go from [0,2*\pi]*[0,2*\pi] grid to [-0.5,0.5]*[-0.5,0.5] grid
  */

  pl.f_hat[0]=0.0;

  for(s=0;s<=2*(*lm);s++){
    q=(s+1);
    pl.f_hat[q]=0.0;
  }

  for(m=0;m<=2*(*lm);m++){
    q=2*(*lm+1)*(m+1);
    pl.f_hat[q]=0.0;
  }

  for(m=0;m<=2*(*lm);m++){
    delp=(2*(*lm)+m+3);
    for(s=0;s<=2*(*lm);s++){
      p=(2*(*lm)+1)*m+s;
      q=p+delp;
      mps=(m+s)%2;
      pl.f_hat[q]=(double)(1-2*mps)*conj(tms[p]);
     }
  }  

  /*Do nfft for temperature after lensing*/

  nfft_trafo(&pl);

  /*Computation of temperature at the non-equispaced nodes after lensing
  Imaginary part of pl.f[i] should be zero, because temperature T(\hat n) 
  is a real field on the surface of sphere
  */

  for (i = 0;i<=(np-1);i++){
    ltmap[i]= creal(pl.f[i]);
    if(sqrt(cimag(pl.f[i])*cimag(pl.f[i]))>pow(10.0,-10)){
      printf("output data is not real");
    }
  }

  /*Finalization of NFFT*/

  nfft_finalize(&pl);

}

void get_lensed_pol_map_(int *lm, 
                         double complex *tms, double complex *pms,
		         int *ns, int *sig, int *kap, 
                         double *thmap, double *phmap,
		         double *ltmap, double *lqmap, double *lumap){

  int np;
  nfft_plan pl;
  int N[2],n[2];
  int i,m,s,p,q,mps,delp;
  time_t timer1,timer2;
  double th, ph;
  /*HEALPix C Subroutine to compute number of pixel given te nside*/

  np=nside2npix(*ns); 

  /* Dimensions of fourier modes */

  N[0]=2*(*lm+1);
  N[1]=N[0];
  n[0]=(*sig)*N[0];
  n[1]=(*sig)*N[1];

  /*Initialization of NFFT*/

  nfft_init_guru(&pl,2, N, np, n,*kap,
		 PRE_PHI_HUT| PRE_PSI| MALLOC_F_HAT| MALLOC_X| MALLOC_F |
		 FFTW_INIT| FFT_OUT_OF_PLACE,
		 FFTW_ESTIMATE| FFTW_DESTROY_INPUT);

 /*Computaion of non-equisapced nodes before lensing
  The the range of  new theta and phi are scaled down to 
  [-0.5,0.5]*[-0.5,0.5]*/

  for (i = 0;i<=(np-1);i++){
    th=thmap[i];
    ph=phmap[i];
    if(th>=2.0*PI){
      th=th-2.0*PI;    
    }
    if(th<0.0){
      th=2.0*PI+th;
    }
    if(ph>=2.0*PI){
      ph=ph-2.0*PI; 
    }
    if(phmap[i]<0.0){
      ph=2.0*PI+ph;
    }
    th=-0.5+th/(2.0*PI);
    ph=-0.5+ph/(2.0*PI);
    pl.x[2*i]=ph;
    pl.x[2*i+1]=th;
  }

  /*Checks a transform plan for frequently used bad parameter*/

  nfft_check(&pl);

  /*Precompute psi, the entries of the matrix B*/

  if(pl.nfft_flags & PRE_ONE_PSI) nfft_precompute_one_psi(&pl);

  /*Computation of fourier modes for temperature after lensing
  The extra factor (double)(1-2*mps) and complex conjugation are required  
  to go from [0,2*\pi]*[0,2*\pi] grid to [-0.5,0.5]*[-0.5,0.5] grid
  */

  pl.f_hat[0]=0.0;

  for(s=0;s<=2*(*lm);s++){
    q=(s+1);
    pl.f_hat[q]=0.0;
  }

  for(m=0;m<=2*(*lm);m++){
    q=2*(*lm+1)*(m+1);
    pl.f_hat[q]=0.0;
  }

  for(m=0;m<=2*(*lm);m++){
    delp=(2*(*lm)+m+3);
    for(s=0;s<=2*(*lm);s++){
      p=(2*(*lm)+1)*m+s;
      q=p+delp;
      mps=(m+s)%2;
      pl.f_hat[q]=(double)(1-2*mps)*conj(tms[p]);
     }
  }  

  /*Do nfft for temperature after lensing*/

  nfft_trafo(&pl);

  /*Computation of temperature at the non-equispaced nodes after lensing
  Imaginary part of pl.f[i] should be zero, because temperature T(\hat n) 
  is a real field on the surface of sphere
  */

  for (i = 0;i<=(np-1);i++){
    ltmap[i]= creal(pl.f[i]);
    if(sqrt(cimag(pl.f[i])*cimag(pl.f[i]))>pow(10.0,-10)){
      printf("output data is not real");
    }
  }

  /*Computation of fourier modes for polarization after lensing
  The extra factor (double)(1-2*mps) and complex conjugation are required  
  to go from [0,2*\pi]*[0,2*\pi] grid to [-0.5,0.5]*[-0.5,0.5] grid
  */

  pl.f_hat[0]=0.0;

  for(s=0;s<=2*(*lm);s++){
    q=(s+1);
    pl.f_hat[q]=0.0;
  }

  for(m=0;m<=2*(*lm);m++){
    q=2*(*lm+1)*(m+1);
    pl.f_hat[q]=0.0;
  }

  for(m=0;m<=2*(*lm);m++){
    delp=(2*(*lm)+m+3);
    for(s=0;s<=2*(*lm);s++){
      p=(2*(*lm)+1)*m+s;
      q=p+delp;
      mps=(m+s)%2;
      pl.f_hat[q]=(double)(1-2*mps)*conj(pms[p]);
     }
  }
 
  /*Do nfft for polarization after lensing*/

  nfft_trafo(&pl);

  /*Computation of polarization at the non-equispaced nodes after lensing*/

  for (i = 0;i<=(np-1);i++){
    lqmap[i]=creal(pl.f[i]);
    lumap[i]=cimag(pl.f[i]);
  }

  /*Finalization of NFFT*/

  nfft_finalize(&pl);

}

void norm_angle(double *th_ang, double *ph_ang){
  /*Computation of non-equispaced nodes:
  The the range of  new theta and phi are scaled down 
  to [-0.5,0.5]*[-0.5,0.5]*/

  double temp;

  if(*th_ang>=2.0*PI){
    temp=*th_ang;
    temp=temp-2.0*PI;
    *th_ang=temp;
  }
  if(*th_ang<0.0){
    temp=*th_ang;
    temp=2.0*PI+temp;
    *th_ang=temp;
  }
  if(*ph_ang>=2.0*PI){
    temp=*ph_ang;
    temp=temp-2.0*PI;
    *ph_ang=temp; 
  }
  if(*ph_ang<0.0){
    temp=*ph_ang;
    temp=2.0*PI+temp;
    *ph_ang=temp;
  }
  *th_ang=-0.5+*th_ang/(2.0*PI);
  *ph_ang=-0.5+*ph_ang/(2.0*PI);
}

void get_fourier_mode(double blimit, double complex *hmode,double complex *fmode){
  /*Computation of 2-d fourier modes from harmonic modes*/

  int m,s,p,q;
  int delp, mps;

  fmode[0]=0.0;

  for(s=0;s<=2*(blimit);s++){
    q=(s+1);
    fmode[q]=0.0;
  }

  for(m=0;m<=2*(blimit);m++){
    q=2*(blimit+1)*(m+1);
    fmode[q]=0.0;
  }

  for(m=0;m<=2*(blimit);m++){
    delp=(2*(blimit)+m+3);
    for(s=0;s<=2*(blimit);s++){
      p=(2*(blimit)+1)*m+s;
      q=p+delp;
      mps=(m+s)%2;
      fmode[q]=(double)(1-2*mps)*conj(hmode[p]);
     }
  }  

}
