; -----------------------------------------------------------------------------
;
; Copyright (C) 2010-2012
;   Guillaume Castex
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	DRAW_SOURCES
;
; PURPOSE:
;	Software being developed, not to be used.
;
;-
PRO DRAW_SOURCES, lon, lat, flux, fwhm, pointer_map, nside=nside, polar_angle=polar_angle, polar_percent=polar_percent, method=method, double=double, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'DRAW_SOURCES: procedure to create an healpix map with point sources on it'
     PSM_INFO_MESSAGE, 'DRAW_SOURCES, lon, lat, flux, fwhm, nside=, polar=, psi=, method='
     PSM_INFO_MESSAGE, "lon and lat: input array of longitudes and latitudes of the point sources"
     PSM_INFO_MESSAGE, "flux: input array of fluxes"
     PSM_INFO_MESSAGE, "fwhm in arcmin"
     PSM_INFO_MESSAGE, "method can be 'ANGLE' (default) or 'EULER'."
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN
  ENDIF

  IF NOT KEYWORD_SET(nside) THEN BEGIN
     PSM_INFO_MESSAGE, 'nside needed for drawing point sources'
     RETURN
  ENDIF

  npix = NSIDE2NPIX(nside)
  nsources=n_elements(lon)

  IF NOT DEFINED(polar_angle) THEN BEGIN
     IF NOT KEYWORD_SET(double) THEN carte = FLTARR(npix) ELSE carte = DBLARR(npix)
  ENDIF ELSE BEGIN
     IF NOT KEYWORD_SET(double) THEN carte = FLTARR(npix,3) ELSE carte = DBLARR(npix,3)
  ENDELSE
   
  pointer_map = PTR_NEW(carte,/no_copy)

  theta_gal=!pi/2-lat
   
  symmetric_source_convolver, flux, theta_gal, lon, pointer_map, fwhm, polar_percent=polar_percent, psi=polar_angle, method=method
  ;elliptic_source_convolver, flux, theta_gal, lon, pointer_map, fwhm, b_on_a, angle, polar_percent=polar_percent, psi=polar_angle, method=method


end
