PRO EMLAWSTRUCT_IN_BAND, em, freqband, output, polarised=polarised, double=double

  IF ISPOINTER(freqband) THEN band=*freqband ELSE band=freqband

  params = EMLAW_PARAMLIST(em.law)
  wh = WHERE(params NE 'NUREF', nwh)
  IF nwh NE 0 THEN params = params[wh] ELSE BEGIN
     IF STRLOWCASE(em.law) NE 'dirac' THEN MESSAGE, 'No parameters found for emission law ' + em.law
  ENDELSE
  
  PSM_DEBUG_MESSAGE, 'Check that integral is OK when numin is not 0 and numax not infinity', routine='EMLAWSTRUCT_IN_BAND', infolimit=2, stoplimit=3
  ;IF DEFINED(*em.numin.value) THEN IF *em.numin.value NE 0 THEN MESSAGE, 'Case not implemented'
  ;IF DEFINED(*em.numax.value) THEN IF *em.numax.value NE DOUBLE('Inf') THEN  MESSAGE, 'Case not implemented'

  IF NOT DEFINED(double) THEN double = PRECISION_ISDOUBLE()

  CASE STRLOWCASE(em.law) OF     

     ;;-----------------------
     ;; Integrate a power law
     ;;-----------------------

     'powerlaw': BEGIN

        ;; Reference frequency
        ;;---------------------
        nuref = *(em.nuref.value)
        IF N_ELEMENTS(nuref) NE 1 THEN MESSAGE, 'Case not implemented'

        ;; Polarised emission law ?
        ;;--------------------------
        IF (polarised AND ISPOLARISED(*em.specind.value)) THEN BEGIN
           signal = 'ALL' 
           lawpol = 1B
        ENDIF ELSE BEGIN
           signal = 'T'
           lawpol = 0B
        ENDELSE

        ;; Get spectral index
        ;;--------------------
        IF ISFITSFILE(*em.specind.value) THEN BEGIN
           form = GET_FORMAT(*em.specind.value)
           CASE STRLOWCASE(form) OF
              'map': BEGIN
                 READ_MAP, *em.specind.value, specind, signal=signal, /pointer
                 fmt = 'map'
              END
              'alm': BEGIN
                 READ_ALM, *em.specind.value, specind, signal=signal, /pointer
                 fmt = 'alm'
              END
              ELSE: MESSAGE, 'Case not implemented'
           ENDCASE
        ENDIF ELSE BEGIN
           specind = em.specind.value
        ENDELSE
        minspecind = MIN(*specind,max=maxspecind)
        
        ;; Make a table of spectral indexes for which the band integral will be computed (twice more points for double precision)
        ;;------------------------------------------------------------------------------------------------------------------------
        npt = LONG(1000.*(1+double)*(maxspecind-minspecind)+1)
        dspind = (maxspecind-minspecind) / ((npt-1D0)>1D0)
        spind = minspecind + FINDGEN(npt) * dspind
        IF double THEN table = DBLARR(npt) ELSE table = FLTARR(npt) 
        FOR p=0L, npt-1L DO table[p] = BANDINTEGRATED_EMLAW(band, emission_name=em.law, double=double, nuref=nuref, specind=spind[p], numin = *(em.numin.value), numax = *(em.numax.value))
        
        ;; Perform interpolation
        ;;-----------------------
        IF dspind ne 0 THEN value = (*specind-minspecind)/dspind ELSE value = 0
        w1 = ((value-LONG(value))>0)
        value = (1-w1) * table[LONG(value)] + w1 * table[CEIL(value)]
        w1 = 0
        npixvalue = (SIZE(value,/dimensions))[0]
               
        ;; Polarised amplitude ?
        ;;-----------------------
        amplpol = (polarised AND ISPOLARISED(*(em.ampl.value)))
        IF amplpol THEN signalampl='ALL' ELSE signalampl='T'
        
        ;; Get amplitude
        ;;---------------
        IF ISFITSFILE(*em.ampl.value) THEN BEGIN
           form = GET_FORMAT(*em.ampl.value)
           CASE STRLOWCASE(form) OF
              'map': BEGIN
                 READ_MAP, *em.ampl.value, ampl, signal=signalampl, /pointer
                 fmt = 'map'
              END
              'alm': BEGIN
                 READ_ALM, *em.ampl.value, ampl, signal=signalampl, /pointer
                 fmt = 'alm'
              END
              ELSE: MESSAGE, 'Case not implemented'
           ENDCASE
        ENDIF ELSE BEGIN
           ampl = em.ampl.value
        ENDELSE

        ;; Check dimensions (number of pixels, or number of alms)
        ;;--------------------------------------------------------
        IF (ISARRAY(*ampl) AND ISARRAY(value)) THEN BEGIN
           npixampl = (SIZE(*ampl,/dimensions))[0]
           IF npixampl NE npixvalue THEN MESSAGE, 'Case not implemented'
        ENDIF

        ;; Make output
        ;;-------------
        IF (amplpol AND lawpol) THEN output = (*ampl) * TEMPORARY(value)
        IF ((NOT amplpol) AND (NOT lawpol)) THEN output = (*ampl) * TEMPORARY(value)
        IF (amplpol AND (NOT lawpol)) THEN BEGIN
           CASE fmt OF 
              'map': BEGIN
                 FOR k=0,1 DO (*ampl)[*,k] = (*ampl)[*,k] * (value)
                 (*ampl)[*,2] = (*ampl)[*,2] * TEMPORARY(value)
                 output = (*ampl)
              END
              'alm': BEGIN
                 FOR k=0,1 DO (*ampl)[*,*,k] = (*ampl)[*,*,k] * (value)
                 (*ampl)[*,*,2] = (*ampl)[*,*,2] * TEMPORARY(value)
                 output = (*ampl)
              END
           ENDCASE
        ENDIF
        IF ((NOT amplpol) AND lawpol) THEN BEGIN
           MESSAGE, 'Case not implemented'
        ENDIF

        IF ISFITSFILE(*em.ampl.value) THEN PTR_FREE, ampl
        IF ISFITSFILE(*em.specind.value) THEN PTR_FREE, specind

     END


     ;;------------------------------
     ;; Integrate a curved power law
     ;;------------------------------

     'curvpowerlaw': BEGIN

        ;; Reference frequencies
        ;;-----------------------
        nuref = *(em.nuref.value)
        IF N_ELEMENTS(nuref) NE 1 THEN MESSAGE, 'Case not implemented'
        curvnuref = *(em.curvnuref.value)
        IF N_ELEMENTS(curvnuref) NE 1 THEN MESSAGE, 'Case not implemented'

        ;; Polarised emission law ?
        ;;--------------------------
        IF (polarised AND ISPOLARISED(*em.specind.value)) THEN BEGIN
           signal = 'ALL' 
           lawpol = 1B
        ENDIF ELSE BEGIN
           signal = 'T'
           lawpol = 0B
        ENDELSE

        ;; Get spectral index
        ;;--------------------
        IF ISFITSFILE(*em.specind.value) THEN BEGIN
           form = GET_FORMAT(*em.specind.value)
           CASE STRLOWCASE(form) OF
              'map': BEGIN
                 READ_MAP, *em.specind.value, specind, signal=signal, /pointer
                 fmt = 'map'
              END
              'alm': BEGIN
                 READ_ALM, *em.specind.value, specind, signal=signal, /pointer
                 fmt = 'alm'
              END
              ELSE: MESSAGE, 'Case not implemented'
           ENDCASE
        ENDIF ELSE BEGIN
           specind = em.specind.value
        ENDELSE
        minspecind = MIN(*specind,max=maxspecind)
        
        ;; Make a table of spectral indexes for which the band integral will be computed (twice more points for double precision)
        ;;------------------------------------------------------------------------------------------------------------------------
        npt = LONG(1000.*(1+double)*(maxspecind-minspecind)+1)
        dspind = (maxspecind-minspecind) / ((npt-1D0)>1D0)
        spind = minspecind + FINDGEN(npt) * dspind

        ;; Get curvature amplitude
        ;;-------------------------
        IF ISFITSFILE(*em.curvampl.value) THEN BEGIN
           form = GET_FORMAT(*em.curvampl.value)
           CASE STRLOWCASE(form) OF
              'map': BEGIN
                 READ_MAP, *em.curvampl.value, curvampl, signal=signal, /pointer
                 fmt = 'map'
              END
              'alm': BEGIN
                 READ_ALM, *em.curvampl.value, curvampl, signal=signal, /pointer
                 fmt = 'alm'
              END
              ELSE: MESSAGE, 'Case not implemented'
           ENDCASE
        ENDIF ELSE BEGIN
           curvampl = em.curvampl.value
        ENDELSE
        mincurvampl = MIN(*curvampl,max=maxcurvampl)

        IF mincurvampl EQ maxcurvampl THEN BEGIN
           npt2 = 1
           usecurvampl = mincurvampl
        ENDIF ELSE STOP

        IF double THEN table = DBLARR(npt) ELSE table = FLTARR(npt) 

        FOR p=0L, npt-1L DO table[p] = BANDINTEGRATED_EMLAW(band, emission_name=em.law, double=double, nuref=nuref, specind=spind[p], $
                                                            curvnuref = curvnuref, curvampl=usecurvampl, $
                                                            numin = *(em.numin.value), numax = *(em.numax.value))
        
        ;; Perform interpolation
        ;;-----------------------
        value = (*specind-minspecind)/dspind
        w1 = ((value-LONG(value))>0)
        value = (1-w1) * table[LONG(value)] + w1 * table[CEIL(value)]
        w1 = 0
        npixvalue = (SIZE(value,/dimensions))[0]
               
        ;; Polarised amplitude ?
        ;;-----------------------
        amplpol = (polarised AND ISPOLARISED(*(em.ampl.value)))
        IF amplpol THEN signalampl='ALL' ELSE signalampl='T'
        
        ;; Get amplitude
        ;;---------------
        IF ISFITSFILE(*em.ampl.value) THEN BEGIN
           form = GET_FORMAT(*em.ampl.value)
           CASE STRLOWCASE(form) OF
              'map': BEGIN
                 READ_MAP, *em.ampl.value, ampl, signal=signalampl, /pointer
                 fmt = 'map'
              END
              'alm': BEGIN
                 READ_ALM, *em.ampl.value, ampl, signal=signalampl, /pointer
                 fmt = 'alm'
              END
              ELSE: MESSAGE, 'Case not implemented'
           ENDCASE
        ENDIF ELSE BEGIN
           ampl = em.ampl.value
        ENDELSE

        ;; Check dimensions (number of pixels, or number of alms)
        ;;--------------------------------------------------------
        IF (ISARRAY(*ampl) AND ISARRAY(value)) THEN BEGIN
           npixampl = (SIZE(*ampl,/dimensions))[0]
           IF npixampl NE npixvalue THEN MESSAGE, 'Case not implemented'
        ENDIF

        ;; Make output
        ;;-------------
        IF (amplpol AND lawpol) THEN output = (*ampl) * TEMPORARY(value)
        IF ((NOT amplpol) AND (NOT lawpol)) THEN output = (*ampl) * TEMPORARY(value)
        IF (amplpol AND (NOT lawpol)) THEN BEGIN
           CASE fmt OF 
              'map': BEGIN
                 FOR k=0,1 DO (*ampl)[*,k] = (*ampl)[*,k] * (value)
                 (*ampl)[*,2] = (*ampl)[*,2] * TEMPORARY(value)
                 output = (*ampl)
              END
              'alm': BEGIN
                 FOR k=0,1 DO (*ampl)[*,*,k] = (*ampl)[*,*,k] * (value)
                 (*ampl)[*,*,2] = (*ampl)[*,*,2] * TEMPORARY(value)
                 output = (*ampl)
              END
           ENDCASE
        ENDIF
        IF ((NOT amplpol) AND lawpol) THEN BEGIN
           MESSAGE, 'Case not implemented'
        ENDIF

        IF ISFITSFILE(*em.ampl.value) THEN PTR_FREE, ampl
        IF ISFITSFILE(*em.specind.value) THEN PTR_FREE, specind
        IF ISFITSFILE(*em.curvampl.value) THEN PTR_FREE, curvampl

     END

     ;;----------------------
     ;; Integrate a greybody
     ;;----------------------

     'greybody': BEGIN

        ;; Reference frequency
        ;;---------------------
        nuref = *(em.nuref.value)
        IF N_ELEMENTS(nuref) NE 1 THEN MESSAGE, 'Case not implemented'

        ;; Spectral index
        ;;----------------
        listind = PSM_UNIQ(*(em.specind.value))
        IF N_ELEMENTS(listind) NE 1 THEN BEGIN
           MESSAGE, 'Case not implemented'
        ENDIF ELSE BEGIN
           specind = (*(em.specind.value))[listind]
        ENDELSE
        
        ;; Polarised emission law ?
        ;;--------------------------
        IF (polarised AND ISPOLARISED(*em.temp.value)) THEN BEGIN
           signal = 'ALL' 
           lawpol = 1B
        ENDIF ELSE BEGIN
           signal = 'T'
           lawpol = 0B
        ENDELSE

        ;; Get temperature
        ;;-----------------
        IF ISFITSFILE(*em.temp.value) THEN BEGIN
           form = GET_FORMAT(*em.temp.value)
           CASE STRLOWCASE(form) OF
              'map': BEGIN
                 READ_MAP, *em.temp.value, temp, signal=signal, /pointer
                 fmt = 'map'
              END
              'alm': BEGIN
                 READ_ALM, *em.temp.value, temp, signal=signal, /pointer
                 fmt = 'alm'
              END
              ELSE: MESSAGE, 'Case not implemented'
           ENDCASE
        ENDIF ELSE BEGIN
           temp = em.temp.value
        ENDELSE
        mintemp = MIN(*temp,max=maxtemp)

        ;; Make a table of temperatures for which the band integral will be computed (twice more points for double precision)
        ;;--------------------------------------------------------------------------------------------------------------------
        PSM_DEBUG_MESSAGE, 'Check number of points for required precision in integration', routine='EMLAWSTRUCT_IN_BAND', infolimit=2, stoplimit=3
        npt = LONG(1000.*(1+double)*(maxtemp-mintemp)+1)
        dt = (maxtemp-mintemp) / ((npt-1D0)>1D0)
        t = mintemp + FINDGEN(npt) * dt
        IF double THEN table = DBLARR(npt) ELSE table = FLTARR(npt) 
        FOR p=0L, npt-1L DO table[p] = BANDINTEGRATED_EMLAW(band, emission_name=em.law, double=double, nuref=nuref, temp=t[p], specind=specind, numin = *(em.numin.value), numax = *(em.numax.value))

        ;; Perform interpolation
        ;;-----------------------
        value = (*temp-mintemp)/dt
        w1 = ((value-LONG(value))>0)
        value = (1-w1) * table[LONG(value)] + w1 * table[CEIL(value)]
        w1 = 0
        npixvalue = (SIZE(value,/dimensions))[0]
               
        ;; Polarised amplitude ?
        ;;-----------------------
        amplpol = (polarised AND ISPOLARISED(*(em.ampl.value)))
        IF amplpol THEN signalampl='ALL' ELSE signalampl='T'
        
        ;; Get amplitude
        ;;---------------
        IF ISFITSFILE(*em.ampl.value) THEN BEGIN
           form = GET_FORMAT(*em.ampl.value)
           CASE STRLOWCASE(form) OF
              'map': BEGIN
                 READ_MAP, *em.ampl.value, ampl, signal=signalampl, /pointer
                 fmt = 'map'
              END
              'alm': BEGIN
                 READ_ALM, *em.ampl.value, ampl, signal=signalampl, /pointer
                 fmt = 'alm'
              END
              ELSE: MESSAGE, 'Case not implemented'
           ENDCASE
        ENDIF ELSE BEGIN
           ampl = em.ampl.value
        ENDELSE


        ;; Check dimensions (number of pixels, or number of alms)
        ;;--------------------------------------------------------
        IF (ISARRAY(*ampl) AND ISARRAY(value)) THEN BEGIN
           npixampl = (SIZE(*ampl,/dimensions))[0]
           IF npixampl NE npixvalue THEN MESSAGE, 'Case not implemented'
        ENDIF

        ;; Make output
        ;;-------------
        IF (amplpol AND lawpol) THEN output = (*ampl) * TEMPORARY(value)
        IF ((NOT amplpol) AND (NOT lawpol)) THEN output = (*ampl) * TEMPORARY(value)
        IF (amplpol AND (NOT lawpol)) THEN BEGIN
           CASE fmt OF 
              'map': BEGIN
                 FOR k=0,1 DO (*ampl)[*,k] = (*ampl)[*,k] * (value)
                 (*ampl)[*,2] = (*ampl)[*,2] * TEMPORARY(value)
                 output = TEMPORARY(*ampl)
              END
              'alm': BEGIN
                 FOR k=0,1 DO (*ampl)[*,*,k] = (*ampl)[*,*,k] * (value)
                 (*ampl)[*,*,2] = (*ampl)[*,*,2] * TEMPORARY(value)
                 output = TEMPORARY(*ampl)
              END
           ENDCASE
        ENDIF
        IF ((NOT amplpol) AND lawpol) THEN BEGIN
           MESSAGE, 'Case not implemented'
        ENDIF

        IF ISFITSFILE(*em.ampl.value) THEN PTR_FREE, ampl
        IF ISFITSFILE(*em.temp.value) THEN PTR_FREE, temp
 
         
     END

     ;;------------------------------
     ;; Integrate a dirac law
     ;;------------------------------

     'dirac': BEGIN

        ;; Reference frequency
        ;;---------------------
        nuref = *(em.nuref.value)
        IF N_ELEMENTS(nuref) NE 1 THEN MESSAGE, 'Case not implemented'
        
        res = BANDINTEGRATED_EMLAW(freqband,emission_name='dirac',nuref=nuref,double=double)
        IF ISFITSFILE(*em.ampl.value) THEN BEGIN
           form = GET_FORMAT(*em.ampl.value)
           CASE STRLOWCASE(form) OF
              'map': BEGIN
                 READ_MAP, *em.ampl.value, ampl, signal=signal, /pointer
                 fmt = 'map'
              END
              'alm': BEGIN
                 READ_ALM, *em.ampl.value, ampl, signal=signal, /pointer
                 fmt = 'alm'
              END
              ELSE: MESSAGE, 'Case not implemented'
           ENDCASE
        ENDIF ELSE BEGIN
           ampl = em.ampl.value
        ENDELSE
        output = (*ampl)*res
     END

     ;;------------------------------
     ;; Integrate a submmlapi law
     ;;------------------------------

     'submmlapi': BEGIN

        nuref = *(em.nuref.value)
        IF N_ELEMENTS(nuref) NE 1 THEN MESSAGE, 'Case not implemented'
        
        res = BANDINTEGRATED_EMLAW(freqband,emission_name='submmlapi',nuref=nuref,redshift=*(em.redshift.value), double=double)
        IF ISFITSFILE(*em.ampl.value) THEN BEGIN
           form = GET_FORMAT(*em.ampl.value)
           CASE STRLOWCASE(form) OF
              'map': BEGIN
                 READ_MAP, *em.ampl.value, ampl, signal=signal, /pointer
                 fmt = 'map'
              END
              'alm': BEGIN
                 READ_ALM, *em.ampl.value, ampl, signal=signal, /pointer
                 fmt = 'alm'
              END
              ELSE: MESSAGE, 'Case not implemented'
           ENDCASE
        ENDIF ELSE BEGIN
           ampl = em.ampl.value
        ENDELSE
        output = (*ampl)*res
     END

     ;;------------------------------
     ;; Integrate a spiralgal law
     ;;------------------------------

     'spiralgal': BEGIN

        nuref = *(em.nuref.value)
        IF N_ELEMENTS(nuref) NE 1 THEN MESSAGE, 'Case not implemented'
        
        res = BANDINTEGRATED_EMLAW(freqband,emission_name='spiralgal',nuref=nuref,redshift=*(em.redshift.value), double=double) 
        IF ISFITSFILE(*em.ampl.value) THEN BEGIN
           form = GET_FORMAT(*em.ampl.value)
           CASE STRLOWCASE(form) OF
              'map': BEGIN
                 READ_MAP, *em.ampl.value, ampl, signal=signal, /pointer
                 fmt = 'map'
              END
              'alm': BEGIN
                 READ_ALM, *em.ampl.value, ampl, signal=signal, /pointer
                 fmt = 'alm'
              END
              ELSE: MESSAGE, 'Case not implemented'
           ENDCASE
        ENDIF ELSE BEGIN
           ampl = em.ampl.value
        ENDELSE
        output = (*ampl)*res
     END


     ;;------------------------------
     ;; Integrate a spiralgal law
     ;;------------------------------

     'starburstgal': BEGIN

        nuref = *(em.nuref.value)
        IF N_ELEMENTS(nuref) NE 1 THEN MESSAGE, 'Case not implemented'
        
        res = BANDINTEGRATED_EMLAW(freqband,emission_name='starburstgal',nuref=nuref,redshift=*(em.redshift.value), double=double)
        IF ISFITSFILE(*em.ampl.value) THEN BEGIN
           form = GET_FORMAT(*em.ampl.value)
           CASE STRLOWCASE(form) OF
              'map': BEGIN
                 READ_MAP, *em.ampl.value, ampl, signal=signal, /pointer
                 fmt = 'map'
              END
              'alm': BEGIN
                 READ_ALM, *em.ampl.value, ampl, signal=signal, /pointer
                 fmt = 'alm'
              END
              ELSE: MESSAGE, 'Case not implemented'
           ENDCASE
        ENDIF ELSE BEGIN
           ampl = em.ampl.value
        ENDELSE
        output = (*ampl)*res
     END

     ;;------------------------------
     ;; Integrate a submm Cai law
     ;;------------------------------

     'submmcai': BEGIN

        nuref = *(em.nuref.value)
        IF N_ELEMENTS(nuref) NE 1 THEN MESSAGE, 'Case not implemented'
        
        res = BANDINTEGRATED_EMLAW(freqband,emission_name='submmcai',nuref=nuref,redshift=*(em.redshift.value), double=double)
        IF ISFITSFILE(*em.ampl.value) THEN BEGIN
           form = GET_FORMAT(*em.ampl.value)
           CASE STRLOWCASE(form) OF
              'map': BEGIN
                 READ_MAP, *em.ampl.value, ampl, signal=signal, /pointer
                 fmt = 'map'
              END
              'alm': BEGIN
                 READ_ALM, *em.ampl.value, ampl, signal=signal, /pointer
                 fmt = 'alm'
              END
              ELSE: MESSAGE, 'Case not implemented'
           ENDCASE
        ENDIF ELSE BEGIN
           ampl = em.ampl.value
        ENDELSE
        output = (*ampl)*res
     END

  ENDCASE
  

END

