PRO OBSERVE_MODEL_CAT, model_cat, instrband, result, whichsrc=whichsrc, nopolar=nopolar, polar=polar, n_integ_pts=n_integ_pts, approx=approx, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'OBSERVE_MODEL_CAT: procedure to get the in-band flux (in mJy) of sources in a model catalog'
     PSM_INFO_MESSAGE, 'SYNTAX: OBSERVE_MODEL_CAT, model_cat, band, result, whichsrc=, n_integ_pts=, /nopolar, /polar'
     PSM_INFO_MESSAGE, "set 'whichsrc' to an array of indices to observe only a fraction of the sources"
     PSM_INFO_MESSAGE, "'model_cat' is the input model catalogue (created with CREATE_PS_MODEL_CAT)"
     PSM_INFO_MESSAGE, "'instrband' is the input band (created with GET_BAND_STRUCT)"
     PSM_INFO_MESSAGE, "'result' is the output array of dim [n_sources] if I only, [n_sources,3] if IQU"
     PSM_INFO_MESSAGE, "set /nopolar or /polar to force the output to be unpolarised or polarised"
     PSM_INFO_MESSAGE, "set 'n_integ_pts' to the number of points used to perform band integration (100 is the default)"
     PSM_INFO_MESSAGE, "set 'approx' to compute the sum of the emission law at tabulated nu values (faster) instead of using INT_TABULATED"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN
  ENDIF

  IF instrband.shape EQ 'INSTR' THEN BEGIN
     IF NOT PTR_VALID(instrband.data) THEN FILL_BAND_DATA, instrband
     band = *instrband.data
  ENDIF ELSE band=instrband

  IF NOT KEYWORD_SET(n_integ_pts) THEN n_integ_pts=100

  nlaw = model_cat.nlaw
  IF NOT DEFINED(whichsrc) THEN BEGIN
     nsources = model_cat.nsources
     whichsrc = LINDGEN(nsources)
  ENDIF ELSE BEGIN
     nsources = N_ELEMENTS(whichsrc)
     IF ((MAX(whichsrc) GE model_cat.nsources) OR (MIN(whichsrc) LT 0)) THEN BEGIN
        PSM_INFO_MESSAGE, 'Invalid source index encountered in source list'
        RETURN
     ENDIF
  ENDELSE

  IF model_cat.polarised THEN BEGIN
     IF NOT KEYWORD_SET(nopolar) THEN BEGIN
        ispol = 1B
        result = DBLARR(nsources,3)
     ENDIF ELSE BEGIN
        ispol = 0B
        result = DBLARR(nsources)
     ENDELSE
  ENDIF ELSE BEGIN
     IF NOT KEYWORD_SET(polar) THEN BEGIN
        ispol = 0B
        result = DBLARR(nsources)
     ENDIF ELSE BEGIN
        ispol = 0B
        result = DBLARR(nsources,3)
     ENDELSE
  ENDELSE

  ;;----------------------------------------------------------------------
  ;; First algorithm: extract each source and integrate 
  ;; Slower for large number of sources (extracting sources is costly)
  ;;----------------------------------------------------------------------
;;   IF algo EQ 1 THEN BEGIN
;;      FOR i=0L, nsources-1 DO BEGIN
        
;;         source = EXTRACT_SOURCE(model_cat,i)
;;         em = 0.

;;         FOR j=1, nlaw DO BEGIN
;;            ok = EXECUTE('emi = source.e'+NUMBER2STRING(j))
;;            CASE emi.ampl.unit OF
;;               'mJy': factor=1
;;               'Jy': factor=1000
;;            ENDCASE
;;            em = em + BANDINTEGRATED_EMLAW(band,emi, n_integ_pts=n_integ_pts ) * (*emi.ampl.value) * factor
;;         ENDFOR
;;         IF ispol THEN result[i,*] = result[i,*] + em ELSE result[i] = result[i] + em
                
;;      ENDFOR
     
;;   ENDIF

  ;;----------------------------------------------------------------------
  ;; Second algorithm: integrate a multidimensional table and interpolate
  ;; Faster for large number of sources
  ;;----------------------------------------------------------------------
;;   IF algo EQ 2 THEN BEGIN
;;      FOR j=1, nlaw DO BEGIN
;;         ;stop
;;         ok = EXECUTE('EMLAWSTRUCT_IN_BAND, model_cat.e'+NUMBER2STRING(j)+', band, output, polarised=ispol')
;;         result = result + output
;;      ENDFOR
;;   ENDIF
     
  ;;----------------------------------------------------------------------
  ;; Third algorithm: simple integral for each source
  ;; Slower for large number of sources, but simpler and always works
  ;;----------------------------------------------------------------------

  doglobal = 1B
  IF model_cat.polarised THEN doglobal = (model_cat.polar_model EQ 'GLOBAL') ELSE doglobal = 1B

  IF doglobal THEN BEGIN

        FOR j=1, nlaw DO BEGIN
           BAND_NUMINMAX, band, numinband, numaxband
           ok = EXECUTE('em = model_cat.e'+NUMBER2STRING(j))
           CASE em.ampl.unit OF
              'mJy': factor=1
              'Jy': factor=1000
           ENDCASE
           
           ;; Check size of parameters of the emission law + avoid computing if result is 0 because of nu range
           ;;---------------------------------------------------------------------------------------------------
           ndim = SIZE(*em.numin.value,/n_dimensions) 
           IF ndim GE 2 THEN BEGIN
              MESSAGE, 'CASE NOT IMPLEMENTED'
           ENDIF ELSE BEGIN
              IF numaxband LT *em.numin.value THEN GOTO, do_nothing
           ENDELSE
           ndim = SIZE(*em.numax.value,/n_dimensions)
           IF ndim GE 2 THEN BEGIN
              MESSAGE, 'CASE NOT IMPLEMENTED'
           ENDIF ELSE BEGIN
              IF numinband GT *em.numax.value THEN GOTO, do_nothing
           ENDELSE
           ndim = SIZE(*em.nuref.value,/n_dimensions) & IF ndim GE 2 THEN MESSAGE, 'CASE NOT IMPLEMENTED'
           ndim = SIZE(*em.specind.value,/n_dimensions) & IF ndim GE 2 THEN MESSAGE, 'CASE NOT IMPLEMENTED'
           nspecind = N_ELEMENTS((*em.specind.value)[*,0])
           
           
           CASE STRUPCASE(em.law) OF
              
              ;;----------------------------
              "POWERLAW": BEGIN                 
                 IF STRUPCASE(band.shape) EQ 'DIRAC' THEN BEGIN
                    res = BANDINTEGRATED_EMLAW(band,emission_name=em.law,numin=(*em.numin.value),numax=(*em.numax.value),$
                                               nuref=(*em.nuref.value),specind=(*em.specind.value)[whichsrc,0])
                    result[*,0] = result[*,0] + ((*em.ampl.value)[whichsrc,0] * (res[*,0] * factor))
                 ENDIF ELSE BEGIN
                    CASE nspecind OF
                       model_cat.nsources: BEGIN
                          IF NOT KEYWORD_SET(approx) THEN BEGIN
                             FOR s=0L, nsources-1 DO BEGIN
                                specind = (*em.specind.value)[whichsrc[s],0]
                                nnuref = N_ELEMENTS(*em.nuref.value)
                                maxi = 0
                                IF nnuref EQ nsources THEN maxi=nsources
                                res = BANDINTEGRATED_EMLAW(band,emission_name=em.law,numin=(*em.numin.value),numax=(*em.numax.value),$
                                                           nuref=(*em.nuref.value)[whichsrc[s]<maxi],specind=specind)
                                IF PSM_DEBUG_VALUE() GE 1 THEN BEGIN
                                   wh = WHERE(FINITE(res) EQ 0, nwh) 
                                   IF nwh NE 0 THEN STOP
                                ENDIF
                                result[s,0] = result[s,0] + ((*em.ampl.value)[whichsrc[s],0] * res * factor)
                             ENDFOR
                          ENDIF ELSE BEGIN
                             specind = (*em.specind.value)[*,0]
                             res = BANDINTEGRATED_EMLAW(band,emission_name=em.law,numin=(*em.numin.value),numax=(*em.numax.value),$
                                                        nuref=(*em.nuref.value),specind=specind,/approx)
                             IF PSM_DEBUG_VALUE() GE 1 THEN BEGIN
                                wh = WHERE(FINITE(res) EQ 0, nwh) 
                                IF nwh NE 0 THEN STOP
                             ENDIF
                             result[*,0] = result[*,0] + ((*em.ampl.value)[whichsrc,0] * (res * factor))
                          ENDELSE
                       END
                       1: BEGIN
                          specind = (*em.specind.value)[*,0]
                          res = BANDINTEGRATED_EMLAW(band,emission_name=em.law,numin=(*em.numin.value),numax=(*em.numax.value),$
                                                     nuref=(*em.nuref.value),specind=specind)
                          IF PSM_DEBUG_VALUE() GE 1 THEN BEGIN
                             wh = WHERE(FINITE(res) EQ 0, nwh) 
                             IF nwh NE 0 THEN STOP
                          ENDIF
                          result[*,0] = result[*,0] + ((*em.ampl.value)[whichsrc,0] * (res * factor))
                       END
                    ENDCASE
                 ENDELSE
              END
              ;;----------------------------
              "GREYBODY": BEGIN
                 ndim = SIZE(*em.temp.value,/n_dimensions) & IF ndim GE 2 THEN MESSAGE, 'CASE NOT IMPLEMENTED'
                 ntemp = N_ELEMENTS(*em.temp.value)
                 ncalc = ntemp > nspecind
                 IF nsources NE 1 THEN BEGIN
                    CASE ncalc OF
                       model_cat.nsources: BEGIN
                          FOR s=0L, nsources-1 DO BEGIN
                             specind = (*em.specind.value)[whichsrc[s]<(nspecind-1),0]
                             temp = (*em.temp.value)[whichsrc[s]<(nspecind-1),0]
                             res = BANDINTEGRATED_EMLAW(band,emission_name=em.law,numin=(*em.numin.value),numax=(*em.numax.value),$
                                                        nuref=(*em.nuref.value),specind=specind,temp=temp)
                             IF PSM_DEBUG_VALUE() GE 1 THEN BEGIN
                                wh = WHERE(FINITE(res) EQ 0, nwh) 
                                IF nwh NE 0 THEN STOP
                             ENDIF
                             result[s,0] = result[s,0] + ((*em.ampl.value)[whichsrc[s],0] * res * factor)
                          ENDFOR
                       END
                       1: BEGIN
                          res = BANDINTEGRATED_EMLAW(band,emission_name=em.law,numin=(*em.numin.value),numax=(*em.numax.value),$
                                                     nuref=(*em.nuref.value),specind=(*em.specind.value)[whichsrc,0],temp=(*em.temp.value))
                          IF PSM_DEBUG_VALUE() GE 1 THEN BEGIN
                             wh = WHERE(FINITE(res) EQ 0, nwh) 
                             IF nwh NE 0 THEN STOP
                          ENDIF
                          result[s,0] = result[s,0] + ((*em.ampl.value)[whichsrc[s],0] * (res * factor))
                       END
                    ENDCASE
                 ENDIF ELSE BEGIN
                    res = BANDINTEGRATED_EMLAW(band,emission_name=em.law,numin=(*em.numin.value),numax=(*em.numax.value),$
                                               nuref=(*em.nuref.value),specind=(*em.specind.value)[whichsrc,0],temp=(*em.temp.value))
                    IF PSM_DEBUG_VALUE() GE 1 THEN BEGIN
                       wh = WHERE(FINITE(res) EQ 0, nwh) 
                       IF nwh NE 0 THEN STOP
                    ENDIF
                    result[s,0] = result[s,0] + ((*em.ampl.value)[whichsrc[s],0] * (res * factor))
                 ENDELSE
              END
              ;;-------------------------
           ENDCASE
           do_nothing:
        ENDFOR

        ;; Compute the polarisation signal from global polar amplitude and angle
        ;;-----------------------------------------------------------------------
        IF ispol THEN BEGIN 
           result[*,1] = result[*,0] * (*model_cat.pol_fraction)[whichsrc] * COS(2*(*model_cat.pol_angle)[whichsrc])
           result[*,2] = result[*,0] * (*model_cat.pol_fraction)[whichsrc] * SIN(2*(*model_cat.pol_angle)[whichsrc])
        ENDIF
     
     ENDIF ELSE BEGIN
        MESSAGE, 'Case not implemented'
     ENDELSE
  
  
END
