; Copyright 2009 Jacques Delabrouille, CNRS

; This file is part of the Planck Sky Model.
;
; The Planck Sky Model is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; version 2 of the License.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY, without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model. If not, write to the Free Software
; Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

;+
; Software to write a PSM point source data catalogue in the appropriate fits file format
;
; @categories ps
;
; @param filename {in}{required}{type=string} Name of the file to be written
; @param cat {in}{required}{type=array of structures} The catalogue to be written
; @keyword psm_hdr {in}{required}{type=structure} The PSM header of the catalogue
; @uses NUMBER2STRING (PSM), MWRFITS, HEADFITS, MODFITS, SXADDPAR, SXPAR (ASTRON)
; @author Jacques Delabrouille
; @history <p>15/04/2009: First version, Jacques Delabrouille</p>
; @copyright Jacques Delabrouille, CNRS, 2009
;-
PRO WRITE_OLD_PS_DATA_CAT, filename, cat, psm_hdr=psm_hdr


  psm_datatype = psm_hdr.datatype
  IF psm_datatype NE 'PS_DATA_CAT' THEN MESSAGE, 'DATATYPE IS NOT WHAT EXPECTED'

  ;; Remove existing file if any, write catalog
  FILE_DELETE, filename, /allow_nonexistent
  MWRFITS, cat, filename

  ;; Update header of extension 0 with PSM specific header
  psm_header = HEADFITS(filename,exten=0)
  tags = TAG_NAMES(psm_hdr)
  FOR i=0, N_ELEMENTS(tags)-1 DO BEGIN
     IF NOT ISARRAY(psm_hdr.(i)) THEN BEGIN
        res=EXECUTE("SXADDPAR, psm_header, '"+tags(i)+"', psm_hdr."+tags(i))
     ENDIF ELSE BEGIN
        nel = N_ELEMENTS(psm_hdr.(i))
        FOR j=0, nel-1 DO BEGIN
           res=EXECUTE("SXADDPAR, psm_header, '"+tags(i)+NUMBER2STRING(j)+"', psm_hdr."+tags(i)+'['+NUMBER2STRING(j)+']')
        ENDFOR           
     ENDELSE
  ENDFOR
  MODFITS, filename, 0, psm_header, exten_no=0

  ;; Update header of extension 1 with units for useful fields
  h = HEADFITS(filename, exten=1)
  fields = STRTRIM(SXPAR(h, 'TTYPE*', count=nfields),2)

  wh = WHERE(fields EQ 'GLON_RAD', nwh)
  IF nwh LE 1 THEN BEGIN
     IF nwh EQ 1 THEN SXADDPAR, h, 'TUNIT'+NUMBER2STRING(wh[0]+1), 'radian', after = 'TTYPE'+NUMBER2STRING(wh[0]+1)
  ENDIF ELSE MESSAGE, 'AMBIGUOUS FIELD: GLON_RAD'

  wh = WHERE(fields EQ 'GLAT_RAD', nwh)
  IF nwh LE 1 THEN BEGIN
     IF nwh EQ 1 THEN SXADDPAR, h, 'TUNIT'+NUMBER2STRING(wh[0]+1), 'radian', after = 'TTYPE'+NUMBER2STRING(wh[0]+1)
  ENDIF ELSE MESSAGE, 'AMBIGUOUS FIELD: GLAT_RAD'

  wh = WHERE(fields EQ 'IFLUX', nwh)
  IF nwh LE 1 THEN BEGIN
     IF nwh EQ 1 THEN SXADDPAR, h, 'TUNIT'+NUMBER2STRING(wh[0]+1), psm_hdr.units[wh(0)], after = 'TTYPE'+NUMBER2STRING(wh[0]+1)
  ENDIF ELSE MESSAGE, 'AMBIGUOUS FIELD: IFLUX'

  wh = WHERE(fields EQ 'IERR', nwh)
  IF nwh LE 1 THEN BEGIN
     IF nwh EQ 1 THEN SXADDPAR, h, 'TUNIT'+NUMBER2STRING(wh[0]+1), psm_hdr.units[wh(0)], after = 'TTYPE'+NUMBER2STRING(wh[0]+1)
  ENDIF ELSE MESSAGE, 'AMBIGUOUS FIELD: IERR'

  MODFITS, filename, 0, h, exten_no=1

END
