; Copyright 2009 Jacques Delabrouille, CNRS

; This file is part of the Planck Sky Model.
;
; The Planck Sky Model is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; version 2 of the License.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY, without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model. If not, write to the Free Software
; Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

;+
; Reads the specific parameters used for the generation of the CMB in the PSM
;
; @categories pipe
;   
; @param table {in}{required}{type=strarr(*,2)} vector of couple (keyword, value)
; @param pixp {out}{required}{type=struct} pixelisation specific parameters
; @keyword psm_global_param {in}{optional}{type=structure} global parameters of the PSM run (from which sim_resolution and sim_lmax are read if needed). 
; @keyword carefulness {in}{optional}{type=integer} level of carefulness in handling missing or repeated keywords in the table. 
;
; @uses fill_struct
; 
; @history <p>22/05/2009: First version, Jacques Delabrouille </p>
; @copyright <p>Jacques Delabrouille, CNRS, 2009</p>
;-
FUNCTION READ_PIX_PARAM, table, carefulness=carefulness

which_pix = {sky_pixelisation: 'HEALPIX'}  ; for now only HEALPIX {'HEALPIX','PATCH'}
FILL_STRUCT, table, which_pix, tags=['SKY_PIXELISATION'], carefulness=carefulness

CASE which_pix.sky_pixelisation OF
   'HEALPIX': BEGIN
      which_nside = {healpix_nside: 512L}
      FILL_STRUCT, table, which_nside, tags='HEALPIX_'+['NSIDE'], carefulness=carefulness
      
      pixp = GET_PIX_STRUCT('HEALPIX', hpx_nside=which_nside.healpix_nside, hpx_ordering='RING', hpx_coordsys='G')

      pixp.npix = NSIDE2NPIX(pixp.nside)
      pixp.lastpix = pixp.npix-1L
   END
   ;'PATCH': BEGIN

   ;END
   ;'COMBINED': BEGIN
   
   ;END
   ELSE: MESSAGE, 'Pixelisation not allowed for sky model'
ENDCASE

RETURN, pixp
END
