PRO GET_PSM_DATA, private=private, user=user, passwd=passwd $
                  ;, cmb=cmb, gal=gal, sz=sz, ps=ps $
                  ;, minimal=minimal, light=light, medium=medium, heavy=heavy, full=full $
                  , nside=nside, datafile=datafile, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'GET_PSM_DATA: procedure to download PSM data files'
     PSM_INFO_MESSAGE, 'SYNTAX: GET_PSM_DATA, private=, user=, passwd=, nside=, datafile='
     PSM_INFO_MESSAGE, "Set /private, and provide user and passwd, for data restricted to the Planck collaboration"
     PSM_INFO_MESSAGE, "Set nside to healpix nside to get maps at a specific nside only"
     PSM_INFO_MESSAGE, "Optionnally set datafile to name of a single data file to be retrieved."
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN
  ENDIF


  datadir = GETPSMDIR('psmdata')

  IF KEYWORD_SET(private) NE 0 THEN BEGIN
     IF NOT KEYWORD_SET(user) THEN BEGIN
        user = ''
        READ, prompt='Access to restricted PSM data (for Planck collaboration only) -- type username :', user
     ENDIF
     IF NOT KEYWORD_SET(passwd) THEN BEGIN
        passwd = ''
        READ, 'Access to restricted PSM data (for Planck collaboration only) -- type password :', passwd
     ENDIF
  ENDIF

  ;;----------------------------------------------------------------------------------------------------------------
  ;; If we want to get a specific file
  ;;----------------------------------------------------------------------------------------------------------------
  IF KEYWORD_SET(datafile) NE 0 THEN BEGIN

     IF NOT DEFINED(datafile) THEN MESSAGE, 'Data file not defined'

     short_datafile = PSM_STR_REPLACE(datafile, datadir, '')
     basename = FILE_BASENAME(datafile)
     subdir = PSM_STR_REPLACE(short_datafile, basename, '')
     result = STRSPLIT(subdir, '/', /extract)
     
     ;; Get data in normal PSM directory
     ;;----------------------------------
     IF NOT KEYWORD_SET(private) THEN BEGIN
        ftpsite = " ftp://planck-wg2:compsep@planck-wg2.planck.fr/PSM-Data/v1_7/"

        ccc = NUMBER2STRING(2+N_ELEMENTS(result))
        IF PSM_VERBOSITY() EQ 0 THEN BEGIN 
           ftpopt = " --passive-ftp -np -N --cut-dirs="+ ccc + $
                    " -nH -r -l Inf --quiet --no-proxy " + $
                    "--directory-prefix='"+datadir+subdir+"'"
        ENDIF
        IF PSM_VERBOSITY() EQ 1 THEN BEGIN 
           ftpopt = " --passive-ftp -np -N --cut-dirs="+ ccc + $
                    " -nH -r -l Inf --no-verbose --no-proxy " + $
                    "--directory-prefix='"+datadir+subdir+"'"
        ENDIF
        IF PSM_VERBOSITY() GE 2 THEN BEGIN 
           ftpopt = " --passive-ftp -np -N --cut-dirs="+ ccc + $
                    " -nH -r -l Inf --verbose --no-proxy " + $
                    "--directory-prefix='"+datadir+subdir+"'"
        ENDIF
        ftpsite = ftpsite+subdir
        selection = " --accept='"+basename+"'"
        cmd = 'wget'+selection+ftpsite+ftpopt

     ;; Get data in private PSM directory
     ;;-----------------------------------
     ENDIF ELSE BEGIN
        ftpsite = " https://www.apc.univ-paris7.fr/Downloads/Planck/PSM-Data/v1_7/"
        cmd = "wget --no-check-certificate --passive-ftp --http-user="+user+" --http-passwd="+passwd+ftpsite+short_datafile+" --directory-prefix='"+datadir+subdir+"'"
     ENDELSE

     PSM_INFO_MESSAGE, 'checking file in PSM data repository: '+datafile
     cmd = cmd+' --tries=2 --connect-timeout=20 --dns-timeout=20'
     
     ;; Limit the download rate for external users
     ;; (non-developers, using other version that the head cvs)
     IF PSM_VERSION() NE 'head' THEN BEGIN
        cmd = cmd+' --limit-rate=10M'
     END

     ;; print, cmd

     SPAWN, cmd, result, errresult

     IF errresult[0] EQ "wget: unrecognized option `--no-proxy'" THEN BEGIN
        newcmd = PSM_STR_REPLACE(cmd,'--no-proxy', '')
        SPAWN, newcmd, newresult, newerrresult
     ENDIF

     IF errresult[0] EQ "wget: unrecognized option `--no-check-certificate'" THEN BEGIN
        newcmd = PSM_STR_REPLACE(cmd,'--no-check-certificate', '')
        SPAWN, newcmd, newresult, newerrresult
     ENDIF

  ;;----------------------------------------------------------------------------------------------------------------
  ;; Getting PSM data according to some selection rules
  ;;----------------------------------------------------------------------------------------------------------------
  ENDIF ELSE BEGIN

     selection = ""
     
     ;; If we want to get a specific nside
     ;;------------------------------------
     IF KEYWORD_SET(nside) NE 0 THEN BEGIN
        nsidelist = ['16','32','64','128','256','512','1024','2048','4096','8192']
        wh = WHERE(nsidelist NE NUMBER2STRING(nside), nwh)
        IF nwh NE 0 THEN BEGIN
           selection = " --reject="
           selection = selection+"'"+nsidelist[wh[0]]+".fits'"
           FOR i=1, nwh-1 DO selection = selection+",'"+nsidelist[wh[i]]+".fits'"
        ENDIF
        docmd = 1
     ENDIF
     
     ;; Command to get data in normal PSM directory
     ;;---------------------------------------------
     IF NOT KEYWORD_SET(private) THEN BEGIN
        ftpsite = " ftp://planck-wg2:compsep@planck-wg2.planck.fr/PSM-Data/v1_7/"
        ftpopt = " --passive-ftp -np -N --cut-dirs=2 -nH -r -l Inf --verbose --no-proxy --directory-prefix='"+datadir+"'"
        cmd = 'wget'+selection+ftpsite+ftpopt
        docmd= 1

     ;; Command to get data in private PSM directory
     ;;----------------------------------------------
     ENDIF ELSE BEGIN
        ftpopt = " -np -N --cut-dirs=4 -nH -r --verbose --no-proxy --directory-prefix='"+datadir+"'"
        ftpsite = " https://www.apc.univ-paris7.fr/Downloads/Planck/PSM-Data/v1_7/"
        selection = " --accept '*.fits','*.pdf','*.txt'"
        cmd = "wget"+selection+" --no-check-certificate --http-user="+user+" --http-passwd="+passwd + selection + ftpsite + ftpopt
        print, cmd
        docmd=1 
 
     ENDELSE

     ;; Execute the wget command to get the PSM data
     ;;----------------------------------------------
     IF docmd THEN BEGIN
        PSM_INFO_MESSAGE, 'command is : '+cmd  
        ;; (Secretely) limit the download rate to 5M for external users
        ;; (non-developers, using other version that the head cvs)
        ;; to avoid consumption of the entire bandwidth by a single user
        IF PSM_VERSION() NE 'head' THEN BEGIN
           cmd = cmd+' --limit-rate=5M'
        END

        ;SPAWN, cmd
        SPAWN, cmd, result, errresult
        IF errresult[0] EQ "wget: unrecognized option `--no-proxy'" THEN BEGIN
           newcmd = PSM_STR_REPLACE(cmd,'--no-proxy', '')
           SPAWN, newcmd, newresult, newerrresult
        ENDIF
        IF KEYWORD_SET(file) NE 0 THEN BEGIN
           ;; Check we got the file allright
           ok = FILE_TEST(file)
           IF NOT ok THEN PSM_INFO_MESSAGE, 'File not retrieved'
        ENDIF
     ENDIF ELSE BEGIN
        PSM_INFO_MESSAGE, 'The command is : '+cmd  
        PRINT, 'This command could not be executed, aborting'
        STOP
     ENDELSE
     
  ENDELSE
  ;;----------------------------------------------------------------------------------------------------------------

END 
