FUNCTION MERGE_FITSHDRS, hdr1, hdr2

; keywords or PSM header blocks present in hdr2 and not in hdr1 are added to hdr1
; comments in hdr2 are ignored

  carefulness = PSM_CAREFULNESS()

  fits_endline = PSM_STRING_TO_LENGTH('END',80)

  ;; Check that both headers are OK

  ok1 = CHECK_FITSHDR(hdr1)
  ok2 = CHECK_FITSHDR(hdr2)
  IF NOT ok1  THEN MESSAGE, 'First header not OK'
  IF NOT ok2  THEN MESSAGE, 'Second header not OK'

  ;; Separate the PSM part in each header
  SPLIT_FITSHDR, hdr1, hout=hdr1_out, hpsm=ph1, hend=endline1
  SPLIT_FITSHDR, hdr2, hout=hdr2_out, hpsm=ph2, hend=endline2
  ;; Initiate header to non-psm part of hdr1
  hdr = hdr1_out

  ;; Read header 2 and process each parameter
  nel = N_ELEMENTS(hdr2_out)

  FOR i=0, nel-1 DO BEGIN
     iscomment = (STRMID(hdr2_out[i],0,8) EQ 'COMMENT ') 
     isend =  (STRMID(hdr2_out[i],0,8) EQ 'END     ') 
     isempty = (strlen(hdr2_out[i]) EQ 0)
     IF NOT ((iscomment OR isend) OR isempty) THEN BEGIN
        parname = STRTRIM(STRMID(hdr2_out[i],0,8), 2)
        ;print, parname
        v1 = SXPAR([hdr,fits_endline], parname, count=c1)
        v2 = SXPAR([hdr2_out,fits_endline], parname, count=c2, comment=comment)
        IF c1 NE 0 THEN BEGIN
           IF carefulness GE 1 THEN BEGIN
              IF v2 NE v1 THEN MESSAGE, 'Incompatibility between the two headers for keyword '+parname
           ENDIF           
        ENDIF ELSE BEGIN
            SXADDPAR, hdr, parname, v2, comment
        ENDELSE
     ENDIF
  ENDFOR

;  nn = N_ELEMENTS(hdr)-2
  whendline = WHERE(STRMID(hdr,0,8) EQ 'END     ', iend)
  IF iend GE 1 THEN BEGIN
     endline = hdr[whendline[0]]
     hdr = hdr[0:whendline[0]-1]
  ENDIF ELSE BEGIN
     endline = PSM_STRING_TO_LENGTH('END',80)
  END

  ;; Do the merging of the psm part
  IF N_ELEMENTS(ph1) LE 1 THEN BEGIN
     ph1 = [PSMHDR_HEADLINE(),PSMHDR_ENDLINE()]
     addph1 = 0
  ENDIF ELSE BEGIN
     addph1 = 1
  ENDELSE
  list = VALID_PSMHDRBLOCKS()
  FOR i=0, N_ELEMENTS(list)-1 DO BEGIN
     EXTRACT_PSMHDRBLOCK, ph1, block1, blockname=list[i], whereblock=whereblock1, /silent
     IF whereblock1[0] EQ -1 THEN BEGIN
        EXTRACT_PSMHDRBLOCK, ph2, block2, blockname=list[i], whereblock=whereblock2, /silent
        IF whereblock2[0] NE -1 THEN BEGIN
           ;ph1 = [ph1,ph2[whereblock2]]
           UPDATE_PSMHDRBLOCK, ph1, ph2[whereblock2]
           addph1 = 1
        ENDIF
     ENDIF
  ENDFOR
  IF addph1 THEN hdr = [hdr,ph1]
     
  IF ((STRLEN(endline1[0]) GT 0) OR (STRLEN(endline2[0]) GT 0)) THEN hdr = [hdr,endline]

RETURN, hdr

END
