FUNCTION PSM_ALM_HDR, lmax=lmax, pxwin=pxwin, help=help

  IF (KEYWORD_SET(help)) NE 0 OR (N_PARAMS() GE 1) THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'PSM_ALM_HDR: function to generate PSM alm header information array'
     PSM_INFO_MESSAGE, 'SYNTAX: result = PSM_ALM_HDR(lmax=,pxwin=)'
     PSM_INFO_MESSAGE, "lmax can be any positive number (default is invalid value -1)"
     PSM_INFO_MESSAGE, "pxwin gives info about the pixel window function applied to the data"
     PSM_INFO_MESSAGE, "pxwin can be 0 (no pixel window function), or, 1, 2, 4, 8, ... (nside of the pixel window)"
     PSM_INFO_MESSAGE, "(default is pxwin=0)"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF

  hd = [PSMHDRBLOCK_HEADLINE('alm')]

  ;; General alm tags
  ;;------------------
  SXADDPAR, hd, 'ALM_FLD', '', "Name of alm field stored in this extension"
  IF DEFINED(lmax) THEN SXADDPAR, hd, 'PSM_LMAX', lmax, "Maximum multipole number" ELSE $
     SXADDPAR, hd, 'PSM_LMAX', -1L, "Maximum multipole number" 
  IF DEFINED(pxwin) THEN BEGIN
     wh = WHERE([0,1,2,4,8,16,32,64,128,256,512,1024,2048,4096,8192] EQ pxwin, nwh)
     IF nwh EQ 1 THEN SXADDPAR, hd, 'PXWIN', pxwin, "Pixel window function" ELSE MESSAGE, 'Invalid pixel window function parameter'
  ENDIF ELSE SXADDPAR, hd, 'PXWIN', 0, "Pixel window function"
  hd = hd[0:N_ELEMENTS(hd)-2]

RETURN, hd
END
