PRO SPLIT_FITSHDR, hdr_in, hout=hout, hpsm=hpsm, hend=hend, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'SPLIT_FITSHDR: procedure to find distance to a mask, up to some max (for apodisation purposes)'
     PSM_INFO_MESSAGE, 'SYNTAX: SPLIT_FITSHDR, hdr_in, hout=, hpsm=, hend='
     PSM_INFO_MESSAGE, 'hdr_in is the input header'
     PSM_INFO_MESSAGE, "hout is the (output) non-psm header part"
     PSM_INFO_MESSAGE, "hpsm is the (output) psm header part"
     PSM_INFO_MESSAGE, 'hend is the (output) end line'
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN
  ENDIF

; procedure to split fits header into subparts

  ;; Check that input header is defined
  IF UNDEFINED(hdr_in) THEN MESSAGE, 'Input header is not defined'
  
  ;; Check that the header is OK
  ok = CHECK_FITSHDR(hdr_in)
  IF NOT ok THEN MESSAGE, 'Input header not OK'

  ;; Define default outputs
  hout = ['']
  hend = ['']
  hpsm = ['']

  ;; Find endline and update hend
  isendline = WHERE(STRUPCASE(STRMID(hdr_in,0,8)) EQ 'END     ',nend)
  IF nend NE 0 THEN BEGIN
     hend = [hend,hdr_in[isendline]]
     keep = REPLICATE(1,N_ELEMENTS(hdr_in))
     keep[isendline] = 0
     keep = WHERE(keep NE 0, nkeep)
  ENDIF ELSE BEGIN
     nkeep = N_ELEMENTS(hdr_in)
     keep = INDGEN(nkeep)
  ENDELSE

  IF nkeep NE 0 THEN BEGIN
     hout = [hout,hdr_in[keep]]

     ;; Find all possible psm header blocks, and update hpsm and hout if needed
     list = VALID_PSMHDRBLOCKS()
     FOR i=0, N_ELEMENTS(list)-1 DO BEGIN
        EXTRACT_PSMHDRBLOCK, hout, block, blockname=list[i], whereblock=whereblock, wherenotblock=wherenotblock, /silent
        IF whereblock[0] NE -1 THEN hpsm = [hpsm,hout[whereblock]]
        IF wherenotblock[0] NE -1 THEN hout = hout[wherenotblock] ELSE hout=['']
     ENDFOR
     
  ENDIF
  ;; Remove PSM head and end lines from hout if needed
  wh = WHERE(hout NE PSMHDR_HEADLINE(),nwh) & hout = hout[wh]
  wh = WHERE(hout NE PSMHDR_ENDLINE(),nwh) & hout = hout[wh]

  ;; If the output headers have more than 1 element, remove the first one (and add psm first and last ine to psm header)
  nel = N_ELEMENTS(hout) & IF nel GT 1 THEN hout = hout[1:*]
  nel = N_ELEMENTS(hend) & IF nel GT 1 THEN hend = hend[1:*]
  nel = N_ELEMENTS(hpsm) & IF nel GT 1 THEN hpsm = [PSMHDR_HEADLINE(),hpsm[1:*],PSMHDR_ENDLINE()]


END
