FUNCTION FITSFILE_PIX, fitsfile, info_exists=info_exists, exten=exten

  IF NOT DEFINED(exten) THEN exten=1
  hd = HEADFITS(fitsfile, exten=exten)
  info_exists = 1B

  pixtype = STRTRIM(SXPAR(hd,'PIXTYPE',count=count),2)
  IF count EQ 0 THEN BEGIN
     info_exists = 0B
     RETURN, GET_PIX_STRUCT('UNKNOWN')
  ENDIF
  IF count EQ 1 THEN BEGIN
     CASE pixtype OF
        'HEALPIX': BEGIN
           ;; nside
           ;;-------
           hpx_nside = SXPAR(hd,'NSIDE',count=count)
           IF count NE 1 THEN BEGIN
              info_exists = 0B
              RETURN, GET_PIX_STRUCT('UNKNOWN')
           ENDIF  
           ;; npix
           ;;------
           hpx_npix = SXPAR(hd,'NPIX',count=count)
           IF count NE 1 THEN BEGIN
              info_exists = 0B
              RETURN, GET_PIX_STRUCT('UNKNOWN')
           ENDIF  
           ;; ordering
           ;;----------
           hpx_ordering = STRTRIM(STRUPCASE(SXPAR(hd,'ORDERING',count=count)),2)
           IF count NE 1 THEN BEGIN
              info_exists = 0B
              RETURN, GET_PIX_STRUCT('UNKNOWN')
           ENDIF  
           ;; object
           ;;--------
           hpx_object = STRTRIM(STRUPCASE(SXPAR(hd,'OBJECT',count=count)),2)
           IF count NE 1 THEN BEGIN
              info_exists = 0B
              RETURN, GET_PIX_STRUCT('UNKNOWN')
           ENDIF  
           ;; firstpix
           ;;----------
           hpx_firstpix = STRTRIM(STRUPCASE(SXPAR(hd,'FIRSTPIX',count=count)),2)
           IF count NE 1 THEN BEGIN
              hpx_firstpix = 0L
           ENDIF  
           ;; lastpix
           ;;---------
           hpx_lastpix = STRTRIM(STRUPCASE(SXPAR(hd,'LASTPIX',count=count)),2)
           IF count NE 1 THEN BEGIN
              hpx_lastpix = npix-1L
           ENDIF  
           ;; indxschm
           ;;----------
           hpx_indxschm = STRTRIM(STRUPCASE(SXPAR(hd,'INDXSCHM',count=count)),2)
           IF count NE 1 THEN BEGIN
              hpx_indxschm = 'IMPLICIT'
           ENDIF  
           ;; indxschm
           ;;----------
           hpx_indxschm = STRTRIM(STRUPCASE(SXPAR(hd,'INDXSCHM',count=count)),2)
           IF count NE 1 THEN BEGIN
              hpx_indxschm = 'IMPLICIT'
           ENDIF  
           ;; coordsys
           ;;----------
           hpx_coordsys = STRTRIM(STRUPCASE(SXPAR(hd,'COORDSYS',count=count)),2)
           IF count NE 1 THEN BEGIN
              IF PSM_CAREFULNESS() GE 2 THEN MESSAGE, 'Coordinate system missing in fits file header'
              hpx_coordsys = ''
           ENDIF  

           pix = { $
                 pixtype: 'HEALPIX', $
                 nside: LONG(hpx_nside), $
                 npix: LONG(hpx_npix), $
                 ordering: hpx_ordering, $
                 object: hpx_object, $
                 firstpix: hpx_firstpix, $
                 lastpix: hpx_lastpix, $
                 indxschm: hpx_indxschm, $
                 coordsys: hpx_coordsys $ 
                 }
           
           RETURN, pix

        END
        ELSE: PSM_DEBUG_MESSAGE, 'Case not implemented', routine='FITSFILE_PIX'
     ENDCASE
  ENDIF
  

END
