PRO READ_CLASS_CL, name, cl, tt=tt, ee=ee, bb=bb, te=te, dd=dd, dt=dt, de=de, lmin=lmin, lmax=lmax, matter=matter, nz=nz, format=format

  IF NOT KEYWORD_SET(format) THEN format = 'class'

  ;; Read l indices in the file
  ;;----------------------------
  READCOL, name, l

  ;; Define the indices to be read in the input file columns
  ;;---------------------------------------------------------
  IF NOT DEFINED(lmin) THEN lmin=MIN(l)
  IF NOT DEFINED(lmax) THEN lmax=MAX(l)

  lindex = LONARR( (lmax < MAX(l)) - (lmin > (MIN(l))) + 1)
  FOR i=0L, N_ELEMENTS(lindex)-1L DO BEGIN
     wh = WHERE(l EQ i) 
     lindex[i] = wh[0]
  ENDFOR

  ;; Define units
  ;;--------------
  CASE format OF
     'class': cmbunit = 'K/KCMB'
     'camb': cmbunit =  'uK_CMB'
  ENDCASE

  ;; CMB and lensing spectra
  ;;-------------------------
  IF DEFINED(matter) EQ 0 THEN BEGIN
     CASE format OF
        'class': BEGIN
           cl = CL_STRUCTURE(fieldnames=['T','E','B','P'], lmin=lmin, lmax=lmax, /double, units=[cmbunit,cmbunit,cmbunit,'unitless'])
           readcol, name, l, tt, ee, te, bb, dd, dt, de
        END
        'camb': BEGIN
           cl = CL_STRUCTURE(fieldnames=['T','E','B','D'], lmin=lmin, lmax=lmax, /double, units=[cmbunit,cmbunit,cmbunit,'unitless'])
           readcol, name, l, tt, ee, bb, te, dd, dt, de
        END
     ENDCASE

     lindex=WHERE((cl.l ge min(l)) AND (cl.l le max(l)))
     lindex2=WHERE((l ge min(cl.l)) AND (l le max(cl.l)))
     IF N_ELEMENTS(lindex) NE N_ELEMENTS(lindex2) THEN STOP

     ;cl.l = l
     cl.C_1_1[lindex] = tt[lindex2]*2*!dpi/(cl.l[lindex]*(cl.l[lindex]+1))
     cl.C_1_2[lindex] = te[lindex2]*2*!dpi/(cl.l[lindex]*(cl.l[lindex]+1))
     cl.C_2_2[lindex] = ee[lindex2]*2*!dpi/(cl.l[lindex]*(cl.l[lindex]+1))
     cl.C_3_3[lindex] = bb[lindex2]*2*!dpi/(cl.l[lindex]*(cl.l[lindex]+1))
     cl.C_4_4[lindex] = dd[lindex2]*2*!dpi/(cl.l[lindex]*(cl.l[lindex]+1))
     cl.C_1_4[lindex] = dt[lindex2]*2*!dpi/(cl.l[lindex]*(cl.l[lindex]+1))
     cl.C_2_4[lindex] = de[lindex2]*2*!dpi/(cl.l[lindex]*(cl.l[lindex]+1))

  ;; Matter shells
  ;;---------------
  ENDIF ELSE BEGIN
     IF defined(nz) eq 0 THEN nz=1
     IF nz LT 0 THEN BEGIN
        print, 'Something must be wrong, buddy...'
        STOP    
     ENDIF
     
     cl = CL_STRUCTURE(nfields=nz, lmin=lmin, lmax=lmax, /double)

     lindex=WHERE((cl.l ge min(l)) AND (cl.l le max(l)))
     lindex2=WHERE((l ge min(cl.l)) AND (l le max(cl.l)))
     IF N_ELEMENTS(lindex) NE N_ELEMENTS(lindex2) THEN STOP

     j_s=0
     j_b=0
     last_i=0
     cmd="readcol, name, F='X,X,X,X,X,X,X,X"
     
     iarr=intarr(20,2)
     FOR i_cell=0, nz-1 DO BEGIN
        FOR j_cell=i_cell, nz-1 DO BEGIN
           
           iarr[j_s,0]=i_cell+1
           iarr[j_s,1]=j_cell+1
           j_s=j_s+1
           
           new_i = ((nz-j_cell)*i_cell+(i_cell+2)*(i_cell+1)/2)+(i_cell+1)*(j_cell-i_cell-1) ;nz*i_cell+j_cell
           na = new_i-last_i-1
           IF na GT 0 THEN BEGIN
              a=replicate(',X', na)
              cmd=cmd+strjoin(a)
           ENDIF
           cmd=cmd+',D'
           last_i=new_i
           
           IF j_s EQ 20 THEN BEGIN
              j_s=0
              j_b=j_b+1
              cmd=cmd+"', c1,c2,c3,c4,c5,c6,c7,c8,c9,c10,c11,c12,c13,c14,c15,c16,c17,c18,c19,c20"
              ok=execute(cmd)
              ;; reinitialize command
              cmd="readcol, name, F='X,X,X,X,X,X,X,X"
              a=replicate(',X', last_i+1)
              cmd=cmd+strjoin(a)
              
              FOR i_s=0,19 DO BEGIN
                 scmd='cl.c_'+ NUMBER2STRING(iarr[i_s,0]) + '_' + NUMBER2STRING(iarr[i_s,1]) + '[lindex]=c'+NUMBER2STRING(i_s+1)+'[lindex2]*2*!dpi/(cl.l[lindex]*(cl.l[lindex]+1))' 
                 ok=execute(scmd)
              ENDFOR
              iarr[*,*]=0	   
           ENDIF
           
        ENDFOR
     ENDFOR
          
     IF j_s ne 0 THEN BEGIN
        a=number2string(indgen(j_s)+1)
        tcmd=',c'+a
        
        cmd=cmd+"'"+strjoin(tcmd)
        ok=execute(cmd)
        FOR i_s=0,j_s-1 DO BEGIN
           scmd='cl.c_'+ NUMBER2STRING(iarr[i_s,0]) + '_' + NUMBER2STRING(iarr[i_s,1]) + '[lindex]=c'+NUMBER2STRING(i_s+1)+'[lindex2]*2*!dpi/(cl.l[lindex]*(cl.l[lindex]+1))'
           ok=execute(scmd)
        ENDFOR
        
     ENDIF
     
  ENDELSE
  
END
