PRO DATANPIX, data, npix

  IF ISPOINTER(data) THEN BEGIN
     ndim = SIZE(*data,/n_dimensions)
     IF ndim EQ 1 THEN npix = N_ELEMENTS(*data) ELSE npix = N_ELEMENTS((*data)[*,0])
  ENDIF ELSE BEGIN
     ndim = SIZE(data,/n_dimensions)
     IF ndim EQ 1 THEN npix = N_ELEMENTS(data) ELSE npix = N_ELEMENTS(data[*,0])
  ENDELSE

END


PRO WRITE_HPX_BINTABLE, filename, xhdr=xhdr, tqustokes=tqustokes, tstokes=tstokes, qstokes=qstokes, ustokes=ustokes, coordsys=coordsys, ordering=ordering


  IF DEFINED(tqustokes) THEN BEGIN
     number = 3 
  ENDIF ELSE BEGIN
     tqu = DEFINED(tstokes) AND DEFINED(qstokes) AND DEFINED(ustokes)
     IF tqu THEN number=3 ELSE number=1
  ENDELSE

  ;; Initialize astron library
  ;;---------------------------
  INIT_ASTROLIB

  ;; Primary unit (empty)
  ;;----------------------

    ;; Minimal header
    WRITEFITS, filename, 0
    h0 = HEADFITS(filename)
    CHECK_FITS, image, h0, /UPDATE, /FITS

    ;; Date
    fdate = TODAY_FITS()
    SXADDPAR, h0, 'DATE', fdate,'Creation date (CCYY-MM-DD) of FITS header'

  ;; First extension (contains healpix map)
  ;;----------------------------------------
    IF DEFINED(xhdr) EQ 0 THEN xhdr=['']
    IF (N_ELEMENTS(xhdr) EQ 1) then xhdr = [xhdr]      ; make sure it is an array

    nside = SXPAR(xhdr,'NSIDE',count=c_nside)
    object = STRTRIM(STRING(SXPAR(xhdr,'OBJECT',count=c_object)),2)

    IF (object EQ 'PARTIAL') THEN fullsky=0 ELSE fullsky=1

    npix = SXPAR(xhdr,'NPIX',count=c_npix)
    IF DEFINED(tqustokes) THEN DATANPIX, tqustokes, dnpix ELSE DATANPIX, tstokes, dnpix
    IF c_npix NE 1 THEN npix = dnpix

    nside = NPIX2NSIDE(npix,err=errpix)
    IF errpix NE 0 THEN BEGIN
       PRINT,' npix = ',npix,' nside = ',nside
       MESSAGE, 'Non-HEALPix data set - File NOT written'
    ENDIF

    ;; Add ordering
    ADD_ORDERING_FITS, xhdr, ordering=ordering, error=error
    IF error ne 0 THEN BEGIN
       PRINT,' *** file NOT written '
       MESSAGE,'Abort'
    ENDIF
    
    ;; Add NSIDE
    ADD_NSIDE_FITS, xhdr, nside=nside, partial = 1-fullsky, obs_npix = npix

    ;; Add coordsys information
    IF DEFINED(coordsys) THEN ADD_COORDSYS_FITS, xhdr, coordsys=coordsys

    ;; Remove reserved keywords that will be added automatically later on
    IF (UNDEFINED(xhdr)) THEN SXADDPAR,xhdr,'COMMENT','  '
    SXDELPAR, xhdr,['XTENSION','BITPIX','NAXIS','NAXIS1','NAXIS2','PCOUNT','GCOUNT','TFIELDS']
    SXDELPAR, xhdr,'TFORM'+STRTRIM(STRING(INDGEN(20)+1,form='(i2)'),2)

    ;; Speed up the IO by putting blocks of elements per entry
    ;; As FXB routines do not support variable length arrays,
    ;; use fixed number of elements, i.e. a divider of the number of elements per field
    nentry_hpx = 1024L                                    ; number of elements per block written
    IF ((npix MOD nentry_hpx) EQ 0 ) THEN BEGIN
       nrows = npix / nentry_hpx 
       nentry = nentry_hpx
    ENDIF ELSE BEGIN
       nrows = npix
       nentry = 1L
    ENDELSE

    ;; Create extension header
    FXBHMAKE, xthdr, nrows
    ;; Update the header for 1 column with TFORMi = ' 1024x'
    IF DEFINED(tqustokes) THEN BEGIN
       IF ISPOINTER(tqustokes) THEN FOR i=1, 3 DO FXBADDCOL, col, xthdr, REPLICATE((*tqustokes)[0],nentry) $
          ELSE FOR i=1, 3 DO FXBADDCOL, col, xthdr, REPLICATE(tqustokes[0],nentry)
    ENDIF ELSE BEGIN
       IF tqu THEN BEGIN
          IF ISPOINTER(tstokes) THEN FOR i=1, 3 DO FXBADDCOL, col, xthdr, REPLICATE((*tstokes)[0],nentry) $
          ELSE FOR i=1, 3 DO FXBADDCOL, col, xthdr, REPLICATE(tstokes[0],nentry)
       ENDIF ELSE BEGIN
          IF ISPOINTER(tstokes) THEN FXBADDCOL, col, xthdr, REPLICATE((*tstokes)[0],nentry) ELSE FXBADDCOL, col, xthdr, REPLICATE(tstokes[0],nentry)
       ENDELSE
    ENDELSE

    ;; Put the value for TTYPE1, TTYPE2 and TTYPE3
    name =  ['TEMPERATURE',  'Q_POLARISATION',  'U_POLARISATION']
    FOR i=1, number DO BEGIN 
       ttype_in = SXPAR(xhdr,'TTYPE'+NUMBER2STRING(i),count=count)
       IF (count EQ 0) THEN SXADDPAR, xthdr, 'TTYPE'+NUMBER2STRING(i), name[i-1]
    ENDFOR

    ;; Add the user defined header 
    iend = WHERE(STRUPCASE(STRMID(xthdr,0,8)) EQ 'END     ', nend)
    xh_tmp = (nend EQ 1) ? xthdr[0:iend[0]-1] : xthdr
    FOR i=0, n_elements(xhdr)-1 DO BEGIN
       IF (STRTRIM(xhdr[i]) NE '') THEN xh_tmp = [xh_tmp, xhdr[i]]
    ENDFOR
    xthdr = xh_tmp
    REORDER_FITSHDR, xthdr
    ;; Put the header in extension 1
    xtn_id = 0                                              ;zero-based
    FXBCREATE, unit, filename, xthdr, xtn_id

    nloop = (LONG(nrows) * nentry * (number-1L)) / 1.e4     ; write 10^4 values at a time
    nloop = LONG(nloop+1) < nrows
    nw = nrows/nloop + 1L
    ip1   = INDGEN(number)+1
    stcol = '['+STRING(ip1,form='(50(i2, :, ","))')+']'
    FOR i=0L, nloop-1 DO BEGIN
       low = LONG64(i * nw)
       IF (low GE nrows) THEN GOTO, done
       hi  = LONG64(low + nw - 1) < (nrows-1)
       
; uses FXBWRITM rather than FXBWRITE, and write a few rows at a time
       srange = STRCOMPRESS(STRING(low*nentry,hi*nentry+nentry-1,form='(i15,":",i15)'),/remove_all)
                                ;starg =  STRCOMPRESS(STRING(ip1,form='(50(:, ",exten_st.(",i2,")['+srange+']"))'),/remove_all)
       IF DEFINED(tqustokes) THEN BEGIN
          IF ISPOINTER(tqustokes) THEN starg = ",(*tqustokes)["+srange+",0]" + ",(*tqustokes)["+srange+",1]" + ",(*tqustokes)["+srange+",2]" $
          ELSE starg = ",(tqustokes)["+srange+",0]" + ",(tqustokes)["+srange+",1]" + ",(tqustokes)["+srange+",2]"       
       ENDIF ELSE BEGIN
          IF ISPOINTER(tstokes) THEN starg = ",(*tstokes)["+srange+"]" ELSE starg = ",(tstokes)["+srange+"]"
          IF tqu THEN BEGIN
             IF ISPOINTER(qstokes) THEN starg = starg+ ",(*qstokes)["+srange+"]" ELSE starg = starg+ ",(qstokes)["+srange+"]" 
             IF ISPOINTER(ustokes) THEN starg = starg+ ",(*ustokes)["+srange+"]" ELSE starg = starg+ ",(ustokes)["+srange+"]"
          ENDIF 
       ENDELSE
       rows =   STRCOMPRESS(STRING(low+1L,hi+1L,form='(",row=[",i12,",",i12,"]")'),/remove_all)
       command = 'FXBWRITM, unit, '+stcol+starg+rows
       ok = EXECUTE(command)
       IF NOT ok THEN MESSAGE,'Error in FITS writing'
       done:
    ENDFOR
    IF (hi NE (nrows-1)) THEN MESSAGE,'Error in FITS writing'

; closes the file
FXBFINISH,unit

RETURN


END



;;-------------------------------------------------------------------------------------------------------------------------------------------
PRO WRITE_HPXMAP, fitsfile, tqustokes=tqustokes, tstokes=tstokes, qstokes=qstokes, ustokes=ustokes, beam=beam, $
                  datatype=datatype, psm_dtid=psm_dtid, coordsys=coordsys, ordering=ordering, xhdr=xhdr, units=units, lmax=lmax, help=help

  ON_ERROR, 2

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'WRITE_HPXMAP: procedure to write a healpix map with proper PSM header information'
     PSM_INFO_MESSAGE, 'SYNTAX: WRITE_HPXMAP, fitsfile, tqustokes=, tstokes=, qstokes=, ustokes=, $'
     PSM_INFO_MESSAGE, '                      datatype=, psm_dtid=, coordsys=, ordering=, xhdr=, units=, lmax='
     PSM_INFO_MESSAGE, "fitsfile is the file name of the file to be written"
     PSM_INFO_MESSAGE, 'set either tqustokes, or tstokes to the data array to be written'
     PSM_INFO_MESSAGE, "set 'beam' to the beam of the map, to be written in the map header"
     PSM_INFO_MESSAGE, "set 'datatype' to either 'COMP' or 'OBS' (component or observation) if appropriate"
     PSM_INFO_MESSAGE, "set 'psm_dtid' to the PSM data identification key if appropriate"
     PSM_INFO_MESSAGE, "set 'coordsys' to the coordinate system used ('G','E', or 'C' for a sky map)"
     PSM_INFO_MESSAGE, "set 'ordering' to the HEALPix ordering used ('RING' (the default) or 'NESTED')"
     PSM_INFO_MESSAGE, "set 'xhdr' to the extension header to be used, if any"
     PSM_INFO_MESSAGE, "set 'units' to the units of the map if any (e.g. 'MJy/sr','K_CMB','K_RJ','K/KCMB','y_sz', ...)"
     PSM_INFO_MESSAGE, "set 'lmax' to the maximum ell present in the map, if any"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN
  ENDIF

  IF NOT DEFINED(lmax) THEN BEGIN
     IF DEFINED(xhdr) THEN BEGIN
        lmax = HEADER_LMAX(xhdr, ok=ok)
        IF NOT ok THEN PSM_DEBUG_MESSAGE, 'Set the value of lmax in call', routine='WRITE_HPXMAP'
     ENDIF ELSE BEGIN
        PSM_DEBUG_MESSAGE, 'Set the value of lmax in call', routine='WRITE_HPXMAP'
     ENDELSE
  ENDIF
  carefulness = PSM_CAREFULNESS()
  verbosity = PSM_VERBOSITY()
  IF NOT DEFINED(ordering) THEN BEGIN
     IF DEFINED(xhdr) THEN BEGIN
        ordering = SXPAR(xhdr,'ORDERING', count=count, /silent) 
        IF count NE 1 THEN BEGIN
           IF PSM_CAREFULNESS() LE 1 THEN BEGIN
              ordering = 'RING'
              IF PSM_VERBOSITY() GE 2 THEN PSM_INFO_MESSAGE, "WRITE_HPXMAP: ordering set to 'RING' by default"
           ENDIF ELSE BEGIN
              PSM_DEBUG_MESSAGE, 'Set the ordering in call', routine='WRITE_HPXMAP'
           ENDELSE
        ENDIF
     ENDIF ELSE BEGIN
        IF PSM_CAREFULNESS() LE 1 THEN BEGIN
           ordering = 'RING'
           IF PSM_VERBOSITY() GE 2 THEN PSM_INFO_MESSAGE, "WRITE_HPXMAP: ordering set to 'RING' by default"
        ENDIF ELSE BEGIN
           PSM_DEBUG_MESSAGE, 'Set the ordering in call', routine='WRITE_HPXMAP'
        ENDELSE
     ENDELSE
  ENDIF
                  
  ;; IF THE INPUT IS TQU
  ;;---------------------
  IF DEFINED(tqustokes) THEN BEGIN
     IF KEYWORD_SET(Tstokes) NE 0 THEN MESSAGE, 'Cannot set both TQUstokes and Tstokes as inputs'
     IF KEYWORD_SET(Qstokes) NE 0 THEN MESSAGE, 'Cannot set both TQUstokes and Qstokes as inputs'
     IF KEYWORD_SET(Ustokes) NE 0 THEN MESSAGE, 'Cannot set both TQUstokes and Ustokes as inputs'

     nmap = 3

  ;; IF THE INPUT IS T, OR T,Q, and U
  ;;----------------------------------
  ENDIF ELSE BEGIN
     IF KEYWORD_SET(Qstokes) NE KEYWORD_SET(Ustokes) THEN MESSAGE, 'Set either both Qstokes and Ustokes, or none'
     IF NOT KEYWORD_SET(Tstokes) THEN MESSAGE, 'Set either TQUstokes or Tstokes'
     IF NOT KEYWORD_SET(Q) THEN nmap = 1 ELSE nmap=3
  ENDELSE


  ;; IF THE INPUT IS T, Q, AND U
  ;;-----------------------------

  n_ext = 1
  i_ext0 = 0
  
  ;; Create a default header
  ;;-------------------------
  defhdr = [' ']

  ;; Create structure for extension
  ;SXADDPAR, defhdr,'EXTNAME', 'SIGNAL'
  ;IF nmap EQ 3 THEN SXADDPAR, defhdr,'POLAR','T' ELSE SXADDPAR, defhdr,'POLAR','F'
  
  ;; Add UNITS information
  colnum = INDGEN(nmap)+1
  IF DEFINED(units) THEN BEGIN
     ADD_UNITS_FITS, defhdr, units=units, error=error, colnum=colnum
     IF error NE 0 THEN RETURN
  ENDIF
  
  ;; Now create a default PSM header
  dataform = 'map'
  block1 = PSM_BASE_HDR(datatype=datatype, dataform=dataform, /setval)
  IF DEFINED(lmax) EQ 0 THEN BEGIN
     IF DEFINED(xhdr) THEN lll = SXPAR(xhdr, 'PSM_LMAX', count=count) ELSE count=0
     IF count GE 1 THEN lmax = lll[0]
  ENDIF
  block2 = PSM_MAP_HDR(pixtype='HEALPIX',lmax=lmax)
  IF DEFINED(beam) THEN BEGIN
     block3 = PSM_BEAM_HDR(beam)
  ENDIF ELSE BEGIN
     block3 = PSM_BEAM_HDR(0.)
  ENDELSE
  MK_PSMHDR, psmhdr, block1, block2, block3

  ;; Merge it into default hdr
  hdr = MERGE_FITSHDRS(defhdr,psmhdr)
  
  ;; Add default PSM header at the end of the header
  IF DEFINED(xhdr) THEN hdr = MERGE_FITSHDRS(xhdr,hdr)
  
  IF DEFINED(psm_dtid) THEN SXADDPAR, hdr, 'PSM_DTID', psm_dtid
  IF DEFINED(psm_dttp) THEN SXADDPAR, hdr, 'PSM_DTTP', psm_dttp
  
  IF NOT KEYWORD_SET(ordering) THEN BEGIN
     IF carefulness EQ 0 THEN BEGIN
        ordering='RING'
        IF verbosity GE 1 THEN PSM_INFO_MESSAGE, 'WRITE_HPXMAP: Healpix ordering assumed to be RING by default'
     ENDIF ELSE BEGIN
        PSM_INFO_MESSAGE, 'WRITE_HPXMAP: Healpix ordering is not defined'
        PSM_INFO_MESSAGE, 'type .c to assume RING ordering and proceed'
        STOP
        ordering='RING'       
     ENDELSE
  ENDIF

  ;; Create the directory if it does not exist
  ;;-------------------------------------------
  direc = FILE_DIRNAME(fitsfile)
  ok1 = FILE_TEST(direc)
  ok2 = FILE_TEST(direc,/directory)
  IF NOT (ok1 AND ok2) THEN FILE_MKDIR, direc

  ;; Write the file
  ;;----------------

  ;WRITE_HPX_BINTABLE, fitsfile, xhdr=hdr, tqustokes=tqustokes, tstokes=tstokes, qstokes=qstokes, ustokes=ustokes, coordsys=coordsys, ordering=ordering
  
  IF DEFINED(tqustokes) THEN BEGIN
     IF ISPOINTER(tqustokes) THEN WRITE_TQU, fitsfile, *tqustokes, ordering=ordering, coordsys=coordsys, units=units ELSE WRITE_TQU, fitsfile, tqustokes, ordering=ordering, coordsys=coordsys, units=units
     h = HEADFITS(fitsfile, exten=1)
     hh = MERGE_FITSHDRS(h,hdr)
     MODFITS, fitsfile, 0, hh, exten_no=1
  ENDIF ELSE BEGIN
     IF NOT KEYWORD_SET(qstokes) THEN BEGIN
        IF ISPOINTER(tstokes) THEN WRITE_FITS_MAP, fitsfile, *tstokes, ordering=ordering, coordsys=coordsys, units=units ELSE  WRITE_FITS_MAP, fitsfile, tstokes, ordering=ordering, coordsys=coordsys, units=units
     ENDIF ELSE BEGIN
        IF ISPOINTER(tstokes) THEN WRITE_TQU, fitsfile, [[*tstokes],[*qstokes],[*ustokes]], ordering=ordering, coordsys=coordsys, units=units ELSE WRITE_TQU, fitsfile, [[tstokes],[qstokes],[ustokes]], ordering=ordering, coordsys=coordsys, units=units       
     ENDELSE
     h = HEADFITS(fitsfile, exten=1)
     hh = MERGE_FITSHDRS(h,hdr)
     MODFITS, fitsfile, 0, hh, exten_no=1
  ENDELSE

closing: 
RETURN
END

