PRO COMPARE_WMAP, output_dir, double=double

  ;; Restore the sky model and define the CMB map file
  ;;---------------------------------------------------
  RESTORE_SKY_MODEL, model, output_dir=output_dir
  cmbmapfile = GETPSMDIR('cmb',psm_output_dir=output_dir)+'cmb_map.fits'


  figures_dir = GETPSMDIR('figures',psm_output_dir=output_dir)+'comparison_wmap/'

  exists = FILE_TEST(figures_dir,/directory)
  IF NOT exists THEN FILE_MKDIR, figures_dir


  exists = FILE_TEST(cmbmapfile)
  
  IF exists THEN BEGIN
     ;; CMB power spectrum
     ;;--------------------
     clfile = GETPSMDIR('cmb',psm_output_dir=output_dir)+'cmb_cl.fits'
     READ_CL_ANYFORMAT, clfile, c
     llp1_2pi = c.l*(c.l+1L)/2./!pi

     ;; Plot with linear x scale
     DEVICE, get_decomposed=d
     DEVICE, decomposed=1
     WINDOW, /free, title = 'PSM Cl vs. WMAP 7 year data'
     PLOT, c.l[0:1500], (llp1_2pi*c.c_1_1)[0:1500]
     DEVICE, decomposed=d
     file_tt_binned = GETPSMDIR('psmdata')+'ancillary/observations/WMAP/wmap_tt_spectrum_7yr_v4p1/wmap_binned_tt_spectrum_7yr_v4p1.txt'
     READCOL, file_tt_binned, l, lmin, lmax, llp1clover2pi, error, insterror, cosmicerror
     FOR i=0, N_ELEMENTS(l)-1 DO BEGIN & $
        OPLOT, [l[i],l[i]], [llp1clover2pi[i]-error[i],llp1clover2pi[i]+error[i]] & $
        OPLOT, [lmin[i],lmax[i]], [llp1clover2pi[i],llp1clover2pi[i]] & $
        ENDFOR

     ;; Plot with log x scale
     DEVICE, get_decomposed=d
     DEVICE, decomposed=1
     WINDOW, /free, title = 'PSM Cl vs. WMAP 7 year data'
     PLOT, c.l[0:1500], (llp1_2pi*c.c_1_1)[0:1500], /xlog, xrange=[1,1500], /xstyle
     DEVICE, decomposed=d
     file_tt_binned = GETPSMDIR('psmdata')+'ancillary/observations/WMAP/wmap_tt_spectrum_7yr_v4p1/wmap_binned_tt_spectrum_7yr_v4p1.txt'
     READCOL, file_tt_binned, l, lmin, lmax, llp1clover2pi, error, insterror, cosmicerror
     FOR i=0, N_ELEMENTS(l)-1 DO BEGIN & $
        OPLOT, [l[i],l[i]], [llp1clover2pi[i]-error[i],llp1clover2pi[i]+error[i]] & $
        OPLOT, [lmin[i],lmax[i]], [llp1clover2pi[i],llp1clover2pi[i]] & $
        ENDFOR

     ;; CMB map empirical power spectrum
     ;;----------------------------------
     PSM_MAP2CL, cmbmapfile, cl, /debeam, double=double
     cl.units[*] = model.units.cmb
     VISU_CL, cl, psym=1, symsize=.5, title = 'PSM map Cl vs. WMAP best fit model'
     
     std_lcdm_clfile = GETPSMDIR('psmdata')+'ancillary/models/cmb/standard_lcdm/camb_42481064_lensedcls.dat'
     READ_CL_ANYFORMAT, std_lcdm_clfile, c, /camb
     llp1_2pi = c.l*(c.l+1L)/2./!pi
     DEVICE, get_decomposed=d
     DEVICE, decomposed=1
     OPLOT, c.l, llp1_2pi*c.c_1_1, color=GET_COLOR('black')
     OPLOT, c.l, llp1_2pi*c.c_2_2, color=GET_COLOR('green')
     OPLOT, c.l, llp1_2pi*c.c_3_3, color=GET_COLOR('blue')
     OPLOT, c.l, llp1_2pi*c.c_1_2, color=GET_COLOR('red')
     OPLOT, c.l, -llp1_2pi*c.c_1_2, color=GET_COLOR('red'), linestyle=2
     DEVICE, decomposed=d
     
     ;; CMB map
     ;;---------
     READ_MAP, cmbmapfile, cmb, hdr=hdr, ordering=order, signal='T'
     lmax = FITSFILE_LMAX(cmbmapfile) < 2000
     cmbbeam = FITSFILE_BEAM(cmbmapfile)
     cmbunits = STRTRIM(SXPAR(hdr,'TUNIT1',/silent),2)
     IF cmbunits NE model.units.cmb THEN MESSAGE, 'Problem with CMB units'
     nside = SXPAR(hdr,'NSIDE',/silent)
     blref = PSM_GET_BL('WMAP5yr-W', lmax=lmax)
     refbeam = GET_BEAM_STRUCT('TABULATED', ell=LINDGEN(lmax+1), bl=blref, dim=1)
     refmap0 = !PSMROOT+'Data/ancillary/observations/WMAP/NILC-CMB-5yr/mapilc5yr.fits'
     globalfactor = CONVERSION_FACTOR('mK_CMB',cmbunits)
     CHANGE_MAP_RESOLUTION, refmap0, refmap, beam_in=refbeam, beam_out=cmbbeam, lmax=lmax, ordering_out=order, nside_out=nside, /nohdr
     refmap = refmap*globalfactor
     max_mK = 0.4
     
     ;; Oplot difference power spectrum for constrained model
     ;;-------------------------------------------------------
     IF 0 THEN BEGIN
        IF STRLOWCASE(model.cmb.model) EQ 'gaussian' THEN BEGIN
           IF STRUPCASE(model.cmb.cmb_constrained) EQ 'YES' THEN BEGIN
              dif = cmb-refmap
              maxdif = MAX(ABS(dif)) < max_mK*globalfactor
              MOLLVIEW, dif, units=cmbunits, min = -maxdif, max=maxdif, title = 'CMB difference (simulation - reference)'
              PSM_MAP2CL, dif, cldif, lmax=lmax
              bl = PSM_GET_BL(cmbbeam,lmax=lmax) 
                                ; VISU_CL, cldif, psym=1, symsize=.5, title = 'Cl of CMB map difference (PSM - WMAP)'
              clbl2 = cl & clbl2.c_1_1 = clbl2.c_1_1*bl^2
              VISU_CL, clbl2, psym=1, symsize=.5, title = 'Cl of PSM map and of difference with WMAP NILC5', /nopolar, yrange=[1e-5*globalfactor^2,1e-2*globalfactor^2]
              OPLOT, cldif.l, llp1_2pi*cldif.c_1_1, color=GET_COLOR('black'), line=2
              READCOL, GETPSMDIR('psmdata')+'ancillary/observations/WMAP/NILC-CMB-5yr/noise_nl.txt', ll, nl, format='L,F'
              DEVICE, get_decomposed=d
              DEVICE, decomposed=1
              OPLOT, ll, llp1_2pi*nl/bl^2*blref^2*globalfactor^2, color=GET_COLOR('black')
              DEVICE, decomposed=d       
           ENDIF
        ENDIF
     ENDIF
     


        MOLLVIEW, cmbmapfile, units=cmbunits, min=-max_mK*globalfactor, max=max_mK*globalfactor, title = 'CMB Map (PSM Simulation)', png=figures_dir+'cmb_psm.png'
        MOLLVIEW, refmap, units=cmbunits, min=-max_mK*globalfactor, max=max_mK*globalfactor, title = 'CMB reference Map (WMAP 5yr Needlet ILC)', png=figures_dir+'cmb_wmap.png'
  
  ENDIF

  ;; Foreground maps
  ;;-----------------
  
     ;; First check whether observations with WMAP5yr_IDEALBANDS exist
     obsdir = GETPSMDIR('observations',psm_output_dir=output_dir)
     exists = FILE_TEST(obsdir+'WMAP5yr_IDEALBANDS',/directory)

     ;IF NOT exists THEN BEGIN
     ;   PSM_INFO_TITLE, 'GENERATING 5YR WMAP OBSERVATIONS OF THE SKY MODEL'
     ;   PSM_MAIN, !PSMROOT+'Soft/psm/config/psm_test_wmap.psm', output_dir=output_dir
     ;ENDIF

     IF NOT exists THEN BEGIN
        PSM_INFO_MESSAGE, 'Cannot compare foreground data to WMAP maps'
        PSM_INFO_MESSAGE, 'Observe the sky with WMAP5yr_IDEALBANDS instrument first'
        RETURN
     ENDIF ELSE BEGIN
        psm_wmap_dir = obsdir+'WMAP5yr_IDEALBANDS/'
        wmap_dir = !PSMROOT+'Data/ancillary/observations/WMAP/NILC-FG-5yr/'
        wmap_bands=['k','ka','q','v','w']

        nbands=n_elements(wmap_bands)
        FOR i=0,nbands-1 DO BEGIN
           read_fits_map,psm_wmap_dir+wmap_bands[i]+'/obs_'+wmap_bands[i]+'.fits',psm_map
           read_fits_map,wmap_dir+'/foreground_'+wmap_bands[i]+'.fits',wmap_map


              mollview, psm_map,min=-0.3,max=0.3,title= 'PSM bande '+wmap_bands[i], png=figures_dir+'psm_'+wmap_bands[i]+'.png'
              mollview, wmap_map,min=-0.3,max=0.3,title= 'WMAP bande '+wmap_bands[i], png=figures_dir+'wmap_'+wmap_bands[i]+'.png'
              mollview, psm_map-wmap_map,min=-0.3,max=0.3,title= 'Difference bande '+wmap_bands[i], png=figures_dir+'diff_'+wmap_bands[i]+'.png'

        ENDFOR


        STOP
     ENDELSE

END
