; Copyright 2007 ----  Universit Paris Diderot-Paris 7

; This file is part of the Planck Sky Model.
;
; The Planck Sky Model is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; version 2 of the License.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY, without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model. If not, write to the Free Software
; Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

;+
;generate ALM_E (CMB) with the constraints ALM_T (CMB observation), CL_E and CL_T (theorical CL_CMB i.e. CMBFAST)
;
; (Alm_T, Alm_E)^t \equiv N((0,0), S) with S=(Cl_T,Cl_TE;CL_TE,CL_E). 
; P(Alm_E|ALM_T) \equiv N(m,s^2) with m=\frac{CL_TE}{CL_T}*Alm_T and s^2=CL_E-\frac{CL_TE^2}{CL_T}
; 
; @author F. DODU, dodu\@apc.univ-paris7.fr
;
; @history Fri Jan 26, 2007  4:40 PM
; @history added predict keyword, JD, 27/01/2011
; @copyright Universit Paris Diderot-Paris 7, 2007
; 
;-

PRO GENERATE_ALM_E_GIVEN_ALM_T, ell, cl_E, cl_T, cl_TE, alm_hat, alm, my_seed=my_seed, write_fits_file=write_fits_file, predict=predict


  nb_ell = N_ELEMENTS(ell)
  l_max = LONG(MAX(ell))
  nb_alm = (l_max+1L)*(l_max+2L)/2L 
  IF PRECISION_ISDOUBLE() THEN  alm = DBLARR(nb_alm,2) ELSE alm = FLTARR(nb_alm,2)
  
  IF NOT KEYWORD_SET(predict) THEN BEGIN
     IF NOT DEFINED(my_seed) THEN my_seed = RANDOMU(seed)
     randomnumbers = RANDOMN(my_seed,2L*nb_alm)
     alm[0:nb_alm-1,0] = randomnumbers(0:nb_alm-1)
     alm[0:nb_alm-1,1] = randomnumbers(nb_alm:*)
  ENDIF ELSE BEGIN
     ;; Do nothing. The random part vanishes 
  ENDELSE

  use_ell = LINDGEN(l_max+1) 
  use_cl_E = FLTARR(l_max+1)
  use_cl_T = FLTARR(l_max+1)
  use_cl_TE = FLTARR(l_max+1)

  use_cl_E(ell)  = cl_E
  use_cl_T(ell)  = cl_T
  use_cl_TE(ell) = cl_TE

  ;;alm(0,0) = SQRT(use_cl_E(0)-use_cl_TE(0)^2/use_cl_T(0))*alm(0,0)+alm_hat(0,0)*use_cl_TE(0)/use_cl_T(0)
  alm(0,0) = 0.
  alm(0,1) = 0.
  alm(1,0) = 0.
  alm(1,1) = 0.
  alm(2,0) = 0.
  alm(2,1) = 0.

  IF nb_ell GT 2 THEN BEGIN
     FOR i_ell = 2L, nb_ell-1 DO BEGIN
        start_ind = (i_ell+1)*(i_ell)/2
        end_ind = start_ind + (i_ell)
        alm(start_ind,0) = SQRT(use_cl_E(i_ell)-use_cl_TE(i_ell)^2/use_cl_T(i_ell))*alm(start_ind,0)+alm_hat(start_ind,0)*use_cl_TE(i_ell)/use_cl_T(i_ell)
        alm(start_ind,1) = 0.
        
        alm(start_ind+1:end_ind,0) = SQRT(1/2.*(use_cl_E(i_ell)-use_cl_TE(i_ell)^2/use_cl_T(i_ell)))*alm(start_ind+1:end_ind,0)+alm_hat(start_ind+1:end_ind,0)*use_cl_TE(i_ell)/use_cl_T(i_ell)
        alm(start_ind+1:end_ind,1) = SQRT(1/2.*(use_cl_E(i_ell)-use_cl_TE(i_ell)^2/use_cl_T(i_ell)))*alm(start_ind+1:end_ind,1)+alm_hat(start_ind+1:end_ind,1)*use_cl_TE(i_ell)/use_cl_T(i_ell)
     ENDFOR
  ENDIF
  IF  KEYWORD_SET(write_fits_file) THEN BEGIN
     index = ALM_INDEX(0,nb_ell-1) 
     ALM2FITS ,index, alm,write_fits_file, HDR = hdr, XHDR = xhdr
  ENDIF
  
END

