FUNCTION NILC5_ALM, lmax=lmax, beam_out=beam_out, pixwin_out=pixwin_out, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'NILC5_ALM: function to get the spherical harmonic coefficients of the NILC5 CMB map of Delabrouille et al. (2008)'
     PSM_INFO_MESSAGE, 'SYNTAX: result = NILC5_ALM(beam_out=, pixwin_out=)'
     PSM_INFO_MESSAGE, 'Set beam_out to the beam of the output map (e.g. beam_out=15 for 15 arcmin)'
     PSM_DEBUG_MESSAGE, "or e.g., beam_out=GET_BEAM_STRUCT('INSTR',instr='WMAP5yr',channel='Q')"
     PSM_INFO_MESSAGE, 'set pixwin_out to the nside of the output window function if any (can be 1, 2, 4, ... 512, ...)'
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF

  IF NOT KEYWORD_SET(beam_out) THEN beam_out = GET_BEAM_STRUCT('GAUSSIAN', beamsize=0., sizeunits='arcmin')
  lmaxnilc = 1000L
  
  ;; Get alm of NILC5 map, at the resolution of the PSM sky
  ;;--------------------------------------------------------
  nilc5_mapfile = !PSMROOT+'Data/ancillary/observations/WMAP/NILC-CMB-5yr/mapilc5yr.fits'
  ok = TEST_PSM_DATA(nilc5_mapfile, /get)
  globalfactor = CONVERSION_FACTOR('mK_CMB',CMB_UNITS())
  blref = PSM_GET_BL('WMAP5yr-W', lmax=lmaxnilc)
  refbeam = GET_BEAM_STRUCT('INSTR',instr='WMAP',channel='W', version='5yr')
  PSM_MAP2ALM, nilc5_mapfile, nilc5_alm, lmax=lmaxnilc, /nohdr
  CHANGE_ALM_RESOLUTION, nilc5_alm, alm_fix, beam_in=refbeam, beam_out=beam_out
  alm_fix = TEMPORARY(alm_fix) * globalfactor
  ind = ALM_INDEX(0,lmaxnilc)
  INDEX2L_ONLY, ind, lll
  
  ;; Correct for pixel window function (nside=512 for the
  ;; WMAP NILC5 map, although this is ignored in the NILC5
  ;; analysis)
  ;;-------------------------------------------------------
  nilcpixwin = HEALPIXWINDOW(512)
  nilcpixwin = nilcpixwin[0:lmaxnilc,*]
  factor = FLTARR(lmaxnilc+1)
  IF KEYWORD_SET(pixwin_out) NE 0 THEN BEGIN
     pixwinl = HEALPIXWINDOW(pixwin_out)
     lim = MIN([N_ELEMENTS(pixwinl[*,0]),N_ELEMENTS(nilcpixwin[*,0])])
     factor[0L:lim-1L] = pixwinl[0L:lim-1L,0]/nilcpixwin[0L:lim-1L,0]
  ENDIF ELSE BEGIN
     lim = MIN([N_ELEMENTS(factor),N_ELEMENTS(nilcpixwin[*,0])])
     factor[0L:lim-1L] = 1./nilcpixwin[0L:lim-1L,0]
  ENDELSE
  alm_fix[*,0] = alm_fix[*,0] * factor[lll]
  alm_fix[*,1] = alm_fix[*,1] * factor[lll]
  
  IF NOT KEYWORD_SET(lmax) THEN lmax=lmaxnilc ELSE BEGIN
     IF lmax GT lmaxnilc THEN MESSAGE, 'lmax for NILC5_ALM should be less than or equal to 1000'
  ENDELSE
  nalm = LMAX2NALM(lmax)

  RETURN, alm_fix[0:nalm-1,*]
  
END
