FUNCTION NILC5_NL, ell, beam_out=beam_out, pixwin_out=pixwin_out, help=help

  IF (KEYWORD_SET(help) NE 0) OR (N_PARAMS() NE 1) THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'NILC5_NL: function to get the Noise power spectrum for the NILC5 CMB map of Delabrouille et al. (2008)'
     PSM_INFO_MESSAGE, 'SYNTAX: result = NILC5_NL(ell)'
     PSM_INFO_MESSAGE, 'Set beam_out to the beam of the output map (e.g. beam_out=15 for 15 arcmin)'
     PSM_DEBUG_MESSAGE, "or e.g., beam_out=GET_BEAM_STRUCT('INSTR',instr='WMAP5yr',channel='Q')"
     PSM_INFO_MESSAGE, 'set pixwin_out to the nside of the output window function if any (can be 1, 2, 4, ... 512, ...)'
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF

  IF NOT KEYWORD_SET(beam_out) THEN beam_out=0.

  datafile = GETPSMDIR('psmdata')+'ancillary/observations/WMAP/NILC-CMB-5yr/noise_nl.txt'
  ok = TEST_PSM_DATA(datafile, /get)

  READCOL, datafile, l, nl, format='L,D', comment='#'
  lmaxnilc = 1000L
  nl = nl[0:lmaxnilc-1]

  out = REPLICATE(FLOAT('Inf'),N_ELEMENTS(ell))
  wh = WHERE(ell LE lmaxnilc, nwh)
  IF nwh NE 0 THEN out[wh]=nl[wh]

  refbeam = GET_BEAM_STRUCT('INSTR',instr='WMAP',channel='W', version='5yr')

  ;; Correct for pixel window function (nside=512 for the
  ;; WMAP NILC5 map, although this is ignored in the NILC5
  ;; analysis)
  ;;-------------------------------------------------------
  nilcpixwin = HEALPIXWINDOW(512)
  nilcpixwin = nilcpixwin[0:lmaxnilc,*]
  factor = FLTARR(lmaxnilc+1)
  IF KEYWORD_SET(pixwin_out) NE 0 THEN BEGIN
     pixwinl = HEALPIXWINDOW(pixwin_out)
     lim = MIN([N_ELEMENTS(pixwinl[*,0]),N_ELEMENTS(nilcpixwin[*,0])])
     factor[0L:lim-1L] = pixwinl[0L:lim-1L,0]/nilcpixwin[0L:lim-1L,0]
  ENDIF ELSE BEGIN
     lim = MIN([N_ELEMENTS(factor),N_ELEMENTS(nilcpixwin[*,0])])
     factor[0L:lim-1L] = 1./nilcpixwin[0L:lim-1L,0]
  ENDELSE

  blratio = PSM_BL_RATIO(beam_out, refbeam, lmax=lmaxnilc, double=PRECISION_ISDOUBLE())

  globalfactor = (CONVERSION_FACTOR('mK_CMB',CMB_UNITS()))^2

  out = out * (blratio[ell] * factor[ell])^2 * globalfactor




  RETURN, out

END

