; Copyright 2009 Jacques Delabrouille, CNRS

; This file is part of the Planck Sky Model.
;
; The Planck Sky Model is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; version 2 of the License.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY, without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model. If not, write to the Free Software
; Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

;+
; Writes a parameter file for running CAMB
;
; @categories cmb
; @param param {in}{required}{type=structure} parameters of the PSM run
; @param filename {in}{required}{type=string} Name of the file to be written
; @uses WRITE_STRUCT (PSM)
; @author Jacques Delabrouille
; @history <p>19/01/2009: First version, Jacques Delabrouille </p>
; @history <p>05/02/2009: adjusted default params to match WMAP power spectrum; Jacques Delabrouille </p>
; @copyright Jacques Delabrouille, CNRS
;-
PRO WRITE_CAMB_PARAM_FILE, cosmo, filename, lmaxs=lmaxs, lmaxt=lmaxt, polar=polar, transfer=transfer, $
                           redshifts_transfer=redshifts_transfer,  output_root=output_root, k_max_hinvMpc=k_max_hinvMpc

  ;; Define default cosmology
  ;;--------------------------
  cp = DEFINE_COSMO(/silent)

  ;; Update parameters
  ;;-------------------
  IF DEFINED(cosmo) THEN BEGIN
     tags = TAG_NAMES(cosmo)
     ntag = N_ELEMENTS(tags)
     IF ntag NE 0 THEN FOR i=0, ntag-1 DO ok=EXECUTE('cp.'+tags[i]+' = cosmo.'+tags[i])
  ENDIF

  If NOT KEYWORD_SET(output_root) then output_root = FIX_SEPARATOR(GETPSMDIR('cosmo'))+'camb'

  IF NOT KEYWORD_SET(lmaxs) THEN lmaxs=3000
  IF NOT KEYWORD_SET(lmaxt) THEN lmaxt=500

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Writing camb param file
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
cambpar={output_root:output_root, $
         get_scalar_cls:'T', $
         get_vector_cls:'F', $
         get_tensor_cls:'T', $
         get_transfer:'F', $
         do_lensing:'T', $
         do_nonlinear:0, $
         l_max_scalar:lmaxs, $
         k_eta_max_scalar:2*(lmaxs+50l), $
         l_max_tensor:lmaxt, $
         k_eta_max_tensor:(2*(lmaxt+50l)), $
         use_physical:'F' , $
         omega_baryon:cp.Omega_b , $
         omega_cdm:cp.Omega_m - cp.Omega_b, $
         omega_lambda:cp.omega_de, $
         omega_neutrino:cp.omega_nu, $
         hubble:100d*cp.h, $
         w: cp.w_dark_energy , $                         ; dark energy equation of state
         cs2_lam:1 , $                                   ; comoving sound speed of DE
         temp_cmb:cp.T_cmb, $                            ; CMB temperature
         helium_fraction:cp.he_fraction , $              ; Helium fraction
         massless_neutrinos:cp.n_massless_nu , $         ; Effective number of massless neutrinos species
         massive_neutrinos:cp.n_massive_nu , $           ; Number of massive neutrinos
         nu_mass_eigenstates:1 , $                       ;
         nu_mass_degeneracies:0, $                       ;
         nu_mass_fractions:1, $                          ;
         reionization:'T', $                             ;
         re_use_optical_depth:'T', $                     ;
         re_optical_depth:cp.tau_reion, $                ;
         re_ionization_frac:1, $                         ;
         pivot_scalar:cp.k_pivot, $                      ;
         pivot_tensor:cp.k_pivot, $                      ; these two pivot points affect the normalisation of the output power spectrum
         initial_condition:1, $                          ; (adiabatic=1, CDM isocurvature=2, baryon isocurvature=3
                                                         ; neutrino density isocurvature=4, neutrino velocity isocurvature=5)
                                                         ; if 0 use the proportions below
         initial_vector:'-1 0 0 0 0', $                  ;
         vector_mode:0, $                                ;
         scalar_output_file:'scalCls.txt', $             ;
         vector_output_file:'vecCls.txt', $              ;
         tensor_output_file:'tensCls.txt', $             ;
         total_output_file:'totCls.txt', $               ;
         lensed_output_file:'lensedCls.txt', $           ;
         lensed_total_output_file:'lensedtotCls.txt', $  ;
         feedback_level:0, $                             ;
         lensing_method:1, $                             ;
         accurate_BB:'T', $                              ;
         recombination:1, $                              ;
         massive_nu_approx:1, $                          ;
         accurate_polarization:'T', $                    ;
         accurate_reionization:'T', $                    ;
         do_tensor_neutrinos:'F', $                      ;
         do_late_rad_truncation:'T', $                   ;
         number_of_threads:0, $                          ;
         accuracy_boost:2, $                             ;
         l_accuracy_boost:2, $                           ;
         l_sample_boost:2 $                              ;
        }


IF KEYWORD_SET(transfer) NE 0 THEN BEGIN
   cambpar.get_transfer = 'T'
ENDIF
;Do not compute polarisation accurately if only temperature outputs are requested
  IF NOT KEYWORD_SET(polar) THEN BEGIN
     cambpar.accurate_BB = 'F'
     cambpar.accurate_polarization = 'F'
  ENDIF

;Write the structure in camb parameter file
  ini = filename
  OPENW, unit, ini, /get_lun
  WRITE_STRUCT, unit, cambpar

;Initial power spectrum, amplitude, spectral index and running
  PRINTF, unit, 'initial_power_num         = 1'
  PRINTF, unit, 'scalar_amp(1)             = '+NUMBER2STRING(cp.scalar_amplitude)
  PRINTF, unit, 'scalar_spectral_index(1)  = '+NUMBER2STRING(cp.n_s)
  PRINTF, unit, 'scalar_nrun(1)            = '+NUMBER2STRING(cp.n_s_running)
  PRINTF, unit, 'tensor_spectral_index(1)  = '+NUMBER2STRING(cp.n_t)
;ratio is that of the initial tens/scal power spectrum amplitudes
  PRINTF, unit, 'initial_ratio(1)          = '+NUMBER2STRING(cp.r)
;note vector modes use the scalar settings above

;CASE SENSITIVE
  PRINTF, unit, 'COBE_normalize = F'                         ;
  ;; PRINTF, unit, 'CMB_outputscale = 7.4311e12'                ;

;Transfer function settings, transfer_kmax=0.5 is enough for sigma_8

  IF KEYWORD_SET(transfer) NE 0 THEN BEGIN
     IF KEYWORD_SET(k_max_hinvMpc) NE 0 THEN BEGIN
        PRINTF, unit, 'transfer_high_precision = T'
        PRINTF, unit, 'transfer_kmax           = ' + NUMBER2STRING(k_max_hinvMpc)
        PRINTF, unit, 'transfer_k_per_logint   = 50'
     ENDIF ELSE BEGIN
        PRINTF, unit, 'transfer_high_precision = T'
        PRINTF, unit, 'transfer_kmax           = 1'
        PRINTF, unit, 'transfer_k_per_logint   = 50'
     ENDELSE
     
     nredshifts = N_ELEMENTS(redshifts_transfer)
     PRINTF, unit, 'transfer_num_redshifts  = '+STRING(nredshifts)
     FOR i=0, nredshifts-1 DO BEGIN
        PRINTF, unit, 'transfer_redshift('+strtrim(i+1,2)+')    = '+STRTRIM(redshifts_transfer[nredshifts-1-i],2)
        PRINTF, unit, 'transfer_filename('+strtrim(i+1,2)+')    = Z_'+STRTRIM(redshifts_transfer[nredshifts-1-i],2)+'_transfer_out.txt'
;Matter power spectrum output against k/h in units of h^3 Mpc^{-3}
        PRINTF, unit, 'transfer_matterpower('+strtrim(i+1,2)+') = Z_'+STRTRIM(redshifts_transfer[nredshifts-1-i],2)+'_matterpower.txt'
     ENDFOR
  ENDIF ELSE BEGIN
     PRINTF, unit, 'transfer_high_precision = F'
     PRINTF, unit, 'transfer_kmax           = 4'
     PRINTF, unit, 'transfer_k_per_logint   = 5'
     PRINTF, unit, 'transfer_num_redshifts  = 1'
     PRINTF, unit, 'transfer_redshift(1)    = 0'
     PRINTF, unit, 'transfer_filename(1)    = transfer_out.txt'
;Matter power spectrum output against k/h in units of h^3 Mpc^{-3}
     PRINTF, unit, 'transfer_matterpower(1) = matterpower.txt'
  ENDELSE



;;IF KEYWORD_SET(fits) NE 0 THEN PRINTF, unit, 'FITS_filename = totalCls.fits'
   
  CLOSE, unit
  FREE_LUN, unit

END
