; This procedure is used in ADD_POSITIONS_TO_CATALOG in order to
; take matter from positive delta parts to fill the few negative delta parts.
; The method used here give a good approximation of what the real linear
; field should be.


;*******************************************************************************************************************
FUNCTION EDGEPIX_BOX, dimensions
;*******************************

pixel = LINDGEN(dimensions)

RETURN,  [REFORM(pixel[0,*,*],LONG(dimensions[1])*LONG(dimensions[2])), $
          REFORM(pixel[dimensions[0]-1,*,*],LONG(dimensions[1])*LONG(dimensions[2])), $
          REFORM(pixel[1:dimensions[0]-2,0,*],(LONG(dimensions[0])-2)*LONG(dimensions[2])), $
          REFORM(pixel[1:dimensions[0]-2,dimensions[1]-1,*],(LONG(dimensions[0])-2)*LONG(dimensions[2])), $
          REFORM(pixel[1:dimensions[0]-2,1:dimensions[1]-2,0],(LONG(dimensions[0])-2)*(LONG(dimensions[1])-2)), $
          REFORM(pixel[1:dimensions[0]-2,1:dimensions[1]-2,dimensions[2]-1], $
                 (LONG(dimensions[0])-2)*(LONG(dimensions[1])-2))]
END
;*******************************************************************************************************************

;*******************************************************************************************************************
FUNCTION NEIGHBOUR, x, y, z, dimensions
;***************************************

pixel = LINDGEN(dimensions)

RETURN, REFORM([pixel[x-1,y,z], pixel[x+1,y,z], pixel[x,y-1,z], pixel[x,y+1,z], pixel[x,y,z-1], pixel[x,y,z+1]], $
        N_ELEMENTS(x),6)

END
;*******************************************************************************************************************

;*******************************************************************************************************************
FUNCTION PRIVE, neigh, is_empty
;******************************

s = SIZE(neigh)
result = DBLARR(s[1], s[2],2)
result[*,*,0] = neigh
res = neigh
res[*] = 1

wh = WHERE(is_empty[neigh] EQ 1, nwh)
IF nwh GE 1 THEN BEGIN
  res[wh] = 0
ENDIF

result[*,*,1] = res

RETURN, result

END
;*******************************************************************************************************************

;*******************************************************************************************************************
FUNCTION BOX_HOLES_FILLING, proba
  av = AVG(proba)
  s = SIZE(proba)
  IF s[0] EQ 2 THEN s[3]=1
  dimensions = [s[1]+2, s[2]+2, s[3]+2]
  xbox = 0*LONARR(dimensions)
  ybox = 0*LONARR(dimensions)
  zbox = 0*LONARR(dimensions)
  FOR i=0, dimensions(0)-1 DO xbox(i,*,*)=i
  FOR i=0, dimensions(1)-1 DO ybox(*,i,*)=i
  FOR i=0, dimensions(2)-1 DO zbox(*,*,i)=i
  prob = DBLARR(dimensions)
  prob[1:dimensions(0)-2,1:dimensions(1)-2,1:dimensions(2)-2]=proba
  is_empty = INTARR(dimensions)
  is_forbidden = INTARR(dimensions)
  is_edge = INTARR(dimensions)
  wh = EDGEPIX_BOX(dimensions)
  is_empty[wh] = 1
  is_forbidden[wh] = 1
  is_edge[wh] = 1
  wh = WHERE(prob LT 0, nwh)
  IF nwh GE 1 THEN is_empty[wh] = 1
  old_nwh = nwh+100
  WHILE ((ABS(old_nwh-nwh) GE 10) AND (nwh GE 1)) DO BEGIN
    values = prob[wh]
    x = xbox[wh]
    y = ybox[wh]
    z = zbox[wh]
    neigh = NEIGHBOUR(x, y, z, dimensions)
    new_neigh = PRIVE(neigh, is_empty)
    tot = TOTAL(new_neigh[*,*,1],2)
    whn = WHERE(tot EQ 0,nwhn)
    IF nwhn GE 1 THEN BEGIN
      is_forbidden[wh[whn]] = 1
      neigh_forbidden = neigh[whn,*]
      new_neigh_forbidden = PRIVE(neigh_forbidden, is_forbidden)
      new_neigh[whn,*,*] = new_neigh_forbidden
      totm = TOTAL(new_neigh_forbidden[*,*,1],2)
      whm = WHERE(totm EQ 0,nwhm)
      IF nwhm GE 1 THEN BEGIN
        neigh_int = neigh_forbidden[whm,*]
        new_neigh_int = PRIVE(neigh_int, is_edge)
        new_neigh[whn[whm],*,*] = new_neigh_int
      ENDIF
    ENDIF
    weight = 1./((TOTAL(new_neigh[*,*,1],2)#REPLICATE(1., 6)))*new_neigh[*,*,1]
    values = values#REPLICATE(1., 6)
    trans  = weight*values
    prob[wh] = 0
    neigh = REFORM(neigh, 6.*nwh)
    trans = REFORM(trans, 6.*nwh)
    FOR i=0L, 6.*nwh-1 DO BEGIN
      prob[neigh[i]] = prob[neigh[i]] + trans[i]
    ENDFOR
    old_nwh = nwh
    wh = WHERE(prob LT 0, nwh)
    IF nwh GE 1 THEN is_empty[wh] = 1
  ENDWHILE
  new_proba = proba
  new_proba = prob[1:dimensions(0)-2,1:dimensions(1)-2,1:dimensions(2)-2]
  new_proba = new_proba>0
  ap = AVG(new_proba)
  new_proba = new_proba*av/ap
RETURN, new_proba

END

;*******************************************************************************************************************
