; This procedure is called in CREATE_SZ_SIMULATION in order to
; create the box Universe
; cluster keyword added, JBM, February 3 2006

;-------------------------------------------------------------------------------------------
FUNCTION SUBFUNC_ARRAY2STRING, array

IF NOT ISARRAY(array) THEN BEGIN
  PRINT, 'The input has to be an array !'
  GOTO, closing
ENDIF

s = SIZE(array)

IF s[0] GE 3 THEN BEGIN
  PRINT, 'Too many dimensions !'
  GOTO, closing
ENDIF

IF s[0] EQ 1 THEN BEGIN
  sarray = STRING(array)
  result = '[' + sarray[0]
  IF s[1] GT 1 THEN BEGIN
    FOR i=1, s[1]-1 DO BEGIN
      result = result+ ',' + sarray[i]
    ENDFOR
  ENDIF
  result = result + ']'
ENDIF

IF s[0] EQ 2 THEN BEGIN
  sarray = STRING(array)
  result = '['
  FOR i=0, s[2]-1 DO BEGIN
     result = result + '[' + sarray[0,i]
     IF s[1] GT 1 THEN BEGIN
       FOR j=1, s[1]-1 DO BEGIN
         result = result + ',' + sarray[j,i] 
       ENDFOR
     ENDIF
     result = result + ']'
     IF  i EQ s[2]-1 THEN result = result + ']' ELSE result = result + ','
  ENDFOR
ENDIF

RETURN, result

closing:
END
;--------------------------------------------------------------------------------------------------------

FUNCTION SUBFUNC_STRUCT2STRING, structure

; Written for simple structure (with arrays of less than 2 dim for exemple)

result = '{'
tag = TAG_NAMES(structure)
nt = N_ELEMENTS(tag)

result = result + tag[0] + ':' + STRING(structure.(0))

IF  nt GE 2 THEN BEGIN
  FOR i=1, nt-1 DO BEGIN
    IF ISARRAY(structure.(i)) THEN value = SUBFUNC_ARRAY2STRING(structure.(i)) ELSE value = STRING(structure.(i))
    result = result + ',' + tag[i] + ':' + value
  ENDFOR
ENDIF

result = result + '}'

RETURN, result

END

;--------------------------------------------------------------------------------------------------------
FUNCTION DENSITY_FLUCTUATIONS, dimensions, cell_size, cosmo=cosmo, cluster=cluster, $
                                 iseed=iseed, smooth_size=smooth_size

;Generate a density fluctuations field with cosmological spectrum P(k)

IF NOT KEYWORD_SET(iseed) THEN iseed=LONG(SYSTIME(1))

max_dim = MAX(dimensions)
snormalize = STRTRIM(STRING(2*!pi/(cell_size*max_dim)),1)
scosmo = SUBFUNC_STRUCT2STRING(cosmo)

scluster = SUBFUNC_STRUCT2STRING(cluster)

use_function='cosmo_pk((k>1)*'+snormalize+',cosmo='+scosmo+',cluster='+scluster+')'

IF KEYWORD_SET(smooth_size) NE 0 THEN BEGIN
	rr = STRTRIM(STRING(smooth_size/cosmo.h),1)
	Wk2  = '(3./(abs((k>1)*'+snormalize+')*'+rr+')^2*(sin(abs((k>1)*'+snormalize+')*'+rr+')/(abs((k>1)*' $
               +snormalize+')*'+rr+') - cos(abs((k>1)*'+snormalize+')*'+rr+')))^2 '
	use_function=use_function+'*'+Wk2
ENDIF

result = GAUSSIAN_FIELD(dimensions, use_function=use_function, iseed=iseed)

;When white noise of PSD 1 is entered, the output RMS is 1
;In standard PSD units, it should be SQRT(integral(d3k)/8./!pi^3)
;therefore multiply the result by SQRT((2*kmax)^3 * 1/8./!pi^3)
;or by SQRT( (2*!pi/cell_size_Mpc)^3 * 1/8./!pi^3 ) =  1/SQRT(cell_size_Mpc^3)

;cell_size_Mpc = cell_size/Mpc2m
;result = result/SQRT(cell_size_Mpc^3)
result = result/SQRT(cell_size^3)

RETURN, result
END

;-----------------------------------------------------------------------------------
FUNCTION CLIP_NEGATIVE, field

; This procedure is not used anymore
; keep only the positive part of a field and change the field average 

wh = WHERE(field LT 0, nwh)
av = AVG(field)
field = field>0
ap = AVG(field)

field = field*av/ap
RETURN, field
END

;-----------------------------------------------------------------------------------
PRO DENSITY_CONTRAST_BOX, dimensions, cell_size, map_ang_size_x, map_ang_size_y, $
                            z_cut=z_cut, map_type=map_type, $
                            cosmo=cosmo

; Give dimensions and cell_size for the future density contrast box

; default value for z_cut
IF NOT KEYWORD_SET(z_cut) THEN z_cut = 5.

; compute the maximum comoving distance corresponding to z_cut
max_dist = (1+z_cut)*ANG_DIST(z_cut, cosmo=cosmo)

; the default map type is 'small_map'
IF NOT KEYWORD_SET(map_type) THEN map_type='small-map'

IF (STRLOWCASE(map_type) EQ 'all-sky') THEN BEGIN
  ;generate a 64*64*64 box
  cell_size = max_dist/32.
  nx = 64 & ny = 64 & nz = 64
ENDIF ELSE BEGIN
  IF (map_type EQ 'half_sky') THEN BEGIN
    ;generate a 64*64*32 box
    cell_size = max_dist/32.
    nx = 64 & ny = 64 & nz = 32
  ENDIF ELSE BEGIN
    IF DEFINED(map_ang_size_x) EQ 0 OR DEFINED(map_ang_size_y) EQ 0 THEN BEGIN
      PRINT, '  WARNING, MAP DIMENSIONS NOT SPECIFIED'
      PRINT, '  RETURNING'
      GOTO, closing
    ENDIF
    box_size_y = map_ang_size_x*!dtor*max_dist
    box_size_z = map_ang_size_y*!dtor*max_dist
    box_size_x = max_dist
    cell_size = (box_size_x*box_size_y*box_size_z)^(1./3.)/64.
    nx = CEIL(box_size_x/cell_size)
    ny = CEIL(box_size_y/cell_size)
    nz = CEIL(box_size_z/cell_size)
  ENDELSE
ENDELSE

dimensions = [nx,ny,nz]
closing:
END

;-------------------------------------------------------------------------------------------------------
PRO DO_DENSITY_CONTRAST_BOX, map_ang_size_x, map_ang_size_y, $
			     dimensions, cell_size, $
			     box, D_g_box, f_box, lon_box, lat_box, redshift_box, map_type = map_type, $
			     cosmo=cosmo, cluster=cluster, iseed=iseed, smooth_size=smooth_size

;generate the box of density contrast
;------------------------------------
z_cut = 5.

DENSITY_CONTRAST_BOX, dimensions, cell_size, map_ang_size_x, map_ang_size_y, $
                        z_cut=z_cut, map_type=map_type, $
                        cosmo=cosmo

box=DENSITY_FLUCTUATIONS(dimensions,cell_size,cosmo=cosmo,cluster=cluster, $
                           iseed=iseed,smooth_size=smooth_size)
box = box - AVG(box)
;print, STDDEV(box), MAX(box), MIN(box)
xbox = 0*LONG(box)
ybox = 0*LONG(box)
zbox = 0*LONG(box)
FOR i=0, dimensions(0)-1 DO xbox(i,*,*)=i
FOR i=0, dimensions(1)-1 DO ybox(*,i,*)=i
FOR i=0, dimensions(2)-1 DO zbox(*,*,i)=i
BOX_PIX2COORD, xbox, ybox, zbox, x, y, z, dimensions, cell_size, map_type=map_type
xbox=0 & ybox=0 & zbox=0 ;to save memory space
COMOVING2OBSERVABLE, x, y, z, lon_box, lat_box, redshift_box, cosmo=cosmo
z_list = FINDGEN(CEIL(MAX(redshift_box)/0.001)+1)*0.001
GROWTH_FACTOR, z_list, D_g_list, f_list, cosmo=cosmo, /slope
D_g_box = INTERPOL(D_g_list, z_list, redshift_box)
f_box = INTERPOL(f_list, z_list, redshift_box)
;box = CLIP_NEGATIVE(1+box*D_g_box)-1
;box = box/D_g_box
END
