; This procedure is called in ADD_POSITIONS_TO_CATALOG in order to
; pass from underpixelized box maps to full pixelized sky maps

PRO ARRAY_PIX2COORD, pixel, coord1, coord2, dim1, dim2, reverse=reverse

dimone = FIX(dim1)
dimtwo = FIX(dim2)

IF KEYWORD_SET(reverse) THEN BEGIN
  pixel = LONG(coord2+dimtwo/2.)*dimone + LONG(coord1+dimone/2.)
ENDIF ELSE BEGIN  
  coord2 = pixel/dimone
  coord1 = pixel - coord2*dimone
  coord1 = coord1 - dimone/2. + 0.5
  coord2 = coord2 - dimtwo/2. + 0.5
ENDELSE
END



FUNCTION EDGEPIX, underpix_map_dim_x, underpix_map_dim_y

coord1 = [0.5,0.5,underpix_map_dim_x-0.5,underpix_map_dim_x-0.5]
coord2 = [0.5,underpix_map_dim_y-0.5,underpix_map_dim_y-0.5,0.5]
coord1 = [coord1, REPLICATE(0.5,underpix_map_dim_y-2)]
coord2 = [coord2, 1.5+FINDGEN(underpix_map_dim_y-2)]
coord1 = [coord1,1.5+FINDGEN(underpix_map_dim_x-2)]
coord2 = [coord2,REPLICATE(underpix_map_dim_y-0.5,underpix_map_dim_x-2)]
coord1 = [coord1,REPLICATE(underpix_map_dim_x-0.5,underpix_map_dim_y-2)]
coord2 = [coord2,underpix_map_dim_y-1.5-FINDGEN(underpix_map_dim_y-2)]
coord1 = [coord1,underpix_map_dim_x-1.5-FINDGEN(underpix_map_dim_x-2)]
coord2 = [coord2,REPLICATE(0.5,underpix_map_dim_x-2)]

coord1 = coord1-underpix_map_dim_x/2.
coord2 = coord2-underpix_map_dim_y/2.


ARRAY_PIX2COORD, result, coord1, coord2, underpix_map_dim_x, underpix_map_dim_y, /reverse
RETURN, result
END


FUNCTION UNDERPIX_MAP2MAP, pixel, map_dim_x, map_dim_y, $
                           underpix_pix, underpix_map_dim_x, underpix_map_dim_y, iseed=iseed, reverse=reverse
  IF KEYWORD_SET(reverse) THEN BEGIN
    ARRAY_PIX2COORD, pixel, coord1, coord2, map_dim_x, map_dim_y
    underpix_coord1 = coord1/underpix_pix
    underpix_coord2 = coord2/underpix_pix
    ARRAY_PIX2COORD, result, underpix_coord1, underpix_coord2, underpix_map_dim_x, underpix_map_dim_y, /reverse
  ENDIF ELSE BEGIN
    ; for all the pixels
    n_pix = N_ELEMENTS(pixel)
    ran = underpix_pix*RANDOMU(iseed, 2, n_pix)
    ARRAY_PIX2COORD, pixel, underpix_coord1, underpix_coord2, underpix_map_dim_x, underpix_map_dim_y
    coord1 = underpix_coord1*underpix_pix
    coord2 = underpix_coord2*underpix_pix
    ARRAY_PIX2COORD, real_pix, coord1-underpix_pix/2.+ran[0,*], coord2-underpix_pix/2.+ran[1,*], $
                     map_dim_x, map_dim_y, /reverse
    ; The result is wrong for the side pixels. They must be computed separately

    bord = EDGEPIX(underpix_map_dim_x, underpix_map_dim_y)
    gapx = (underpix_pix*underpix_map_dim_x-map_dim_x)/2
    gapy = (underpix_pix*underpix_map_dim_y-map_dim_y)/2

    ; left bottom corner
    wh = WHERE(bord[0] EQ pixel,nwh)
    IF nwh GE 1 THEN BEGIN
      ranx = (underpix_pix-gapx)*RANDOMU(iseed, nwh)
      rany = (underpix_pix-gapy)*RANDOMU(iseed, nwh)
      ARRAY_PIX2COORD, bord[0], underpix_coord1, underpix_coord2, underpix_map_dim_x, underpix_map_dim_y
      coord1 = underpix_coord1*underpix_pix
      coord2 = underpix_coord2*underpix_pix
      ARRAY_PIX2COORD, pix, coord1-(underpix_pix/2.-gapx)+ranx, coord2-(underpix_pix/2.-gapy)+rany, $
                       map_dim_x, map_dim_y, /reverse
      real_pix[wh] = pix
    ENDIF
    
    ; left up corner
    wh = WHERE(bord[1] EQ pixel,nwh)
    IF nwh GE 1 THEN BEGIN
      ranx = (underpix_pix-gapx)*RANDOMU(iseed, nwh)
      rany = (underpix_pix-gapy)*RANDOMU(iseed, nwh)
      ARRAY_PIX2COORD, bord[1], underpix_coord1, underpix_coord2, underpix_map_dim_x, underpix_map_dim_y
      coord1 = underpix_coord1*underpix_pix
      coord2 = underpix_coord2*underpix_pix
      ARRAY_PIX2COORD, pix, coord1-(underpix_pix/2.-gapx)+ranx, coord2+(underpix_pix/2.-gapy)-rany, $
                       map_dim_x, map_dim_y, /reverse
      real_pix[wh] = pix
    ENDIF
    
    ; right up corner
    wh = WHERE(bord[2] EQ pixel,nwh)
    IF nwh GE 1 THEN BEGIN
      ranx = (underpix_pix-gapx)*RANDOMU(iseed, nwh)
      rany = (underpix_pix-gapy)*RANDOMU(iseed, nwh)
      ARRAY_PIX2COORD, bord[2], underpix_coord1, underpix_coord2, underpix_map_dim_x, underpix_map_dim_y
      coord1 = underpix_coord1*underpix_pix
      coord2 = underpix_coord2*underpix_pix
      ARRAY_PIX2COORD, pix, coord1+(underpix_pix/2.-gapx)-ranx, coord2+(underpix_pix/2.-gapy)-rany, $
                       map_dim_x, map_dim_y, /reverse
      real_pix[wh] = pix
    ENDIF
    
    ; right bottom corner
    wh = WHERE(bord[3] EQ pixel,nwh)
    IF nwh GE 1 THEN BEGIN
      ranx = (underpix_pix-gapx)*RANDOMU(iseed, nwh)
      rany = (underpix_pix-gapy)*RANDOMU(iseed, nwh)
      ARRAY_PIX2COORD, bord[3], underpix_coord1, underpix_coord2, underpix_map_dim_x, underpix_map_dim_y
      coord1 = underpix_coord1*underpix_pix
      coord2 = underpix_coord2*underpix_pix
      ARRAY_PIX2COORD, pix, coord1+(underpix_pix/2.-gapx)-ranx, coord2-(underpix_pix/2.-gapy)+rany, $
                       map_dim_x, map_dim_y, /reverse
      real_pix[wh] = pix
    ENDIF
    
    ; left side
    istart = 4
    FOR i=0, underpix_map_dim_y-3 DO BEGIN
      wh = WHERE(bord[istart+i] EQ pixel,nwh)
      IF nwh GE 1 THEN BEGIN
        ranx = (underpix_pix-gapx)*RANDOMU(iseed, nwh)
        rany = underpix_pix*RANDOMU(iseed, nwh)
        ARRAY_PIX2COORD, bord[istart+i], underpix_coord1, underpix_coord2, underpix_map_dim_x, underpix_map_dim_y
        coord1 = underpix_coord1*underpix_pix
        coord2 = underpix_coord2*underpix_pix
        ARRAY_PIX2COORD, pix, coord1-(underpix_pix/2.-gapx)+ranx, coord2-underpix_pix/2.+rany, $
                         map_dim_x, map_dim_y, /reverse
        real_pix[wh] = pix
      ENDIF
    ENDFOR

    ; side up
    istart = 2+underpix_map_dim_y
    FOR i=0, underpix_map_dim_x-3 DO BEGIN
      wh = WHERE(bord[istart+i] EQ pixel,nwh)
      IF nwh GE 1 THEN BEGIN
        ranx = underpix_pix*RANDOMU(iseed, nwh)
        rany = (underpix_pix-gapy)*RANDOMU(iseed, nwh)
        ARRAY_PIX2COORD, bord[istart+i], underpix_coord1, underpix_coord2, underpix_map_dim_x, underpix_map_dim_y
        coord1 = underpix_coord1*underpix_pix
        coord2 = underpix_coord2*underpix_pix
        ARRAY_PIX2COORD, pix, coord1-underpix_pix/2.+ranx, coord2+(underpix_pix/2.-gapy)-rany, $
                         map_dim_x, map_dim_y, /reverse
        real_pix[wh] = pix
      ENDIF
    ENDFOR

    ; right side
    istart = underpix_map_dim_y+underpix_map_dim_x
    FOR i=0, underpix_map_dim_y-3 DO BEGIN
      wh = WHERE(bord[istart+i] EQ pixel,nwh)
      IF nwh GE 1 THEN BEGIN
        ranx = (underpix_pix-gapx)*RANDOMU(iseed, nwh)
        rany = underpix_pix*RANDOMU(iseed, nwh)
        ARRAY_PIX2COORD, bord[istart+i], underpix_coord1, underpix_coord2, underpix_map_dim_x, underpix_map_dim_y
        coord1 = underpix_coord1*underpix_pix
        coord2 = underpix_coord2*underpix_pix
        ARRAY_PIX2COORD, pix, coord1+(underpix_pix/2.-gapx)-ranx, coord2-underpix_pix/2.+rany, $
                         map_dim_x, map_dim_y, /reverse
        real_pix[wh] = pix
      ENDIF
    ENDFOR

    ; low side
    istart = 2*underpix_map_dim_y+underpix_map_dim_x-2
    FOR i=0, underpix_map_dim_x-3 DO BEGIN
      wh = WHERE(bord[istart+i] EQ pixel,nwh)
      IF nwh GE 1 THEN BEGIN
        ranx = underpix_pix*RANDOMU(iseed, nwh)
        rany = (underpix_pix-gapy)*RANDOMU(iseed, nwh)
        ARRAY_PIX2COORD, bord[istart+i], underpix_coord1, underpix_coord2, underpix_map_dim_x, underpix_map_dim_y
        coord1 = underpix_coord1*underpix_pix
        coord2 = underpix_coord2*underpix_pix
        ARRAY_PIX2COORD, pix, coord1-underpix_pix/2.+ranx, coord2-(underpix_pix/2.-gapy)+rany, $
                         map_dim_x, map_dim_y, /reverse
        real_pix[wh] = pix
      ENDIF
    ENDFOR
    result = real_pix
  ENDELSE
RETURN, result
END

