FUNCTION COMOVING_SEPARATION, co_dist1, co_dist2, lon1, lon2, lat1, lat2, cosmo=cosmo

;+
; NAME:
;    COMOVING_SEPARATION
; ONE LINE HELP:
;    give the comoving separation of two sources
; PURPOSE:
;    give the comoving separation of two sources
; CALLING SEQUENCE:
;    com_sep = COMOVING_SEPARATION(co_dist1, co_dist2, lon1, lon2, lat1, lat2, cosmo=cosmo)
; INPUT:
;    co_dist1, co_dist2 : comoving distances of the sources
;    lon1, lon2 : longitudes of the sources
;    lat1, lat2 : latitudes of the sources
;
; OUTPUT:
;    com_sep : the comoving separation of the two sources.
;    
; KEYWORDS:
;    cosmo : the structure containing the cosmological parameters
; EXAMPLE:
;    
; RESTRICTIONS:
;    Not fully tested yet. Report any bugs to Jacques Delabrouille
;    j.delabrouille@cdf.in2p3.fr
;
; PROCEDURES CALLED:
;    ANG_DIST
;
; REVISION HISTORY
;    Written, Jean-Baptiste Melin, January 2002
;-

c_light = 2.9979246d+08 ; in m/s
c_light = 2.9979246d+08/1000. ; in km/s

k = (100.*cosmo.h/c_light)^2.*(cosmo.omega_m+cosmo.omega_de-1.)

x1 = co_dist1 ;(1+z1)*ANG_DIST(z1, cosmo=cosmo)
x2 = co_dist2 ;(1+z2)*ANG_DIST(z2, cosmo=cosmo)

cos_theta12 = COS((lon2-lon1)*!dtor)*COS((lat2-lat1)*!dtor)

result = x1^2.+x2^2.-k*x1^2.*x2^2.*(1.+cos_theta12^2.) $
         -2*x1*x2*SQRT(1.-k*x1^2.)*SQRT(1.-k*x2^2.)*cos_theta12>0

result = SQRT(result)

RETURN, result

END
