; Same function as PK_SIGMA but it uses Efstathiou power spectrum
; The result is nearly the same
; cluster keyword added, JBM, Feb 6 2006

;------------------------------------------------------------------------------
function N_PK_SIGMA_I_SIGMA, x
;---------------------------
common N_PK_SIGMA_COSMO, n_pk_sigma_cosmo, n_pk_sigma_cluster
common N_PK_SIGMA_SCALE, n_pk_sigma_r

k = x/n_pk_sigma_r
Pk = EFSPK(k, cosmo=n_pk_sigma_cosmo, cluster=n_pk_sigma_cluster, /nonorm)

if (x lt 0.001) then Wk = 1. else Wk  = 3./x^2*(sin(x)/x - cos(x))  

return, k^2*Pk*Wk^2
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;------------------------------------------------------------------------------
function N_PK_SIGMA_I_SLOPE, x
;---------------------------

common N_PK_SIGMA_COSMO, n_pk_sigma_cosmo, n_pk_sigma_cluster
common N_PK_SIGMA_SCALE, n_pk_sigma_r

k = x/n_pk_sigma_r
Pk = EFSPK(k, cosmo=n_pk_sigma_cosmo, cluster=n_pk_sigma_cluster, /nonorm)

if (x lt 0.001) then Wk = 1. else Wk  = 3./x^2*(sin(x)/x - cos(x))  

return, k*Pk*Wk*sin(x)
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

PRO EFSIGMA, mass, sigma, alpha, cosmo=cosmo, cluster=cluster, slope=slope

common N_PK_SIGMA_COSMO, n_pk_sigma_cosmo, n_pk_sigma_cluster
common N_PK_SIGMA_SCALE, n_pk_sigma_r


n_pk_sigma_cosmo=cosmo
n_pk_sigma_cluster=cluster

rho = 2.76e-4*n_pk_sigma_cosmo.omega_m*(n_pk_sigma_cosmo.h)^2
; in units of 10^15 solar masses/Mpc^3

x1 = 0.0                       ; roots of Wth(x)
x2 = 4.496
x3 = 7.723
x4 = 10.902
x5 = 14.066
x6 = 17.221
x7 = 20.371

; compute the spectrum normalization 
norm = EFSPK(1., cosmo=n_pk_sigma_cosmo, cluster=n_pk_sigma_cluster) / EFSPK(1., cosmo=n_pk_sigma_cosmo, cluster=n_pk_sigma_cluster, /nonorm)

; Calculation of sig

n_pk_sigma_r  = (3.*mass/(4.*!pi*rho))^(1./3)
sig1 = qromo('N_PK_SIGMA_I_SIGMA',x1,x2)
sig2 = qromo('N_PK_SIGMA_I_SIGMA',x2,x3)
sig3 = qromo('N_PK_SIGMA_I_SIGMA',x3,x4)
sig4 = qromo('N_PK_SIGMA_I_SIGMA',x4,x5)
sig5 = qromo('N_PK_SIGMA_I_SIGMA',x5,x6)
if (sig5/sig1 gt 0.1) then stop, 'sig not converging'
sigma = SQRT(norm/(2.*!pi^2)*(1./n_pk_sigma_r)*(sig1+sig2+sig3+sig4+sig5))



; Calculation of the slope

IF KEYWORD_SET(slope) THEN BEGIN
slope1 = qromo('N_PK_SIGMA_I_SLOPE',x1,x2)
slope2 = qromo('N_PK_SIGMA_I_SLOPE',x2,x3)
slope3 = qromo('N_PK_SIGMA_I_SLOPE',x3,x4)
slope4 = qromo('N_PK_SIGMA_I_SLOPE',x4,x5)
slope5 = qromo('N_PK_SIGMA_I_SLOPE',x5,x6)
slope6 = qromo('N_PK_SIGMA_I_SLOPE',x6,x7)
if (slope6/slope1 gt 0.1) then stop, 'slope not converging'
alpha = norm/(2.*!pi^2)*1./(sigma*n_pk_sigma_r)^2*(slope1+slope2+slope3+slope4 $
        +slope5+slope6)
alpha = 1. - alpha
ENDIF

END

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;