;FUNCTION JENKINS, delta_c, sigma, z, cosmo=cosmo
;
;+
; NAME:
;  JENKINS
; ONE LINE HELP:
;  give M/rho0*dNclu/(d_covolume*dln(sigma^(-1))) using the Jenkins et al. 2001 fitting
; PURPOSE:
;  give M/rho0*dNclu/(d_covolume*dln(sigma^(-1))) using the Jenkins et al. 2001 fitting
; CALLING SEQUENCE:
;  result = JENKINS(delta_c, sigma, z, cosmo=cosmo)
;
; INPUT:
;  delta_c : the critical density contrast at redshift z
;  sigma : fluctuation amplitude for scale M and redshift z
;  z : the redshift of computation
;
; OUTPUT:
;  M/rho0*dNclu/(d_covolume*dln(sigma^(-1)))
;    
; KEYWORDS:
;  cosmo : the structure containing the cosmological parameters
; EXAMPLES:
;  
; RESTRICTIONS:
;  Not fully tested yet. Report any bugs to Jacques Delabrouille
;  j.delabrouille@cdf.in2p3.fr
;
; PROCEDURES CALLED:
;  No procedure called 
; REVISION HISTORY
;  Written, Jean-Baptiste Melin, September 11 2001
;-

; fitting formula given by Jenkins et al. 2001
;*******************************************************************************
; be carefull the condition -1.2 LE ALOG(sigma^(-1)) GE 1.05 has to be completed
;*******************************************************************************

FUNCTION JENKINS, delta_c, sigma, z, cosmo=cosmo

IF ( N_PARAMS() EQ 0 ) THEN BEGIN
    print,'  CALL IS : JENKINS(delta_c, sigma, z)'
    print,'  KEYWORD : cosmo=cosmo'
    GOTO, closing
ENDIF

RETURN, 0.315*EXP(-1*(ABS(ALOG(1./sigma)+0.61))^3.8)

closing:
END
