; -----------------------------------------------------------------------------
;
; Copyright (C) 2011-2012
;   Guillaume Castex
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	PK2CL
;
; PURPOSE:
;	Software being developed, not to be used.
;
;-
;;
;;Converts PK to CL for a given r distance or cross CL if two
;;distances are given in parameters


FUNCTION PK2CL, k, pk, r1, pk2, r2, lmax=lmax

  ON_ERROR, 2
  MESSAGE, 'Not fully implemented'
  PSM_DEBUG_MESSAGE, 'Code needs development, validation and documentation', routine=PK2CL, infolimit=0, stoplimit=0

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'PK2CL: function to create a cl structure'
     PSM_INFO_MESSAGE, 'SYNTAX: result = PK2CL(k, pk, r1 [,pk2,r2])'
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF

  IF N_ELEMENTS(k) NE N_ELEMENTS(pk) THEN BEGIN
     print, "number of k elements not consistent with pk"
     STOP
  ENDIF


  IF DEFINED(lmax) EQ 0 THEN lmax=1000
  ell = LINDGEN(lmax+1)
  cl=DBLARR(N_ELEMENTS(ell))
  FOR l=0,lmax DO BEGIN
     If N_PARAMS() eq 3 THEN BEGIN     
        f = k*k*pk*((bessel_sph(k*r1,l))^2)  
 
     ENDIF ELSE BEGIN
        f = k*k*sqrt(pk)*sqrt(pk2)*bessel_sph(k*r1,l)*bessel_sph(k*r2,l)
     ENDELSE
     integral = int_tabulated(k, f, /double, /sort)
     IF (integral lt 0) AND (min(f) ge 0) THEN BEGIN
        print, "*$%ù§&# of int_tabulated"
        stop
     ENDIF
     cl[l] = (2./!dpi)*integral
     IF (finite(cl[l]) eq 0) then stop
     IF cl[l] lt 0 THEN cl[l]=0
  ENDFOR
stop
  RETURN, cl
END
