PRO PLOT_MASS_FUNCTION, mass_function, z_list, mass_list, mf_plot, $
                        cosmo=cosmo, cluster=cluster, $ 
                        mass=mass, redshift=redshift

;+
; NAME:
;  PLOT_MASS_FUNCTION
; ONE LINE HELP:
;  plot a given mass function
; PURPOSE:
;  plot a given mass function
; CALLING SEQUENCE:
;  PLOT_MASS_FUNCTION, mass_function, z_list, mass_list, mf_plot, $
;                      cosmo=cosmo, cluster=cluster, mass=mass, redshift=redshift
; INPUT:
;  mass_function : the mass function
; OUTPUT:
;  z_list : the redshift list
;  mass_list : the mass list
;  mf_plot : the mass function plot
; KEYWORDS:
;  cosmo : the structure containing the cosmological parameters
;  cluster : the structure containing the structure formation parameters (cf. DEFINE_CLUSTER)
;  mass : set this keyword to integrate on masses
;         the plot will be density vs resdhifts
;  redshift : set this keyword to integrate on redshifts
;             the plot will be density vs masses
; EXAMPLES:
;
; RESTRICTIONS:
;  Not fully tested yet. Report any bugs to Jacques Delabrouille
;  j.delabrouille@cdf.in2p3.fr
;
; PROCEDURES CALLED:
;  MF_DENSITY
;  INT_MASS_FUNCTION
; REVISION HISTORY
;  Written, JBM, September 11 2001 
;  Keyword cosmo introduced, JBM, Sept 27 2001
;  Cluster structure added, JBM, February 2006
;-

IF ( N_PARAMS() EQ 0 ) THEN BEGIN
 PRINT,'  CALL IS : PLOT_MASS_FUNCTION, mass_function, z_list, mass_list, mf_plot'
 PRINT,'  Keywords :'
 PRINT,'  cosmo=cosmo, cluster=cluster'
 PRINT,'  mass=mass, redshift=redshift'
 GOTO, closing
ENDIF

IF KEYWORD_SET(redshift) THEN BEGIN
  INT_MASS_FUNCTION, z_list, mass_list, mf_plot, $
  cosmo=cosmo, cluster=cluster, $
  mass_function=mass_function, $
  /redshift
  PLOT, mass_list, mf_plot, background=255, color=0, charsize=2.
ENDIF ELSE BEGIN
  IF KEYWORD_SET(mass) THEN BEGIN
    INT_MASS_FUNCTION, z_list, mass_list, mf_plot, $
    cosmo=cosmo, cluster=cluster, $
    mass_function=mass_function
    PLOT, z_list, mf_plot, background=255, color=0, charsize=2.
  ENDIF ELSE BEGIN
    z_sup=4.
    z_inf = 0.01
    step_z = 0.05
    step_log10 = 20.
    M_inf = 0.1
    M_sup = 1.
    z_list = (FINDGEN(CEIL((z_sup-z_inf)/step_z)+1)*step_z)+z_inf
    delta_lnM = ALOG(10)/step_log10
    log10_m = (FINDGEN(CEIL((ALOG10(M_sup)-ALOG10(M_inf))*step_log10)+1)/step_log10)+ALOG10(M_inf)
    mass_list = EXP(ALOG(10)*log10_m)
    MF_DENSITY, z_list, mass_list, mf_plot, $
    cosmo=cosmo, cluster=cluster, mass_function=mass_function
    SURFACE, mf_plot, z_list, mass_list, /ylog, background=255, color=0, charsize=2.
    ENDELSE
ENDELSE

closing:
END
