PRO PSM_PEP_FIRB, nside, nurefs, cl, outputdir=outputdir

  COMMON psm, param

  ;; Number of frequencies for the FIRB reference maps
  ;; (alm to be generated from Cl)
  ;;---------------------------------------------------
  nfreq = N_ELEMENTS(nurefs)
  IF nfreq NE N_ELEMENTS(cl.field) THEN MESSAGE, 'Incompatible inputs'

  ;; Check units
  ;;-------------
  wh = WHERE(cl.units NE 'MJy/sr', nwh)
  IF nwh NE 0 THEN MESSAGE, 'Wrong units in input FIRB Cl'

  ;; Put the input Cl to SKY_RESOLUTION()
  ;;--------------------------------------
  clsmooth = cl
  bl = PSM_GET_BL(SKY_RESOLUTION(), lmax = MAX(cl.l))
  bl2 = bl[cl.l]^2
  FOR i=1, nfreq DO FOR j=i, nfreq DO BEGIN
     ok = EXECUTE('clsmooth.c_'+NUMBER2STRING(i)+'_'+NUMBER2STRING(j)+' = clsmooth.c_'+NUMBER2STRING(i)+'_'+NUMBER2STRING(j)+' * bl2')
     IF NOT ok THEN MESSAGE, 'Command could not be executed'   
  ENDFOR

  ;; Generate alm for all reference frequencies
  ;;--------------------------------------------
  stop
  PSM_CL2ALM, clsmooth, alm, seed=GETSEED(), tolerance=.1
  
  ;; Generate and write maps
  ;;-------------------------
  IF NOT KEYWORD_SET(outputdir) THEN outputdir = GETPSMDIR('firb')
  firbfiles = outputdir + param.names.firb_basename + '_map_'+NUMBER2STRING(nurefs/1e9)+'GHz.fits'
  FOR i=0, nfreq-1 DO BEGIN
     PSM_ALM2MAP, alm[*,*,i], map, lmax=SKY_LMAX(), ordering=SKY_ORDERING(), hpx_nside=SKY_NSIDE(), /overwrite    
     basehdr = PSM_BASE_HDR(datatype='comp',dataform='map',/setval)
     maphdr = PSM_MAP_HDR(pixtype='HEALPIX',lmax=SKY_LMAX())
     xhdr = MERGE_FITSHDRS(basehdr,maphdr)
     bndhdr = PSM_BAND_HDR(nurefs[i])
     xhdr = MERGE_FITSHDRS(xhdr,bndhdr)
     beamhdr = PSM_BEAM_HDR(SKY_RESOLUTION())
     xhdr = MERGE_FITSHDRS(xhdr,beamhdr)
     skypix=SKY_PIX()
     stop
     WRITE_HPXMAP, firbfiles[i], tstokes=map, datatype='COMP', psm_dtid=firb_id, ordering=SKY_ORDERING(), units='MJy_sr', lmax=SKY_LMAX(), xhdr=xhdr, coordsys=skypix.coordsys
  ENDFOR
  DELVARX, alm

  STOP ;; powerlaws are not adequate for firb fluctuations. Need to change this.

  ;; FIRB maps and emission laws 
  ;;-----------------------------
  READ_MAP, firbfiles[0], ampmin
  FOR i=1, nfreq-1 DO BEGIN
     em = EMLAW_PARAMSTRUCT('POWERLAW',/include_nurange,/include_ampl)
     *em.numax.value = nurefs[i]
     *em.numin.value = nurefs[i-1]
     *em.nuref.value = nurefs[PREVIOUS_EVEN(i-1)+1]
     *em.ampl.value = firbfiles[PREVIOUS_EVEN(i-1)+1]
     em.ampl.unit = 'MJy/sr'
     READ_MAP, firbfiles[i], ampmax
     specind = ALOG10(ampmax/ampmin) / ALOG10(*em.numax.value/*em.numin.value)

     basehdr = PSM_BASE_HDR(datatype='comp',dataform='map',/setval)
     maphdr = PSM_MAP_HDR(pixtype='HEALPIX',lmax=SKY_LMAX())
     xhdr = MERGE_FITSHDRS(basehdr,maphdr)
     beamhdr = PSM_BEAM_HDR(0)  ; It is assumed that only the amplitude maps are smoothed, not the spectral index
     xhdr = MERGE_FITSHDRS(xhdr,beamhdr)

     fitsfile = outputdir + param.names.firb_basename + '_specind_e'+NUMBER2STRING(i)+'.fits'
     WRITE_HPXMAP, fitsfile, tstokes=specind, datatype='COMP', psm_dtid=firb_id, ordering=SKY_ORDERING(), units='MJy_sr', lmax=SKY_LMAX(), xhdr=xhdr, coordsys=skypix.coordsys
     
     *em.specind.value = fitsfile
     ok = EXECUTE('e'+NUMBER2STRING(i)+'=em')
     ampmin = TEMPORARY(ampmax)
  ENDFOR



END 
