; -----------------------------------------------------------------------------
;
; Copyright (C) 2011, 2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	IS_STRONGPS_COMP
;
; PURPOSE:
;	Check whether a given component is a strong point source
;	component. 
;
; CATEGORY:
;	Sky model
;
; CALLING SEQUENCE:
;	IS_STRONGPS_COMP, Comp
;
; INPUTS:
;	Comp:	A component name, among the accepted PSM component
;	        names list returned by PSM_ALLCOMP.
;
; KEYWORD PARAMETERS:
;	None
;
; OUTPUTS:
;	This function returns 1B (TRUE) if the input component 'comp'
;	is a strong point source component, and 0B (FALSE) otherwise.
;
; RESTRICTIONS:
;       Report any bugs to Jacques Delabrouille
;       delabrouille@apc.univ-paris7.fr
;
; MODIFICATION HISTORY:
; 	13/05/2011:	Jacques Delabrouille, First version.
; 	06/01/2012:	JD, doc and PSM license header.
;-
FUNCTION IS_STRONGPS_COMP, comp

  complist = STRONGPS_COMP_LIST()
  wh = WHERE(complist EQ comp, nwh)
  IF nwh EQ 1 THEN RETURN, 1B ELSE RETURN, 0B

END
