; -----------------------------------------------------------------------------
;
; Copyright (C) 2009, 2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	SAVE_COMP
;
; PURPOSE:
;	This procedure is an I/O tool for saving to file a PSM component
;	structure, hiding the technical detail of the format of the
;	model representation, as well as the format and location of
;	the file saved and the name of the saved structure. The
;	component structure can then be restored with RESTORE_COMP.
;
; CATEGORY:
;	SKY_MODEL
;
; CALLING SEQUENCE:
;	SAVE_COMP
;
; INPUTS:
;	Comp:	A sky emission component structure
;
; COMMON BLOCKS:
;	None
;
; SIDE EFFECTS:
;	Creates a file in the current PSM output directory
;
; RESTRICTIONS:
;       Report any bugs to Jacques Delabrouille
;       delabrouille@apc.univ-paris7.fr
;
; PROCEDURE:
;
; MODIFICATION HISTORY:
; 	18/09/2009:	Jacques Delabrouille, First version.
; 	02/12/2009:	JD, Added sophistication to handle directory
; 	                in file names, which must be handled at the
; 	                PSM param COMMON level and not here.
; 	18/12/2009:	JD, call PSM_STR_REPLACE instead of STR_REPLACE.
; 	06/01/2012:	JD, doc and PSM license header.
;-
PRO SAVE_COMP, comp
  
  ;; Remove component directory from file names strings written in the structure
  ;; (necessary so that it still works if the output directory is moved or renamed)
  FOR i=1, comp.nlaw DO BEGIN
     ok = EXECUTE('em = comp.e'+NUMBER2STRING(i))
     IF NOT ok THEN STOP
     params = EMLAW_PARAMLIST(em.law)
     params = ['AMPL',params]
     taglist = TAG_NAMES(em)
     FOR j=0, N_ELEMENTS(params)-1 DO BEGIN
        wh = WHERE(taglist EQ params[j],nwh)
        IF nwh EQ 1 THEN BEGIN
           ok = EXECUTE('value = em.'+params[j]+'.value')
           IF NOT ok THEN STOP
           IF NOT ISARRAY(*value) THEN BEGIN
              IF ISFITSFILE(*value) THEN BEGIN
                 *value = PSM_STR_REPLACE(*value,GETPSMDIR(comp.name),'')
                 ok = EXECUTE('*(comp.e'+NUMBER2STRING(i)+'.'+params[j]+'.value) = *value')
                 IF NOT ok THEN STOP
              ENDIF
           ENDIF ELSE BEGIN
              FOR k=0, N_ELEMENTS(*value)-1 DO BEGIN
                 IF ISFITSFILE((*value)[k]) THEN BEGIN
                    (*value)[k] = PSM_STR_REPLACE((*value)[k],GETPSMDIR(comp.name),'')
                    ok = EXECUTE('(*(comp.e'+NUMBER2STRING(i)+'.'+params[j]+'.value))[k] = (*value)[k]')
                    IF NOT ok THEN STOP
                 ENDIF
              ENDFOR
           ENDELSE
        ENDIF
     ENDFOR
  ENDFOR
  
  SAVE, comp, filename=GETPSMDIR(comp.name)+comp.name+'.sav'

END
