; -----------------------------------------------------------------------------
;
; Copyright (C) 2009, 2011-2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	SKY_PIX
;
; PURPOSE:
;	Obtain the sky pixelisation used in a PSM session. This
;	routine is a convenient interface that gives access to the
;	information while hiding the (non-stable) practical way in
;	which the information is stored internally by the PSM. 
;
; CATEGORY:
;	Sky model
;
; CALLING SEQUENCE:
;	result = SKY_PIX()
;
; INPUTS:
;	None
;
; KEYWORD PARAMETERS:
;	None
;
; OUTPUTS:
;	A structure describing the pixelisation scheme
;
; COMMON BLOCKS:
;	SKY:	model
;	PSM:	param
;
; RESTRICTIONS:
;       Report any bugs to Jacques Delabrouille
;       delabrouille@apc.univ-paris7.fr
;
; PROCEDURE:
;
; MODIFICATION HISTORY:
; 	20/11/2009:	Jacques Delabrouille, First version.
; 	24/11/2009:	JD, Add test for model mapping type.
; 	12/05/2011:	JD, Handle case where COMMON sky model is not defined.
; 	06/01/2012:	JD, doc and PSM license header.
;-
FUNCTION SKY_PIX

  COMMON sky, model

  IF NOT DEFINED(model) THEN BEGIN
     COMMON psm, param
     IF DEFINED(param) THEN RESTORE_SKY_MODEL, model, output_dir=param.run.output_directory ELSE MESSAGE, 'No information available about sky model' 
  ENDIF

  CASE model.mapping.type OF
     'MAP': BEGIN
        RETURN, model.mapping.pix
     END
     ELSE: MESSAGE, 'Mapping type not implemented for sky model'
  ENDCASE 

END
