; -----------------------------------------------------------------------------
;
; Copyright (C) 2009, 2011, 2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	SKY_RESOLUTION
;
; PURPOSE:
;	Obtain the resolution of the sky simulation in a PSM
;	session. This routine is a convenient interface that gives
;	access to the information while hiding the (non-stable)
;	practical way in which the information is stored internally by
;	the PSM. 
;
; CATEGORY:
;	Sky model
;
; CALLING SEQUENCE:
;	result = SKY_RESOLUTION()
;
; INPUTS:
;	None
;
; KEYWORD PARAMETERS:
;	None
;
; OUTPUTS:
;	The resolution of the sky simulation in a PSM session
;
; RESTRICTIONS:
;       Report any bugs to Jacques Delabrouille
;       delabrouille@apc.univ-paris7.fr
;
; MODIFICATION HISTORY:
; 	29/10/2009:	Jacques Delabrouille, First version.
; 	05/09/2011:	JD, use system variable !SKY_RESOLUTION.
; 	04/01/2012:	JD, return NaN if !SKY_RESOLUTION not defined.
; 	06/01/2012:	JD, doc and PSM licence headers, + general clean-up.
;-
FUNCTION SKY_RESOLUTION

  DEFSYSV, '!sky_resolution', exists=ok
  IF ok THEN BEGIN
     RETURN, FLOAT(!SKY_RESOLUTION) 
  ENDIF ELSE BEGIN
     IF PSM_VERBOSITY() GE 1 THEN PSM_INFO_MESSAGE, 'WARNING: The PSM sky resolution is not defined'
     IF PSM_CAREFULNESS() LE 1 THEN RETURN, FLOAT('NaN') ELSE STOP
  ENDELSE

END
