; -----------------------------------------------------------------------------
;
; Copyright (C) 2011
;   Guillaume Castex
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
;-
;; Functions for fitting the observational data
;;----------------------------------------------

FUNCTION FIT_PWLAW_AND_GREYBODY, nu, p
  flux = p[1] * PSM_EMLAW(nu, emission_name='POWERLAW',nuref=p[0],specind=p[2])
  flux = flux + p[4] * PSM_EMLAW(nu, emission_name='GREYBODY', nuref=p[3], specind=p[5], temp=p[6])
RETURN, flux
END

PRO PSM_ERCSC_SOURCES, polarised=polarised

  COMMON psm, param

  gp = param.global
  runp = param.run
  names = param.names
  


  ;; Read ERCSC catalog

  allercsc = ['f030','f044','f070','f100','f143','f217','f353','f545','f857','ECC','ESZ']
 
  cat = GET_PLANCK_CAT(release='ERCSC')
  IF N_ELEMENTS(cat) NE 11 THEN MESSAGE, 'Unexpected number of catalogues ! '+number2string(N_ELEMENTS(cat))
  catECC = MRDFITS(cat[0],1)
  cat30 = MRDFITS(cat[1],1)
  cat44 = MRDFITS(cat[2],1)
  cat70 = MRDFITS(cat[3],1)
  cat100 = MRDFITS(cat[4],1)
  cat143 = MRDFITS(cat[5],1)
  cat217 = MRDFITS(cat[6],1)
  cat353 = MRDFITS(cat[7],1)
  cat545 = MRDFITS(cat[8],1)
  cat857 = MRDFITS(cat[9],1)

  catESZ = MRDFITS(cat[10],1)




;  tcat1 = get_ps_id_cat((cat30.glon)*!dpi/180,(cat30.glat)*!dpi/180,((cat30.pos_err)[0])*!dpi/180)
;  tcat2 = get_ps_id_cat((cat44.glon)*!dpi/180,(cat44.glat)*!dpi/180,((cat44.pos_err)[0])*!dpi/180)

  cat_ext=[30,44,70,100,143,217,353,545,857]

  FOR i=0,8 DO BEGIN
     command = 'tcat'+number2string(cat_ext[i]) + ' = get_ps_id_cat((cat'+number2string(cat_ext[i])+'.glon)*!dpi/180,(cat'+number2string(cat_ext[i])+'.glat)*!dpi/180,((cat'+number2string(cat_ext[i])+'.pos_err)[0])*!dpi/180)'
     ok = EXECUTE(command)
  ENDFOR
 

 ens_cat=[tcat30,tcat44,tcat70,tcat100,tcat143,tcat217,tcat353,tcat545,tcat857]

 result=source_identification(ens_cat)

 nsrc = n_elements(result.indices[*,0])


  ;; Create ERCSC catalog    
;; non polaris dans tous les cas, pour l'instant
  ercsc_cat = CREATE_PS_MODEL_CAT(nsrc, ['POWERLAW','GREYBODY'], name='strong'+names.ercscps_basename, id=id, polarised=0, elliptic=1)

  *ercsc_cat.e1.nuref.value = 3e10  ;;30GHz is the reference frequency for each source
  *ercsc_cat.e1.numin.value = 0.
  *ercsc_cat.e1.numax.value = FLOAT('INF')
  (ercsc_cat.e1.ampl.unit)='mJy'
  (*ercsc_cat.e1.specind.value)=dblarr(nsrc)
  (*ercsc_cat.e1.ampl.value)=dblarr(nsrc)

  *ercsc_cat.e2.nuref.value = 3e10  ;;30GHz is the reference frequency for each source
  *ercsc_cat.e2.numin.value = 0.
  *ercsc_cat.e2.numax.value = FLOAT('INF')
  (*ercsc_cat.e2.specind.value)=dblarr(nsrc)
  (*ercsc_cat.e2.ampl.value)=dblarr(nsrc)
  (ercsc_cat.e2.ampl.unit)='mJy'
  (*ercsc_cat.e2.temp.value)=dblarr(nsrc)

  (*ercsc_cat.el_fwhm_maj)=dblarr(nsrc)
  (*ercsc_cat.el_fwhm_min)=dblarr(nsrc)
  (*ercsc_cat.el_theta)=dblarr(nsrc)


  no_fwhm = -1

  FOR i=0, nsrc-1 DO BEGIN

     good=where(result.indices[i,*] ne -1)
     nu=cat_ext[good]*1.e9
     flux=0.D
     err=0.D
     FOR j=0, n_elements(good)-1 DO BEGIN
        indix=result.indices[i,good[j]]
        IF j eq 0 THEN BEGIN
           command ='(*ercsc_cat.lon.value)[i] = (cat' + number2string(cat_ext[good[j]]) + '.glon)[indix]*!dpi/180'
           ok = EXECUTE(command)     
           command ='(*ercsc_cat.lat.value)[i] = (cat' + number2string(cat_ext[good[j]]) + '.glat)[indix]*!dpi/180'
           ok = EXECUTE(command)
           
           finished_elipticity = 0
           jbis = j
           WHILE (finished_elipticity EQ 0) AND (jbis LE  n_elements(good)-1) DO BEGIN
              indixbis=result.indices[i,good[jbis]]
              command ='fin_maj = finite((cat' + number2string(cat_ext[good[jbis]]) + '.gau_fwhmmaj)[indixbis])'
              ok = EXECUTE(command)
              command ='fin_min = finite((cat' + number2string(cat_ext[good[jbis]]) + '.gau_fwhmmin)[indixbis])'
              ok = EXECUTE(command)
              
              IF (fin_maj EQ 1) AND (fin_min EQ 1) THEN BEGIN
                 command ='(*ercsc_cat.el_fwhm_maj)[i] = (cat' + number2string(cat_ext[good[jbis]]) + '.gau_fwhmmaj)[indixbis]'
                 ok = EXECUTE(command)
                 command ='(*ercsc_cat.el_fwhm_min)[i] = (cat' + number2string(cat_ext[good[jbis]]) + '.gau_fwhmmin)[indixbis]'
                 ok = EXECUTE(command)
                 command ='(*ercsc_cat.el_theta)[i] =(cat' + number2string(cat_ext[good[jbis]]) + '.gau_theta)[indixbis]*!dpi/180'
                 ok = EXECUTE(command)
                 finished_elipticity = 1
              ENDIF 
              jbis=jbis+1

           ENDWHILE
           IF finished_elipticity EQ 0 THEN no_fwhm=[no_fwhm,i]
              
              
        ENDIF
        command = 'flux = [flux,(cat' + number2string(cat_ext[good[j]]) + '.flux)[indix]]'
        ok = EXECUTE(command)
        command = 'err = [err,(cat' + number2string(cat_ext[good[j]]) + '.flux_err)[indix]]'
        ok = EXECUTE(command)
     ENDFOR
    flux=flux[1:n_elements(flux)-1]
    err=err[1:n_elements(err)-1]

    par = {value:0.,fixed:0B,limited:[0B,0B],limits:[0.,0.]}
    parinfo = REPLICATE(par,7)

    parinfo.value = [3e9, 0., -2.15, nu[0], flux[0], 1.5, 35.]
    parinfo.fixed = [1,0,1,1,0,1,0]  

    parms = MPFITFUN('FIT_PWLAW_AND_GREYBODY', nu, flux, err, ftol=1.e-8, parinfo=parinfo, status=status, /quiet)

    (*ercsc_cat.e1.ampl.value)[i] = parms[1]
    (*ercsc_cat.e1.specind.value)[i] = parms[2]
    (*ercsc_cat.e2.ampl.value)[i] = parms[4]
    (*ercsc_cat.e2.specind.value)[i] = parms[5]
    (*ercsc_cat.e2.temp.value)[i] = parms[6]



 ENDFOR






  IF ercsc_cat.polarised THEN BEGIN
   ;  (*ercsc_cat.pol_fraction) = (*(*model_cat).pol_fraction)[candidates]
   ;  (*ercsc_cat.pol_angle) = (*(*model_cat).pol_angle)[candidates]
  ENDIF


  ;indix = where(candidates ge 0)
  ;candidates = candidates[indix]
  ;remove_src_cat, candidates, model_cat

  ecat = PTR_NEW(ercsc_cat)

  remove_src_cat, no_fwhm[1:n_elements(no_fwhm)-1], ecat

  filename = FIX_SEPARATOR(GETPSMDIR('ps')) +'strong'+ names.ercscps_basename + '.sav'
  WRITE_CATALOG, *ecat, filename, /sav, name='comp'
  DESTROY_DATA, ercsc_cat
  DESTROY_DATA, ecat
  
  IF runp.verbosity GE 1 THEN PSM_INFO_MESSAGE, 'ERCSC model catalogue written'

END
