; -----------------------------------------------------------------------------
;
; Copyright (C) 2011
;   Guillaume Castex
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
;-
FUNCTION SOURCE_IDENTIFICATION, cat, help=help


  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'CONVERSION_FACTOR: function which identificate point sources from various catalogs'
     PSM_INFO_MESSAGE, 'SYNTAX: result = SOURCE_IDENTIFICATION([cat1, cat2, cat3, ...])'
     PSM_INFO_MESSAGE, "each cat must be a structure"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF


  ncat=(SIZE(cat))[1]


  indices=dblarr(1,ncat)
  FOR i=0, ncat-1 DO BEGIN
     thecat=cat[i]
     FOR i_src=0, N_ELEMENTS(*thecat.lat)-1 DO BEGIN
        lon=(*thecat.lon)[i_src]
        lat=(*thecat.lat)[i_src]        
        IF lat NE 100 THEN BEGIN
           indix=dblarr(1,ncat)
           indix[*]=-1
           indix[i]=i_src
           IF i LE ncat-2 THEN BEGIN
              FOR j=i+1,ncat-1 DO BEGIN
                 ocat=cat[j]
                 dist=*thecat.pos_err+*ocat.pos_err
                 wh_lat=WHERE(abs((*ocat.lat)-lat) lt dist)
                 IF wh_lat[0] NE -1 THEN BEGIN
                    gcirc, 0, lon, lat,(*ocat.lon)[wh_lat],(*ocat.lat)[wh_lat],dist_pos
                    wh_lon=WHERE(dist_pos le dist)
                    IF wh_lon[0] NE -1 THEN BEGIN
                       wh_can=wh_lat[wh_lon]
                       ncan=n_elements(wh_can)
                       best_can=WHERE(dist_pos[wh_lon] eq min(dist_pos[wh_lon]))
                       best_can=wh_can[best_can]
                       indix[j]=best_can
                       (*ocat.lat)[best_can]=100
;gcirc, 0, lon, lat,(*ocat.lon)[best_can],(*ocat.lat)[best_can],dist_test
;if lat gt 1 then stop
                    ENDIF
                 ENDIF   
              ENDFOR        
   
           ENDIF
           indices=[indices, indix]
        ENDIF
     ENDFOR
  ENDFOR
  nsrc=n_elements(indices[*,0])
  indices=indices[1:nsrc-1,*]
  result={indices:indices}

  RETURN, result

END
