; -----------------------------------------------------------------------------
;
; Copyright (C) 2011
;   Jean-Baptiste Melin
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	MERGE_INTERMEDIATE_CLUSTER_CATALOGS
;
; CATEGORY:
;	SZ effect
;
; CALLING SEQUENCE:
;	MERGE_INTERMEDIATE_CLUSTER_CATALOGS, Local_catalog, Sub_catalog

; MODIFICATION HISTORY:
; 	02/02/2011:	Jean-Baptiste Melin, First version.
; 	05/01/2012:	Jacques Delabrouille, updated doc header.
;-

FUNCTION MERGE_INTERMEDIATE_CLUSTER_CATALOGS, local_catalog, sub_catalog

delta_r = 5.    ; arcmin
delta_z = 0.1

remove_from_sub_catalog = BYTARR(N_ELEMENTS(sub_catalog))

FOR i=0, N_ELEMENTS(local_catalog)-1 DO BEGIN
  dival=2*ASIN(SQRT((SIN((local_catalog[i].lat*!dtor-sub_catalog.lat*!dtor)/2.))^2+COS(local_catalog[i].lat*!dtor)*COS(sub_catalog.lat*!dtor)*(SIN((local_catalog[i].lon*!dtor-sub_catalog.lon*!dtor)/2.))^2))/!dtor*60.
  whr = WHERE(dival LE delta_r, nwhr)
  IF nwhr GE 1 THEN BEGIN
    whz = WHERE(ABS(local_catalog[i].z-sub_catalog[whr].z) LE delta_z, nwhz)
    IF nwhz GE 1 THEN BEGIN
      ok = MIN(dival[whr[whz]], whm)
      remove_from_sub_catalog[whr[whz[whm]]]=1
      ;IF nwhz GE 2 THEN BEGIN
      ;  print, i
      ;  print, nwhz
      ;  print, whr[whz[whm]]
      ;  print, local_catalog[i].lon, sub_catalog[whr[whz[whm]]].lon
      ;  print, local_catalog[i].lat, sub_catalog[whr[whz[whm]]].lat
      ;  print, local_catalog[i].z, sub_catalog[whr[whz[whm]]].z
      ;  print, '*****'
      ;ENDIF
    ENDIF
  ENDIF
ENDFOR

whm = WHERE(remove_from_sub_catalog EQ 0, nwhm)
IF nwhm GE 1 THEN BEGIN
  merged_catalog = [local_catalog, sub_catalog[whm]]
ENDIF ELSE BEGIN
  merged_catalog = local_catalog
ENDELSE

RETURN, merged_catalog

END
