PRO RUN_DMB_SZ_CASE, resol, output_directory, fields, mf, cp, np, pb, pol, rosat, sdss, relativistic

  PSM_INFO_MESSAGE, 'FIELDS = ' + fields
  PSM_INFO_MESSAGE, 'MASS_FUNCTION = ' + mf
  PSM_INFO_MESSAGE, 'CLUSTER_PROFILE = ' + cp
  PSM_INFO_MESSAGE, 'NORM_PROFILE = ' + np
  PSM_INFO_MESSAGE, 'PROFILE_BOUNDS = ' + pb
  PSM_INFO_MESSAGE, 'SZ_INCLUDE_POLARISED = ' + pol
  PSM_INFO_MESSAGE, 'SZ_RELATIVISTIC = ' + NUMBER2STRING(relativistic)

  IF rosat+sdss GE 1 THEN BEGIN
     PSM_INFO_MESSAGE, 'SZ_CONSTRAINED = yes'
     sz_constrained = 'yes'
     sz_input_cat = ''
     IF rosat THEN sz_input_cat = sz_input_cat + 'rosat '
     IF sdss THEN sz_input_cat = sz_input_cat + 'sdss '
     PSM_INFO_MESSAGE, 'SZ_INPUT_CAT = ' + sz_input_cat
  ENDIF ELSE BEGIN
     PSM_INFO_MESSAGE, 'SZ_CONSTRAINED = no'
     sz_constrained = 'no'
  ENDELSE

  GET_LUN, unit
  paramfile = PSM_TMPFILENAME(extension='.psm')
  OPENW, unit, paramfile
  PRINTF, unit, 'OUTPUT_DIRECTORY = ' + output_directory
  PRINTF, unit, 'FIELDS = ' + fields
  PRINTF, unit, 'CLEAR_ALL = yes'
  PRINTF, unit, 'VISU = 2'
  PRINTF, unit, 'OUTPUT_VISU = ps'
  PRINTF, unit, 'SKY_RESOLUTION = ' + NUMBER2STRING(resol)
  PRINTF, unit, 'SKY_LMAX = ' + NUMBER2STRING(RESOL2LMAX(resol))
  PRINTF, unit, 'SKY_PIXELISATION = HEALPIX'
  PRINTF, unit, 'HEALPIX_NSIDE = ' + NUMBER2STRING(RESOL2NSIDE(resol))
  ;PRINTF, unit, 'SKY_PIXWINDOW = ' + sky_pixwindow
  PRINTF, unit, 'SZ_MODEL = dmb'
  PRINTF, unit, 'MASS_FUNCTION = ' + mf
  PRINTF, unit, 'CLUSTER_PROFILE = ' + cp
  PRINTF, unit, 'NORM_PROFILE = ' + np
  PRINTF, unit, 'PROFILE_BOUNDS = ' + pb
  PRINTF, unit, 'SZ_INCLUDE_POLARISED = ' + pol
  PRINTF, unit, 'SZ_CONSTRAINED = ' + sz_constrained
  IF sz_constrained EQ 'yes' THEN PRINTF, unit, 'SZ_INPUT_CAT = ' + sz_input_cat
  PRINTF, unit, 'SZ_RELATIVISTIC = ' + NUMBER2STRING(relativistic)

  PRINTF, unit, 'OBS_TASK = new'
  PRINTF, unit, 'WHAT_OBS = bandinteg'
  PRINTF, unit, 'INSTRUMENT = PSM_IDEAL'
  PRINTF, unit, 'OBS_FREQUENCIES =  150'
  PRINTF, unit, 'OBS_STOKES = ' + fields
  PRINTF, unit, 'CLUSTER_M_INF = 0.5'

  CLOSE, unit
  FREE_LUN, unit
  
  PSM_MAIN, paramfile, /debug, verbosity=0

END

PRO TEST_DMB_SZ, resol=resol, nsample=nsample, $
                 exhaustive=exhaustive, pickruncase=pickruncase, $
                 fields=fields, mass_function=mass_function, $
                 cluster_profile=cluster_profile, norm_profile=norm_profile, profile_bounds=profile_bounds, $
                 polarised=polarised, rosat=rosat, sdss=sdss, relativistic=relativistic
  
  IF NOT KEYWORD_SET(resol) THEN resol=120         ; 120 arcminutes
  output_directory = GETPSMDIR('psmroot') + 'Test/PSM_OUTPUT_dmb_sz_'
  
  ;listparam = ['FIELDS', 'MASS_FUNCTION', 'CLUSTER_PROFILE', 'NORM_PROFILE', 'PROFILE_BOUNDS', 'POLARISED', 'ROSAT', 'SDSS', 'RELATIVISITC']
  
  fields_list = ['T','TP']
  mf_list = ['press_schechter', 'sheth_tormen', 'evrard', 'jenkins', 'tinker']
  cp_list = ['xmm', 'chandra', 'beta']
  np_list = ['yes', 'no']
  pb_list = ['5r500', 'rvir']
  pol_list = ['yes', 'no']
  rosat_list = [0,1]
  sdss_list = [0,1]
  relativistic_list = [0,1,2,3,4]

  runcase=1L

  ncases = LONG(N_ELEMENTS(fields_list)) * LONG(N_ELEMENTS(mf_list)) * LONG(N_ELEMENTS(cp_list)) * LONG(N_ELEMENTS(np_list)) * $
           LONG(N_ELEMENTS(pb_list)) * LONG(N_ELEMENTS(pol_list)) * LONG(N_ELEMENTS(rosat_list)) * LONG(N_ELEMENTS(sdss_list)) * LONG(N_ELEMENTS(relativistic_list))

  IF ((KEYWORD_SET(exhaustive)) OR (KEYWORD_SET(pickruncase))) THEN BEGIN

     FOR f=0,1 DO FOR mf=0,4 DO FOR cp=0,2 DO FOR np=0,1 DO FOR pb=0,1 DO FOR p=0,1 DO FOR r=0,1 DO FOR s=0,1 DO FOR rel=0,4 DO BEGIN
        
        IF KEYWORD_SET(pickruncase) NE 0 THEN BEGIN
           IF runcase NE pickruncase THEN GOTO, nextcase
        ENDIF

        fields = fields_list[f]
        mass_fun = mf_list[mf]
        clus_prof = cp_list[cp]
        norm_prof = np_list[np]
        prof_bound = pb_list[pb]
        pol = pol_list[p]
        rosat = rosat_list[r]
        sdss = sdss_list[s]
        relativistic = relativistic_list[rel]

        PSM_INFO_TITLE, 'RUN CASE '+NUMBER2STRING(runcase)+' / '+NUMBER2STRING(ncases)+': '
        
        output_directory_n = output_directory + NUMBER2STRING(runcase)
        
        RUN_DMB_SZ_CASE, resol, output_directory_n, fields, mass_fun, clus_prof, norm_prof, prof_bound, pol, rosat, sdss, relativistic
        
        ;; check that the catalogue and the maps have been created
        ok = FILE_TEST(GETPSMDIR('sz')+'sz_cat.fits')
        IF NOT ok THEN MESSAGE, 'DMB SZ CASE '+NUMBER2STRING(runcase)+': SZ catalogue missing'
        IF relativistic EQ 0 THEN BEGIN
           ok = FILE_TEST(GETPSMDIR('sz')+'thermalsz_map.fits')
           IF NOT ok THEN MESSAGE, 'DMB SZ CASE '+NUMBER2STRING(runcase)+': Thermal SZ map missing'
        ENDIF ELSE BEGIN
           FOR i=0, relativistic DO BEGIN
              ok = FILE_TEST(GETPSMDIR('sz')+'relatsz'+NUMBER2STRING(relativistic)+'_map.fits')
              IF NOT ok THEN MESSAGE, 'DMB SZ CASE '+NUMBER2STRING(runcase)+': Thermal SZ map missing (relatsz'+NUMBER2STRING(relativistic)+'_map.fits)'
           ENDFOR
        ENDELSE
        ok = FILE_TEST(GETPSMDIR('sz')+'kineticsz_map.fits')
        IF NOT ok THEN MESSAGE, 'DMB SZ CASE '+NUMBER2STRING(runcase)+': Kinetic SZ map missing'

        nextcase:
        runcase = runcase + 1L

     ENDFOR

  ENDIF ELSE BEGIN

     IF NOT KEYWORD_SET(nsample) THEN nsample=1
     draw = RANDOMU(noseed,nsample)
     cases = LONG(ncases*draw)

     FOR i=0L, nsample-1 DO BEGIN
        TEST_DMB_SZ, resol=resol, pickruncase = cases[i]
     ENDFOR

  ENDELSE


END
