PRO RUN_GAUSSIAN_CMB_CASE, resol, output_directory, fields, sky_pixwindow, cmb_lensing, cmb_constrained, cmb_cl_source, r_ts=r_ts

  IF DEFINED(r_ts) THEN BEGIN
     r_is_0 = r_ts EQ 0
     r_is_005 = r_ts EQ 0.05
     ok = r_is_0 OR r_is_005
     IF NOT ok THEN MESSAGE, 'r_ts must be either 0 or 0.05 in this test'
  ENDIF

  PSM_INFO_MESSAGE, 'FIELDS = ' + fields
  PSM_INFO_MESSAGE, 'SKY_PIXWINDOW = ' + sky_pixwindow
  PSM_INFO_MESSAGE, 'CMB_LENSING = ' + cmb_lensing
  PSM_INFO_MESSAGE, 'CMB_CONSTRAINED = ' + cmb_constrained
  PSM_INFO_MESSAGE, 'CMB_CL_SOURCE = ' + cmb_cl_source
  IF KEYWORD_SET(r_ts) NE 0 THEN PSM_INFO_MESSAGE, 'R = ' + NUMBER2STRING(r_ts)
  
  GET_LUN, unit
  paramfile = PSM_TMPFILENAME(extension='.psm')
  OPENW, unit, paramfile
  PRINTF, unit, 'OUTPUT_DIRECTORY = ' + output_directory
  PRINTF, unit, 'FIELDS = ' + fields
  PRINTF, unit, 'CLEAR_ALL = yes'
  PRINTF, unit, 'VISU = 2'
  PRINTF, unit, 'OUTPUT_VISU = ps'
  PRINTF, unit, 'SKY_RESOLUTION = ' + NUMBER2STRING(resol)
  PRINTF, unit, 'SKY_LMAX = ' + NUMBER2STRING(RESOL2LMAX(resol))
  PRINTF, unit, 'SKY_PIXELISATION = HEALPIX'
  PRINTF, unit, 'HEALPIX_NSIDE = ' + NUMBER2STRING(RESOL2NSIDE(resol))
  PRINTF, unit, 'SKY_PIXWINDOW = ' + sky_pixwindow
  CASE cmb_cl_source OF
     'CAMB': PRINTF, unit, 'RUN_CAMB = yes'
     'CLASS': BEGIN
        PRINTF, unit, 'RUN_CLASS = yes'
        PRINTF, unit, 'RUN_CAMB = yes'
     END
     'standard-LCDM': 
  ENDCASE
  PRINTF, unit, 'CMB_MODEL = gaussian'
  PRINTF, unit, 'CMB_LENSING = ' + cmb_lensing
  PRINTF, unit, 'CMB_CONSTRAINED = ' + cmb_constrained
  PRINTF, unit, 'CMB_CL_SOURCE = ' + cmb_cl_source
  IF KEYWORD_SET(r_ts) NE 0 THEN PRINTF, unit, 'R = ' + NUMBER2STRING(r_ts)
  
  PRINTF, unit, 'OBS_TASK = new'
  PRINTF, unit, 'WHAT_OBS = bandinteg'
  PRINTF, unit, 'INSTRUMENT = PSM_IDEAL'
  PRINTF, unit, 'OBS_FREQUENCIES =  150'
  PRINTF, unit, 'OBS_STOKES = ' + fields


  CLOSE, unit
  FREE_LUN, unit
  PSM_MAIN, paramfile, /debug, verbosity=0
  
END


PRO TEST_GAUSSIAN_CMB, resol=resol, $
                       nsample=nsample, $
                       exhaustive=exhaustive, pickruncase=pickruncase, $
                       fields=fields, sky_pixwindow=sky_pixwindow, cmb_lensing=cmb_lensing, cmb_constrained=cmb_constrained, cmb_cl_source=cmb_cl_source, r_ts=r_ts
  
  IF NOT KEYWORD_SET(resol) THEN resol=120         ; 120 arcminutes
  output_directory = GETPSMDIR('psmroot') + 'Test/PSM_OUTPUT_gaussian_cmb_'
  
  listparam = ['FIELDS', 'SKY_PIXWINDOW', 'CMB_LENSING', 'CMB_CONSTRAINED', 'CMB_CL_SOURCE', 'R_TS']
  
  fields_list = ['T','TP']
  sky_pixwindow_list = ['0','1']
  cmb_lensing_list = ['none','cl','ilens']
  cmb_constrained_list = ['yes', 'no']
  cmb_cl_source_list = ['standard-LCDM', 'CAMB', 'CLASS']
  r_list = [0, 0.05]

  runcase=1L

  n1 = LONG(N_ELEMENTS(fields_list))
  n2 = LONG(N_ELEMENTS(sky_pixwindow_list))
  n3 = LONG(N_ELEMENTS(cmb_lensing_list))
  n4 = LONG(N_ELEMENTS(cmb_constrained_list))
  n5 = LONG(N_ELEMENTS(cmb_cl_source_list))
  n6 = LONG(N_ELEMENTS(r_list))

  ncases = n1 * n2 * n3 * n4 * n5 * n6
           
  IF ((KEYWORD_SET(exhaustive)) OR (KEYWORD_SET(pickruncase))) THEN BEGIN

     FOR r=0,1 DO FOR ccs=0,n5-1 DO FOR cc=0,n4-1 DO FOR cl=0,n3-1 DO FOR s=0,n2-1 DO FOR f=0,n1-1 DO BEGIN
        
        IF KEYWORD_SET(pickruncase) NE 0 THEN BEGIN
           IF runcase NE pickruncase THEN GOTO, nextcase
        ENDIF

        fields = fields_list[f]
        sky_pixwindow = sky_pixwindow_list[s]
        cmb_lensing = cmb_lensing_list(cl)
        cmb_constrained = cmb_constrained_list(cc)
        cmb_cl_source = cmb_cl_source_list(ccs)
        r_ts = r_list(r)

        PSM_INFO_TITLE, 'RUN CASE '+NUMBER2STRING(runcase)+': '
        
        output_directory_n = output_directory + NUMBER2STRING(runcase)
        
        IF ccs NE 0 THEN BEGIN
           ;; PRINT, 'fields = '+fields
           ;; PRINT, 'sky_pixwindow = '+sky_pixwindow
           ;; PRINT, 'cmb_lensing = '+cmb_lensing
           ;; PRINT, 'cmb_constrained = '+cmb_constrained
           ;; PRINT, 'cmb_cl_source = '+cmb_cl_source
           ;; return
           ;; stop
           RUN_GAUSSIAN_CMB_CASE, resol, output_directory_n, fields, sky_pixwindow, cmb_lensing, cmb_constrained, cmb_cl_source, r_ts=r_ts
        ENDIF ELSE BEGIN
           ;; PRINT, 'fields = '+fields
           ;; PRINT, 'sky_pixwindow = '+sky_pixwindow
           ;; PRINT, 'cmb_lensing = '+cmb_lensing
           ;; PRINT, 'cmb_constrained = '+cmb_constrained
           ;; PRINT, 'cmb_cl_source = '+cmb_cl_source
           ;; return
           ;; stop
           RUN_GAUSSIAN_CMB_CASE, resol, output_directory_n, fields, sky_pixwindow, cmb_lensing, cmb_constrained, cmb_cl_source
        ENDELSE

        nextcase:
        runcase = runcase + 1L
     ENDFOR

  ENDIF ELSE BEGIN

     casenumber = LINDGEN(ncases)+1L
     casenumber = REFORM(casenumber, n1, n2, n3, n4, n5, n6)
     i1 = casenumber & FOR i=0, n1-1 DO i1[i,*,*,*,*,*] = i
     i2 = casenumber & FOR i=0, n2-1 DO i2[*,i,*,*,*,*] = i
     i3 = casenumber & FOR i=0, n3-1 DO i3[*,*,i,*,*,*] = i
     i4 = casenumber & FOR i=0, n4-1 DO i4[*,*,*,i,*,*] = i
     i5 = casenumber & FOR i=0, n5-1 DO i5[*,*,*,*,i,*] = i
     i6 = casenumber & FOR i=0, n6-1 DO i6[*,*,*,*,*,i] = i

     IF NOT KEYWORD_SET(nsample) THEN nsample=1
     draw = RANDOMU(noseed,nsample)
     cases = LONG(ncases*draw)

     FOR i=0L, nsample-1 DO BEGIN

        IF NOT DEFINED(fields) THEN usei1 = i1[cases[i]] ELSE usei1 = (WHERE(fields_list EQ fields))[0]
        IF NOT DEFINED(sky_pixwindow) THEN usei2 = i2[cases[i]] ELSE usei2 =  (WHERE(sky_pixwindow_list EQ sky_pixwindow))[0]
        IF NOT DEFINED(cmb_lensing) THEN usei3 = i3[cases[i]] ELSE usei3 =  (WHERE(cmb_lensing_list EQ cmb_lensing))[0]
        IF NOT DEFINED(cmb_constrained) THEN usei4 = i4[cases[i]] ELSE usei4 =  (WHERE(cmb_constrained_list EQ cmb_constrained))[0]
        IF NOT DEFINED(cmb_cl_source) THEN usei5 = i5[cases[i]] ELSE usei5 =  (WHERE(cmb_cl_source_list EQ cmb_cl_source))[0]
        IF NOT DEFINED(r_ts) THEN usei6 = i6[cases[i]] ELSE usei6 =  (WHERE(r_list EQ r_ts))[0]

        ;; print, usei1, usei2, usei3, usei4, usei5     
        ;; PRINT, casenumber[usei1,usei2,usei3,usei4,usei5]

        TEST_GAUSSIAN_CMB, resol=resol, pickruncase = casenumber[usei1,usei2,usei3,usei4,usei5,usei6]
     ENDFOR

  ENDELSE
  
END
