; -----------------------------------------------------------------------------
;
; Copyright (C) 1997, 2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	COMMONWHERE
;
; PURPOSE:
;	Find indexes common to the result of two distinct "WHERE" statements.
;
; CATEGORY:
;	Array handling
;
; CALLING SEQUENCE:
;	Result = COMMONWHERE(wh1, wh2, nc)
;
; INPUTS:
;	Wh1:	Result of a first WHERE statement
;	Wh2:	Result of a second WHERE statement
;
; KEYWORD PARAMETERS:
;	None
;
; OUTPUTS:
;	Result:	list of indexes common to wh1 and wh2.
;
; RESTRICTIONS:
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; EXAMPLE:
;	wh1 = WHERE((FINDGEN(100) MOD 15) EQ 1)
;	wh2 = WHERE((FINDGEN(100) MOD 6) EQ 1)
;	result = COMMONWHERE(wh1,wh2,nnn) 
;
; MODIFICATION HISTORY:
; 	October 1997:	Jacques Delabrouille, First version.
;	05/01/2012	JD, Updated doc header.
;-
FUNCTION COMMONWHERE, wh1, wh2, nc

  tab1 = BYTARR(MAX([wh1,wh2])+2)
  tab2 = tab1
  IF ISARRAY(wh1) EQ 1 THEN tab1(wh1) = 1
  IF ISARRAY(wh2) EQ 1 THEN tab2(wh2) = 1
  commonwhere = WHERE(tab1*tab2 EQ 1, nc)
  RETURN, commonwhere

END
