; -----------------------------------------------------------------------------
;
; Copyright (C) 2009, 2010
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	ROUTINE_NAME
;
; PURPOSE:
;	...
;
;-
FUNCTION PSM_UNIQ, array, index=index, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'PSM_UNIQ: function which returns subscripts for unique elements in an array '
     PRINT, '          one subscript per value present is returned (similar to UNIQ but not necessarily adjacent)'
     PSM_INFO_MESSAGE, 'SYNTAX: result = PSM_UNIQ(array, index=)'
     PSM_INFO_MESSAGE, 'array is an input array of numbers or strings'
     PSM_INFO_MESSAGE, 'index is an output keyword, which indexes the elements of array (same index means equal elements)'
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF

  nel = N_ELEMENTS(array)
  s = SORT(array)

  index = LONARR(nel)
  u = UNIQ(array[s])

  nu = N_ELEMENTS(u)
  IF nu GE 2 THEN FOR i=1L, nu-1L DO index[s[LINDGEN(u[i]-u[i-1])+u[i-1]+1L]]=i

  RETURN, s[u]

END
