; -----------------------------------------------------------------------------
;
; Copyright (C) 2011
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	SET_PRECISION
;
; PURPOSE:
;	...
;
;-
PRO SET_PRECISION, precision

;; This sets the precision of some calculations with PSM software.
;; Warning: during a PSM run, this is superseded by the precision of
;; the run itself.
;; The precision can be obtained with GET_PRECISION or with
;; PRECISION_ISDOUBLE.
;; The two do not do the exact same thing. PRECISION_ISDOUBLE is a
;; function for a PSM run, which reads first the precision parameter
;; as defined in the PSM run, and then the !precision system variable
;; if the former is not defined.

  DEFSYSV, '!precision', exists=exists
  IF DEFINED(precision) THEN BEGIN
     IF exists THEN BEGIN
        CASE STRUPCASE(precision) OF
           'DOUBLE': !precision = 'double' 
           'SINGLE': !precision = 'single'
        ENDCASE
     ENDIF ELSE BEGIN
        CASE STRUPCASE(precision) OF
           'DOUBLE': DEFSYSV, '!precision', 'double'
           'SINGLE': DEFSYSV, '!precision', 'single'
        ENDCASE
     ENDELSE
  ENDIF

END
