; -----------------------------------------------------------------------------
;
; Copyright (C) 2012
;   Guillaume Castex
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	ROUTINE_NAME
;
; PURPOSE:
;	...
;
; CATEGORY:
;	...
;
; CALLING SEQUENCE:
;	ROUTINE_NAME, Parm1, Parm2, Parm3
;
; INPUTS:
;	Parm1:	...
;
; OPTIONAL INPUTS:
;	Parm2:	...
;	
; KEYWORD PARAMETERS:
;	KEY1:	...
;
; OUTPUTS:
;	...
;
; OPTIONAL OUTPUTS:
;	...
;
; COMMON BLOCKS:
;	BLOCK1:	...
;
; SIDE EFFECTS:
;	...
;
; RESTRICTIONS:
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; PROCEDURE:
;	...
;
; EXAMPLE:
;	... 
;
; MODIFICATION HISTORY:
; 	13/04/2012:	Jacques Delabrouille & Guillaume Castex, First version.
;-
PRO READ_SED_STARBURST, sed

  dir = GETPSMDIR('psmdata')+'ancillary/models/firb/GC-2012/'
  filename = dir+'starburst_template_NGC6090.dat'

  ok = TEST_PSM_DATA(filename, /get)
  IF NOT ok THEN MESSAGE, 'Starburst Submillimeter SED data missing'

  a = read_ascii(filename)
  wh = where((finite(a.field1[1,*])) ne 0)
  flux = reform(a.field1[1,wh])
  log_freq = reform(a.field1[0,wh])
  freq = 10.^(log_freq)
  flux = 10.^flux

  sed={freq:freq, S:flux}

END

FUNCTION STARBURST_SED, nu, redshift=redshift

  ;; Read the data file
	READ_SED_STARBURST, sed
  ;; Redshift the data
	IF DEFINED(redshift) THEN BEGIN
		freq=sed.freq/(1+redshift)
		sed.freq=freq
	ENDIF
  ;; Write the interpolation for the values of nu
	result=interpol(sed.s,sed.freq, nu) 
  ;; Return the emission at nu
  RETURN, result

END
