; -----------------------------------------------------------------------------
;
; Copyright (C) 2009, 2011, 2012
;   Jacques Delabrouille
;   Guillaume Castex
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	VECTOR_GET_ANGLE
;
; PURPOSE:
;	Procedure to compute the angle between two vectors.
;
; CATEGORY:
;	Mathematics
;
; CALLING SEQUENCE:
;	VECTOR_GET_ANGLE, Vector1, Vector2, Angle
;
; INPUTS:
;	Vector1:	First vector.
;	Vector2:	Second vector.
;
; OUTPUTS:
;	Angle:  	The angle, in radians, between the two vectors
;	                (comprised between 0 and PI).
;
; RESTRICTIONS:
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; PROCEDURE:
;	Simple mathematics.
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
; 	03/07/2009:   	Guillaume Castex, First version.
;	January 2012:	Jacques Delabrouille, some general clean-up,
;	                updated header documentation. 
;-

PRO VECTOR_GET_ANGLE, vector1, vector2, angle, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'VECTOR_GET_ANGLE: procedure to get the angle between two vectors'
     PSM_INFO_MESSAGE, 'SYNTAX: VECTOR_GET_ANGLE, vector1, vector2, angle'
     PSM_INFO_MESSAGE, "The output angle is in radians, between 0 and Pi"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN
  ENDIF

  cos_angle = TOTAL(vector2*vector1) / SQRT(TOTAL(vector1*vector1)*TOTAL(vector2*vector2))
  cos_angle = cos_angle<1
  cos_angle = cos_angle>(-1)
  angle = ACOS(cos_angle)

END
