; -----------------------------------------------------------------------------
;
; Copyright (C) 2010
;   Jacques Delabrouille
;   Soumen Basak
;   Guillaume Castex
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	DISTANCE_TO_MASK
;
; PURPOSE:
;	...
;
;-
PRO DISTANCE_TO_MASK, mask, dist, maxdist=maxdist, deg=deg, ordering=ordering, help=help


  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'DISTANCE_TO_MASK: procedure to find distance to a mask, up to some max (for apodisation purposes)'
     PSM_INFO_MESSAGE, 'SYNTAX: DISTANCE_TO_MASK, mask, dist, maxdist=, /deg, ordering='
     PSM_INFO_MESSAGE, "mask is an input HEALPix map containing zeros and ones (can be file or memory array)"
     PSM_INFO_MESSAGE, "set ordering to either 'RING' (the default) or 'NESTED' for input in memory array format"
     PSM_INFO_MESSAGE, 'dist is the (output) distance to the closest pixel for which mask=0'
     PSM_INFO_MESSAGE, 'set /deg for dist and maxdist in degrees'
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN
  ENDIF

  IF NOT KEYWORD_SET(ordering) THEN ordering='RING'
  IF NOT KEYWORD_SET(deg) THEN BEGIN
     IF NOT KEYWORD_SET(maxdist) THEN maxdist=!pi
  ENDIF ELSE BEGIN
     IF NOT KEYWORD_SET(maxdist) THEN maxdist=180
  ENDELSE
  nside = GET_NSIDE(mask)
  BORDER_OF_MASK, mask, p0, p1, np0=np0, np1=np1

  s = SIZE(mask, /type)
  IF s LE 5 THEN dist = FLOAT(maxdist)*mask
  IF s EQ 7 THEN BEGIN
     READ_MAP, mask, m
     dist = FLOAT(maxdist)*m
  ENDIF
  IF np0 NE 0 THEN BEGIN
     FOR p=0L, np0-1L DO BEGIN
        IF ordering EQ 'RING' THEN BEGIN
           PIX2VEC_RING, nside, p0[p], vec
           QUERY_DIST, nside, vec, maxdist, listpix, nlist, d, deg=deg
        ENDIF ELSE BEGIN
           PIX2VEC_NEST, nside, p0[p], vec
           QUERY_DIST, nside, vec, maxdist, listpix, nlist, d, deg=deg,/nested
        ENDELSE
        IF nlist NE 0 THEN dist(listpix) = d < dist(listpix)
     ENDFOR
  ENDIF
  IF s LE 5 THEN dist = TEMPORARY(dist)*mask
  IF s EQ 7 THEN BEGIN
     dist = TEMPORARY(dist)*TEMPORARY(m)
  ENDIF


END
