; -----------------------------------------------------------------------------
;
; Copyright (C) 2010
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	GET_NPIX
;
; PURPOSE:
;	...
;
;-
FUNCTION GET_NPIX, map, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'GET_NPIX: function to get the number of pixels in a healpix map'
     PSM_INFO_MESSAGE, 'SYNTAX: result = GET_NPIX(map)'
     PSM_INFO_MESSAGE, "map can be an array, a pointer, or a fits file"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF

  s = SIZE(map,/type)

  ;; Array
  ;;-------
  IF s LE 5 THEN BEGIN
     npix = ((SIZE(map,/dim))[0])
  ENDIF

  ;; Pointer
  ;;---------
  IF s EQ 10 THEN BEGIN
     npix = ((SIZE(*map,/dim))[0])
  ENDIF

  ;; File
  ;;------
  IF s EQ 7 THEN BEGIN
     h = HEADFITS(map, exten=1)
     npix = SXPAR(h,'NPIX')
  ENDIF

  RETURN, npix

END
