; -----------------------------------------------------------------------------
;
; Copyright (C) 2010-2011
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	HPX_VIEWPIX
;
; PURPOSE:
;	...
;
;-
PRO HPX_VIEWPIX, map, pixnum, ordering=ordering, title=title, xpos=xpos, ypos=ypos, scale=scale, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'HPX_VIEWPIX: procedure to visualize the neighbourhood of an healpix pixel'
     PSM_INFO_MESSAGE, 'SYNTAX: HPX_VIEWPIX, map, pixnum, ordering=, title=, xpos=, ypos=, scale='
     PSM_INFO_MESSAGE, 'Set scale to a zooming factor for more or less pixels in the window, e.g. scale=0.5 for zooming out'
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN
  ENDIF

  IF NOT KEYWORD_SET(ordering) THEN ordering='RING'
  npix = N_ELEMENTS(map)
  nside = NPIX2NSIDE(npix)

  IF NOT KEYWORD_SET(scale) THEN scale=1

  IF NOT DEFINED(pixnum) THEN BEGIN
     PSM_INFO_MESSAGE, 'Pixnum is not defined: picking a random pixel'
     pixnum = LONG(RANDOMU(noseed)*npix)
  ENDIF

  CASE STRUPCASE(ordering) OF
     'RING': PIX2ANG_RING, nside, pixnum, theta, phi
     'NESTED': PIX2ANG_NEST, nside, pixnum, theta, phi
  ENDCASE

  rtod = 180d0/!dpi
  rot = [phi*rtod,(!dpi/2d0-theta)*rtod]

  pixsize_rad = SQRT(4*!pi/N_ELEMENTS(map))
  pixsize_arcmin = pixsize_rad*60.*rtod
  CASE STRUPCASE(ordering) OF
     'RING': GNOMVIEW, map, rot=rot, reso_arcmin=pixsize_arcmin/30./scale, title=title, subtitle='PIX # '+NUMBER2STRING(pixnum)+' AT NSIDE='+NUMBER2STRING(nside), xpos=xpos, ypos=ypos
     'NESTED': GNOMVIEW, map, rot=rot, reso_arcmin=pixsize_arcmin/30./scale, title=title, subtitle='PIX # '+NUMBER2STRING(pixnum)+' AT NSIDE='+NUMBER2STRING(nside), /nested, xpos=xpos, ypos=ypos
  ENDCASE

END
