; -----------------------------------------------------------------------------
;
; Copyright (C) 2010
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	NSIDE2PIXSIZE
;
; PURPOSE:
;	...
;
;-
FUNCTION NSIDE2PIXSIZE, nside, deg=deg, arcmin=arcmin, rad=rad, arcsec=arcsec, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, ''
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'NSIDE2PIXSIZE: function to get the typical pixel size for a HEALPix map of a given nside'
     PSM_INFO_MESSAGE, 'SYNTAX: result = NSIDE2PIXSIZE(nside, /deg, /arcmin, /rad, /arcsec)'
     PSM_INFO_MESSAGE, "set at most one unit keyword (default = 'rad')"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF

  unitsset = LONG(KEYWORD_SET(deg)) + LONG(KEYWORD_SET(arcmin)) + LONG(KEYWORD_SET(rad)) + LONG(KEYWORD_SET(arcsec))
  
  IF unitsset GT 1L THEN MESSAGE, "Set only one of /rad, /deg, /arcmin, or /arcsec (default=rad)"
  IF unitsset EQ 0L THEN factor=1D0
  IF unitsset EQ 1L THEN BEGIN
     IF KEYWORD_SET(rad) THEN factor=1D0
     IF KEYWORD_SET(deg) THEN factor=180D0/!DPI
     IF KEYWORD_SET(arcmin) THEN factor=10800D0/!DPI    
     IF KEYWORD_SET(arcsec) THEN factor=648000D0/!DPI     
  ENDIF
  
  pixsize = factor*SQRT(4D0*!DPI/NSIDE2NPIX(nside))

  RETURN, pixsize

END

