FUNCTION IRAS_COVERAGE_MASK, band, nside=nside, ordering=ordering, help=help

  IF ((KEYWORD_SET(help) NE 0) OR (N_PARAMS() NE 1)) THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'IRAS_COVERAGE_MASK: function to make a mask for the IRAS coverage'
     PSM_INFO_MESSAGE, 'SYNTAX: result = IRAS_COVERAGE_MASK(band, nside=, ordering=)'
     PSM_INFO_MESSAGE, "band is the IRAS band (1 to 4). 1 corresponds to 12 micron, 4 to 100 micron."
     PSM_INFO_MESSAGE, "Use input keyword 'nside' for specifying HEALPix nside parameter of the mask (default 2048)."
     PSM_INFO_MESSAGE, "Use input keyword 'ordering' to 'RING' (default) or 'NESTED'."
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF

    ON_ERROR, 2    

    IF NOT DEFINED(nside) THEN nside=2048
    IF NOT DEFINED(ordering) THEN ordering='RING'

    maskfile = GETPSMDIR('psmdata')+'ancillary/observations/IRAS/ISSA_hpx1024/ISSA_B'+NUMBER2STRING(band)+'mask_ns1024.fits'
    ok = TEST_PSM_DATA(maskfile, /get)
    IF NOT ok THEN MESSAGE, 'IRAS coverage map not available'

    READ_FITS_MAP, maskfile, mask
    IF ((nside NE 1024) OR (STRUPCASE(ordering) NE 'NESTED')) THEN BEGIN
       UD_GRADE, TEMPORARY(mask), mask, order_in='NESTED', order_out=ordering, nside_out=nside
       mask = mask EQ 1
    ENDIF

    RETURN, BYTE(mask)

END
