; -----------------------------------------------------------------------------
;
; Copyright (C) 2010-2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	ALOG2
;
; PURPOSE:
;	Compute the logarithm in base 2 of a number. The result is
;	such that Number = 2^Result
;
; CATEGORY:
;	Mathematics
;
; CALLING SEQUENCE:
;	Result = ALOG2(Number)
;
; INPUTS:
;	Number:	A number
;
; KEYWORD PARAMETERS:
;	None
;
; OUTPUTS:
;	Result:	The value such that Number = 2^Result
;
; RESTRICTIONS:
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; PROCEDURE:
;	Simple algebra
;
; MODIFICATION HISTORY:
; 	01/10/2010:	Jacques Delabrouille, First version.
;-
FUNCTION ALOG2, number, help=help

  IF ((KEYWORD_SET(help) NE 0) OR (N_PARAMS() EQ 0)) THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'ALOG2: function which returns the basis-2 logarithm (such that ALOG2(2)=1)'
     PSM_INFO_MESSAGE, 'SYNTAX: result=ALOG2(number)'
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF

  RETURN, ALOG(number)/ALOG(2)

END
