; -----------------------------------------------------------------------------
;
; Copyright (C) 2009, 2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	ALM_INDEX
;
; PURPOSE:
;	Get the indexes corresponding to a range of l in an alm array
;
; CATEGORY:
;	Spherical transforms
;
; CALLING SEQUENCE:
;	Result = ALM_INDEX(Lmin, Lmax)
;
; INPUTS:
;	Lmin:	Minimum value of l in the l range
;	Lmax:	Maximum value of l in the l range
;
; OUTPUTS:
;	Result is an array of indexes
;
; RESTRICTIONS:
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; MODIFICATION HISTORY:
; 	29/10/2009:	Jacques Delabrouille, First version.
;-
FUNCTION ALM_INDEX, lmin, lmax, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'ALM_INDEX: function to calculate index for sph. harm. coefficients between lmin and lmax'
     PSM_INFO_MESSAGE, 'SYNTAX: result = ALM_INDEX(lmin, lmax)'
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF

  ni = ((lmax+1L)*(lmax+2L))/2L - ((lmin)*(lmin+1L))/2L
  
  index = LONARR(ni)
  i=0L
  FOR l=lmin, lmax DO FOR m=0, l DO BEGIN
     index[i] = LONG(l)^2 + l + m + 1L
     i=i+1L
  ENDFOR

RETURN, index

END
