; -----------------------------------------------------------------------------
;
; Copyright (C) 2009, 2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	CHECK_CL_STRUCT
;
; PURPOSE:
;	...
;
;-
FUNCTION CHECK_CL_STRUCT, cl, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'CHECK_CL_STRUCT: function to check whether a structure is a valid cl structure (as created by cl_structure.pro)'
     PSM_INFO_MESSAGE, 'SYNTAX: result = CHECK_CL_STRUCT(cl)'
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF

  tagnames = TAG_NAMES(cl)
  ;; Check that 'FIELD' tag exists
  wh = WHERE(STRUPCASE(tagnames) EQ 'FIELD', nwh)
  IF nwh NE 1 THEN RETURN, 0

  ;; Check that cl has correct number of tags
  cl0 = CL_STRUCTURE(fieldnames=cl.field)
  tagnames0 = TAG_NAMES(cl0)
  n = N_ELEMENTS(tagnames)
  n0 = N_ELEMENTS(tagnames0)
  IF n NE n0 THEN RETURN, 0

  ;; Check that all tags are identical
  FOR i=0, n-1 DO IF tagnames[i] NE tagnames0[i] THEN RETURN, 0

  ;; If everything was ok
  RETURN, 1

END	

