; -----------------------------------------------------------------------------
;
; Copyright (C) 2009, 2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	CL_VALUE
;
; PURPOSE:
;	...
;
;-
FUNCTION CL_VALUE, cl, field1, field2, l=l, help=help


  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'CL_VALUE: function to extract from a cl structure the cross spectrum between field1 and field2'
     PSM_INFO_MESSAGE, 'SYNTAX: result = CL_VALUE(cl, field1, field2, l=l)'
     PSM_INFO_MESSAGE, "cl is a structure, created by cl_structure.pro"
     PSM_INFO_MESSAGE, "field1 and field2 are strings"
     PSM_INFO_MESSAGE, "Set l keyword to get the result for a specific value of l"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF

  wh1 = WHERE(STRUPCASE(cl.field) EQ STRUPCASE(field1), nwh1)
  wh2 = WHERE(STRUPCASE(cl.field) EQ STRUPCASE(field2), nwh2)

  IF (nwh1 NE 1) OR (nwh2 NE 1) THEN RETURN, REPLICATE(0.,N_ELEMENTS(cl.l))

  wh = [wh1+1,wh2+1]
  wh = wh[SORT(wh)]

  IF NOT DEFINED(l) THEN BEGIN
     ok = EXECUTE('res = cl.c_'+NUMBER2STRING(wh[0])+'_'+NUMBER2STRING(wh[1]))
  ENDIF ELSE BEGIN
     whl = WHERE(cl.l EQ l, nwhl)
     IF nwhl EQ 1 THEN ok = EXECUTE('res = cl.c_'+NUMBER2STRING(wh[0])+'_'+NUMBER2STRING(wh[1])+'[whl[0]]') ELSE res=FLOAT('NaN')
  ENDELSE

  RETURN, res

END
