; -----------------------------------------------------------------------------
;
; Copyright (C) 2009, 2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	INDEX2L_ONLY
;
; PURPOSE:
;	Give value of l for a given alm index (duplicates the
;	INDEX2_LM healpix facility, but does not compute m)
;
; CATEGORY:
;	Spherical transforms
;
; CALLING SEQUENCE:
;	INDEX2L_ONLY, Index, L
;
; INPUTS:
;	Index:	ALM coefficient index
;
; OUTPUTS:
;	L:      Corresponding harmonic mode 
;
; OPTIONAL OUTPUTS:
;	Describe optional outputs here.  If the routine doesn't have any, 
;	just delete this section.
;
; COMMON BLOCKS:
;	BLOCK1:	Describe any common blocks here. If there are no COMMON
;		blocks, just delete this entry.
;
; RESTRICTIONS:
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; MODIFICATION HISTORY:
; 	05/02/2009:	Jacques Delabrouille, First version
;
;-
PRO INDEX2L_ONLY, index, l, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, 'INDEX2L_ONLY: Procedure to compute l corresponding to a given spherical harmonics index'
     PRINT, 'SYNTAX: INDEX2L_ONLY, index, l'
     RETURN
  ENDIF

  l = LONG(SQRT(index-1.D0))

END
