; -----------------------------------------------------------------------------
;
; Copyright (C) 2010-212
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	LASTCHAR
;
; PURPOSE:
;	Extract the last character(s) of a given string
;
; CATEGORY:
;	String handling
;
; CALLING SEQUENCE:
;	Result = LASTCHAR(String)
;
; INPUTS:
;	String:	A string.
;
; OPTIONAL INPUTS:
;	N:	Number of last characters in the output string
;		(default=1). If N=0 then the function returns the null
;		string, ''.
;	
; KEYWORD PARAMETERS:
;	HELP:	Set for online short help
;
; OUTPUTS:
;	Result:	A string that contain only the
;		first N characters of the original input.
;
; SIDE EFFECTS:
;	None
;
; RESTRICTIONS:
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; PROCEDURE:
;	Simple string manipulation
;
; EXAMPLE:
;	IDL> PRINT, LASTCHAR('PSM',2)
;       IDL prints: SM 
;
; MODIFICATION HISTORY:
; 	16/06/2010:	Jacques Delabrouille, First version.
;-
FUNCTION LASTCHAR, string, n, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'LASTCHAR: function to get the last character(s) in a string'
     PSM_INFO_MESSAGE, 'SYNTAX: result = LASTCHAR(string, n)'
     PSM_INFO_MESSAGE, "'string' is the input string"
     PSM_INFO_MESSAGE, "'n' (optional, default=1) is the number of characters"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF

  IF DEFINED(n) EQ 0 THEN n=1
  m = n<STRLEN(string)
  RETURN, STRMID(string,STRLEN(string)-m,m)	 	   
  
END
