; -----------------------------------------------------------------------------
;
; Copyright (C) 2009, 2010, 2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	RECURSIVE_TAG_NAMES
;
; PURPOSE:
;	...
;
;-
FUNCTION RECURSIVE_TAG_NAMES, struct, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'RECURSIVE_TAG_NAMES: function which returns the full tree array of tags and subtags in a structure'
     PSM_INFO_MESSAGE, 'SYNTAX: result = RECURSIVE_TAG_NAMES(struct)'
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF

  s = SIZE(struct,/type)
  IF s NE 8 THEN BEGIN
     MESSAGE, 'Input is not a structure', /info
     IF PSM_CAREFULNESS() GE 1 THEN STOP
     RETURN, 0
  ENDIF

  t = TAG_NAMES(struct)
  nt = N_ELEMENTS(t)
  keepgoing = 1

  WHILE keepgoing DO BEGIN
     struc = BYTARR(nt)
     FOR i=0, nt-1 DO BEGIN
        ok = EXECUTE('struc[i] = SIZE(struct.'+t[i]+',/type) EQ 8')
     ENDFOR
     wh = WHERE(struc EQ 1, nwh)
     IF nwh EQ 0 THEN BEGIN
        keepgoing = 0 
     ENDIF ELSE BEGIN
        tt = ['']
        FOR i=0, nt-1 DO BEGIN
           IF struc[i] EQ 0 THEN BEGIN
              tt = [tt, t[i]]
           ENDIF ELSE BEGIN
              ;stop
              res = EXECUTE("newt = '"+t[i]+"'+'.'+TAG_NAMES(struct."+t[i]+")")
              tt = [tt, newt]
           ENDELSE
        ENDFOR
        t = tt[1:*]
        nt = N_ELEMENTS(t)
     ENDELSE
  ENDWHILE

  RETURN, t
END





