; -----------------------------------------------------------------------------
;
; Copyright (C) 2009, 2010, 2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	UNIT2PSMUNIT
;
; PURPOSE:
;	...
;
;-
PRO UNIT2PSMUNIT, unit, psmunit, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'UNIT2PSMUNIT: procedure to get the normalised PSM unit string for some commonly used units'
     PSM_INFO_MESSAGE, 'SYNTAX: UNIT2PSMUNIT, unit, psmunit'
     PSM_INFO_MESSAGE, "unit is an input string (e.g. 'mK, thermodynamic', 'Kelvin-squared')"
     PSM_INFO_MESSAGE, "psmunit is the output correct PSM unit (e.g. 'mK_CMB', '(K_CMB)**2')"
     PSM_INFO_MESSAGE, "Useful, but to be used with caution !"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN
  ENDIF


CASE unit OF
   'mK, thermodynamic': psmunit = 'mK_CMB'
   'Kelvin-squared': psmunit = '(K_CMB)**2'
   'DT/T': psmunit = 'K/KCMB'
   ELSE: BEGIN
      ok = CHECKUNIT(unit)
      IF ok THEN psmunit = unit ELSE BEGIN
         psmunit = 'unknown'
         MESSAGE, 'input unit not recognised'
      ENDELSE
   END
ENDCASE

IF PSM_VERBOSITY() GE 1 THEN BEGIN
   IF unit NE psmunit THEN MESSAGE, unit + ' interpreted as ' + psmunit, /info
ENDIF

END

