; -----------------------------------------------------------------------------
;
; Copyright (C) 2009, 2010, 2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	GET_COLOR
;
; PURPOSE:
;	...
;
;-
FUNCTION GET_COLOR, colorname, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'GET_COLOR: function to get the LONG number corresponding to a color for color display'
     PSM_INFO_MESSAGE, 'SYNTAX: result = GET_COLOR(colorname)'
     PSM_INFO_MESSAGE, "colorname is a string corresponding to any usual color (e.g. red, white, green, pink, ...)"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF

ncol = N_ELEMENTS(colorname)

col = LONARR(ncol,3)

FOR i=0, ncol-1 DO BEGIN
CASE STRLOWCASE(colorname[i]) OF
   'black':  col[i,*] = [0,0,0]
   'white':  col[i,*] = [255,255,255]
   'red':    col[i,*] = [255,0,0]
   'green':  col[i,*] = [0,255,0]
   'blue':   col[i,*] = [0,0,255]
   'yellow': col[i,*] = [255,255,0]
   'purple': col[i,*] = [170,0,255]
   'cyan':   col[i,*] = [0,255,255]
   'orange': col[i,*] = [255,110,0]
   'grey':   col[i,*] = [128,128,128]
   'gray':   col[i,*] = [128,128,128] ;same as previous
   'pink':   col[i,*] = [255,140,220]
   'brown':  col[i,*] = [110,40,20]
ENDCASE
ENDFOR
RETURN, col[*,0] + 256L*col[*,1] + 256L^2*col[*,2]

END
