; -----------------------------------------------------------------------------
;
; Copyright (C) 2009-2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	PLOT_CMB_CL
;
; PURPOSE:
;	...
;
;-
PRO PLOT_CMB_CL, cl_in, filename, _extra=_extra, output=output, nocolor=nocolor, nopolar=nopolar, useunits=useunits, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'PLOT_CMB_CL: procedure to visualize (multivariate) angular power spectrum'
     PSM_INFO_MESSAGE, 'SYNTAX: PLOT_CMB_CL, cl_in, [filename,] output=, _extra=, output=, useunits=, /nocolor'
     PSM_INFO_MESSAGE, "cl_in can be a cl structure, or a file containing cl (of 'any format')"
     PSM_INFO_MESSAGE, "output can be 'screen' [default] or 'ps' (in which case the plot is in 'filename'"
     PSM_INFO_MESSAGE, "If filename is undefined then it is set to 'plot_cl.ps' for output='ps'"
     PSM_INFO_MESSAGE, "Set 'useunits' to a string which will be used to label units if those are unknown"
     PSM_INFO_MESSAGE, "Set '/nopolar' for visualisation of TT only"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN
  ENDIF

  ON_ERROR, 2

  DEFSYSV, '!debug', exists=exists
  IF exists THEN IF !debug THEN MESSAGE, 'Implement general case, implement ps', /info

  IF NOT KEYWORD_SET(output) THEN output='screen'
  IF output EQ 'ps' THEN IF DEFINED(filename) EQ 0 THEN filename='plot_cl.ps'
  IF output EQ 'png' THEN IF DEFINED(filename) EQ 0 THEN filename='plot_cl.png'

  cltype = SIZE(cl_in, /type)

  IF cltype EQ 8 THEN BEGIN
     ;; Check that it is compatible with expected cl structure
     ok = CHECK_CL_STRUCT(cl_in)
     IF NOT ok THEN BEGIN
        MESSAGE, 'Sorry, cl format not supported', /info
        RETURN
     ENDIF ELSE c=cl_in
  ENDIF 

  IF cltype EQ 7 THEN BEGIN
     ;; Assume that the input is a cl_file
     ;; READ_CL_ANYFORMAT, cl_in, c
     STOP
  ENDIF

  nc=0
  ;; look for T
  ;;------------
  wt = WHERE(c.field EQ 'T', nwt)
  IF nwt EQ 1 THEN BEGIN
     nc = nc+1
     i = NUMBER2STRING(wt[0]+1)
     ok = EXECUTE('tt = c.c_'+i+'_'+i)
     IF NOT ok THEN STOP
  ENDIF
  ;; look for E
  ;;------------
  we = WHERE(c.field EQ 'E', nwe)
  IF nwe EQ 1 THEN BEGIN
     nc = nc+1
     i = NUMBER2STRING(we[0]+1)
     ok = EXECUTE('ee = c.c_'+i+'_'+i)
     IF NOT ok THEN STOP
     j = NUMBER2STRING(wt[0]+1)
     ok = EXECUTE('te = c.c_'+j+'_'+i)
     IF NOT ok THEN STOP
  ENDIF
  ;; look for B
  ;;------------
  wb = WHERE(c.field EQ 'B', nwb)
  IF nwb EQ 1 THEN BEGIN
     nc = nc+1
     i = NUMBER2STRING(wb[0]+1)
     ok = EXECUTE('bb = c.c_'+i+'_'+i)
     IF NOT ok THEN STOP
     j = NUMBER2STRING(wt[0]+1)
     ok = EXECUTE('tb = c.c_'+j+'_'+i)
     IF NOT ok THEN STOP
     k = NUMBER2STRING(we[0]+1)
     ok = EXECUTE('eb = c.c_'+k+'_'+i)
     IF NOT ok THEN STOP
  ENDIF

  ;; nc = N_ELEMENTS(c.field)
  IF KEYWORD_SET(nopolar) NE 0 THEN nc=1

  current_device = !d.name
  CASE output OF
     'screen': BEGIN
        DEVICE, get_decomposed=d
        WINDOW, /free, xsize=600, ysize=600           
        DEVICE, decomposed=1
        SET_PLOT, 'x'
     END
     'png': BEGIN
        DEVICE, get_decomposed=d
        WINDOW, /free, xsize=600, ysize=600           
        DEVICE, decomposed=1
        SET_PLOT, 'x'
     END
     'ps':BEGIN
        SET_PLOT, 'ps'
        DEVICE, /color, xsize=16, ysize=16, decomposed=1, filename=filename
     END
  ENDCASE
  
  ;; Three components (T, E, B)
  ;;----------------------------
  IF nc EQ 3 THEN BEGIN
     ;;wt = WHERE(STRUPCASE(c.field) EQ 'T', nwt)
     ;;we = WHERE(STRUPCASE(c.field) EQ 'E', nwe)
     ;;wb = WHERE(STRUPCASE(c.field) EQ 'B', nwb)
     
     ;; log plot of all
     ;;-----------------
     ;; find range for plotting
     llp1_2pi = c.l*(c.l+1L)/2/!pi
     ;cltab = [llp1_2pi*c.c_1_1, llp1_2pi*c.c_1_2, llp1_2pi*c.c_1_3, llp1_2pi*c.c_2_2, llp1_2pi*c.c_2_3, llp1_2pi*c.c_3_3]
     cltab = [llp1_2pi*tt, llp1_2pi*te, llp1_2pi*tb, llp1_2pi*ee, llp1_2pi*eb, llp1_2pi*bb]

     mini = MIN(cltab)
     maxi = MAX(cltab)

     mini = MIN(ABS(cltab(WHERE(cltab NE 0)))) 
     maxi = MAX(cltab)
     expmin = CEIL(ALOG10(ABS(mini)))-1
     mini = 10.^expmin
     expmax = CEIL(ALOG10(maxi))
     maxi = 10.^expmax
     lmax = MAX(c.l)
     xrange=[1,2*lmax]

     IF NOT KEYWORD_SET(useunits) THEN BEGIN
        useunits = c.units(0)
     ENDIF ELSE BEGIN
        IF PSM_CAREFULNESS() GE 1 THEN BEGIN
           IF c_units[0] NE 'unknown' THEN IF NOT (useunits EQ c_units[0]) THEN BEGIN
              PSM_INFO_MESSAGE, 'Warning, units set different from those present in cl'
              IF PSM_CAREFULNESS() GE 2 THEN STOP
           ENDIF
        ENDIF
     ENDELSE

     ;; IF NOT KEYWORD_SET(nocolor) THEN BEGIN
     ;;    PLOT, c.l, llp1_2pi*c.c_1_1, background=GET_COLOR('white'), color=GET_COLOR('black'), /xlog, $
     ;;          yrange=[mini,maxi], /ylog, _extra=_extra, title = 'POWER SPECTRUM', $
     ;;          xtitle = 'L', ytitle = 'L(L+1)CL/2pi [('+useunits+')**2]', xrange=xrange, /ystyle, /xstyle
     ;;    OPLOT, c.l, llp1_2pi*c.c_2_2, _extra=_extra, color=GET_COLOR('green')
     ;;    OPLOT, c.l, llp1_2pi*c.c_3_3, _extra=_extra, color=GET_COLOR('blue')
     ;;    OPLOT, c.l, llp1_2pi*c.c_1_2, _extra=_extra, color=GET_COLOR('red')
     ;;    OPLOT, c.l, -llp1_2pi*c.c_1_2, _extra=_extra, color=GET_COLOR('red'), linestyle=2
     ;; ENDIF ELSE BEGIN
     ;;    PLOT, c.l, llp1_2pi*c.c_1_1, background=GET_COLOR('white'), color=GET_COLOR('black'), /xlog, $
     ;;          yrange=[mini,maxi], /ylog, _extra=_extra, title = 'POWER SPECTRUM', $
     ;;          xtitle = 'L', ytitle = 'L(L+1)CL/2pi [('+useunits+')**2]', xrange=xrange, /ystyle, /xstyle
     ;;    OPLOT, c.l, llp1_2pi*c.c_2_2, _extra=_extra, color=GET_COLOR('black')
     ;;    OPLOT, c.l, llp1_2pi*c.c_3_3, _extra=_extra, color=GET_COLOR('black')
     ;;    OPLOT, c.l, llp1_2pi*c.c_1_2, _extra=_extra, color=GET_COLOR('black')
     ;;    OPLOT, c.l, -llp1_2pi*c.c_1_2, _extra=_extra, linestyle=2, color=GET_COLOR('black')     
     ;; ENDELSE
     IF NOT KEYWORD_SET(nocolor) THEN BEGIN
        PLOT, c.l, llp1_2pi*tt, background=GET_COLOR('white'), color=GET_COLOR('black'), /xlog, $
              yrange=[mini,maxi], /ylog, _extra=_extra, title = 'POWER SPECTRUM', $
              xtitle = 'L', ytitle = 'L(L+1)CL/2pi [('+useunits+')**2]', xrange=xrange, /ystyle, /xstyle
        OPLOT, c.l, llp1_2pi*ee, _extra=_extra, color=GET_COLOR('green')
        OPLOT, c.l, llp1_2pi*bb, _extra=_extra, color=GET_COLOR('blue')
        OPLOT, c.l, llp1_2pi*te, _extra=_extra, color=GET_COLOR('red')
        OPLOT, c.l, -llp1_2pi*te, _extra=_extra, color=GET_COLOR('red'), linestyle=2
     ENDIF ELSE BEGIN
        PLOT, c.l, llp1_2pi*tt, background=GET_COLOR('white'), color=GET_COLOR('black'), /xlog, $
              yrange=[mini,maxi], /ylog, _extra=_extra, title = 'POWER SPECTRUM', $
              xtitle = 'L', ytitle = 'L(L+1)CL/2pi [('+useunits+')**2]', xrange=xrange, /ystyle, /xstyle
        OPLOT, c.l, llp1_2pi*ee, _extra=_extra, color=GET_COLOR('black')
        OPLOT, c.l, llp1_2pi*bb, _extra=_extra, color=GET_COLOR('black')
        OPLOT, c.l, llp1_2pi*te, _extra=_extra, color=GET_COLOR('black')
        OPLOT, c.l, -llp1_2pi*te, _extra=_extra, linestyle=2, color=GET_COLOR('black')     
     ENDELSE
  ENDIF

  ;; Two components (T, E)
  ;;----------------------------
  IF nc EQ 2 THEN BEGIN
     ;;wt = WHERE(STRUPCASE(c.field) EQ 'T', nwt)
     ;;we = WHERE(STRUPCASE(c.field) EQ 'E', nwe)
     
     ;; log plot of all
     ;;-----------------
     ;; find range for plotting
     llp1_2pi = c.l*(c.l+1L)/2/!pi
     ;;cltab = [llp1_2pi*c.c_1_1, llp1_2pi*c.c_1_2, llp1_2pi*c.c_2_2]
     cltab = [llp1_2pi*tt, llp1_2pi*te, llp1_2pi*ee]

     mini = MIN(cltab)
     maxi = MAX(cltab)

     mini = MIN(ABS(cltab(WHERE(cltab NE 0)))) 
     maxi = MAX(cltab)
     expmin = CEIL(ALOG10(ABS(mini)))-1
     mini = 10.^expmin
     expmax = CEIL(ALOG10(maxi))
     maxi = 10.^expmax
     lmax = MAX(c.l)
     xrange=[1,2*lmax]

     IF NOT KEYWORD_SET(useunits) THEN BEGIN
        useunits = c.units(0)
     ENDIF ELSE BEGIN
        IF PSM_CAREFULNESS() GE 1 THEN BEGIN
           IF c_units[0] NE 'unknown' THEN IF NOT (useunits EQ c_units[0]) THEN BEGIN
              PSM_INFO_MESSAGE, 'Warning, units set different from those present in cl'
              IF PSM_CAREFULNESS() GE 2 THEN STOP
           ENDIF
        ENDIF
     ENDELSE

     ;; IF NOT KEYWORD_SET(nocolor) THEN BEGIN
     ;;    PLOT, c.l, llp1_2pi*c.c_1_1, background=GET_COLOR('white'), color=GET_COLOR('black'), /xlog, $
     ;;          yrange=[mini,maxi], /ylog, _extra=_extra, title = 'POWER SPECTRUM', $
     ;;          xtitle = 'L', ytitle = 'L(L+1)CL/2pi [('+useunits+')**2]', xrange=xrange, /ystyle, /xstyle
     ;;    OPLOT, c.l, llp1_2pi*c.c_2_2, _extra=_extra, color=GET_COLOR('green')
     ;;    OPLOT, c.l, llp1_2pi*c.c_1_2, _extra=_extra, color=GET_COLOR('red')
     ;;    OPLOT, c.l, -llp1_2pi*c.c_1_2, _extra=_extra, color=GET_COLOR('red'), linestyle=2
     ;; ENDIF ELSE BEGIN
     ;;    PLOT, c.l, llp1_2pi*c.c_1_1, background=GET_COLOR('white'), color=GET_COLOR('black'), /xlog, $
     ;;          yrange=[mini,maxi], /ylog, _extra=_extra, title = 'POWER SPECTRUM', $
     ;;          xtitle = 'L', ytitle = 'L(L+1)CL/2pi [('+useunits+')**2]', xrange=xrange, /ystyle, /xstyle
     ;;    OPLOT, c.l, llp1_2pi*c.c_2_2, _extra=_extra, color=GET_COLOR('black')
     ;;    OPLOT, c.l, llp1_2pi*c.c_1_2, _extra=_extra, color=GET_COLOR('black')
     ;;    OPLOT, c.l, -llp1_2pi*c.c_1_2, _extra=_extra, linestyle=2, color=GET_COLOR('black')     
     ;; ENDELSE
     IF NOT KEYWORD_SET(nocolor) THEN BEGIN
        PLOT, c.l, llp1_2pi*tt, background=GET_COLOR('white'), color=GET_COLOR('black'), /xlog, $
              yrange=[mini,maxi], /ylog, _extra=_extra, title = 'POWER SPECTRUM', $
              xtitle = 'L', ytitle = 'L(L+1)CL/2pi [('+useunits+')**2]', xrange=xrange, /ystyle, /xstyle
        OPLOT, c.l, llp1_2pi*ee, _extra=_extra, color=GET_COLOR('green')
        OPLOT, c.l, llp1_2pi*te, _extra=_extra, color=GET_COLOR('red')
        OPLOT, c.l, -llp1_2pi*te, _extra=_extra, color=GET_COLOR('red'), linestyle=2
     ENDIF ELSE BEGIN
        PLOT, c.l, llp1_2pi*tt, background=GET_COLOR('white'), color=GET_COLOR('black'), /xlog, $
              yrange=[mini,maxi], /ylog, _extra=_extra, title = 'POWER SPECTRUM', $
              xtitle = 'L', ytitle = 'L(L+1)CL/2pi [('+useunits+')**2]', xrange=xrange, /ystyle, /xstyle
        OPLOT, c.l, llp1_2pi*ee, _extra=_extra, color=GET_COLOR('black')
        OPLOT, c.l, llp1_2pi*te, _extra=_extra, color=GET_COLOR('black')
        OPLOT, c.l, -llp1_2pi*te, _extra=_extra, linestyle=2, color=GET_COLOR('black')     
     ENDELSE
  ENDIF

  ;; One component (T)
  ;;-------------------
  IF nc EQ 1 THEN BEGIN
     
     ;; log plot 
     ;;----------
     ;; find range for plotting
     llp1_2pi = c.l*(c.l+1L)/2/!pi
     ;;cltab = [llp1_2pi*c.c_1_1[2:*]]
     cltab = [llp1_2pi*tt[2:*]]

     mini = MIN(cltab)
     maxi = MAX(cltab)

     mini = MIN(ABS(cltab(WHERE(cltab NE 0)))) 
     maxi = MAX(cltab)
     expmin = CEIL(ALOG10(ABS(mini)))-1
     mini = 10.^expmin
     expmax = CEIL(ALOG10(maxi))
     maxi = 10.^expmax
     lmax = MAX(c.l)
     xrange=[1,2*lmax]

     ;; IF NOT KEYWORD_SET(nocolor) THEN BEGIN
     ;;    PLOT, c.l, llp1_2pi*c.c_1_1, background=GET_COLOR('white'), color=GET_COLOR('black'), /xlog, $
     ;;          yrange=[mini,maxi], /ylog, _extra=_extra, title = 'POWER SPECTRUM', $
     ;;          xtitle = 'L', ytitle = 'L(L+1)CL/2pi [('+c.units(0)+')**2]', xrange=xrange, /ystyle, /xstyle
     ;; ENDIF ELSE BEGIN
     ;;    PLOT, c.l, llp1_2pi*c.c_1_1, background=GET_COLOR('white'), color=GET_COLOR('black'), /xlog, $
     ;;          yrange=[mini,maxi], /ylog, _extra=_extra, title = 'POWER SPECTRUM', $
     ;;          xtitle = 'L', ytitle = 'L(L+1)CL/2pi [('+c.units(0)+')**2]', xrange=xrange, /ystyle, /xstyle
     ;; ENDELSE
     IF NOT KEYWORD_SET(nocolor) THEN BEGIN
        PLOT, c.l, llp1_2pi*tt, background=GET_COLOR('white'), color=GET_COLOR('black'), /xlog, $
              yrange=[mini,maxi], /ylog, _extra=_extra, title = 'POWER SPECTRUM', $
              xtitle = 'L', ytitle = 'L(L+1)CL/2pi [('+c.units(0)+')**2]', xrange=xrange, /ystyle, /xstyle
     ENDIF ELSE BEGIN
        PLOT, c.l, llp1_2pi*tt, background=GET_COLOR('white'), color=GET_COLOR('black'), /xlog, $
              yrange=[mini,maxi], /ylog, _extra=_extra, title = 'POWER SPECTRUM', $
              xtitle = 'L', ytitle = 'L(L+1)CL/2pi [('+c.units(0)+')**2]', xrange=xrange, /ystyle, /xstyle
     ENDELSE
  ENDIF
    
  PSM_DEBUG_MESSAGE, 'Reset the device parameters in case of error in plot_cmb_cl', routine='PLOT_CMB_CL', infolimit=2, stoplimit=3

  CASE output OF
     'ps':BEGIN
        DEVICE, /close
     END
     'png':BEGIN
        WRITE_PNG, filename, TVRD(/true)
        ;WDEL, !d.window
     END
     ELSE: ;do nothing
  ENDCASE
  IF output NE 'ps' THEN ok=EXECUTE('DEVICE, decomposed=d')
  SET_PLOT, current_device

END
