; -----------------------------------------------------------------------------
;
; Copyright (C) 2009-2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	VISU_CL
;
; PURPOSE:
;	...
;
;-
PRO VISU_CL, cl_in, _extra=_extra, ps=ps, diag=diag, nolegend=nolegend, cmbstyle=cmbstyle, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'VISU_CL: procedure to visualize (multivariate) angular power spectrum'
     PSM_INFO_MESSAGE, 'SYNTAX: VISU_CL, cl_in, _extra=_extra, /ps, /diag, /nolegend, /cmbstyle'
     PSM_INFO_MESSAGE, "cl_in can be a cl structure, or a file containing cl (of 'any format')"
     PSM_INFO_MESSAGE, "set ps to a valid file name for the outmut to be written in a postscript file"
     PSM_INFO_MESSAGE, "set /diag to plot only auto power spectra (no cross-spectrum)"
     PSM_INFO_MESSAGE, "set /nolegend to skip the boxed legend"
     PSM_INFO_MESSAGE, "set /cmbstyle to plot l(l+1)Cl/2pi instead of Cl"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN
  ENDIF

  ON_ERROR, 2

  DEFSYSV, '!debug', exists=exists
  IF exists THEN IF !debug THEN MESSAGE, 'Implement general case, implement ps', /info

  IF NOT KEYWORD_SET(output) THEN output='screen'
  IF output EQ 'ps' THEN IF DEFINED(filename) EQ 0 THEN filename='plot_cl.ps'
  IF output EQ 'png' THEN IF DEFINED(filename) EQ 0 THEN filename='plot_cl.png'

  cltype = SIZE(cl_in, /type)

  IF cltype EQ 8 THEN BEGIN
     ;; Check that it is compatible with expected cl structure
     ok = CHECK_CL_STRUCT(cl_in)
     IF NOT ok THEN BEGIN
        MESSAGE, 'Sorry, cl format not supported', /info
        RETURN
     ENDIF ELSE c=cl_in
  ENDIF 

  IF cltype EQ 7 THEN BEGIN
     ;; Assume that the input is a cl_file
     ;; READ_CL_ANYFORMAT, cl_in, c
     STOP
  ENDIF

  ;; Make a table
  ;;--------------
  nfields = N_ELEMENTS(c.field)
  IF NOT KEYWORD_SET(diag) THEN ntab = (nfields*(nfields+1L))/2L ELSE ntab=nfields
  nl = N_ELEMENTS(c.l)
  tab = DBLARR(nl,ntab)
  tit = STRARR(ntab)
  IF NOT KEYWORD_SET(cmbstyle) THEN factorl=1 ELSE factorl = c.l*(c.l+1L)/2/!pi

  ;; Fill the table
  ;;----------------
  IF NOT KEYWORD_SET(diag) THEN BEGIN
     nn = 0
     FOR i=1, nfields DO FOR j=i, nfields DO BEGIN
        IF c.units(j-1) EQ c.units(i-1) THEN tit[nn] = 'C!Dl!N ['+c.field(i-1)+' x '+c.field(j-1)+'] ('+STRTRIM(c.units(i-1),2)+')!U2!N' $
        ELSE  tit[nn] = 'C!Dl!N ['+c.field(i-1)+' x '+c.field(j-1)+'] ('+STRTRIM(c.units(i-1),2)+'.'+STRTRIM(c.units(j-1),2)+')'
        ok = EXECUTE('tab[*,nn] = factorl * c.c_'+NUMBER2STRING(i)+'_'+NUMBER2STRING(j))
        IF NOT ok THEN MESSAGE, 'Error accessing the Cl values'
        nn = nn+1
     ENDFOR
  ENDIF ELSE BEGIN
     FOR i=1, nfields DO BEGIN
        tit[i-1] = 'C!Dl!N ['+c.field(i-1)+' x '+c.field(i-1)+'] ('+STRTRIM(c.units(i-1),2)+')!U2!N'
        ok = EXECUTE('tab[*,i-1] = factorl * c.c_'+NUMBER2STRING(i)+'_'+NUMBER2STRING(i))
        IF NOT ok THEN MESSAGE, 'Error accessing the Cl values'
     ENDFOR
  ENDELSE

  IF NOT KEYWORD_SET(autorange) THEN BEGIN
     minabs = MIN(ABS(tab))
     maxabs = MAX(ABS(tab))
  ENDIF

  idlversion = !version.release
  version_numbers = STRSPLIT(idlversion,'.', /extract)
  main_version = LONG(version_numbers[0])

  IF main_version GE 7 THEN BEGIN
     DEVICE, get_decomposed=d
     current_device = !d.name

     IF DEFINED(ps) THEN BEGIN
        SET_PLOT, 'ps'
        dirname = FILE_DIRNAME(ps)
        IF NOT FILE_TEST(dirname,/directory) THEN FILE_MKDIR, dirname
        DEVICE, /color, xsize=20, ysize=20, decomposed=1, filename=ps, /encapsulated
     ENDIF ELSE BEGIN
        WINDOW, /free, xsize=800, ysize=800           
        DEVICE, decomposed=1
        SET_PLOT, 'x'
     ENDELSE

     colors = ['black','brown','orange','red','purple','blue','green','cyan','grey']
     IF ntab EQ 2 THEN colors=['red','blue']
     IF ntab EQ 3 THEN colors=['red','blue','green']
     IF ntab EQ 4 THEN colors=['brown','red','blue','green']
     IF ntab EQ 5 THEN colors=['brown','red','purple','blue','green']
     IF ntab EQ 6 THEN colors=['brown','red','purple','blue','green','cyan']
     IF ntab EQ 7 THEN colors=['brown','orange','red','purple','blue','green','cyan']
     IF ntab EQ 8 THEN colors=['brown','orange','red','purple','blue','green','cyan','grey']
     IF ntab EQ 9 THEN colors=['brown','orange','red','purple','blue','green','cyan','grey','black']
     IF ntab EQ 10 THEN colors=['brown','orange','red','purple','blue','green','cyan','grey','black','pink']
     IF ntab EQ 11 THEN colors=['brown','orange','red','purple','blue','green','cyan','grey','black','pink','yellow']
     ncolors = N_ELEMENTS(colors)

     IF NOT KEYWORD_SET(cmbstyle) THEN ytitle='C!Dl!N' ELSE ytitle='l(l+1)C!Dl!N/2pi'

     ;; Extract info from _extra keywords to set the value of xrange
     IF DEFINED(_EXTRA) THEN BEGIN
        extra_tags = TAG_NAMES(_extra)
        matchlength = LONARR(N_ELEMENTS(extra_tags))
        FOR i=0, N_ELEMENTS(extra_tags)-1 DO BEGIN
           tag_matches_xrange = extra_tags[i] EQ FIRSTCHAR('XRANGE', STRLEN(extra_tags[i])<6)
           IF tag_matches_xrange THEN matchlength[i] = STRLEN(extra_tags[i]) ELSE matchlength[i]=0
        ENDFOR
        len = MAX(matchlength,index)
        IF len NE 0 THEN xrange=_extra.(index) ELSE BEGIN
           matchlength = LONARR(N_ELEMENTS(extra_tags))
           FOR i=0, N_ELEMENTS(extra_tags)-1 DO BEGIN
              tag_matches_xlog = extra_tags[i] EQ FIRSTCHAR('XLOG', STRLEN(extra_tags[i])<4)
              IF tag_matches_xlog THEN matchlength[i] = STRLEN(extra_tags[i]) ELSE matchlength[i]=0
           ENDFOR
           len = MAX(matchlength,index)        
           IF len NE 0 THEN xrange = [1,MAX(c.l)]
        ENDELSE
     ENDIF ELSE BEGIN
     ENDELSE
     PLOT, c.l, tab[*,0], /nodata, _extra=_extra, xrange=xrange, yrange=[minabs,maxabs], xtitle='Harmonic Mode l', ytitle=ytitle, background=GET_COLOR('white'), color=GET_COLOR('black')
     FOR i=0, ntab-1 DO OPLOT, c.l, tab[*,i], color=GET_COLOR(colors[i MOD ncolors]), _extra=_extra
     FOR i=0, ntab-1 DO OPLOT, c.l, -tab[*,i], color=GET_COLOR(colors[i MOD ncolors]), linestyle=2, _extra=_extra
     alli = INDGEN(ntab)
     IF NOT KEYWORD_SET(nolegend) THEN AL_LEGEND, tit, color=GET_COLOR(colors[alli MOD ncolors]), linestyle=0, _extra=_extra, textcolors=GET_COLOR('black')

     PSM_DEBUG_MESSAGE, 'Reset the device parameters in case of error in visu_cl', routine='VISU_CL', infolimit=2, stoplimit=3
     
     IF DEFINED(ps) THEN DEVICE, /close

     IF NOT KEYWORD_SET(ps) THEN ok=EXECUTE('DEVICE, decomposed=d')
     SET_PLOT, current_device

  ENDIF ELSE BEGIN

     PSM_DEBUG_MESSAGE, 'Plot not possible for IDL version older than 7.0', routine='VISU_CL', infolimit=2, stoplimit=3

  ENDELSE

    

END
