; -----------------------------------------------------------------------------
;
; Copyright (C) 2010-2012
;   Guillaume Castex
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	VISU_COMP
;
; PURPOSE:
;	...
;
;-
PRO VISU_COMP, comp, level=level, output=output, psm_output_dir=psm_output_dir, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'VISU_COMP: procedure to visualize a PSM component'
     PSM_INFO_MESSAGE, 'SYNTAX: VISU_COMP, comp, level=, output=, psm_output_dir='
     PSM_INFO_MESSAGE, "comp can be: 'dipole', 'cmb', 'thermalsz', 'kineticsz', 'sz'"
     PSM_INFO_MESSAGE, "'synchrotron', 'galaxy', 'all'"
     PSM_INFO_MESSAGE, "set 'level' to 0, 1 or 2 to bypass the standard level of PSM visualisation"
     PSM_INFO_MESSAGE, "'output' can be 'screen', 'ps' or 'png' (default 'screen')"
     PSM_INFO_MESSAGE, "set psm_output_dir to the PSM output directory if needed (default read from PSM param)"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN
  ENDIF

  COMMON psm, param

  IF NOT DEFINED(param) THEN BEGIN
     IF KEYWORD_SET(psm_output_dir) NE 0 THEN RESTORE_PSM_PARAM, psm_output_dir=psm_output_dir $
     ELSE BEGIN
        MESSAGE, "Path to component not found -- set the 'psm_output_dir' keyword or define PSM parameters", /info
        RETURN
     ENDELSE
  ENDIF ELSE BEGIN
     currentparam = param
     IF KEYWORD_SET(psm_output_dir) NE 0 THEN RESTORE_PSM_PARAM, psm_output_dir=psm_output_dir
  ENDELSE

  IF NOT DEFINED(level) THEN BEGIN
     IF DEFINED(param) THEN level=param.run.visu ELSE level=1
  ENDIF
  
  IF level EQ 0 THEN RETURN

  IF NOT DEFINED(output) THEN BEGIN
     IF DEFINED(param) THEN output_visu = param.run.output_visu ELSE output_visu = 'screen'
  ENDIF ELSE output_visu = output

  CASE comp OF

     ;;------------------------------------------------------------------------------------
     ;; CMB 
     ;;------------------------------------------------------------------------------------

     'cmb': BEGIN

        ;; View CMB map
        ;;--------------
        IF DEFINED(param) THEN cmb_map_file = GETPSMDIR(comp, psm_output_dir = psm_output_dir)+param.names.cmb_map_file
        IF FILE_TEST(cmb_map_file) THEN BEGIN
           ;; Find number of fields
           h = HEADFITS(cmb_map_file,exten=1)
           nmaps = SXPAR(h, 'TFIELDS')
           ;; Make figures
           CASE output_visu OF
              'screen': BEGIN
                 VISU_MAP, cmb_map_file, title = 'CMB anisotropies', subtitle = 'Intensity', /moll
                 VISU_MAP, cmb_map_file, title = 'CMB anisotropies (galactic centre)', subtitle = 'Intensity', /gnom
                 VISU_MAP, cmb_map_file, title = 'CMB anisotropies (NGP)', subtitle = 'Intensity', /gnom, rotate=[0,90]
                 IF nmaps EQ 3 THEN BEGIN
                    VISU_MAP, cmb_map_file, 2, title = 'CMB anisotropies', subtitle = 'Q Polarisation', /moll
                    VISU_MAP, cmb_map_file, 3, title = 'CMB anisotropies', subtitle = 'U Polarisation', /moll
                 ENDIF
              END
              'png': BEGIN
                 dirname = GETPSMDIR('cmb')+'figures/'
                 IF NOT FILE_TEST(dirname,/directory) THEN FILE_MKDIR, dirname
                 VISU_MAP, cmb_map_file, title = 'CMB anisotropies', /moll, png=GETPSMDIR('cmb')+'figures/'+param.names.cmb_basename + '_T_map.png'
                 IF nmaps EQ 3 THEN BEGIN
                    VISU_MAP, cmb_map_file, 2, title = 'CMB anisotropies', subtitle = 'Q Polarisation', /moll, $
                              png=GETPSMDIR('cmb')+'figures/'+param.names.cmb_basename + '_Q_map.png'
                    VISU_MAP, cmb_map_file, 3, title = 'CMB anisotropies', subtitle = 'U Polarisation', /moll, $
                              png=GETPSMDIR('cmb')+'figures/'+param.names.cmb_basename + '_U_map.png'
                 ENDIF
              END
              'ps': BEGIN
                 dirname = GETPSMDIR('cmb')+'figures/'
                 IF NOT FILE_TEST(dirname,/directory) THEN FILE_MKDIR, dirname
                 VISU_MAP, cmb_map_file, title = 'CMB anisotropies', /moll, ps=GETPSMDIR('cmb')+'figures/'+param.names.cmb_basename + '_T_map.ps'
                 IF nmaps EQ 3 THEN BEGIN
                    VISU_MAP, cmb_map_file, 2, title = 'CMB anisotropies', subtitle = 'Q Polarisation', /moll, $
                              ps=GETPSMDIR('cmb')+'figures/'+param.names.cmb_basename + '_Q_map.ps'
                    VISU_MAP, cmb_map_file, 3, title = 'CMB anisotropies', subtitle = 'U Polarisation', /moll, $
                              ps=GETPSMDIR('cmb')+'figures/'+param.names.cmb_basename + '_U_map.ps'
                 ENDIF
                 filename_cl = GETPSMDIR('cmb')+'figures/'+param.names.cmb_basename + '_Cl.ps'
              END
           ENDCASE
        ENDIF ELSE PSM_INFO_MESSAGE, 'No cmb map found'
        
        ;; View theoretical Cl, unlensed
        ;;-------------------------------
        IF DEFINED(param) THEN clfile = GETPSMDIR(comp, psm_output_dir = psm_output_dir)+param.names.cmb_unlensed_cl_file
        IF FILE_TEST(clfile) THEN BEGIN
           READ_CL, clfile, cl
           CASE output_visu OF
              'screen': VISU_CL, cl, title = 'CMB unlensed Cl', thick=2, charsize=1.8, charthick=1, xthick=2, ythick=2
              'png': BEGIN
                 dirname = GETPSMDIR('cmb')+'figures/'
                 IF NOT FILE_TEST(dirname,/directory) THEN FILE_MKDIR, dirname
                 filename_cl = GETPSMDIR('cmb')+'figures/'+param.names.cmb_basename+'_unlensed_Cl.png'
                 filename_cl_ps = GETPSMDIR('cmb')+'figures/'+param.names.cmb_basename + '_unlensed_Cl.png'
                 PLOT_CMB_CL, cl, filename_cl, title = 'CMB unlensed Cl', output='png', thick=2, charsize=1.8, charthick=1, xthick=2, ythick=2
                 SPAWN, 'convert '+filename_cl_ps+' '+filename_cl_png
              END
              'ps': BEGIN
                 dirname = GETPSMDIR('cmb')+'figures/'
                 IF NOT FILE_TEST(dirname,/directory) THEN FILE_MKDIR, dirname
                 filename_cl = GETPSMDIR('cmb')+'figures/'+param.names.cmb_basename + '_unlensed_Cl.ps'
                 PLOT_CMB_CL, cl, filename_cl, title = 'CMB unlensed Cl', output='ps', thick=2, charsize=1.8, charthick=1, xthick=2, ythick=2
                 ;VISU_CL, cl, ps=filename_cl, title = 'CMB unlensed Cl', thick=2, charsize=1.2, charthick=2, xthick=2, ythick=2, /xlog, /ylog, xrange=[1,2*SKY_LMAX()], /cmbstyle
              END
           ENDCASE
        ENDIF ELSE PSM_DEBUG_MESSAGE, 'No unlensed cmb power spectrum found', routine='VISU_COMP'

        ;; View theoretical Cl, lensed
        ;;-------------------------------
        IF DEFINED(param) THEN clfile = GETPSMDIR(comp, psm_output_dir = psm_output_dir)+param.names.cmb_lensed_cl_file
        IF FILE_TEST(clfile) THEN BEGIN
           READ_CL, clfile, cl
           CASE output_visu OF
              'screen': VISU_CL, cl, title = 'CMB lensed Cl', thick=2, charsize=1.8, charthick=1, xthick=2, ythick=2
              'png': BEGIN
                 dirname = GETPSMDIR('cmb')+'figures/'
                 IF NOT FILE_TEST(dirname,/directory) THEN FILE_MKDIR, dirname
                 filename_cl = GETPSMDIR('cmb')+'figures/'+param.names.cmb_basename + '_lensed_Cl.png'
                 filename_cl_ps = GETPSMDIR('cmb')+'figures/'+param.names.cmb_basename + '_lensed_Cl.ps'
                 PLOT_CMB_CL, cl, ps=filename_cl_ps, title = 'CMB lensed Cl', thick=2, charsize=1.8, charthick=1, xthick=2, ythick=2
                 SPAWN, 'convert '+filename_cl_ps+' '+filename_cl_png
              END
              'ps': BEGIN
                 dirname = GETPSMDIR('cmb')+'figures/'
                 IF NOT FILE_TEST(dirname,/directory) THEN FILE_MKDIR, dirname
                 filename_cl = GETPSMDIR('cmb')+'figures/'+param.names.cmb_basename + '_lensed_Cl.ps'
                 PLOT_CMB_CL, cl, filename_cl, title = 'CMB lensed Cl', output='ps', thick=2, charsize=1.8, charthick=1, xthick=2, ythick=2
                 ;VISU_CL, cl, ps=filename_cl, title = 'CMB lensed Cl', thick=2, charsize=1.2, charthick=1, xthick=2, ythick=2, /xlog, /ylog, xrange=[1,2*SKY_LMAX()], /cmbstyle
              END
           ENDCASE
        ENDIF ELSE PSM_DEBUG_MESSAGE, 'No lensed cmb power spectrum found', routine='VISU_COMP'

     END

     ;;------------------------------------------------------------------------------------
     ;; SZ 
     ;;------------------------------------------------------------------------------------

     'thermalsz': BEGIN

        ;; View thermal SZ map
        ;;---------------------
        IF DEFINED(param) THEN mapfile = GETPSMDIR(comp,psm_output_dir = psm_output_dir)+param.names.thermalsz_map_file
        IF FILE_TEST(mapfile) THEN BEGIN
           CASE output_visu OF
              'screen': BEGIN
                 VISU_MAP, mapfile, title = 'Thermal SZ map (full sky view)', /moll, min=0, max=2e-5
                 VISU_MAP, mapfile, title = 'Thermal SZ map', /gnom, rot=[0,90], pxsize=600, pysize=600, reso=.5, $
                           min=0, max=2e-5, subtitle='Five degree patch around North Galactic Pole'
              END
              'png': BEGIN
                 dirname = GETPSMDIR('sz')+'figures/'
                 IF NOT FILE_TEST(dirname,/directory) THEN FILE_MKDIR, dirname
                 VISU_MAP, mapfile, title = 'Thermal SZ map (full sky view)', /moll, png=GETPSMDIR('sz')+'figures/'+param.names.thermalsz_basename + '_mollview.png', min=0, max=2e-5
                 VISU_MAP, mapfile, title = 'Thermal SZ map', /gnom, rot=[0,90], pxsize=600, pysize=600, reso=.5, subtitle='Five degree patch around North Galactic Pole', $
                           min=0, max=2e-5, png=GETPSMDIR('sz')+'figures/'+param.names.thermalsz_basename + '_gnomview.png'
              END
              'ps': BEGIN
                 dirname = GETPSMDIR('sz')+'figures/'
                 IF NOT FILE_TEST(dirname,/directory) THEN FILE_MKDIR, dirname
                 VISU_MAP, mapfile, title = 'Thermal SZ map (full sky view)', /moll, ps=GETPSMDIR('sz')+'figures/'+param.names.thermalsz_basename + '_mollview.ps', min=0, max=2e-5
                 VISU_MAP, mapfile, title = 'Thermal SZ map', /gnom, rot=[0,90], pxsize=600, pysize=600, reso=.5, subtitle='Five degree patch around North Galactic Pole', $
                          min=0, max=2e-5,  ps=GETPSMDIR('sz')+'figures/'+param.names.thermalsz_basename + '_gnomview.ps'
              END
           ENDCASE
        ENDIF ELSE PSM_DEBUG_MESSAGE, 'No thermal SZ map found', routine='VISU_COMP'

        ;; View dN/dMdz
        ;;--------------

     END

     'kineticsz': BEGIN

        ;; View kinetic SZ map
        ;;---------------------
        IF DEFINED(param) THEN mapfile = GETPSMDIR(comp, psm_output_dir = psm_output_dir)+param.names.kineticsz_map_file
        IF FILE_TEST(mapfile) THEN BEGIN
           CASE output_visu OF
              'screen': BEGIN
                 VISU_MAP, mapfile, title = 'Kinetic SZ map (full sky view)', /moll
                 VISU_MAP, mapfile, title = 'Kinetic SZ map', /gnom, rot=[0,90], pxsize=600, pysize=600, reso=.5, $
                           min=-10, max=10, subtitle='Five degree patch around North Galactic Pole'
              END
              'png': BEGIN
                 dirname = GETPSMDIR('sz')+'figures/'
                 IF NOT FILE_TEST(dirname,/directory) THEN FILE_MKDIR, dirname
                 VISU_MAP, mapfile, title = 'Kinetic SZ map (full sky view)', /moll, png=GETPSMDIR('sz')+'figures/'+param.names.kineticsz_basename + '_mollview.png'
                 VISU_MAP, mapfile, title = 'Kinetic SZ map', /gnom, rot=[0,90], pxsize=600, pysize=600, reso=.5, $
                           min=-10, max=10, subtitle='Five degree patch around North Galactic Pole', $
                           png=GETPSMDIR('sz')+'figures/'+param.names.kineticsz_basename + '_gnomview.png'
              END
              'ps': BEGIN
                 dirname = GETPSMDIR('sz')+'figures/'
                 IF NOT FILE_TEST(dirname,/directory) THEN FILE_MKDIR, dirname
                 VISU_MAP, mapfile, title = 'Kinetic SZ map (full sky view)', /moll, ps=GETPSMDIR('sz')+'figures/'+param.names.kineticsz_basename + '_mollview.ps'
                 VISU_MAP, mapfile, title = 'Kinetic SZ map', /gnom, rot=[0,90], pxsize=600, pysize=600, reso=.5, $
                           min=-10, max=10, subtitle='Five degree patch around North Galactic Pole', $
                           ps=GETPSMDIR('sz')+'figures/'+param.names.kineticsz_basename + '_gnomview.ps'
              END
           ENDCASE
        ENDIF ELSE PSM_INFO_MESSAGE, 'No kinetic SZ map found'
        
        ;; view velocity v(z)
        ;;--------------------

     END

     'sz': BEGIN

        ;; View both components (thermal and kinetic)
        ;;--------------------------------------------
        VISU_COMP, 'thermalsz', psm_output_dir=psm_output_dir
        VISU_COMP, 'kineticsz', psm_output_dir=psm_output_dir

     END

     ;;------------------------------------------------------------------------------------
     ;; GALAXY 
     ;;------------------------------------------------------------------------------------
     
     'synchrotron': BEGIN

        ;; View amplitude
        ;;----------------
        mapfile = GETPSMDIR(comp,psm_output_dir = psm_output_dir)+'synchrotron_ampl.fits'
        IF FILE_TEST(mapfile) THEN BEGIN

           ;; Find number of fields
           h = HEADFITS(mapfile,exten=1)
           nmaps = SXPAR(h, 'TFIELDS')

           ;; Visu of maps
           CASE output_visu OF
              'screen': BEGIN
                 VISU_MAP, mapfile, title = 'Synchrotron map - 23 GHz', subtitle = 'Intensity', /moll, /hist_equal
                 VISU_MAP, mapfile, title = 'Synchrotron map - 23 GHz', subtitle = 'Intensity', /moll
                 VISU_MAP, mapfile, title = 'Synchrotron map - 23 GHz (galactic centre)', subtitle = 'Intensity', /gnom
                 IF nmaps EQ 3 THEN BEGIN
                    ;; View Q and U
                    VISU_MAP, mapfile, 2, title = 'Synchrotron map - 23 GHz', subtitle = 'Q Polarisation', /moll
                    VISU_MAP, mapfile, 3, title = 'Synchrotron map - 23 GHz', subtitle = 'U Polarisation', /moll
                    VISU_MAP, mapfile, 2, title = 'Synchrotron map - 23 GHz (galactic centre)', subtitle = 'Q Polarisation', /gnom
                    VISU_MAP, mapfile, 3, title = 'Synchrotron map - 23 GHz (galactic centre)', subtitle = 'U Polarisation', /gnom
                    ;; View polarisation degree
                    READ_TQU, mapfile, tqu
                    polardeg = SQRT((tqu[*,1]^2 + tqu[*,2]^2)/tqu[*,0]^2) * 100
                    aver = AVG(polardeg)
                    sbtit = 'average value = '+NUMBER2STRING(aver)+' %'
                    VISU_MAP, polardeg, title = 'Synchrotron polarisation fraction', subtitle = sbtit, /moll, units='%'
                 ENDIF
              END
              'png': BEGIN
                 dirname = GETPSMDIR('synchrotron')+'figures/'
                 IF NOT FILE_TEST(dirname,/directory) THEN FILE_MKDIR, dirname
                 VISU_MAP, mapfile, title = 'Synchrotron map - 23 GHz', subtitle = 'Intensity', /moll, /hist_equal, $
                           png=GETPSMDIR('synchrotron')+'figures/'+'synchrotron_T_map_equalised.png'
                 VISU_MAP, mapfile, title = 'Synchrotron map - 23 GHz', subtitle = 'Intensity', /moll, $
                           png=GETPSMDIR('synchrotron')+'synchrotron_T_map.png'
                 VISU_MAP, mapfile, title = 'Synchrotron map - 23 GHz (galactic centre)', subtitle = 'Intensity', /gnom, $
                           png=GETPSMDIR('synchrotron')+'figures/'+'synchrotron_T_map_GC.png'
                 IF nmaps EQ 3 THEN BEGIN
                    ;; View Q and U
                    VISU_MAP, mapfile, 2, title = 'Synchrotron map - 23 GHz', subtitle = 'Q Polarisation', /moll, $
                              png=GETPSMDIR('synchrotron')+'figures/'+'synchrotron_Q_map.png'
                    VISU_MAP, mapfile, 3, title = 'Synchrotron map - 23 GHz', subtitle = 'U Polarisation', /moll, $
                              png=GETPSMDIR('synchrotron')+'figures/'+'synchrotron_U_map.png'
                    VISU_MAP, mapfile, 2, title = 'Synchrotron map - 23 GHz (galactic centre)', subtitle = 'Q Polarisation', /gnom, $
                              png=GETPSMDIR('synchrotron')+'synchrotron_Q_map_GC.png'
                    VISU_MAP, mapfile, 3, title = 'Synchrotron map - 23 GHz (galactic centre)', subtitle = 'U Polarisation', /gnom, $
                              png=GETPSMDIR('synchrotron')+'synchrotron_U_map_GC.png'
                    ;; View polarisation degree
                    READ_TQU, mapfile, tqu
                    polardeg = SQRT((tqu[*,1]^2 + tqu[*,2]^2)/tqu[*,0]^2) * 100
                    aver = AVG(polardeg)
                    sbtit = 'average value = '+NUMBER2STRING(aver)+' %'
                    VISU_MAP, polardeg, title = 'Synchrotron polarisation fraction', subtitle = sbtit, /moll, units='%', $
                              png=GETPSMDIR('visu')+'figures/'+'synchrotron_polar_fraction_map.png'
                 ENDIF
              END
              'ps': BEGIN
                 dirname = GETPSMDIR('synchrotron')+'figures/'
                 IF NOT FILE_TEST(dirname,/directory) THEN FILE_MKDIR, dirname
                 VISU_MAP, mapfile, title = 'Synchrotron map - 23 GHz', subtitle = 'Intensity', /moll, /hist_equal, $
                           ps=GETPSMDIR('synchrotron')+'figures/'+'synchrotron_T_map_equalised.ps'
                 VISU_MAP, mapfile, title = 'Synchrotron map - 23 GHz', subtitle = 'Intensity', /moll, $
                           ps=GETPSMDIR('synchrotron')+'figures/'+'synchrotron_T_map.ps'
                 VISU_MAP, mapfile, title = 'Synchrotron map - 23 GHz (galactic centre)', subtitle = 'Intensity', /gnom, $
                           ps=GETPSMDIR('synchrotron')+'figures/'+'synchrotron_T_map_GC.ps'
                 IF nmaps EQ 3 THEN BEGIN
                    ;; View Q and U
                    VISU_MAP, mapfile, 2, title = 'Synchrotron map - 23 GHz', subtitle = 'Q Polarisation', /moll, $
                              ps=GETPSMDIR('synchrotron')+'figures/'+'synchrotron_Q_map.ps'
                    VISU_MAP, mapfile, 3, title = 'Synchrotron map - 23 GHz', subtitle = 'U Polarisation', /moll, $
                              ps=GETPSMDIR('synchrotron')+'figures/'+'synchrotron_U_map.ps'
                   VISU_MAP, mapfile, 2, title = 'Synchrotron map - 23 GHz (galactic centre)', subtitle = 'Q Polarisation', /gnom, $
                             ps=GETPSMDIR('synchrotron')+'figures/'+'synchrotron_Q_map_GC.ps'
                    VISU_MAP, mapfile, 3, title = 'Synchrotron map - 23 GHz (galactic centre)', subtitle = 'U Polarisation', /gnom, $
                              ps=GETPSMDIR('synchrotron')+'figures/'+'synchrotron_U_map_GC.ps'
                    ;; View polarisation degree
                    READ_TQU, mapfile, tqu
                    polardeg = SQRT((tqu[*,1]^2 + tqu[*,2]^2)/tqu[*,0]^2) * 100
                    aver = AVG(polardeg)
                    sbtit = 'average value = '+NUMBER2STRING(aver)+' %'
                    VISU_MAP, polardeg, title = 'Synchrotron polarisation fraction', subtitle = sbtit, /moll, units='%', $
                              ps=GETPSMDIR('synchrotron')+'figures/'+'synchrotron_polar_fraction_map.ps'
                 ENDIF
              END
           ENDCASE
        ENDIF ELSE PSM_INFO_MESSAGE, 'No synchrotron emission map found'
           
        ;; View spectral index
        ;;---------------------
        mapfile = GETPSMDIR(comp,psm_output_dir = psm_output_dir)+'synchrotron_specind.fits'
        IF FILE_TEST(mapfile) THEN BEGIN

           ;; Find number of fields
           h = HEADFITS(mapfile,exten=1)
           nmaps = SXPAR(h, 'TFIELDS')

           ;; Visu of maps
           CASE output_visu OF
              'screen': BEGIN
                 VISU_MAP, mapfile, title = 'Synchrotron spectral index', subtitle = 'Spectral index', /moll
                 VISU_MAP, mapfile, title = 'Synchrotron spectral index (galactic centre)', subtitle = 'Spectral index', /gnom
                 IF nmaps EQ 3 THEN BEGIN
                    ;; View Q and U
                    VISU_MAP, mapfile, 2, title = 'Synchrotron spectral index', subtitle = 'Q Spectral index', /moll
                    VISU_MAP, mapfile, 3, title = 'Synchrotron spectral index', subtitle = 'U Spectral index', /moll
                    VISU_MAP, mapfile, 2, title = 'Synchrotron spectral index (galactic centre)', subtitle = 'Q Spectral index', /gnom
                    VISU_MAP, mapfile, 3, title = 'Synchrotron spectral index (galactic centre)', subtitle = 'U Spectral index', /gnom
                 ENDIF
              END
              'png': BEGIN
                 dirname = GETPSMDIR('synchrotron')+'figures/'
                 IF NOT FILE_TEST(dirname,/directory) THEN FILE_MKDIR, dirname
                 VISU_MAP, mapfile, title = 'Synchrotron spectral index', subtitle = 'Spectral index', /moll, $
                           png=GETPSMDIR('synchrotron')+'figures/'+'synchrotron_T_specind_map.png'
                 VISU_MAP, mapfile, title = 'Synchrotron spectral index (galactic centre)', subtitle = 'Spectral index', /gnom, $
                           png=GETPSMDIR('synchrotron')+'figures/'+'synchrotron_T_specind_map_GC.png'
                 IF nmaps EQ 3 THEN BEGIN
                    ;; View Q and U
                    VISU_MAP, mapfile, 2, title = 'Synchrotron spectral index', subtitle = 'Q Polarisation', /moll, $
                              png=GETPSMDIR('synchrotron')+'figures/'+'synchrotron_Q_specind_map.png'
                    VISU_MAP, mapfile, 3, title = 'Synchrotron spectral index', subtitle = 'U Polarisation', /moll, $
                              png=GETPSMDIR('synchrotron')+'figures/'+'synchrotron_U_specind_map.png'
                    VISU_MAP, mapfile, 2, title = 'Synchrotron spectral index (galactic centre)', subtitle = 'Q Polarisation', /gnom, $
                              png=GETPSMDIR('synchrotron')+'figures/'+'synchrotron_Q_specind_map_GC.png'
                    VISU_MAP, mapfile, 3, title = 'Synchrotron spectral index (galactic centre)', subtitle = 'U Polarisation', /gnom, $
                              png=GETPSMDIR('synchrotron')+'figures/'+'synchrotron_U_specind_map_GC.png'
                  ENDIF
              END
              'ps': BEGIN
                 dirname = GETPSMDIR('synchrotron')+'figures/'
                 IF NOT FILE_TEST(dirname,/directory) THEN FILE_MKDIR, dirname
                 VISU_MAP, mapfile, title = 'Synchrotron spectral index', subtitle = 'Intensity', /moll, $
                           ps=GETPSMDIR('synchrotron')+'figures/'+'synchrotron_T_specind_map.ps'
                 VISU_MAP, mapfile, title = 'Synchrotron spectral index (galactic centre)', subtitle = 'Intensity', /gnom, $
                           ps=GETPSMDIR('synchrotron')+'figures/'+'synchrotron_T_specind_map_GC.ps'
                 IF nmaps EQ 3 THEN BEGIN
                    ;; View Q and U
                    VISU_MAP, mapfile, 2, title = 'Synchrotron spectral index', subtitle = 'Q Polarisation', /moll, $
                              ps=GETPSMDIR('synchrotron')+'figures/'+'synchrotron_Q_specind_map.ps'
                    VISU_MAP, mapfile, 3, title = 'Synchrotron spectral index', subtitle = 'U Polarisation', /moll, $
                              ps=GETPSMDIR('synchrotron')+'figures/'+'synchrotron_U_specind_map.ps'
                    VISU_MAP, mapfile, 2, title = 'Synchrotron spectral index (galactic centre)', subtitle = 'Q Polarisation', /gnom, $
                              ps=GETPSMDIR('synchrotron')+'figures/'+'synchrotron_Q_specind_map_GC.ps'
                    VISU_MAP, mapfile, 3, title = 'Synchrotron spectral index (galactic centre)', subtitle = 'U Polarisation', /gnom, $
                              ps=GETPSMDIR('synchrotron')+'figures/'+'synchrotron_U_specind_map_GC.ps'
                 ENDIF
              END
           ENDCASE
        ENDIF ELSE PSM_INFO_MESSAGE, 'No synchrotron spectral index map found'

     END

     'freefree': BEGIN

        ;; View amplitude
        ;;----------------
        mapfile = GETPSMDIR(comp,psm_output_dir = psm_output_dir)+'freefree_ampl.fits'
        IF FILE_TEST(mapfile) THEN BEGIN

           ;; Find number of fields
           h = HEADFITS(mapfile,exten=1)
           nmaps = SXPAR(h, 'TFIELDS')

           ;; Visu of maps
           CASE output_visu OF
              'screen': BEGIN
                 VISU_MAP, mapfile, title = 'Free-free map - 23 GHz', subtitle = 'Intensity', /moll, /hist_equal
                 VISU_MAP, mapfile, title = 'Free-free map - 23 GHz', subtitle = 'Intensity', /moll
                 VISU_MAP, mapfile, title = 'Free-free map - 23 GHz (galactic centre)', subtitle = 'Intensity', /gnom
                 IF nmaps EQ 3 THEN BEGIN
                    ;; View Q and U
                    VISU_MAP, mapfile, 2, title = 'Free-free map - 23 GHz', subtitle = 'Q Polarisation', /moll
                    VISU_MAP, mapfile, 3, title = 'Free-free map - 23 GHz', subtitle = 'U Polarisation', /moll
                    VISU_MAP, mapfile, 2, title = 'Free-free map - 23 GHz (galactic centre)', subtitle = 'Q Polarisation', /gnom
                    VISU_MAP, mapfile, 3, title = 'Free-free map - 23 GHz (galactic centre)', subtitle = 'U Polarisation', /gnom
                    ;; View polarisation degree
                    READ_TQU, mapfile, tqu
                    polardeg = SQRT((tqu[*,1]^2 + tqu[*,2]^2)/tqu[*,0]^2) * 100
                    aver = AVG(polardeg)
                    sbtit = 'average value = '+NUMBER2STRING(aver)+' %'
                    VISU_MAP, polardeg, title = 'Free-free polarisation fraction', subtitle = sbtit, /moll, units='%'
                 ENDIF
              END
              'png': BEGIN
                 dirname = GETPSMDIR('freefree')+'figures/'
                 IF NOT FILE_TEST(dirname,/directory) THEN FILE_MKDIR, dirname
                 VISU_MAP, mapfile, title = 'Free-free map - 23 GHz', subtitle = 'Intensity', /moll, /hist_equal, png=dirname+'freefree_T_map_equalised.png'
                 VISU_MAP, mapfile, title = 'Free-free map - 23 GHz', subtitle = 'Intensity', /moll, png=dirname+'freefree_T_map.png'
                 VISU_MAP, mapfile, title = 'Free-free map - 23 GHz (galactic centre)', subtitle = 'Intensity', /gnom, png=dirname+'freefree_T_map_GC.png'
                 IF nmaps EQ 3 THEN BEGIN
                    ;; View Q and U
                    VISU_MAP, mapfile, 2, title = 'Free-free map - 23 GHz', subtitle = 'Q Polarisation', /moll, png=dirname+'freefree_Q_map.png'
                    VISU_MAP, mapfile, 3, title = 'Free-free map - 23 GHz', subtitle = 'U Polarisation', /moll, png=dirname+'freefree_U_map.png'
                   VISU_MAP, mapfile, 2, title = 'Free-free map - 23 GHz (galactic centre)', subtitle = 'Q Polarisation', /gnom, png=dirname+'freefree_Q_map_GC.png'
                    VISU_MAP, mapfile, 3, title = 'Free-free map - 23 GHz (galactic centre)', subtitle = 'U Polarisation', /gnom, png=dirname+'freefree_U_map_GC.png'
                    ;; View polarisation degree
                    READ_TQU, mapfile, tqu
                    polardeg = SQRT((tqu[*,1]^2 + tqu[*,2]^2)/tqu[*,0]^2) * 100
                    aver = AVG(polardeg)
                    sbtit = 'average value = '+NUMBER2STRING(aver)+' %'
                    VISU_MAP, polardeg, title = 'Free-free polarisation fraction', subtitle = sbtit, /moll, units='%', png=dirname+'freefree_polar_fraction_map.png'
                 ENDIF
              END
              'ps': BEGIN
                 dirname = GETPSMDIR('freefree')+'figures/'
                 IF NOT FILE_TEST(dirname,/directory) THEN FILE_MKDIR, dirname
                 VISU_MAP, mapfile, title = 'Free-free map - 23 GHz', subtitle = 'Intensity', /moll, /hist_equal, ps=dirname+'freefree_T_map_equalised.ps'
                 VISU_MAP, mapfile, title = 'Free-free map - 23 GHz', subtitle = 'Intensity', /moll, ps=dirname+'freefree_T_map.ps'
                 VISU_MAP, mapfile, title = 'Free-free map - 23 GHz (galactic centre)', subtitle = 'Intensity', /gnom, ps=dirname+'freefree_T_map_GC.ps'
                 IF nmaps EQ 3 THEN BEGIN
                    ;; View Q and U
                    VISU_MAP, mapfile, 2, title = 'Free-free map - 23 GHz', subtitle = 'Q Polarisation', /moll, ps=dirname+'freefree_Q_map.ps'
                    VISU_MAP, mapfile, 3, title = 'Free-free map - 23 GHz', subtitle = 'U Polarisation', /moll, ps=dirname+'freefree_U_map.ps'
                    VISU_MAP, mapfile, 2, title = 'Free-free map - 23 GHz (galactic centre)', subtitle = 'Q Polarisation', /gnom, ps=dirname+'freefree_Q_map_GC.ps'
                    VISU_MAP, mapfile, 3, title = 'Free-free map - 23 GHz (galactic centre)', subtitle = 'U Polarisation', /gnom, ps=dirname+'freefree_U_map_GC.ps'
                    ;; View polarisation degree
                    READ_TQU, mapfile, tqu
                    polardeg = SQRT((tqu[*,1]^2 + tqu[*,2]^2)/tqu[*,0]^2) * 100
                    aver = AVG(polardeg)
                    sbtit = 'average value = '+NUMBER2STRING(aver)+' %'
                    VISU_MAP, polardeg, title = 'Free-free polarisation fraction', subtitle = sbtit, /moll, units='%', ps=dirname+'freefree_polar_fraction_map.ps'
                 ENDIF
              END
           ENDCASE
        ENDIF ELSE PSM_INFO_MESSAGE, 'No free-free emission map found'
                     
     END

     'spindust': BEGIN
        ;; View amplitude
        ;;----------------
        mapfile = GETPSMDIR(comp,psm_output_dir = psm_output_dir)+'spindust_ampl.fits'
        IF FILE_TEST(mapfile) THEN BEGIN

           ;; Find number of fields
           h = HEADFITS(mapfile,exten=1)
           nmaps = SXPAR(h, 'TFIELDS')

           ;; Visu of maps
           CASE output_visu OF
              'screen': BEGIN
                 VISU_MAP, mapfile, title = 'Spinning dust map - 23 GHz', subtitle = 'Intensity', /moll, /hist_equal
                 VISU_MAP, mapfile, title = 'Spinning dust map - 23 GHz', subtitle = 'Intensity', /moll
                 VISU_MAP, mapfile, title = 'Spinning dust map - 23 GHz (galactic centre)', subtitle = 'Intensity', /gnom
                 IF nmaps EQ 3 THEN BEGIN
                    ;; View Q and U
                    VISU_MAP, mapfile, 2, title = 'Spinning dust map - 23 GHz', subtitle = 'Q Polarisation', /moll
                    VISU_MAP, mapfile, 3, title = 'Spinning dust map - 23 GHz', subtitle = 'U Polarisation', /moll
                    VISU_MAP, mapfile, 2, title = 'Spinning dust map - 23 GHz (galactic centre)', subtitle = 'Q Polarisation', /gnom
                    VISU_MAP, mapfile, 3, title = 'Spinning dust map - 23 GHz (galactic centre)', subtitle = 'U Polarisation', /gnom
                    ;; View polarisation degree
                    READ_TQU, mapfile, tqu
                    polardeg = SQRT((tqu[*,1]^2 + tqu[*,2]^2)/tqu[*,0]^2) * 100
                    aver = AVG(polardeg)
                    sbtit = 'average value = '+NUMBER2STRING(aver)+' %'
                    VISU_MAP, polardeg, title = 'Spinning dust polarisation fraction', subtitle = sbtit, /moll, units='%'
                 ENDIF
              END
              'png': BEGIN
                 dirname = GETPSMDIR('spindust')+'figures/'
                 IF NOT FILE_TEST(dirname,/directory) THEN FILE_MKDIR, dirname
                 VISU_MAP, mapfile, title = 'Spinning dust map - 23 GHz', subtitle = 'Intensity', /moll, /hist_equal, png=dirname+'spindust_T_map_equalised.png'
                 VISU_MAP, mapfile, title = 'Spinning dust map - 23 GHz', subtitle = 'Intensity', /moll, png=dirname+'spindust_T_map.png'
                 VISU_MAP, mapfile, title = 'Spinning dust map - 23 GHz (galactic centre)', subtitle = 'Intensity', /gnom, png=dirname+'spindust_T_map_GC.png'
                 IF nmaps EQ 3 THEN BEGIN
                    ;; View Q and U
                    VISU_MAP, mapfile, 2, title = 'Spinning dust map - 23 GHz', subtitle = 'Q Polarisation', /moll, png=dirname+'spindust_Q_map.png'
                    VISU_MAP, mapfile, 3, title = 'Spinning dust map - 23 GHz', subtitle = 'U Polarisation', /moll, png=dirname+'spindust_U_map.png'
                   VISU_MAP, mapfile, 2, title = 'Spinning dust map - 23 GHz (galactic centre)', subtitle = 'Q Polarisation', /gnom, png=dirname+'spindust_Q_map_GC.png'
                    VISU_MAP, mapfile, 3, title = 'Spinning dust map - 23 GHz (galactic centre)', subtitle = 'U Polarisation', /gnom, png=dirname+'spindust_U_map_GC.png'
                    ;; View polarisation degree
                    READ_TQU, mapfile, tqu
                    polardeg = SQRT((tqu[*,1]^2 + tqu[*,2]^2)/tqu[*,0]^2) * 100
                    aver = AVG(polardeg)
                    sbtit = 'average value = '+NUMBER2STRING(aver)+' %'
                    VISU_MAP, polardeg, title = 'Spinning dust polarisation fraction', subtitle = sbtit, /moll, units='%', png=dirname+'spindust_polar_fraction_map.png'
                 ENDIF
              END
              'ps': BEGIN
                 dirname = GETPSMDIR('spindust')+'figures/'
                 IF NOT FILE_TEST(dirname,/directory) THEN FILE_MKDIR, dirname
                 VISU_MAP, mapfile, title = 'Spinning dust map - 23 GHz', subtitle = 'Intensity', /moll, /hist_equal, ps=dirname+'spindust_T_map_equalised.ps'
                 VISU_MAP, mapfile, title = 'Spinning dust map - 23 GHz', subtitle = 'Intensity', /moll, ps=dirname+'spindust_T_map.ps'
                 VISU_MAP, mapfile, title = 'Spinning dust map - 23 GHz (galactic centre)', subtitle = 'Intensity', /gnom, ps=dirname+'spindust_T_map_GC.ps'
                 IF nmaps EQ 3 THEN BEGIN
                    ;; View Q and U
                    VISU_MAP, mapfile, 2, title = 'Spinning dust map - 23 GHz', subtitle = 'Q Polarisation', /moll, ps=dirname+'spindust_Q_map.ps'
                    VISU_MAP, mapfile, 3, title = 'Spinning dust map - 23 GHz', subtitle = 'U Polarisation', /moll, ps=dirname+'spindust_U_map.ps'
                    VISU_MAP, mapfile, 2, title = 'Spinning dust map - 23 GHz (galactic centre)', subtitle = 'Q Polarisation', /gnom, ps=dirname+'spindust_Q_map_GC.ps'
                    VISU_MAP, mapfile, 3, title = 'Spinning dust map - 23 GHz (galactic centre)', subtitle = 'U Polarisation', /gnom, ps=dirname+'spindust_U_map_GC.ps'
                    ;; View polarisation degree
                    READ_TQU, mapfile, tqu
                    polardeg = SQRT((tqu[*,1]^2 + tqu[*,2]^2)/tqu[*,0]^2) * 100
                    aver = AVG(polardeg)
                    sbtit = 'average value = '+NUMBER2STRING(aver)+' %'
                    VISU_MAP, polardeg, title = 'Spinning dust polarisation fraction', subtitle = sbtit, /moll, units='%', ps=dirname+'spindust_polar_fraction_map.ps'
                 ENDIF
              END
           ENDCASE
        ENDIF ELSE PSM_INFO_MESSAGE, 'No Spinning dust emission map found'
                     
     END

     'galaxy': BEGIN
        VISU_COMP, 'synchrotron', level=level, output=output, psm_output_dir=psm_output_dir
        VISU_COMP, 'freefree', level=level, output=output, psm_output_dir=psm_output_dir
        VISU_COMP, 'spindust', level=level, output=output, psm_output_dir=psm_output_dir
     END


     ;;------------------------------------------------------------------------------------
     ;; POINT SOURCES 
     ;;------------------------------------------------------------------------------------
     
     'radiops': BEGIN

        ;; At 100 GHz, 1 THz
        ;; Compute flux of all sources
        ;; Make maps of source counts for flux > 10mJy, 100mJy, 1Jy
        catfile_strong = GETPSMDIR('ps') + 'strong_radiops.sav'
        catfile_faint= FIX_SEPARATOR(GETPSMDIR('ps')) + 'faint_radiops.sav'      
    

        IF FILE_TEST(catfile_strong) AND FILE_TEST(catfile_faint) THEN BEGIN
           nside=512
           map=dblarr(nside2npix(nside))
           
           ;; Make maps
           pointer_map = PTR_NEW(map,/no_copy)           


           READ_CATALOG, catfile_strong, cat_strong
           READ_CATALOG, catfile_faint, cat_faint   

           ;; Define bands
     
           freq = [100., 1000.]
           nfreq= N_elements(freq)
           threshold= [10, 100, 1000]


           FOR i=0, nfreq-1 DO BEGIN

              band = GET_BAND_STRUCT('DIRAC', nu_c=freq[i]*1e9)

              
              OBSERVE_MODEL_CAT, cat_strong, band, result_s
              OBSERVE_MODEL_CAT, cat_faint, band, result_f       

              FOR j=0, N_ELEMENTS(threshold)-1 DO BEGIN
                 ;; Clean map
                 (*pointer_map)[*]=0

                 ;; strong sources
                 wh=WHERE(result_s[*,0] GE threshold[j])
                 IF wh[0] NE -1 THEN print_pix_sources, nside, pointer_map, (*cat_strong.lat.value)[wh],(*cat_strong.lon.value)[wh]
                 ;; faint sources
                 wh=WHERE(result_f[*,0] GE threshold[j])
                 IF wh[0] NE -1 THEN print_pix_sources, nside, pointer_map, (*cat_faint.lat.value)[wh],(*cat_faint.lon.value)[wh]
                 
                 ;; Visu of maps
                 CASE output_visu OF
                    'screen': BEGIN
                        VISU_MAP, *pointer_map, title = 'Radio point sources map - '+ number2string(freq[i]) + ' GHz - Flux > ' + number2string(threshold[j]) + ' mJy', subtitle = '', /moll
                       
                    END
                    'png': BEGIN
                        VISU_MAP, *pointer_map, title = 'Radio point sources map - '+ number2string(freq[i]) + ' GHz - Flux > ' + number2string(threshold[j]) + ' mJy', subtitle = '', /moll, $
                                  png=GETPSMDIR('visu') + 'radiops_distribution'+ number2string(freq[i]) + '_GHz_' + number2string(threshold[j]) + '_mJy.png'
                    END
                    'ps': BEGIN
                       VISU_MAP, *pointer_map, title = 'Radio point sources map - '+ number2string(freq[i]) + ' GHz - Flux > ' + number2string(threshold[j]) + ' mJy', subtitle = '', /moll, $
                                 ps=GETPSMDIR('visu') + 'radiops_distribution'+ number2string(freq[i]) + '_GHz_' + number2string(threshold[j]) + '_mJy.ps'
                    END
                 ENDCASE

              ENDFOR
           ENDFOR
           ;; Destroy map
           ptr_free, pointer_map
        ENDIF



     END
     'irps': BEGIN

        ;; At 100 GHz, 1 THz
        ;; Compute flux of all sources
        ;; Make maps of source counts for flux > 10mJy, 100mJy, 1Jy
        catfile_strong = GETPSMDIR('ps') + 'strong_irps.sav'
        catfile_faint= FIX_SEPARATOR(GETPSMDIR('ps')) + 'faint_irps.sav'      
    

        IF FILE_TEST(catfile_strong) AND FILE_TEST(catfile_faint) THEN BEGIN
           nside=512
           map=dblarr(nside2npix(nside))
           
           ;; Make maps
           pointer_map = PTR_NEW(map,/no_copy)           


           READ_CATALOG, catfile_strong, cat_strong
           READ_CATALOG, catfile_faint, cat_faint   

           ;; Define bands
     
           freq = [100., 1000.]
           nfreq= N_elements(freq)
           threshold= [10, 100, 1000]


           FOR i=0, nfreq-1 DO BEGIN

              band = GET_BAND_STRUCT('DIRAC', nu_c=freq[i]*1e9)

              
              OBSERVE_MODEL_CAT, cat_strong, band, result_s
              OBSERVE_MODEL_CAT, cat_faint, band, result_f       

              FOR j=0, N_ELEMENTS(threshold)-1 DO BEGIN
                 ;; Clean map
                 (*pointer_map)[*]=0

                 ;; strong sources
                 wh=WHERE(result_s[*,0] GE threshold[j])
                 IF wh[0] NE -1 THEN print_pix_sources, nside, pointer_map, (*cat_strong.lat.value)[wh],(*cat_strong.lon.value)[wh]
                 ;; faint sources
                 wh=WHERE(result_f[*,0] GE threshold[j])
                 IF wh[0] NE -1 THEN print_pix_sources, nside, pointer_map, (*cat_faint.lat.value)[wh],(*cat_faint.lon.value)[wh]
                 
                 ;; Visu of maps
                 CASE output_visu OF
                    'screen': BEGIN
                        VISU_MAP, *pointer_map, title = 'IR point sources map - '+ number2string(freq[i]) + ' GHz - Flux > ' + number2string(threshold[j]) + ' mJy', subtitle = '', /moll
                       
                    END
                    'png': BEGIN
                        VISU_MAP, *pointer_map, title = 'IR point sources map - '+ number2string(freq[i]) + ' GHz - Flux > ' + number2string(threshold[j]) + ' mJy', $
                                  subtitle = '', /moll, png=GETPSMDIR('visu') + 'irps_distribution'+ number2string(freq[i]) + '_GHz_' + number2string(threshold[j]) + '_mJy.png'
                    END
                    'ps': BEGIN
                       VISU_MAP, *pointer_map, title = 'IR point sources map - '+ number2string(freq[i]) + ' GHz - Flux > ' + number2string(threshold[j]) + ' mJy', $
                                 subtitle = '', /moll, ps=GETPSMDIR('visu') + 'irps_distribution'+ number2string(freq[i]) + '_GHz_' + number2string(threshold[j]) + '_mJy.ps'
                    END
                 ENDCASE

              ENDFOR
           ENDFOR
           ;; Destroy map
           ptr_free, pointer_map
        ENDIF
     END
     'ps': BEGIN
        VISU_COMP, 'radiops', level=level, output=output, psm_output_dir=psm_output_dir
        VISU_COMP, 'irps', level=level, output=output, psm_output_dir=psm_output_dir

        ;; Plot number of sources as a function of flux
        ;; At 10, 100 GHz and 1 THz



        radio_catfile_strong = GETPSMDIR('ps') + 'strong_radiops.sav'
        radio_catfile_faint= FIX_SEPARATOR(GETPSMDIR('ps')) + 'faint_radiops.sav'      
  
        ir_catfile_strong = GETPSMDIR('ps') + 'strong_irps.sav'
        ir_catfile_faint= FIX_SEPARATOR(GETPSMDIR('ps')) + 'faint_irps.sav'      


           READ_CATALOG, radio_catfile_strong, radio_cat_strong
           READ_CATALOG, radio_catfile_faint, radio_cat_faint   
           READ_CATALOG, ir_catfile_strong, ir_cat_strong
           READ_CATALOG, ir_catfile_faint, ir_cat_faint   


        freq = [10,100., 1000.]
        nfreq= N_elements(freq)


        DEVICE, get_decomposed=d
        current_device = !d.name
        
        CASE output OF
           'screen': BEGIN
              WINDOW, /free, xsize=600, ysize=600           
              DEVICE, decomposed=1
              SET_PLOT, 'x'
           END
           'png': BEGIN
              WINDOW, /free, xsize=600, ysize=600           
              DEVICE, decomposed=1
              SET_PLOT, 'x'
           END
           'ps':BEGIN
              SET_PLOT, 'ps'
              DEVICE, /color, filename=GETPSMDIR('visu') + 'logn_logs_' + number2string(freq[i]) + '_GHz.ps'
           END
        ENDCASE


        FOR i=0, nfreq-1 DO BEGIN
           
           band = GET_BAND_STRUCT('DIRAC', nu_c=freq[i]*1e9)
           
           
           OBSERVE_MODEL_CAT, radio_cat_strong, band, result_rad_s
           OBSERVE_MODEL_CAT, radio_cat_faint, band, result_rad_f
           OBSERVE_MODEL_CAT, ir_cat_strong, band, result_ir_s
           OBSERVE_MODEL_CAT, ir_cat_faint, band, result_ir_f           

           fluxes=[result_rad_s,result_rad_f,result_ir_s,result_ir_f]
           LOGN_LOGS, fluxes, title='Number of sources as a flux function - ' + number2string(freq[i]) + ' GHz'

        ENDFOR


        CASE output OF
           'png': BEGIN
              DEVICE, /close
              SET_PLOT, 'x'
           END
           'ps':BEGIN
               WRITE_PNG, GETPSMDIR('visu') + 'logn_logs_' + number2string(freq[i]) + '_GHz.png', TVRD(/true)
           END
        ENDCASE
     END

     'all': BEGIN
        ;VISU_COMP, 'dipole', level=level, output=output, psm_output_dir=psm_output_dir
        VISU_COMP, 'cmb', level=level, output=output, psm_output_dir=psm_output_dir
        VISU_COMP, 'sz', level=level, output=output, psm_output_dir=psm_output_dir
        VISU_COMP, 'galaxy', level=level, output=output, psm_output_dir=psm_output_dir
     END

     ELSE: MESSAGE, 'Visualisation not implemented for component '+comp





  ENDCASE




  PSM_DEBUG_MESSAGE, 'Make sure that param is also restored upon an error in the execution of visu_comp', routine='VISU_COMP', infolimit=2, stoplimit=3
  IF DEFINED(currentparam) THEN param=currentparam

END
