; -----------------------------------------------------------------------------
;
; Copyright (C) 2010-2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	PLOT_ELL_BANDS
;
; PURPOSE:
;	This function plots harmonic bands as used for wavelet
;	(needlet) transforms in the PSM.
;
; CATEGORY:
;	Wavelet (needlet) processing.
;
; CALLING SEQUENCE:
;	PLOT_ELL_BANDS, Bands
;
; INPUTS:
;	Bands:	An arrray of dimension [N_ell, N_bands] that defines
;	        N_bands harmonic bands.
;
; KEYWORD PARAMETERS:
;	None, but the routine accepts all keywords used by the IDL
;	PLOT routine.
;
; OUTPUTS:
;	A plot of the input bands is made.
;
; RESTRICTIONS:
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; EXAMPLE:
;	
;	Create a set of bands:
;	b = COSINE_ELL_BANDS(bandcenters=[2,10,20,40,65,100], /single) 
;	
;	View them using the PLOT_BANDS routine:
;	PLOT_ELL_BANDS, b, yrange=[0,1.2]
;
; MODIFICATION HISTORY:
; 	08/12/2011:	Jacques Delabrouille, First version (name
; 	                changed from other version PLOT_BANDS).
;-
PRO PLOT_ELL_BANDS, bands, _extra=_extra

  ON_ERROR, 2

  nbands = N_ELEMENTS(bands(0,*))
  nell = N_ELEMENTS(bands(*,0))

  PLOT, bands(*,0), _extra=_extra

  IF nbands GT 1 THEN FOR i=1, nbands-1 DO OPLOT, bands(*,i), line=i MOD 2, _extra=_extra

END
